import { Observable } from '../data/observable';
/**
 * The signal class.
 * @see https://dom.spec.whatwg.org/#abortsignal
 */
var AbortSignal = /** @class */ (function (_super) {
    __extends(AbortSignal, _super);
    /**
     * AbortSignal cannot be constructed directly.
     */
    function AbortSignal() {
        return _super.call(this) || this;
    }
    Object.defineProperty(AbortSignal.prototype, "aborted", {
        /**
         * Returns `true` if this `AbortSignal`'s `AbortController` has signaled to abort, and `false` otherwise.
         */
        get: function () {
            var aborted = abortedFlags.get(this);
            if (typeof aborted !== 'boolean') {
                throw new TypeError("Expected 'this' to be an 'AbortSignal' object, but got ".concat(this === null ? 'null' : typeof this));
            }
            return aborted;
        },
        enumerable: false,
        configurable: true
    });
    return AbortSignal;
}(Observable));
export default AbortSignal;
/**
 * Create an AbortSignal object.
 */
export function createAbortSignal() {
    var signal = new AbortSignal();
    abortedFlags.set(signal, false);
    return signal;
}
/**
 * Abort a given signal.
 */
export function abortSignal(signal) {
    if (abortedFlags.get(signal) !== false) {
        return;
    }
    abortedFlags.set(signal, true);
    signal.notify({ eventName: 'abort', type: 'abort' });
}
/**
 * Aborted flag for each instances.
 */
var abortedFlags = new WeakMap();
// Properties should be enumerable.
Object.defineProperties(AbortSignal.prototype, {
    aborted: { enumerable: true },
});
// `toString()` should return `"[object AbortSignal]"`
if (typeof Symbol === 'function' && typeof Symbol.toStringTag === 'symbol') {
    Object.defineProperty(AbortSignal.prototype, Symbol.toStringTag, {
        configurable: true,
        value: 'AbortSignal',
    });
}
//# sourceMappingURL=abortsignal.js.map