import AbortSignal, { abortSignal, createAbortSignal } from './abortsignal';
/**
 * The AbortController.
 * @see https://dom.spec.whatwg.org/#abortcontroller
 */
var AbortController = /** @class */ (function () {
    /**
     * Initialize this controller.
     */
    function AbortController() {
        signals.set(this, createAbortSignal());
    }
    Object.defineProperty(AbortController.prototype, "signal", {
        /**
         * Returns the `AbortSignal` object associated with this object.
         */
        get: function () {
            return getSignal(this);
        },
        enumerable: false,
        configurable: true
    });
    /**
     * Abort and signal to any observers that the associated activity is to be aborted.
     */
    AbortController.prototype.abort = function () {
        abortSignal(getSignal(this));
    };
    return AbortController;
}());
export default AbortController;
/**
 * Associated signals.
 */
var signals = new WeakMap();
/**
 * Get the associated signal of a given controller.
 */
function getSignal(controller) {
    var signal = signals.get(controller);
    if (signal == null) {
        throw new TypeError("Expected 'this' to be an 'AbortController' object, but got ".concat(controller === null ? 'null' : typeof controller));
    }
    return signal;
}
// Properties should be enumerable.
Object.defineProperties(AbortController.prototype, {
    signal: { enumerable: true },
    abort: { enumerable: true },
});
if (typeof Symbol === 'function' && typeof Symbol.toStringTag === 'symbol') {
    Object.defineProperty(AbortController.prototype, Symbol.toStringTag, {
        configurable: true,
        value: 'AbortController',
    });
}
export { AbortController, AbortSignal };
//# sourceMappingURL=index.js.map