import { Application } from '../application';
import { AccessibilityServiceEnabledObservable } from './accessibility-service';
import { FontScaleCategory, getCurrentFontScale, getFontScaleCategory, VALID_FONT_SCALES } from './font-scale';
// CSS-classes
var fontScaleExtraSmallCategoryClass = "a11y-fontscale-xs";
var fontScaleMediumCategoryClass = "a11y-fontscale-m";
var fontScaleExtraLargeCategoryClass = "a11y-fontscale-xl";
var fontScaleCategoryClasses = [fontScaleExtraSmallCategoryClass, fontScaleMediumCategoryClass, fontScaleExtraLargeCategoryClass];
var a11yServiceEnabledClass = "a11y-service-enabled";
var a11yServiceDisabledClass = "a11y-service-disabled";
var a11yServiceClasses = [a11yServiceEnabledClass, a11yServiceDisabledClass];
var accessibilityServiceObservable;
var fontScaleCssClasses;
var currentFontScaleClass = '';
var currentFontScaleCategory = '';
var currentA11YServiceClass = '';
function ensureClasses() {
    if (accessibilityServiceObservable) {
        return;
    }
    fontScaleCssClasses = new Map(VALID_FONT_SCALES.map(function (fs) { return [fs, "a11y-fontscale-".concat(Number(fs * 100).toFixed(0))]; }));
    accessibilityServiceObservable = new AccessibilityServiceEnabledObservable();
}
function applyRootCssClass(cssClasses, newCssClass) {
    var rootView = Application.getRootView();
    if (!rootView) {
        return;
    }
    Application.applyCssClass(rootView, cssClasses, newCssClass);
    var rootModalViews = rootView._getRootModalViews();
    rootModalViews.forEach(function (rootModalView) { return Application.applyCssClass(rootModalView, cssClasses, newCssClass); });
}
function applyFontScaleToRootViews() {
    var rootView = Application.getRootView();
    if (!rootView) {
        return;
    }
    var fontScale = getCurrentFontScale();
    rootView.style.fontScaleInternal = fontScale;
    var rootModalViews = rootView._getRootModalViews();
    rootModalViews.forEach(function (rootModalView) { return (rootModalView.style.fontScaleInternal = fontScale); });
}
export function initAccessibilityCssHelper() {
    ensureClasses();
    Application.on(Application.fontScaleChangedEvent, function () {
        updateCurrentHelperClasses();
        applyFontScaleToRootViews();
    });
    accessibilityServiceObservable.on(AccessibilityServiceEnabledObservable.propertyChangeEvent, updateCurrentHelperClasses);
}
/**
 * Update the helper CSS-classes.
 * Return true is any changes.
 */
function updateCurrentHelperClasses() {
    var fontScale = getCurrentFontScale();
    var fontScaleCategory = getFontScaleCategory();
    var oldFontScaleClass = currentFontScaleClass;
    if (fontScaleCssClasses.has(fontScale)) {
        currentFontScaleClass = fontScaleCssClasses.get(fontScale);
    }
    else {
        currentFontScaleClass = fontScaleCssClasses.get(1);
    }
    if (oldFontScaleClass !== currentFontScaleClass) {
        applyRootCssClass(__spreadArray([], fontScaleCssClasses.values(), true), currentFontScaleClass);
    }
    var oldActiveFontScaleCategory = currentFontScaleCategory;
    switch (fontScaleCategory) {
        case FontScaleCategory.ExtraSmall: {
            currentFontScaleCategory = fontScaleExtraSmallCategoryClass;
            break;
        }
        case FontScaleCategory.Medium: {
            currentFontScaleCategory = fontScaleMediumCategoryClass;
            break;
        }
        case FontScaleCategory.ExtraLarge: {
            currentFontScaleCategory = fontScaleExtraLargeCategoryClass;
            break;
        }
        default: {
            currentFontScaleCategory = fontScaleMediumCategoryClass;
            break;
        }
    }
    if (oldActiveFontScaleCategory !== currentFontScaleCategory) {
        applyRootCssClass(fontScaleCategoryClasses, currentFontScaleCategory);
    }
    var oldA11YStatusClass = currentA11YServiceClass;
    if (accessibilityServiceObservable.accessibilityServiceEnabled) {
        currentA11YServiceClass = a11yServiceEnabledClass;
    }
    else {
        currentA11YServiceClass = a11yServiceDisabledClass;
    }
    if (oldA11YStatusClass !== currentA11YServiceClass) {
        applyRootCssClass(a11yServiceClasses, currentA11YServiceClass);
    }
}
//# sourceMappingURL=accessibility-css-helper.js.map