import { CssProperty, InheritedCssProperty, Property } from '../ui/core/properties';
import { booleanConverter } from '../ui/core/view-base';
import { Style } from '../ui/styling/style';
import { AccessibilityLiveRegion, AccessibilityRole, AccessibilityState } from './accessibility-types';
function makePropertyEnumConverter(enumValues) {
    return function (value) {
        if (!value || typeof value !== 'string') {
            return null;
        }
        for (var _i = 0, _a = Object.entries(enumValues); _i < _a.length; _i++) {
            var _b = _a[_i], enumKey = _b[0], enumValue = _b[1];
            if (typeof enumKey !== 'string') {
                continue;
            }
            if (enumKey === value || "".concat(enumValue).toLowerCase() === "".concat(value).toLowerCase()) {
                return enumValue;
            }
        }
        return null;
    };
}
export var accessibilityEnabledProperty = new CssProperty({
    name: 'accessible',
    cssName: 'a11y-enabled',
    valueConverter: booleanConverter,
});
accessibilityEnabledProperty.register(Style);
export var iosAccessibilityAdjustsFontSizeProperty = new InheritedCssProperty({
    defaultValue: false,
    name: 'iosAccessibilityAdjustsFontSize',
    cssName: 'ios-a11y-adjusts-font-size',
    valueConverter: booleanConverter,
});
iosAccessibilityAdjustsFontSizeProperty.register(Style);
export var iosAccessibilityMinFontScaleProperty = new InheritedCssProperty({
    defaultValue: 0,
    name: 'iosAccessibilityMinFontScale',
    cssName: 'ios-a11y-min-font-scale',
    valueConverter: parseFloat,
});
iosAccessibilityMinFontScaleProperty.register(Style);
export var iosAccessibilityMaxFontScaleProperty = new InheritedCssProperty({
    defaultValue: 0,
    name: 'iosAccessibilityMaxFontScale',
    cssName: 'ios-a11y-max-font-scale',
    valueConverter: parseFloat,
});
iosAccessibilityMaxFontScaleProperty.register(Style);
export var accessibilityHiddenProperty = new (__APPLE__ ? InheritedCssProperty : CssProperty)({
    name: 'accessibilityHidden',
    cssName: 'a11y-hidden',
    valueConverter: booleanConverter,
});
accessibilityHiddenProperty.register(Style);
export var accessibilityIdentifierProperty = new Property({
    name: 'accessibilityIdentifier',
});
export var accessibilityRoleProperty = new CssProperty({
    name: 'accessibilityRole',
    cssName: 'a11y-role',
    valueConverter: makePropertyEnumConverter(AccessibilityRole),
});
accessibilityRoleProperty.register(Style);
export var accessibilityStateProperty = new CssProperty({
    name: 'accessibilityState',
    cssName: 'a11y-state',
    valueConverter: makePropertyEnumConverter(AccessibilityState),
});
accessibilityStateProperty.register(Style);
export var accessibilityLabelProperty = new Property({
    name: 'accessibilityLabel',
});
export var accessibilityValueProperty = new Property({
    name: 'accessibilityValue',
});
export var accessibilityHintProperty = new Property({
    name: 'accessibilityHint',
});
export var accessibilityIgnoresInvertColorsProperty = new Property({
    name: 'accessibilityIgnoresInvertColors',
    valueConverter: booleanConverter,
});
export var accessibilityLiveRegionProperty = new CssProperty({
    name: 'accessibilityLiveRegion',
    cssName: 'a11y-live-region',
    defaultValue: AccessibilityLiveRegion.None,
    valueConverter: makePropertyEnumConverter(AccessibilityLiveRegion),
});
accessibilityLiveRegionProperty.register(Style);
export var accessibilityTraitsProperty = new Property({
    name: 'accessibilityTraits',
});
export var accessibilityLanguageProperty = new CssProperty({
    name: 'accessibilityLanguage',
    cssName: 'a11y-lang',
});
accessibilityLanguageProperty.register(Style);
export var accessibilityMediaSessionProperty = new CssProperty({
    name: 'accessibilityMediaSession',
    cssName: 'a11y-media-session',
});
accessibilityMediaSessionProperty.register(Style);
/**
 * Represents the observable property backing the accessibilityStep property.
 */
export var accessibilityStepProperty = new CssProperty({
    name: 'accessibilityStep',
    cssName: 'a11y-step',
    defaultValue: 10,
    valueConverter: function (v) {
        var step = parseFloat(v);
        if (isNaN(step) || step <= 0) {
            return 10;
        }
        return step;
    },
});
accessibilityStepProperty.register(Style);
//# sourceMappingURL=accessibility-properties.js.map