import { Application } from '../application';
import { Observable } from '../data/observable';
import { Trace } from '../trace';
import * as Utils from '../utils';
import { SDK_VERSION } from '../utils/constants';
import { CommonA11YServiceEnabledObservable, SharedA11YObservable } from './accessibility-service-common';
export function getAndroidAccessibilityManager() {
    var context = Utils.ad.getApplicationContext();
    if (!context) {
        return null;
    }
    return context.getSystemService(android.content.Context.ACCESSIBILITY_SERVICE);
}
var accessibilityStateEnabledPropName = 'accessibilityStateEnabled';
var touchExplorationStateEnabledPropName = 'touchExplorationStateEnabled';
var AndroidSharedA11YObservable = /** @class */ (function (_super) {
    __extends(AndroidSharedA11YObservable, _super);
    function AndroidSharedA11YObservable() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    Object.defineProperty(AndroidSharedA11YObservable.prototype, "accessibilityServiceEnabled", {
        // @ts-ignore todo: fix
        get: function () {
            return !!this[accessibilityStateEnabledPropName] && !!this[touchExplorationStateEnabledPropName];
        },
        set: function (v) {
            return;
        },
        enumerable: false,
        configurable: true
    });
    return AndroidSharedA11YObservable;
}(SharedA11YObservable));
var accessibilityStateChangeListener;
var touchExplorationStateChangeListener;
var sharedA11YObservable;
function updateAccessibilityState() {
    var accessibilityManager = getAndroidAccessibilityManager();
    if (!accessibilityManager) {
        sharedA11YObservable.set(accessibilityStateEnabledPropName, false);
        sharedA11YObservable.set(touchExplorationStateEnabledPropName, false);
        return;
    }
    sharedA11YObservable.set(accessibilityStateEnabledPropName, !!accessibilityManager.isEnabled());
    sharedA11YObservable.set(touchExplorationStateEnabledPropName, !!accessibilityManager.isTouchExplorationEnabled());
}
function ensureStateListener() {
    if (sharedA11YObservable) {
        return sharedA11YObservable;
    }
    var accessibilityManager = getAndroidAccessibilityManager();
    sharedA11YObservable = new AndroidSharedA11YObservable();
    if (!accessibilityManager) {
        sharedA11YObservable.set(accessibilityStateEnabledPropName, false);
        sharedA11YObservable.set(touchExplorationStateEnabledPropName, false);
        return sharedA11YObservable;
    }
    accessibilityStateChangeListener = new android.view.accessibility.AccessibilityManager.AccessibilityStateChangeListener({
        onAccessibilityStateChanged: function (enabled) {
            updateAccessibilityState();
            if (Trace.isEnabled()) {
                Trace.write("AccessibilityStateChangeListener state changed to: ".concat(!!enabled), Trace.categories.Accessibility);
            }
        },
    });
    accessibilityManager.addAccessibilityStateChangeListener(accessibilityStateChangeListener);
    if (SDK_VERSION >= 19) {
        touchExplorationStateChangeListener = new android.view.accessibility.AccessibilityManager.TouchExplorationStateChangeListener({
            onTouchExplorationStateChanged: function (enabled) {
                updateAccessibilityState();
                if (Trace.isEnabled()) {
                    Trace.write("TouchExplorationStateChangeListener state changed to: ".concat(!!enabled), Trace.categories.Accessibility);
                }
            },
        });
        accessibilityManager.addTouchExplorationStateChangeListener(touchExplorationStateChangeListener);
    }
    updateAccessibilityState();
    Application.on(Application.resumeEvent, updateAccessibilityState);
    Application.on(Application.exitEvent, function (args) {
        var activity = args.android;
        if (activity && !activity.isFinishing()) {
            return;
        }
        var accessibilityManager = getAndroidAccessibilityManager();
        if (accessibilityManager) {
            if (accessibilityStateChangeListener) {
                accessibilityManager.removeAccessibilityStateChangeListener(accessibilityStateChangeListener);
            }
            if (touchExplorationStateChangeListener) {
                accessibilityManager.removeTouchExplorationStateChangeListener(touchExplorationStateChangeListener);
            }
        }
        accessibilityStateChangeListener = null;
        touchExplorationStateChangeListener = null;
        if (sharedA11YObservable) {
            sharedA11YObservable.removeEventListener(Observable.propertyChangeEvent);
            sharedA11YObservable = null;
        }
        Application.off(Application.resumeEvent, updateAccessibilityState);
    });
    return sharedA11YObservable;
}
export function isAccessibilityServiceEnabled() {
    return ensureStateListener().accessibilityServiceEnabled;
}
var AccessibilityServiceEnabledObservable = /** @class */ (function (_super) {
    __extends(AccessibilityServiceEnabledObservable, _super);
    function AccessibilityServiceEnabledObservable() {
        return _super.call(this, ensureStateListener()) || this;
    }
    return AccessibilityServiceEnabledObservable;
}(CommonA11YServiceEnabledObservable));
export { AccessibilityServiceEnabledObservable };
//# sourceMappingURL=accessibility-service.android.js.map