import { Application } from '../application';
import { Observable } from '../data/observable';
import { Trace } from '../trace';
import { AccessibilityServiceEnabledPropName, CommonA11YServiceEnabledObservable, SharedA11YObservable } from './accessibility-service-common';
export function isAccessibilityServiceEnabled() {
    return getSharedA11YObservable().accessibilityServiceEnabled;
}
export function getAndroidAccessibilityManager() {
    return null;
}
var sharedA11YObservable;
var nativeObserver;
function getSharedA11YObservable() {
    if (sharedA11YObservable) {
        return sharedA11YObservable;
    }
    sharedA11YObservable = new SharedA11YObservable();
    var isVoiceOverRunning;
    if (typeof UIAccessibilityIsVoiceOverRunning === 'function') {
        isVoiceOverRunning = UIAccessibilityIsVoiceOverRunning;
    }
    else {
        if (typeof UIAccessibilityIsVoiceOverRunning !== 'function') {
            Trace.write("UIAccessibilityIsVoiceOverRunning() - is not a function", Trace.categories.Accessibility, Trace.messageType.error);
            isVoiceOverRunning = function () { return false; };
        }
    }
    sharedA11YObservable.set(AccessibilityServiceEnabledPropName, isVoiceOverRunning());
    var voiceOverStatusChangedNotificationName = null;
    if (typeof UIAccessibilityVoiceOverStatusDidChangeNotification !== 'undefined') {
        // iOS 11+
        voiceOverStatusChangedNotificationName = UIAccessibilityVoiceOverStatusDidChangeNotification;
    }
    else if (typeof UIAccessibilityVoiceOverStatusChanged !== 'undefined') {
        // iOS <11
        voiceOverStatusChangedNotificationName = UIAccessibilityVoiceOverStatusChanged;
    }
    if (voiceOverStatusChangedNotificationName) {
        nativeObserver = Application.ios.addNotificationObserver(voiceOverStatusChangedNotificationName, function () {
            sharedA11YObservable === null || sharedA11YObservable === void 0 ? void 0 : sharedA11YObservable.set(AccessibilityServiceEnabledPropName, isVoiceOverRunning());
        });
        Application.on(Application.exitEvent, function () {
            if (nativeObserver) {
                Application.ios.removeNotificationObserver(nativeObserver, voiceOverStatusChangedNotificationName);
            }
            nativeObserver = null;
            if (sharedA11YObservable) {
                sharedA11YObservable.removeEventListener(Observable.propertyChangeEvent);
                sharedA11YObservable = null;
            }
        });
    }
    Application.on(Application.resumeEvent, function () { return sharedA11YObservable.set(AccessibilityServiceEnabledPropName, isVoiceOverRunning()); });
    return sharedA11YObservable;
}
var AccessibilityServiceEnabledObservable = /** @class */ (function (_super) {
    __extends(AccessibilityServiceEnabledObservable, _super);
    function AccessibilityServiceEnabledObservable() {
        return _super.call(this, getSharedA11YObservable()) || this;
    }
    return AccessibilityServiceEnabledObservable;
}(CommonA11YServiceEnabledObservable));
export { AccessibilityServiceEnabledObservable };
//# sourceMappingURL=accessibility-service.ios.js.map