import { Application } from '../application';
import { FontScaleCategory, getClosestValidFontScale } from './font-scale-common';
export * from './font-scale-common';
var currentFontScale = null;
function fontScaleChanged(origFontScale) {
    var oldValue = currentFontScale;
    currentFontScale = getClosestValidFontScale(origFontScale);
    if (oldValue !== currentFontScale) {
        Application.notify({
            eventName: Application.fontScaleChangedEvent,
            object: Application,
            newValue: currentFontScale,
        });
    }
}
export function getCurrentFontScale() {
    setupConfigListener();
    return currentFontScale;
}
export function getFontScaleCategory() {
    return FontScaleCategory.Medium;
}
function useAndroidFontScale() {
    fontScaleChanged(Number(Application.android.context.getResources().getConfiguration().fontScale));
}
var configChangedCallback;
function setupConfigListener() {
    var _a;
    if (configChangedCallback) {
        return;
    }
    Application.off(Application.launchEvent, setupConfigListener);
    var context = (_a = Application.android) === null || _a === void 0 ? void 0 : _a.context;
    if (!context) {
        Application.on(Application.launchEvent, setupConfigListener);
        return;
    }
    useAndroidFontScale();
    configChangedCallback = new android.content.ComponentCallbacks2({
        onLowMemory: function () {
            // Dummy
        },
        onTrimMemory: function () {
            // Dummy
        },
        onConfigurationChanged: function (newConfig) {
            fontScaleChanged(Number(newConfig.fontScale));
        },
    });
    context.registerComponentCallbacks(configChangedCallback);
    Application.on(Application.resumeEvent, useAndroidFontScale);
}
export function initAccessibilityFontScale() {
    setupConfigListener();
}
//# sourceMappingURL=font-scale.android.js.map