import { queueMacrotask } from '../utils/macrotask-scheduler';
import { FPSCallback } from '../fps-meter/fps-native';
import { getTimeInFrameBase } from './animation-native';
import { Trace } from '../trace';
var animationId = 0;
var currentFrameAnimationCallbacks = {}; // requests that were scheduled in this frame and must be called ASAP
var currentFrameScheduled = false;
var nextFrameAnimationCallbacks = {}; // requests there were scheduled in another request and must be called in the next frame
var shouldStop = true;
var inAnimationFrame = false;
var fpsCallback;
var lastFrameTime = 0;
function getNewId() {
    return animationId++;
}
function ensureNative() {
    if (fpsCallback) {
        return;
    }
    fpsCallback = new FPSCallback(doFrame);
}
function callAnimationCallbacks(thisFrameCbs, frameTime) {
    inAnimationFrame = true;
    for (var animationId_1 in thisFrameCbs) {
        if (thisFrameCbs[animationId_1]) {
            try {
                thisFrameCbs[animationId_1](frameTime);
            }
            catch (err) {
                var msg_1 = err ? err.stack || err : err;
                Trace.write("Error in requestAnimationFrame: ".concat(msg_1), Trace.categories.Error, Trace.messageType.error);
            }
        }
    }
    inAnimationFrame = false;
}
function doCurrentFrame() {
    // if we're not getting accurate frame times
    // set last frame time as the current time
    if (!fpsCallback || !fpsCallback.running) {
        lastFrameTime = getTimeInFrameBase();
    }
    currentFrameScheduled = false;
    var thisFrameCbs = currentFrameAnimationCallbacks;
    currentFrameAnimationCallbacks = {};
    callAnimationCallbacks(thisFrameCbs, lastFrameTime);
}
function doFrame(currentTimeMillis) {
    lastFrameTime = currentTimeMillis;
    shouldStop = true;
    var thisFrameCbs = nextFrameAnimationCallbacks;
    nextFrameAnimationCallbacks = {};
    callAnimationCallbacks(thisFrameCbs, lastFrameTime);
    if (shouldStop) {
        fpsCallback.stop(); // TODO: check performance without stopping to allow consistent frame times
    }
}
function ensureCurrentFrameScheduled() {
    if (!currentFrameScheduled) {
        currentFrameScheduled = true;
        queueMacrotask(doCurrentFrame);
    }
}
export function requestAnimationFrame(cb) {
    var animId = getNewId();
    if (!inAnimationFrame) {
        ensureCurrentFrameScheduled();
        currentFrameAnimationCallbacks[animId] = zonedCallback(cb);
        return animId;
    }
    ensureNative();
    nextFrameAnimationCallbacks[animId] = zonedCallback(cb);
    shouldStop = false;
    fpsCallback.start();
    return animId;
}
export function cancelAnimationFrame(id) {
    delete currentFrameAnimationCallbacks[id];
    delete nextFrameAnimationCallbacks[id];
}
//# sourceMappingURL=index.js.map