import { profile } from '../profiling';
import { isEmbedded } from '../ui/embedding';
import { IOSHelper } from '../ui/core/view/view-helper';
import * as Utils from '../utils';
import { ApplicationCommon } from './application-common';
var CADisplayLinkTarget = /** @class */ (function (_super) {
    __extends(CADisplayLinkTarget, _super);
    function CADisplayLinkTarget() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    CADisplayLinkTarget.initWithOwner = function (owner) {
        var target = CADisplayLinkTarget.new();
        target._owner = owner;
        return target;
    };
    CADisplayLinkTarget.prototype.onDisplayed = function (link) {
        link.invalidate();
        var owner = this._owner.deref();
        if (!owner) {
            return;
        }
        owner.displayedOnce = true;
        owner.notify({
            eventName: owner.displayedEvent,
            object: owner,
            ios: UIApplication.sharedApplication,
        });
        owner.displayedLinkTarget = null;
        owner.displayedLink = null;
    };
    CADisplayLinkTarget.ObjCExposedMethods = {
        onDisplayed: { returns: interop.types.void, params: [CADisplayLink] },
    };
    return CADisplayLinkTarget;
}(NSObject));
var NotificationObserver = /** @class */ (function (_super) {
    __extends(NotificationObserver, _super);
    function NotificationObserver() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    NotificationObserver.initWithCallback = function (onReceiveCallback) {
        var observer = _super.new.call(this);
        observer._onReceiveCallback = onReceiveCallback;
        return observer;
    };
    NotificationObserver.prototype.onReceive = function (notification) {
        this._onReceiveCallback(notification);
    };
    NotificationObserver.ObjCExposedMethods = {
        onReceive: { returns: interop.types.void, params: [NSNotification] },
    };
    return NotificationObserver;
}(NSObject));
var Responder = /** @class */ (function (_super) {
    __extends(Responder, _super);
    function Responder() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    Object.defineProperty(Responder.prototype, "window", {
        get: function () {
            return Application.ios.window;
        },
        set: function (value) {
            // NOOP
        },
        enumerable: true,
        configurable: true
    });
    Responder.ObjCProtocols = [UIApplicationDelegate];
    return Responder;
}(UIResponder));
var iOSApplication = /** @class */ (function (_super) {
    __extends(iOSApplication, _super);
    /**
     * @internal - should not be constructed by the user.
     */
    function iOSApplication() {
        var _this = _super.call(this) || this;
        _this._delegateHandlers = new Map();
        _this._notificationObservers = [];
        _this.displayedOnce = false;
        _this.addNotificationObserver(UIApplicationDidFinishLaunchingNotification, _this.didFinishLaunchingWithOptions.bind(_this));
        _this.addNotificationObserver(UIApplicationDidBecomeActiveNotification, _this.didBecomeActive.bind(_this));
        _this.addNotificationObserver(UIApplicationDidEnterBackgroundNotification, _this.didEnterBackground.bind(_this));
        _this.addNotificationObserver(UIApplicationWillTerminateNotification, _this.willTerminate.bind(_this));
        _this.addNotificationObserver(UIApplicationDidReceiveMemoryWarningNotification, _this.didReceiveMemoryWarning.bind(_this));
        _this.addNotificationObserver(UIApplicationDidChangeStatusBarOrientationNotification, _this.didChangeStatusBarOrientation.bind(_this));
        return _this;
    }
    iOSApplication.prototype.getRootView = function () {
        return this._rootView;
    };
    iOSApplication.prototype.resetRootView = function (view) {
        _super.prototype.resetRootView.call(this, view);
        this.setWindowContent();
    };
    iOSApplication.prototype.run = function (entry) {
        this.mainEntry = typeof entry === 'string' ? { moduleName: entry } : entry;
        this.started = true;
        if (this.nativeApp) {
            this.runAsEmbeddedApp();
        }
        else {
            this.runAsMainApp();
        }
    };
    iOSApplication.prototype.runAsMainApp = function () {
        UIApplicationMain(0, null, null, this.delegate ? NSStringFromClass(this.delegate) : NSStringFromClass(Responder));
    };
    iOSApplication.prototype.runAsEmbeddedApp = function () {
        var _this = this;
        // TODO: this rootView should be held alive until rootController dismissViewController is called.
        var rootView = this.createRootView(this._rootView, true);
        if (!rootView) {
            return;
        }
        this._rootView = rootView;
        // Attach to the existing iOS app
        var window = Utils.ios.getWindow();
        if (!window) {
            return;
        }
        var rootController = window.rootViewController;
        if (!rootController) {
            return;
        }
        var controller = this.getViewController(rootView);
        var embedderDelegate = NativeScriptEmbedder.sharedInstance().delegate;
        rootView._setupAsRootView({});
        rootView.on(IOSHelper.traitCollectionColorAppearanceChangedEvent, function () {
            var userInterfaceStyle = controller.traitCollection.userInterfaceStyle;
            var newSystemAppearance = _this.getSystemAppearanceValue(userInterfaceStyle);
            _this.setSystemAppearance(newSystemAppearance);
        });
        if (embedderDelegate) {
            this.setViewControllerView(rootView);
            embedderDelegate.presentNativeScriptApp(controller);
        }
        else {
            var visibleVC = Utils.ios.getVisibleViewController(rootController);
            visibleVC.presentViewControllerAnimatedCompletion(controller, true, null);
        }
        this.initRootView(rootView);
        this.notifyAppStarted();
    };
    iOSApplication.prototype.getViewController = function (rootView) {
        var viewController = rootView.viewController || rootView.ios;
        if (!(viewController instanceof UIViewController)) {
            // We set UILayoutViewController dynamically to the root view if it doesn't have a view controller
            // At the moment the root view doesn't have its native view created. We set it in the setViewControllerView func
            viewController = IOSHelper.UILayoutViewController.initWithOwner(new WeakRef(rootView));
            rootView.viewController = viewController;
        }
        return viewController;
    };
    iOSApplication.prototype.setViewControllerView = function (view) {
        var viewController = view.viewController || view.ios;
        var nativeView = view.ios || view.nativeViewProtected;
        if (!nativeView || !viewController) {
            throw new Error('Root should be either UIViewController or UIView');
        }
        if (viewController instanceof IOSHelper.UILayoutViewController) {
            viewController.view.addSubview(nativeView);
        }
    };
    iOSApplication.prototype.setMaxRefreshRate = function (options) {
        var _this = this;
        var adjustRefreshRate = function () {
            if (!_this.displayedLink) {
                return;
            }
            var minFrameRateDisabled = NSBundle.mainBundle.objectForInfoDictionaryKey('CADisableMinimumFrameDurationOnPhone');
            if (minFrameRateDisabled) {
                var max = 120;
                var deviceMaxFrames = Utils.ios.getMainScreen().maximumFramesPerSecond;
                if (options === null || options === void 0 ? void 0 : options.max) {
                    if (deviceMaxFrames) {
                        // iOS 10.3
                        max = options.max <= deviceMaxFrames ? options.max : deviceMaxFrames;
                    }
                    else if (_this.displayedLink.preferredFramesPerSecond) {
                        // iOS 10.0
                        max = options.max <= _this.displayedLink.preferredFramesPerSecond ? options.max : _this.displayedLink.preferredFramesPerSecond;
                    }
                }
                if (Utils.SDK_VERSION >= 15 || __VISIONOS__) {
                    var min = (options === null || options === void 0 ? void 0 : options.min) || max / 2;
                    var preferred = (options === null || options === void 0 ? void 0 : options.preferred) || max;
                    _this.displayedLink.preferredFrameRateRange = CAFrameRateRangeMake(min, max, preferred);
                }
                else {
                    _this.displayedLink.preferredFramesPerSecond = max;
                }
            }
        };
        if (this.displayedOnce) {
            adjustRefreshRate();
            return;
        }
        this.displayedLinkTarget = CADisplayLinkTarget.initWithOwner(new WeakRef(this));
        this.displayedLink = CADisplayLink.displayLinkWithTargetSelector(this.displayedLinkTarget, 'onDisplayed');
        adjustRefreshRate();
        this.displayedLink.addToRunLoopForMode(NSRunLoop.mainRunLoop, NSDefaultRunLoopMode);
        this.displayedLink.addToRunLoopForMode(NSRunLoop.mainRunLoop, UITrackingRunLoopMode);
    };
    Object.defineProperty(iOSApplication.prototype, "rootController", {
        get: function () {
            var _a;
            return (_a = this.window) === null || _a === void 0 ? void 0 : _a.rootViewController;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(iOSApplication.prototype, "nativeApp", {
        get: function () {
            return UIApplication.sharedApplication;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(iOSApplication.prototype, "window", {
        get: function () {
            // TODO: consideration
            // may not want to cache this value given the potential of multiple scenes
            // particularly with SwiftUI app lifecycle based apps
            if (!this._window) {
                // Note: NativeScriptViewFactory.getKeyWindow will always be used in SwiftUI app lifecycle based apps
                this._window = Utils.ios.getWindow();
            }
            return this._window;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(iOSApplication.prototype, "delegate", {
        get: function () {
            return this._delegate;
        },
        set: function (value) {
            if (this._delegate !== value) {
                this._delegate = value;
            }
        },
        enumerable: false,
        configurable: true
    });
    iOSApplication.prototype.addDelegateHandler = function (methodName, handler) {
        var _a, _b;
        // safe-guard against invalid handlers
        if (typeof handler !== 'function') {
            return;
        }
        // ensure we have a delegate
        (_a = this.delegate) !== null && _a !== void 0 ? _a : (this.delegate = Responder);
        var handlers = (_b = this._delegateHandlers.get(methodName)) !== null && _b !== void 0 ? _b : [];
        if (!this._delegateHandlers.has(methodName)) {
            var originalHandler = this.delegate.prototype[methodName];
            if (originalHandler) {
                // if there is an original handler, we add it to the handlers array to be called first.
                handlers.push(originalHandler);
            }
            // replace the original method implementation with one that will call all handlers.
            this.delegate.prototype[methodName] = function () {
                var args = [];
                for (var _i = 0; _i < arguments.length; _i++) {
                    args[_i] = arguments[_i];
                }
                var res;
                for (var _a = 0, handlers_1 = handlers; _a < handlers_1.length; _a++) {
                    var handler_1 = handlers_1[_a];
                    if (typeof handler_1 !== 'function') {
                        continue;
                    }
                    res = handler_1.apply(this, args);
                }
                return res;
            };
            // store the handlers
            this._delegateHandlers.set(methodName, handlers);
        }
        handlers.push(handler);
    };
    iOSApplication.prototype.getNativeApplication = function () {
        return this.nativeApp;
    };
    iOSApplication.prototype.addNotificationObserver = function (notificationName, onReceiveCallback) {
        var observer = NotificationObserver.initWithCallback(onReceiveCallback);
        NSNotificationCenter.defaultCenter.addObserverSelectorNameObject(observer, 'onReceive', notificationName, null);
        this._notificationObservers.push(observer);
        return observer;
    };
    iOSApplication.prototype.removeNotificationObserver = function (observer, notificationName) {
        var index = this._notificationObservers.indexOf(observer);
        if (index >= 0) {
            this._notificationObservers.splice(index, 1);
            NSNotificationCenter.defaultCenter.removeObserverNameObject(observer, notificationName, null);
        }
    };
    iOSApplication.prototype.getSystemAppearance = function () {
        // userInterfaceStyle is available on UITraitCollection since iOS 12.
        if ((!__VISIONOS__ && Utils.SDK_VERSION <= 11) || !this.rootController) {
            return null;
        }
        var userInterfaceStyle = this.rootController.traitCollection.userInterfaceStyle;
        return this.getSystemAppearanceValue(userInterfaceStyle);
    };
    iOSApplication.prototype.getSystemAppearanceValue = function (userInterfaceStyle) {
        switch (userInterfaceStyle) {
            case 2 /* UIUserInterfaceStyle.Dark */:
                return 'dark';
            case 1 /* UIUserInterfaceStyle.Light */:
            case 0 /* UIUserInterfaceStyle.Unspecified */:
                return 'light';
        }
    };
    iOSApplication.prototype.getOrientation = function () {
        var statusBarOrientation;
        if (__VISIONOS__) {
            statusBarOrientation = NativeScriptEmbedder.sharedInstance().windowScene.interfaceOrientation;
        }
        else {
            statusBarOrientation = UIApplication.sharedApplication.statusBarOrientation;
        }
        return this.getOrientationValue(statusBarOrientation);
    };
    iOSApplication.prototype.getOrientationValue = function (orientation) {
        switch (orientation) {
            case 3 /* UIInterfaceOrientation.LandscapeRight */:
            case 4 /* UIInterfaceOrientation.LandscapeLeft */:
                return 'landscape';
            case 2 /* UIInterfaceOrientation.PortraitUpsideDown */:
            case 1 /* UIInterfaceOrientation.Portrait */:
                return 'portrait';
            case 0 /* UIInterfaceOrientation.Unknown */:
                return 'unknown';
        }
    };
    iOSApplication.prototype.notifyAppStarted = function (notification) {
        var _a, _b;
        var root = this.notifyLaunch({
            ios: (_b = (_a = notification === null || notification === void 0 ? void 0 : notification.userInfo) === null || _a === void 0 ? void 0 : _a.objectForKey('UIApplicationLaunchOptionsLocalNotificationKey')) !== null && _b !== void 0 ? _b : null,
        });
        if (this._window) {
            if (root !== null && !isEmbedded()) {
                this.setWindowContent(root);
            }
        }
        else {
            this._window = this.window; // UIApplication.sharedApplication.keyWindow;
        }
    };
    iOSApplication.prototype._onLivesync = function (context) {
        // Handle application root module
        var isAppRootModuleChanged = context && context.path && context.path.includes(this.getMainEntry().moduleName) && context.type !== 'style';
        // Set window content when:
        // + Application root module is changed
        // + View did not handle the change
        // Note:
        // The case when neither app root module is changed, nor livesync is handled on View,
        // then changes will not apply until navigate forward to the module.
        if (isAppRootModuleChanged || (this._rootView && !this._rootView._onLivesync(context))) {
            this.setWindowContent();
        }
    };
    iOSApplication.prototype.setWindowContent = function (view) {
        var _this = this;
        if (this._rootView) {
            // if we already have a root view, we reset it.
            this._rootView._onRootViewReset();
        }
        var rootView = this.createRootView(view);
        var controller = this.getViewController(rootView);
        this._rootView = rootView;
        // setup view as styleScopeHost
        rootView._setupAsRootView({});
        this.setViewControllerView(rootView);
        var win = this.window;
        var haveController = win.rootViewController !== null;
        win.rootViewController = controller;
        if (!haveController) {
            win.makeKeyAndVisible();
        }
        this.initRootView(rootView);
        rootView.on(IOSHelper.traitCollectionColorAppearanceChangedEvent, function () {
            var userInterfaceStyle = controller.traitCollection.userInterfaceStyle;
            var newSystemAppearance = _this.getSystemAppearanceValue(userInterfaceStyle);
            _this.setSystemAppearance(newSystemAppearance);
        });
    };
    // Observers
    iOSApplication.prototype.didFinishLaunchingWithOptions = function (notification) {
        this.setMaxRefreshRate();
        // ensures window is assigned to proper window scene
        this._window = this.window;
        if (!this._window) {
            // if still no window, create one
            this._window = UIWindow.alloc().initWithFrame(UIScreen.mainScreen.bounds);
        }
        if (!__VISIONOS__) {
            this.window.backgroundColor = Utils.SDK_VERSION <= 12 || !UIColor.systemBackgroundColor ? UIColor.whiteColor : UIColor.systemBackgroundColor;
        }
        this.notifyAppStarted(notification);
    };
    iOSApplication.prototype.didBecomeActive = function (notification) {
        var additionalData = {
            ios: UIApplication.sharedApplication,
        };
        this.setInBackground(false, additionalData);
        this.setSuspended(false, additionalData);
        var rootView = this._rootView;
        if (rootView && !rootView.isLoaded) {
            rootView.callLoaded();
        }
    };
    iOSApplication.prototype.didEnterBackground = function (notification) {
        var additionalData = {
            ios: UIApplication.sharedApplication,
        };
        this.setInBackground(true, additionalData);
        this.setSuspended(true, additionalData);
        var rootView = this._rootView;
        if (rootView && rootView.isLoaded) {
            rootView.callUnloaded();
        }
    };
    iOSApplication.prototype.willTerminate = function (notification) {
        this.notify({
            eventName: this.exitEvent,
            object: this,
            ios: this.ios,
        });
        // const rootView = this._rootView;
        // if (rootView && rootView.isLoaded) {
        // 	rootView.callUnloaded();
        // }
    };
    iOSApplication.prototype.didReceiveMemoryWarning = function (notification) {
        this.notify({
            eventName: this.lowMemoryEvent,
            object: this,
            ios: this.ios,
        });
    };
    iOSApplication.prototype.didChangeStatusBarOrientation = function (notification) {
        var statusBarOrientation = UIApplication.sharedApplication.statusBarOrientation;
        var newOrientation = this.getOrientationValue(statusBarOrientation);
        this.setOrientation(newOrientation);
    };
    Object.defineProperty(iOSApplication.prototype, "ios", {
        get: function () {
            // ensures Application.ios is defined when running on iOS
            return this;
        },
        enumerable: false,
        configurable: true
    });
    __decorate([
        profile,
        __metadata("design:type", Function),
        __metadata("design:paramtypes", [NSNotification]),
        __metadata("design:returntype", void 0)
    ], iOSApplication.prototype, "didFinishLaunchingWithOptions", null);
    __decorate([
        profile,
        __metadata("design:type", Function),
        __metadata("design:paramtypes", [NSNotification]),
        __metadata("design:returntype", void 0)
    ], iOSApplication.prototype, "didBecomeActive", null);
    return iOSApplication;
}(ApplicationCommon));
export { iOSApplication };
var iosApp = new iOSApplication();
// Attach on global, so it can also be overwritten to implement different logic based on flavor
global.__onLiveSyncCore = function (context) {
    iosApp._onLivesync(context);
};
export * from './application-common';
export var Application = iosApp;
export var AndroidApplication = undefined;
//# sourceMappingURL=application.ios.js.map