//Requires
import { getNodeById } from './dom-node';
import { mainThreadify } from '../utils';
// Use lazy requires for core modules
var getAppRootView = function () { return require('../application').getRootView(); };
var unsetValue;
function unsetViewValue(view, name) {
    if (!unsetValue) {
        unsetValue = require('../ui/core/properties').unsetValue;
    }
    view[name] = unsetValue;
}
function getViewById(nodeId) {
    var node = getNodeById(nodeId);
    var view;
    if (node) {
        view = node.viewRef.get();
    }
    return view;
}
export function getDocument() {
    var appRoot = getAppRootView();
    if (!appRoot) {
        return undefined;
    }
    try {
        appRoot.ensureDomNode();
    }
    catch (e) {
        console.log('ERROR in getDocument(): ' + e);
    }
    return appRoot.domNode.toObject();
}
export function getComputedStylesForNode(nodeId) {
    var view = getViewById(nodeId);
    if (view) {
        return view.domNode.getComputedProperties();
    }
    return [];
}
export var removeNode = mainThreadify(function removeNode(nodeId) {
    var view = getViewById(nodeId);
    if (view) {
        // Avoid importing layout and content view
        var parent_1 = view.parent;
        if (parent_1.removeChild) {
            parent_1.removeChild(view);
        }
        else if (parent_1.content === view) {
            parent_1.content = null;
        }
        else {
            console.log("Can't remove child from " + parent_1);
        }
    }
});
export var setAttributeAsText = mainThreadify(function setAttributeAsText(nodeId, text, name) {
    var view = getViewById(nodeId);
    if (view) {
        // attribute is registered for the view instance
        var hasOriginalAttribute = !!name.trim();
        if (text) {
            var textParts = text.split('=');
            if (textParts.length === 2) {
                var attrName = textParts[0];
                var attrValue = textParts[1].replace(/['"]+/g, '');
                // if attr name is being replaced with another
                if (name !== attrName && hasOriginalAttribute) {
                    unsetViewValue(view, name);
                    view[attrName] = attrValue;
                }
                else {
                    view[hasOriginalAttribute ? name : attrName] = attrValue;
                }
            }
        }
        else {
            // delete attribute
            unsetViewValue(view, name);
        }
        view.domNode.loadAttributes();
    }
});
//# sourceMappingURL=devtools-elements.common.js.map