var inspectorCommands = require('./InspectorBackendCommands');
import * as debuggerDomains from '.';
var getApplicationContext = function () { return require('../utils/android').getApplicationContext(); };
var frameId = 'NativeScriptMainFrameIdentifier';
var loaderId = 'Loader Identifier';
var resources_datas = [];
var documentTypeByMimeType = {
    'text/xml': 'Document',
    'text/plain': 'Document',
    'text/html': 'Document',
    'application/xml': 'Document',
    'application/xhtml+xml': 'Document',
    'text/css': 'Stylesheet',
    'text/javascript': 'Script',
    'text/ecmascript': 'Script',
    'application/javascript': 'Script',
    'application/ecmascript': 'Script',
    'application/x-javascript': 'Script',
    'application/json': 'Script',
    'application/x-json': 'Script',
    'text/x-javascript': 'Script',
    'text/x-json': 'Script',
    'text/typescript': 'Script',
};
var Request = /** @class */ (function () {
    function Request(_networkDomainDebugger, _requestID) {
        this._networkDomainDebugger = _networkDomainDebugger;
        this._requestID = _requestID;
    }
    Object.defineProperty(Request.prototype, "mimeType", {
        get: function () {
            return this._mimeType;
        },
        set: function (value) {
            if (this._mimeType !== value) {
                if (!value) {
                    this._mimeType = 'text/plain';
                    this._resourceType = 'Other';
                    return;
                }
                this._mimeType = value;
                var resourceType = 'Other';
                if (this._mimeType in documentTypeByMimeType) {
                    resourceType = documentTypeByMimeType[this._mimeType];
                }
                if (this._mimeType.indexOf('image/') !== -1) {
                    resourceType = 'Image';
                }
                if (this._mimeType.indexOf('font/') !== -1) {
                    resourceType = 'Font';
                }
                this._resourceType = resourceType;
            }
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(Request.prototype, "requestID", {
        get: function () {
            return this._requestID;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(Request.prototype, "hasTextContent", {
        get: function () {
            return ['Document', 'Stylesheet', 'Script', 'XHR'].indexOf(this._resourceType) !== -1;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(Request.prototype, "data", {
        get: function () {
            return this._data;
        },
        set: function (value) {
            if (this._data !== value) {
                this._data = value;
            }
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(Request.prototype, "resourceType", {
        get: function () {
            return this._resourceType;
        },
        set: function (value) {
            if (this._resourceType !== value) {
                this._resourceType = value;
            }
        },
        enumerable: false,
        configurable: true
    });
    Request.prototype.responseReceived = function (response) {
        if (this._networkDomainDebugger.enabled) {
            this._networkDomainDebugger.events.responseReceived(this.requestID, frameId, loaderId, __inspectorTimestamp(), this.resourceType, response);
        }
    };
    Request.prototype.loadingFinished = function () {
        if (this._networkDomainDebugger.enabled) {
            this._networkDomainDebugger.events.loadingFinished(this.requestID, __inspectorTimestamp());
        }
    };
    Request.prototype.requestWillBeSent = function (request) {
        if (this._networkDomainDebugger.enabled) {
            this._networkDomainDebugger.events.requestWillBeSent(this.requestID, frameId, loaderId, request.url, request, __inspectorTimestamp(), { type: 'Script' });
        }
    };
    return Request;
}());
export { Request };
var NetworkDomainDebugger = /** @class */ (function () {
    function NetworkDomainDebugger() {
        this.events = new inspectorCommands.NetworkDomain.NetworkFrontend();
        // By default start enabled because we can miss the "enable" event when
        // running with `--debug-brk` -- the frontend will send it before we've been created
        this.enable();
    }
    NetworkDomainDebugger_1 = NetworkDomainDebugger;
    Object.defineProperty(NetworkDomainDebugger.prototype, "enabled", {
        get: function () {
            return this._enabled;
        },
        enumerable: false,
        configurable: true
    });
    /**
     * Enables network tracking, network events will now be delivered to the client.
     */
    NetworkDomainDebugger.prototype.enable = function () {
        if (debuggerDomains.getNetwork()) {
            throw new Error('One NetworkDomainDebugger may be enabled at a time.');
        }
        else {
            debuggerDomains.setNetwork(this);
        }
        this._enabled = true;
    };
    /**
     * Disables network tracking, prevents network events from being sent to the client.
     */
    NetworkDomainDebugger.prototype.disable = function () {
        if (debuggerDomains.getNetwork() === this) {
            debuggerDomains.setNetwork(null);
        }
        this._enabled = false;
    };
    /**
     * Specifies whether to always send extra HTTP headers with the requests from this page.
     */
    NetworkDomainDebugger.prototype.setExtraHTTPHeaders = function (params) {
        //
    };
    /**
     * Returns content served for the given request.
     */
    NetworkDomainDebugger.prototype.getResponseBody = function (params) {
        var _a, _b;
        var resource_data = resources_datas[params.requestId];
        // java.io.ByteArrayOutputStream
        var body = resource_data.hasTextContent ? resource_data.data.toString('UTF-8') : android.util.Base64.encodeToString((_b = (_a = resource_data.data) === null || _a === void 0 ? void 0 : _a.buf) === null || _b === void 0 ? void 0 : _b.call(_a), android.util.Base64.NO_WRAP);
        if (resource_data) {
            return {
                body: body,
                base64Encoded: !resource_data.hasTextContent,
            };
        }
    };
    /**
     * Tells whether clearing browser cache is supported.
     */
    NetworkDomainDebugger.prototype.canClearBrowserCache = function () {
        return {
            result: false,
        };
    };
    /**
     * Clears browser cache.
     */
    NetworkDomainDebugger.prototype.clearBrowserCache = function () {
        //
    };
    /**
     * Tells whether clearing browser cookies is supported.
     */
    NetworkDomainDebugger.prototype.canClearBrowserCookies = function () {
        return {
            result: false,
        };
    };
    /**
     * Clears browser cookies.
     */
    NetworkDomainDebugger.prototype.clearBrowserCookies = function () {
        //
    };
    /**
     * Toggles ignoring cache for each request. If <code>true</code>, cache will not be used.
     */
    NetworkDomainDebugger.prototype.setCacheDisabled = function (params) {
        //
    };
    /**
     * Loads a resource in the context of a frame on the inspected page without cross origin checks.
     */
    NetworkDomainDebugger.prototype.loadResource = function (params) {
        var appPath = getApplicationContext().getFilesDir().getCanonicalPath() + '/app';
        var pathUrl = params.url.replace('file://', appPath);
        var file = new java.io.File(pathUrl);
        var is = file.exists() ? new java.io.FileInputStream(file) : undefined;
        var data = is ? Array.create('bytes', file.length()) : undefined;
        var read = data ? is.read(data) : 0;
        var content = read ? new java.lang.String(data) : '';
        return {
            content: content.toString(), // Not sure why however we need to call toString() for NSString
            mimeType: 'application/octet-stream',
            status: 200,
        };
    };
    NetworkDomainDebugger.prototype.create = function () {
        var id = (++NetworkDomainDebugger_1.idSequence).toString();
        var resourceData = new Request(this, id);
        resources_datas[id] = resourceData;
        return resourceData;
    };
    var NetworkDomainDebugger_1;
    NetworkDomainDebugger.idSequence = 0;
    NetworkDomainDebugger = NetworkDomainDebugger_1 = __decorate([
        inspectorCommands.DomainDispatcher('Network'),
        __metadata("design:paramtypes", [])
    ], NetworkDomainDebugger);
    return NetworkDomainDebugger;
}());
export { NetworkDomainDebugger };
//# sourceMappingURL=webinspector-network.android.js.map