import { SDK_VERSION } from '../utils/constants';
var FPSCallback = /** @class */ (function () {
    function FPSCallback(onFrame) {
        var _this = this;
        this.running = false;
        this.onFrame = onFrame;
        this.nativeFramesSupported = SDK_VERSION >= 24 && this._isNativeFramesSupported();
        if (this.nativeFramesSupported) {
            this.impl = function (nanos) {
                _this.handleFrame(nanos);
            };
        }
        else {
            this.impl = new android.view.Choreographer.FrameCallback({
                doFrame: function (nanos) {
                    _this.handleFrame(nanos);
                },
            });
        }
    }
    FPSCallback.prototype._isNativeFramesSupported = function () {
        return typeof global.__postFrameCallback === 'function' && typeof global.__removeFrameCallback === 'function';
    };
    FPSCallback.prototype.start = function () {
        if (this.running) {
            return;
        }
        if (this.nativeFramesSupported) {
            global.__postFrameCallback(this.impl);
        }
        else {
            android.view.Choreographer.getInstance().postFrameCallback(this.impl);
        }
        this.running = true;
    };
    FPSCallback.prototype.stop = function () {
        if (!this.running) {
            return;
        }
        if (this.nativeFramesSupported) {
            global.__removeFrameCallback(this.impl);
        }
        else {
            android.view.Choreographer.getInstance().removeFrameCallback(this.impl);
        }
        this.running = false;
    };
    FPSCallback.prototype.handleFrame = function (nanos) {
        if (!this.running) {
            return;
        }
        // divide by 1 000 000 since the parameter is in nanoseconds
        this.onFrame(nanos / 1000000);
        // add the FrameCallback instance again since it is automatically removed from the Choreographer
        if (this.nativeFramesSupported) {
            global.__postFrameCallback(this.impl);
        }
        else {
            android.view.Choreographer.getInstance().postFrameCallback(this.impl);
        }
    };
    return FPSCallback;
}());
export { FPSCallback };
//# sourceMappingURL=fps-native.android.js.map