import { Observable } from '../data/observable';
import { Screen } from '../platform';
import { Application } from '../application';
import { matchQuery, MediaQueryType } from '../css-mediaquery';
import { Trace } from '../trace';
var mediaQueryLists = [];
var applicationEvents = [Application.orientationChangedEvent, Application.systemAppearanceChangedEvent];
// In browser, developers cannot create MediaQueryList instances without calling matchMedia
var isMediaInitializationEnabled = false;
function toggleApplicationEventListeners(toAdd) {
    for (var _i = 0, applicationEvents_1 = applicationEvents; _i < applicationEvents_1.length; _i++) {
        var eventName = applicationEvents_1[_i];
        if (toAdd) {
            Application.on(eventName, onDeviceChange);
        }
        else {
            Application.off(eventName, onDeviceChange);
        }
    }
}
function onDeviceChange(args) {
    for (var _i = 0, mediaQueryLists_1 = mediaQueryLists; _i < mediaQueryLists_1.length; _i++) {
        var mql = mediaQueryLists_1[_i];
        var matches = checkIfMediaQueryMatches(mql.media);
        if (mql.matches !== matches) {
            mql._matches = matches;
            mql.notify({
                eventName: MediaQueryListImpl.changeEvent,
                object: mql,
                matches: mql.matches,
                media: mql.media,
            });
        }
    }
}
function checkIfMediaQueryMatches(mediaQueryString) {
    var _a = Screen.mainScreen, widthPixels = _a.widthPixels, heightPixels = _a.heightPixels;
    var matches;
    try {
        matches = matchQuery(mediaQueryString, {
            type: MediaQueryType.screen,
            width: widthPixels,
            height: heightPixels,
            'device-width': widthPixels,
            'device-height': heightPixels,
            orientation: Application.orientation(),
            'prefers-color-scheme': Application.systemAppearance(),
        });
    }
    catch (err) {
        matches = false;
        Trace.write(err, Trace.categories.MediaQuery, Trace.messageType.error);
    }
    return matches;
}
function matchMedia(mediaQueryString) {
    isMediaInitializationEnabled = true;
    var mediaQueryList = new MediaQueryListImpl();
    isMediaInitializationEnabled = false;
    mediaQueryList._media = mediaQueryString;
    mediaQueryList._matches = checkIfMediaQueryMatches(mediaQueryString);
    return mediaQueryList;
}
var MediaQueryListImpl = /** @class */ (function (_super) {
    __extends(MediaQueryListImpl, _super);
    function MediaQueryListImpl() {
        var _this = _super.call(this) || this;
        if (!isMediaInitializationEnabled) {
            throw new TypeError('Illegal constructor');
        }
        Object.defineProperties(_this, {
            _media: {
                writable: true,
            },
            _matches: {
                writable: true,
            },
            _onChange: {
                writable: true,
                value: null,
            },
            mediaQueryChangeListeners: {
                value: new Map(),
            },
            _throwInvocationError: {
                value: null,
            },
        });
        return _this;
    }
    Object.defineProperty(MediaQueryListImpl.prototype, "media", {
        get: function () {
            var _a;
            (_a = this._throwInvocationError) === null || _a === void 0 ? void 0 : _a.call(this);
            return this._media;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(MediaQueryListImpl.prototype, "matches", {
        get: function () {
            var _a;
            (_a = this._throwInvocationError) === null || _a === void 0 ? void 0 : _a.call(this);
            return this._matches;
        },
        enumerable: false,
        configurable: true
    });
    // @ts-ignore
    MediaQueryListImpl.prototype.addEventListener = function (eventName, callback, thisArg) {
        var _a;
        (_a = this._throwInvocationError) === null || _a === void 0 ? void 0 : _a.call(this);
        var hasChangeListeners = this.hasListeners(MediaQueryListImpl.changeEvent);
        // Call super method first since it throws in the case of bad parameters
        _super.prototype.addEventListener.call(this, eventName, callback, thisArg);
        if (eventName === MediaQueryListImpl.changeEvent && !hasChangeListeners) {
            mediaQueryLists.push(this);
            if (mediaQueryLists.length === 1) {
                toggleApplicationEventListeners(true);
            }
        }
    };
    // @ts-ignore
    MediaQueryListImpl.prototype.removeEventListener = function (eventName, callback, thisArg) {
        var _a;
        (_a = this._throwInvocationError) === null || _a === void 0 ? void 0 : _a.call(this);
        // Call super method first since it throws in the case of bad parameters
        _super.prototype.removeEventListener.call(this, eventName, callback, thisArg);
        if (eventName === MediaQueryListImpl.changeEvent) {
            var hasChangeListeners = this.hasListeners(MediaQueryListImpl.changeEvent);
            if (!hasChangeListeners) {
                var index = mediaQueryLists.indexOf(this);
                if (index >= 0) {
                    mediaQueryLists.splice(index, 1);
                    if (!mediaQueryLists.length) {
                        toggleApplicationEventListeners(false);
                    }
                }
            }
        }
    };
    MediaQueryListImpl.prototype.addListener = function (callback) {
        var _this = this;
        var _a;
        (_a = this._throwInvocationError) === null || _a === void 0 ? void 0 : _a.call(this);
        // This kind of implementation helps maintain listener registration order
        // regardless of using the deprecated methods or property onchange
        var wrapperCallback = function (data) {
            callback.call(_this, {
                matches: _this.matches,
                media: _this.media,
            });
        };
        // Call this method first since it throws in the case of bad parameters
        this.addEventListener(MediaQueryListImpl.changeEvent, wrapperCallback);
        this.mediaQueryChangeListeners.set(callback, wrapperCallback);
    };
    MediaQueryListImpl.prototype.removeListener = function (callback) {
        var _a;
        (_a = this._throwInvocationError) === null || _a === void 0 ? void 0 : _a.call(this);
        if (this.mediaQueryChangeListeners.has(callback)) {
            // Call this method first since it throws in the case of bad parameters
            this.removeEventListener(MediaQueryListImpl.changeEvent, this.mediaQueryChangeListeners.get(callback));
            this.mediaQueryChangeListeners.delete(callback);
        }
    };
    Object.defineProperty(MediaQueryListImpl.prototype, "onchange", {
        get: function () {
            var _a;
            (_a = this._throwInvocationError) === null || _a === void 0 ? void 0 : _a.call(this);
            return this._onChange;
        },
        set: function (callback) {
            var _a;
            (_a = this._throwInvocationError) === null || _a === void 0 ? void 0 : _a.call(this);
            // Remove old listener if any
            if (this._onChange) {
                this.removeListener(this._onChange);
            }
            if (callback) {
                this.addListener(callback);
            }
            this._onChange = callback;
        },
        enumerable: false,
        configurable: true
    });
    MediaQueryListImpl.prototype._throwInvocationError = function () {
        throw new TypeError('Illegal invocation');
    };
    MediaQueryListImpl.changeEvent = 'change';
    return MediaQueryListImpl;
}(Observable));
export { matchMedia, MediaQueryListImpl as MediaQueryList, checkIfMediaQueryMatches };
//# sourceMappingURL=index.js.map