var MIN_WH = 'minWH';
var MIN_W = 'minW';
var MIN_H = 'minH';
var PRIORITY_STEP = 10000;
var minWidthHeightQualifier = {
    isMatch: function (path) {
        return new RegExp(".".concat(MIN_WH, "\\d+")).test(path);
    },
    getMatchOccurences: function (path) {
        return path.match(new RegExp(".".concat(MIN_WH, "\\d+"), 'g'));
    },
    getMatchValue: function (value, context) {
        var numVal = parseInt(value.substr(MIN_WH.length + 1));
        if (isNaN(numVal)) {
            return -1;
        }
        var actualLength = Math.min(context.width, context.height);
        if (actualLength < numVal) {
            return -1;
        }
        return PRIORITY_STEP - (actualLength - numVal);
    },
};
var minWidthQualifier = {
    isMatch: function (path) {
        return new RegExp(".".concat(MIN_W, "\\d+")).test(path) && !new RegExp(".".concat(MIN_WH, "\\d+")).test(path);
    },
    getMatchOccurences: function (path) {
        return path.match(new RegExp(".".concat(MIN_W, "\\d+"), 'g'));
    },
    getMatchValue: function (value, context) {
        var numVal = parseInt(value.substr(MIN_W.length + 1));
        if (isNaN(numVal)) {
            return -1;
        }
        var actualWidth = context.width;
        if (actualWidth < numVal) {
            return -1;
        }
        return PRIORITY_STEP - (actualWidth - numVal);
    },
};
var minHeightQualifier = {
    isMatch: function (path) {
        return new RegExp(".".concat(MIN_H, "\\d+")).test(path) && !new RegExp(".".concat(MIN_WH, "\\d+")).test(path);
    },
    getMatchOccurences: function (path) {
        return path.match(new RegExp(".".concat(MIN_H, "\\d+"), 'g'));
    },
    getMatchValue: function (value, context) {
        var numVal = parseInt(value.substr(MIN_H.length + 1));
        if (isNaN(numVal)) {
            return -1;
        }
        var actualHeight = context.height;
        if (actualHeight < numVal) {
            return -1;
        }
        return PRIORITY_STEP - (actualHeight - numVal);
    },
};
var platformQualifier = {
    isMatch: function (path) {
        return path.includes('.android') || path.includes('.ios');
    },
    getMatchOccurences: function (path) {
        return path.match(new RegExp('\\.android|\\.ios', 'g'));
    },
    getMatchValue: function (value, context) {
        var val = value.substr(1);
        return val === context.os.toLowerCase() ? 1 : -1;
    },
};
var orientationQualifier = {
    isMatch: function (path) {
        return path.includes('.land') || path.includes('.port');
    },
    getMatchOccurences: function (path) {
        return path.match(new RegExp('\\.land|\\.port', 'g'));
    },
    getMatchValue: function (value, context) {
        var val = value.substr(1);
        var isLandscape = context.width > context.height ? 1 : -1;
        return val === 'land' ? isLandscape : -isLandscape;
    },
};
// List of supported qualifiers ordered by priority
var supportedQualifiers = [minWidthHeightQualifier, minWidthQualifier, minHeightQualifier, orientationQualifier, platformQualifier];
function checkQualifiers(path, context) {
    var result = 0;
    var value;
    for (var i = 0; i < supportedQualifiers.length; i++) {
        var qualifier = supportedQualifiers[i];
        if (qualifier.isMatch(path)) {
            var occurences = qualifier.getMatchOccurences(path);
            // Always get the last qualifier among identical occurences
            value = qualifier.getMatchValue(occurences[occurences.length - 1], context);
            if (value < 0) {
                // Non of the supported qualifiers matched this or the match was not satisfied
                return -1;
            }
            if (value > 0) {
                result += value + (supportedQualifiers.length - i) * PRIORITY_STEP;
            }
        }
    }
    return result;
}
export function stripQualifiers(path) {
    // Strip qualifiers from path if any
    for (var i = 0; i < supportedQualifiers.length; i++) {
        var qualifier = supportedQualifiers[i];
        if (qualifier.isMatch(path)) {
            var occurences = qualifier.getMatchOccurences(path);
            for (var j = 0; j < occurences.length; j++) {
                path = path.replace(occurences[j], '');
            }
        }
    }
    return path;
}
export function findMatch(path, ext, candidates, context) {
    var fullPath = ext ? path + ext : path;
    var bestValue = -1;
    var result = null;
    for (var i = 0; i < candidates.length; i++) {
        var filePath = candidates[i];
        // Check if candidate is correct for given path
        var cleanFilePath = stripQualifiers(filePath);
        if (cleanFilePath !== fullPath) {
            continue;
        }
        var value = checkQualifiers(filePath, context);
        if (value >= 0 && value > bestValue) {
            bestValue = value;
            result = candidates[i];
        }
    }
    return result;
}
//# sourceMappingURL=index.js.map