import { Application } from '../../application';
import { SDK_VERSION } from '../../utils/constants';
import { platformNames } from '../common';
import { Screen } from '../screen';
var MIN_TABLET_PIXELS = 600;
var DeviceRef = /** @class */ (function () {
    function DeviceRef() {
    }
    Object.defineProperty(DeviceRef.prototype, "manufacturer", {
        get: function () {
            if (!this._manufacturer) {
                this._manufacturer = android.os.Build.MANUFACTURER;
            }
            return this._manufacturer;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(DeviceRef.prototype, "os", {
        get: function () {
            return platformNames.android;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(DeviceRef.prototype, "osVersion", {
        get: function () {
            if (!this._osVersion) {
                this._osVersion = android.os.Build.VERSION.RELEASE;
            }
            return this._osVersion;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(DeviceRef.prototype, "model", {
        get: function () {
            if (!this._model) {
                this._model = android.os.Build.MODEL;
            }
            return this._model;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(DeviceRef.prototype, "sdkVersion", {
        get: function () {
            if (!this._sdkVersion) {
                this._sdkVersion = android.os.Build.VERSION.SDK;
            }
            return this._sdkVersion;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(DeviceRef.prototype, "deviceType", {
        get: function () {
            if (!this._deviceType) {
                var dips = Math.min(Screen.mainScreen.widthPixels, Screen.mainScreen.heightPixels) / Screen.mainScreen.scale;
                // If the device has more than 600 dips it is considered to be a tablet.
                if (dips >= MIN_TABLET_PIXELS) {
                    this._deviceType = 'Tablet';
                }
                else {
                    this._deviceType = 'Phone';
                }
            }
            return this._deviceType;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(DeviceRef.prototype, "uuid", {
        get: function () {
            if (!this._uuid) {
                var nativeApp = Application.android.getNativeApplication();
                this._uuid = android.provider.Settings.Secure.getString(nativeApp.getContentResolver(), android.provider.Settings.Secure.ANDROID_ID);
            }
            return this._uuid;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(DeviceRef.prototype, "language", {
        get: function () {
            var defaultNativeLocale;
            if (SDK_VERSION >= 24) {
                defaultNativeLocale = android.content.res.Resources.getSystem().getConfiguration().getLocales().get(0);
            }
            else {
                defaultNativeLocale = android.content.res.Resources.getSystem().getConfiguration().locale;
            }
            return defaultNativeLocale.getLanguage().replace('_', '-');
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(DeviceRef.prototype, "region", {
        get: function () {
            var defaultNativeLocale;
            if (SDK_VERSION >= 24) {
                defaultNativeLocale = android.content.res.Resources.getSystem().getConfiguration().getLocales().get(0);
            }
            else {
                defaultNativeLocale = android.content.res.Resources.getSystem().getConfiguration().locale;
            }
            return defaultNativeLocale.getCountry();
        },
        enumerable: false,
        configurable: true
    });
    return DeviceRef;
}());
export var Device = new DeviceRef();
// This retains compatibility with NS6
export var device = Device;
//# sourceMappingURL=index.android.js.map