export function uptime() {
    return global.android ? org.nativescript.Process.getUpTime() : global.__tns_uptime();
}
export function log(message) {
    var optionalParams = [];
    for (var _i = 1; _i < arguments.length; _i++) {
        optionalParams[_i - 1] = arguments[_i];
    }
    if (global.__nslog) {
        global.__nslog('CONSOLE LOG: ' + message);
    }
    console.log.apply(console, __spreadArray([message], optionalParams, false));
}
// Use object instead of map as it is a bit faster
var timers = {};
var anyGlobal = global;
var profileNames = [];
export var time = (global.__time || Date.now);
export function start(name) {
    var info = timers[name];
    if (info) {
        info.currentStart = time();
        info.runCount++;
    }
    else {
        info = {
            totalTime: 0,
            count: 0,
            currentStart: time(),
            runCount: 1,
        };
        timers[name] = info;
        profileNames.push(name);
    }
}
export function stop(name) {
    var info = timers[name];
    if (!info) {
        throw new Error("No timer started: ".concat(name));
    }
    if (info.runCount) {
        info.runCount--;
        if (info.runCount) {
            info.count++;
        }
        else {
            info.lastTime = time() - info.currentStart;
            info.totalTime += info.lastTime;
            info.count++;
            info.currentStart = 0;
        }
    }
    else {
        throw new Error("Timer ".concat(name, " paused more times than started."));
    }
    return info;
}
export function timer(name) {
    return timers[name];
}
export function print(name) {
    var info = timers[name];
    if (!info) {
        throw new Error("No timer started: ".concat(name));
    }
    console.log("---- [".concat(name, "] STOP total: ").concat(info.totalTime, " count:").concat(info.count));
    return info;
}
export function isRunning(name) {
    var info = timers[name];
    return !!(info && info.runCount);
}
function countersProfileFunctionFactory(fn, name, type) {
    if (type === void 0) { type = 1 /* MemberType.Instance */; }
    profileNames.push(name);
    return function () {
        start(name);
        try {
            return fn.apply(this, arguments);
        }
        finally {
            stop(name);
        }
    };
}
function timelineProfileFunctionFactory(fn, name, type) {
    if (type === void 0) { type = 1 /* MemberType.Instance */; }
    return type === 1 /* MemberType.Instance */
        ? function () {
            var start = time();
            try {
                return fn.apply(this, arguments);
            }
            finally {
                var end = time();
                console.log("Timeline: Modules: ".concat(name, " ").concat(this, "  (").concat(start, "ms. - ").concat(end, "ms.)"));
            }
        }
        : function () {
            var start = time();
            try {
                return fn.apply(this, arguments);
            }
            finally {
                var end = time();
                console.log("Timeline: Modules: ".concat(name, "  (").concat(start, "ms. - ").concat(end, "ms.)"));
            }
        };
}
export var Level;
(function (Level) {
    Level[Level["none"] = 0] = "none";
    Level[Level["lifecycle"] = 1] = "lifecycle";
    Level[Level["timeline"] = 2] = "timeline";
})(Level || (Level = {}));
var tracingLevel = Level.none;
var profileFunctionFactory;
export function enable(mode) {
    if (mode === void 0) { mode = 'counters'; }
    profileFunctionFactory =
        mode &&
            {
                counters: countersProfileFunctionFactory,
                timeline: timelineProfileFunctionFactory,
            }[mode];
    tracingLevel =
        {
            lifecycle: Level.lifecycle,
            timeline: Level.timeline,
        }[mode] || Level.none;
}
try {
    var appConfig = require('~/package.json');
    if (appConfig && appConfig.profiling) {
        enable(appConfig.profiling);
    }
}
catch (e1) {
    try {
        console.log('Profiling startup failed to figure out defaults from package.json, error: ' + e1);
    }
    catch (e2) {
        // We can get here if an exception is thrown in the mksnapshot as there is no console there.
    }
}
export function disable() {
    profileFunctionFactory = undefined;
}
function profileFunction(fn, customName) {
    return profileFunctionFactory(fn, customName || fn.name);
}
var profileMethodUnnamed = function (target, key, descriptor) {
    // save a reference to the original method this way we keep the values currently in the
    // descriptor and don't overwrite what another decorator might have done to the descriptor.
    if (descriptor === undefined) {
        descriptor = Object.getOwnPropertyDescriptor(target, key);
    }
    var originalMethod = descriptor.value;
    var className = '';
    if (target && target.constructor && target.constructor.name) {
        className = target.constructor.name + '.';
    }
    var name = className + (key === null || key === void 0 ? void 0 : key.toString());
    //editing the descriptor/value parameter
    descriptor.value = profileFunctionFactory(originalMethod, name, 1 /* MemberType.Instance */);
    // return edited descriptor as opposed to overwriting the descriptor
    return descriptor;
};
var profileStaticMethodUnnamed = function (ctor, key, descriptor) {
    // save a reference to the original method this way we keep the values currently in the
    // descriptor and don't overwrite what another decorator might have done to the descriptor.
    if (descriptor === undefined) {
        descriptor = Object.getOwnPropertyDescriptor(ctor, key);
    }
    var originalMethod = descriptor.value;
    var className = '';
    if (ctor && ctor.name) {
        className = ctor.name + '.';
    }
    var name = className + (key === null || key === void 0 ? void 0 : key.toString());
    //editing the descriptor/value parameter
    descriptor.value = profileFunctionFactory(originalMethod, name, 0 /* MemberType.Static */);
    // return edited descriptor as opposed to overwriting the descriptor
    return descriptor;
};
function profileMethodNamed(name) {
    return function (target, key, descriptor) {
        // save a reference to the original method this way we keep the values currently in the
        // descriptor and don't overwrite what another decorator might have done to the descriptor.
        if (descriptor === undefined) {
            descriptor = Object.getOwnPropertyDescriptor(target, key);
        }
        var originalMethod = descriptor.value;
        //editing the descriptor/value parameter
        descriptor.value = profileFunctionFactory(originalMethod, name);
        // return edited descriptor as opposed to overwriting the descriptor
        return descriptor;
    };
}
var voidMethodDecorator = function () {
    // no op
};
export function profile(nameFnOrTarget, fnOrKey, descriptor, attrs) {
    if (typeof nameFnOrTarget === 'object' && (typeof fnOrKey === 'string' || typeof fnOrKey === 'symbol')) {
        if (!profileFunctionFactory) {
            return;
        }
        return profileMethodUnnamed(nameFnOrTarget, fnOrKey, descriptor);
    }
    else if (typeof nameFnOrTarget === 'function' && (typeof fnOrKey === 'string' || typeof fnOrKey === 'symbol')) {
        if (!profileFunctionFactory) {
            return;
        }
        return profileStaticMethodUnnamed(nameFnOrTarget, fnOrKey, descriptor);
    }
    else if (typeof nameFnOrTarget === 'string' && typeof fnOrKey === 'function') {
        if (!profileFunctionFactory) {
            return fnOrKey;
        }
        return profileFunction(fnOrKey, nameFnOrTarget);
    }
    else if (typeof nameFnOrTarget === 'function') {
        if (!profileFunctionFactory) {
            return nameFnOrTarget;
        }
        return profileFunction(nameFnOrTarget);
    }
    else if (typeof nameFnOrTarget === 'string') {
        if (!profileFunctionFactory) {
            return voidMethodDecorator;
        }
        return profileMethodNamed(nameFnOrTarget);
    }
    else {
        if (!profileFunctionFactory) {
            return voidMethodDecorator;
        }
        return profileMethodUnnamed;
    }
}
export function dumpProfiles() {
    profileNames.forEach(function (name) {
        var info = timers[name];
        if (info) {
            console.log('---- [' + name + '] STOP total: ' + info.totalTime + ' count:' + info.count);
        }
        else {
            console.log('---- [' + name + '] Never called');
        }
    });
}
export function resetProfiles() {
    profileNames.forEach(function (name) {
        var info = timers[name];
        if (info) {
            if (info.runCount) {
                console.log('---- timer with name [' + name + "] is currently running and won't be reset");
            }
            else {
                timers[name] = undefined;
            }
        }
    });
}
export function startCPUProfile(name) {
    if (anyGlobal.android) {
        __startCPUProfiler(name);
    }
}
export function stopCPUProfile(name) {
    if (anyGlobal.android) {
        __stopCPUProfiler(name);
    }
}
export function level() {
    return tracingLevel;
}
export function trace(message, start, end) {
    log("Timeline: Modules: ".concat(message, "  (").concat(start, "ms. - ").concat(end, "ms.)"));
}
//# sourceMappingURL=index.js.map