import { ActionItemBase, ActionBarBase, isVisible, flatProperty, iosIconRenderingModeProperty, traceMissingIcon } from './action-bar-common';
import { View } from '../core/view';
import { Color } from '../../color';
import { ios as iosBackground } from '../styling/background';
import { LinearGradient } from '../styling/linear-gradient';
import { colorProperty, backgroundInternalProperty, backgroundColorProperty, backgroundImageProperty } from '../styling/style-properties';
import { ios as iosViewUtils } from '../utils';
import { ImageSource } from '../../image-source';
import { layout, iOSNativeHelper, isFontIconURI } from '../../utils';
import { accessibilityHintProperty, accessibilityLabelProperty, accessibilityLanguageProperty, accessibilityValueProperty } from '../../accessibility/accessibility-properties';
export * from './action-bar-common';
var majorVersion = iOSNativeHelper.MajorVersion;
var UNSPECIFIED = layout.makeMeasureSpec(0, layout.UNSPECIFIED);
function loadActionIcon(item) {
    var is = null;
    var img = null;
    var itemIcon = item.icon;
    var itemStyle = item.style;
    if (isFontIconURI(itemIcon)) {
        var fontIconCode = itemIcon.split('//')[1];
        var font = itemStyle.fontInternal;
        var color = itemStyle.color;
        is = ImageSource.fromFontIconCodeSync(fontIconCode, font, color);
    }
    else {
        is = ImageSource.fromFileOrResourceSync(itemIcon);
    }
    if (is && is.ios) {
        img = is.ios;
    }
    else {
        traceMissingIcon(itemIcon);
    }
    return img;
}
var TapBarItemHandlerImpl = /** @class */ (function (_super) {
    __extends(TapBarItemHandlerImpl, _super);
    function TapBarItemHandlerImpl() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    TapBarItemHandlerImpl.initWithOwner = function (owner) {
        var handler = TapBarItemHandlerImpl.new();
        handler._owner = owner;
        return handler;
    };
    TapBarItemHandlerImpl.prototype.tap = function (args) {
        var _a;
        var owner = (_a = this._owner) === null || _a === void 0 ? void 0 : _a.deref();
        if (owner) {
            owner._raiseTap();
        }
    };
    TapBarItemHandlerImpl.ObjCExposedMethods = {
        tap: { returns: interop.types.void, params: [interop.types.id] },
    };
    return TapBarItemHandlerImpl;
}(NSObject));
var ActionItem = /** @class */ (function (_super) {
    __extends(ActionItem, _super);
    function ActionItem() {
        var _this = _super !== null && _super.apply(this, arguments) || this;
        _this._ios = {
            position: 'left',
            systemIcon: undefined,
        };
        return _this;
    }
    Object.defineProperty(ActionItem.prototype, "ios", {
        // @ts-ignore
        get: function () {
            return this._ios;
        },
        set: function (value) {
            throw new Error('ActionItem.ios is read-only');
        },
        enumerable: false,
        configurable: true
    });
    return ActionItem;
}(ActionItemBase));
export { ActionItem };
var NavigationButton = /** @class */ (function (_super) {
    __extends(NavigationButton, _super);
    function NavigationButton() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    NavigationButton.prototype._onVisibilityChanged = function (visibility) {
        if (this._navigationItem) {
            var visible = visibility === 'visible';
            this._navigationItem.setHidesBackButtonAnimated(!visible, true);
        }
    };
    return NavigationButton;
}(ActionItem));
export { NavigationButton };
var ActionBar = /** @class */ (function (_super) {
    __extends(ActionBar, _super);
    function ActionBar() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    Object.defineProperty(ActionBar.prototype, "ios", {
        get: function () {
            var _a;
            var page = this.page;
            if (!page || !page.parent) {
                return;
            }
            var viewController = page.ios;
            return (_a = viewController === null || viewController === void 0 ? void 0 : viewController.navigationController) === null || _a === void 0 ? void 0 : _a.navigationBar;
        },
        enumerable: false,
        configurable: true
    });
    ActionBar.prototype[accessibilityValueProperty.setNative] = function (value) {
        value = value == null ? null : "".concat(value);
        this.nativeViewProtected.accessibilityValue = value;
        var navigationItem = this._getNavigationItem();
        if (navigationItem) {
            navigationItem.accessibilityValue = value;
        }
    };
    ActionBar.prototype[accessibilityLabelProperty.setNative] = function (value) {
        value = value == null ? null : "".concat(value);
        this.nativeViewProtected.accessibilityLabel = value;
        var navigationItem = this._getNavigationItem();
        if (navigationItem) {
            navigationItem.accessibilityLabel = value;
        }
    };
    ActionBar.prototype[accessibilityHintProperty.setNative] = function (value) {
        value = value == null ? null : "".concat(value);
        this.nativeViewProtected.accessibilityHint = value;
        var navigationItem = this._getNavigationItem();
        if (navigationItem) {
            navigationItem.accessibilityHint = value;
        }
    };
    ActionBar.prototype[accessibilityLanguageProperty.setNative] = function (value) {
        value = value == null ? null : "".concat(value);
        this.nativeViewProtected.accessibilityLanguage = value;
        var navigationItem = this._getNavigationItem();
        if (navigationItem) {
            navigationItem.accessibilityLanguage = value;
        }
    };
    ActionBar.prototype.createNativeView = function () {
        return this.ios;
    };
    ActionBar.prototype.disposeNativeView = function () {
        var navBar = this.navBar;
        if (navBar === null || navBar === void 0 ? void 0 : navBar.gradientLayer) {
            navBar.gradientLayer = null;
        }
        _super.prototype.disposeNativeView.call(this);
    };
    ActionBar.prototype._addChildFromBuilder = function (name, value) {
        var _a;
        if (value instanceof NavigationButton) {
            this.navigationButton = value;
        }
        else if (value instanceof ActionItem) {
            (_a = this.actionItems) === null || _a === void 0 ? void 0 : _a.addItem(value);
        }
        else if (value instanceof View) {
            this.titleView = value;
        }
    };
    Object.defineProperty(ActionBar.prototype, "_getActualSize", {
        get: function () {
            var nativeView = this.ios;
            if (!nativeView) {
                return { width: 0, height: 0 };
            }
            var frame = nativeView.frame;
            var size = frame.size;
            var width = layout.toDevicePixels(size.width);
            var height = layout.toDevicePixels(size.height);
            return { width: width, height: height };
        },
        enumerable: false,
        configurable: true
    });
    ActionBar.prototype.layoutInternal = function () {
        var _a = this._getActualSize, width = _a.width, height = _a.height;
        var widthSpec = layout.makeMeasureSpec(width, layout.EXACTLY);
        var heightSpec = layout.makeMeasureSpec(height, layout.EXACTLY);
        this.measure(widthSpec, heightSpec);
        this.layout(0, 0, width, height, false);
    };
    ActionBar.prototype._getIconRenderingMode = function () {
        switch (this.iosIconRenderingMode) {
            case 'alwaysOriginal':
                return 1 /* UIImageRenderingMode.AlwaysOriginal */;
            case 'alwaysTemplate':
                return 2 /* UIImageRenderingMode.AlwaysTemplate */;
            case 'automatic':
            default:
                return 1 /* UIImageRenderingMode.AlwaysOriginal */;
        }
    };
    ActionBar.prototype._getNavigationItem = function () {
        var page = this.page;
        // Page should be attached to frame to update the action bar.
        if (!page || !page.frame) {
            return null;
        }
        var viewController = page.ios;
        return viewController.navigationItem;
    };
    ActionBar.prototype.update = function () {
        var page = this.page;
        // Page should be attached to frame to update the action bar.
        if (!page || !page.frame) {
            return;
        }
        var viewController = page.ios;
        // visionOS may init with different nav stack setup
        if (!viewController) {
            return;
        }
        var navigationItem = viewController.navigationItem;
        var navController = viewController.navigationController;
        if (!navController) {
            return;
        }
        var navigationBar = navController.navigationBar;
        var previousController;
        // Set Title
        navigationItem.title = this.title;
        var titleView = this.titleView;
        if (titleView && titleView.ios) {
            navigationItem.titleView = titleView.ios;
        }
        else {
            navigationItem.titleView = null;
        }
        // Find previous ViewController in the navigation stack
        var indexOfViewController = navController.viewControllers.indexOfObject(viewController);
        if (indexOfViewController > 0 && indexOfViewController < navController.viewControllers.count) {
            previousController = navController.viewControllers[indexOfViewController - 1];
        }
        // Set back button text
        if (previousController) {
            if (this.navigationButton) {
                var tapHandler = TapBarItemHandlerImpl.initWithOwner(new WeakRef(this.navigationButton));
                var barButtonItem = UIBarButtonItem.alloc().initWithTitleStyleTargetAction(this.navigationButton.text + '', 0 /* UIBarButtonItemStyle.Plain */, tapHandler, 'tap');
                previousController.navigationItem.backBarButtonItem = barButtonItem;
            }
            else {
                previousController.navigationItem.backBarButtonItem = null;
            }
        }
        // Set back button image
        var img;
        if (this.navigationButton && isVisible(this.navigationButton) && this.navigationButton.icon) {
            img = loadActionIcon(this.navigationButton);
        }
        // TODO: This could cause issue when canceling BackEdge gesture - we will change the backIndicator to
        // show the one from the old page but the new page will still be visible (because we canceled EdgeBackSwipe gesutre)
        // Consider moving this to new method and call it from - navigationControllerDidShowViewControllerAnimated.
        var image = img ? img.imageWithRenderingMode(1 /* UIImageRenderingMode.AlwaysOriginal */) : null;
        if (majorVersion >= 15) {
            var appearance = this._getAppearance(navigationBar);
            appearance.setBackIndicatorImageTransitionMaskImage(image, image);
            this._updateAppearance(navigationBar, appearance);
        }
        else {
            navigationBar.backIndicatorImage = image;
            navigationBar.backIndicatorTransitionMaskImage = image;
        }
        // Set back button visibility
        if (this.navigationButton) {
            this.navigationButton._navigationItem = navigationItem;
            navigationItem.setHidesBackButtonAnimated(!isVisible(this.navigationButton), false);
        }
        // Populate action items
        this.populateMenuItems(navigationItem);
        // update colors explicitly - they may have to be cleared form a previous page
        this.updateFills(navigationBar);
        // the 'flat' property may have changed in between pages
        this.updateFlatness(navigationBar);
        if (!this.isLayoutValid) {
            this.layoutInternal();
        }
        // Make sure accessibility values are up-to-date on the navigationItem
        navigationItem.accessibilityValue = this.accessibilityValue;
        navigationItem.accessibilityLabel = this.accessibilityLabel;
        navigationItem.accessibilityLanguage = this.accessibilityLanguage;
        navigationItem.accessibilityHint = this.accessibilityHint;
    };
    ActionBar.prototype.populateMenuItems = function (navigationItem) {
        var _a, _b;
        var items = (_b = (_a = this.actionItems) === null || _a === void 0 ? void 0 : _a.getVisibleItems()) !== null && _b !== void 0 ? _b : [];
        var leftBarItems = NSMutableArray.new();
        var rightBarItems = NSMutableArray.new();
        for (var i = 0; i < items.length; i++) {
            var barButtonItem = this.createBarButtonItem(items[i]);
            if (items[i].ios.position === 'left') {
                leftBarItems.addObject(barButtonItem);
            }
            else {
                rightBarItems.insertObjectAtIndex(barButtonItem, 0);
            }
        }
        navigationItem.setLeftBarButtonItemsAnimated(leftBarItems, false);
        navigationItem.setRightBarButtonItemsAnimated(rightBarItems, false);
        if (leftBarItems.count > 0) {
            navigationItem.leftItemsSupplementBackButton = true;
        }
    };
    ActionBar.prototype.createBarButtonItem = function (item) {
        var tapHandler = TapBarItemHandlerImpl.initWithOwner(new WeakRef(item));
        // associate handler with menuItem or it will get collected by JSC.
        item.handler = tapHandler;
        var barButtonItem;
        if (item.actionView && item.actionView.ios) {
            var recognizer = UITapGestureRecognizer.alloc().initWithTargetAction(tapHandler, 'tap');
            item.actionView.ios.addGestureRecognizer(recognizer);
            barButtonItem = UIBarButtonItem.alloc().initWithCustomView(item.actionView.ios);
        }
        else if (item.ios.systemIcon !== undefined) {
            var id = item.ios.systemIcon;
            if (typeof id === 'string') {
                id = parseInt(id);
            }
            barButtonItem = UIBarButtonItem.alloc().initWithBarButtonSystemItemTargetAction(id, tapHandler, 'tap');
        }
        else if (item.icon) {
            var img = loadActionIcon(item);
            if (img) {
                var image = img.imageWithRenderingMode(this._getIconRenderingMode());
                barButtonItem = UIBarButtonItem.alloc().initWithImageStyleTargetAction(image, 0 /* UIBarButtonItemStyle.Plain */, tapHandler, 'tap');
            }
        }
        else {
            barButtonItem = UIBarButtonItem.alloc().initWithTitleStyleTargetAction(item.text + '', 0 /* UIBarButtonItemStyle.Plain */, tapHandler, 'tap');
        }
        if (item.text) {
            barButtonItem.isAccessibilityElement = true;
            barButtonItem.accessibilityLabel = item.text;
            barButtonItem.accessibilityTraits = UIAccessibilityTraitButton;
        }
        return barButtonItem;
    };
    ActionBar.prototype.updateFills = function (navBar) {
        var _this = this;
        var color = this.color;
        this.setColor(navBar, color);
        this._setBackgroundColor(navBar, this.style.backgroundColor);
        this._createBackgroundUIImage(navBar, this.style.backgroundImage, function (image) {
            _this._setBackgroundImage(navBar, image);
        });
    };
    ActionBar.prototype.setColor = function (navBar, color) {
        if (!navBar) {
            return;
        }
        if (color) {
            var titleTextColor = NSDictionary.dictionaryWithObjectForKey(color.ios, NSForegroundColorAttributeName);
            if (majorVersion >= 15) {
                var appearance = this._getAppearance(navBar);
                appearance.titleTextAttributes = titleTextColor;
            }
            navBar.titleTextAttributes = titleTextColor;
            navBar.largeTitleTextAttributes = titleTextColor;
            navBar.tintColor = color.ios;
        }
        else {
            navBar.titleTextAttributes = null;
            navBar.largeTitleTextAttributes = null;
            navBar.tintColor = null;
        }
    };
    ActionBar.prototype._setBackgroundColor = function (navBar, color) {
        if (!navBar) {
            return;
        }
        var nativeColor = color instanceof Color ? color.ios : color;
        if (__VISIONOS__ || majorVersion >= 15) {
            var appearance = this._getAppearance(navBar);
            // appearance.configureWithOpaqueBackground();
            appearance.backgroundColor = nativeColor;
            this._updateAppearance(navBar, appearance);
        }
        else {
            // legacy styling
            navBar.barTintColor = nativeColor;
        }
    };
    ActionBar.prototype._getBackgroundColor = function (navBar) {
        if (!navBar) {
            return null;
        }
        var color;
        if (__VISIONOS__ || majorVersion >= 15) {
            var appearance = this._getAppearance(navBar);
            color = appearance.backgroundColor;
        }
        else {
            // legacy styling
            color = navBar.barTintColor;
        }
        return color;
    };
    ActionBar.prototype._setBackgroundImage = function (navBar, image) {
        if (!navBar) {
            return;
        }
        if (__VISIONOS__ || majorVersion >= 15) {
            var appearance = this._getAppearance(navBar);
            // appearance.configureWithOpaqueBackground();
            appearance.backgroundImage = image;
            this._updateAppearance(navBar, appearance);
        }
        else {
            // legacy styling
            // Set a blank image in case image is null and flatness is enabled
            if (this.flat && !image) {
                image = UIImage.new();
            }
            navBar.setBackgroundImageForBarMetrics(image, 0 /* UIBarMetrics.Default */);
        }
    };
    ActionBar.prototype._getBackgroundImage = function (navBar) {
        if (!navBar) {
            return null;
        }
        var image;
        if (__VISIONOS__ || majorVersion >= 15) {
            var appearance = this._getAppearance(navBar);
            image = appearance.backgroundImage;
        }
        else {
            // legacy styling
            image = navBar.backgroundImageForBarMetrics(0 /* UIBarMetrics.Default */);
        }
        return image;
    };
    ActionBar.prototype._createBackgroundUIImage = function (navBar, value, callback) {
        if (!navBar) {
            return;
        }
        if (value) {
            if (value instanceof LinearGradient) {
                if (!navBar.gradientLayer) {
                    navBar.gradientLayer = CAGradientLayer.new();
                }
                iosViewUtils.drawGradient(navBar, navBar.gradientLayer, value);
                var renderer = UIGraphicsImageRenderer.alloc().initWithSize(navBar.bounds.size);
                var img = renderer.imageWithActions(function (context) {
                    navBar.gradientLayer.renderInContext(context.CGContext);
                });
                callback(img);
                // Return here to avoid unnecessary cleanups
                return;
            }
            // Background image
            iosBackground.createUIImageFromURI(this, value, false, callback);
        }
        else {
            callback(null);
        }
        if (navBar.gradientLayer) {
            navBar.gradientLayer = null;
        }
    };
    ActionBar.prototype._onTitlePropertyChanged = function () {
        var page = this.page;
        if (!page) {
            return;
        }
        if (page.frame) {
            page.frame._updateActionBar(page);
        }
        var navigationItem = page.ios.navigationItem;
        navigationItem.title = this.title;
    };
    ActionBar.prototype.updateFlatness = function (navBar) {
        if (this.flat) {
            if (majorVersion >= 15) {
                var appearance = this._getAppearance(navBar);
                appearance.shadowColor = UIColor.clearColor;
                this._updateAppearance(navBar, appearance);
            }
            else {
                // Do not apply blank image if background image is already set
                if (!this.backgroundImage) {
                    navBar.setBackgroundImageForBarMetrics(UIImage.new(), 0 /* UIBarMetrics.Default */);
                }
                navBar.shadowImage = UIImage.new();
                navBar.translucent = false;
            }
        }
        else {
            if (majorVersion >= 15) {
                if (navBar.standardAppearance) {
                    // Not flat and never been set do nothing.
                    var appearance = navBar.standardAppearance;
                    appearance.shadowColor = UINavigationBarAppearance.new().shadowColor;
                    this._updateAppearance(navBar, appearance);
                }
            }
            else {
                // Do not apply blank image if background image is already set
                if (!this.backgroundImage) {
                    // Bar metrics is needed even when unsetting the image
                    navBar.setBackgroundImageForBarMetrics(null, 0 /* UIBarMetrics.Default */);
                }
                navBar.shadowImage = null;
                navBar.translucent = true;
            }
        }
    };
    ActionBar.prototype._getAppearance = function (navBar) {
        var _a;
        return (_a = navBar.standardAppearance) !== null && _a !== void 0 ? _a : UINavigationBarAppearance.new();
    };
    ActionBar.prototype._updateAppearance = function (navBar, appearance) {
        navBar.standardAppearance = appearance;
        navBar.compactAppearance = appearance;
        navBar.scrollEdgeAppearance = appearance;
    };
    ActionBar.prototype.onMeasure = function (widthMeasureSpec, heightMeasureSpec) {
        var _this = this;
        var _a;
        var width = layout.getMeasureSpecSize(widthMeasureSpec);
        var height = layout.getMeasureSpecSize(heightMeasureSpec);
        if (this.titleView) {
            View.measureChild(this, this.titleView, UNSPECIFIED, UNSPECIFIED);
        }
        (_a = this.actionItems) === null || _a === void 0 ? void 0 : _a.getItems().forEach(function (actionItem) {
            var actionView = actionItem.actionView;
            if (actionView) {
                View.measureChild(_this, actionView, UNSPECIFIED, UNSPECIFIED);
            }
        });
        // We ignore our width/height, minWidth/minHeight dimensions because it is against Apple policy to change height of NavigationBar.
        this.setMeasuredDimension(width, height);
    };
    ActionBar.prototype.onLayout = function (left, top, right, bottom) {
        var _this = this;
        var _a;
        var titleView = this.titleView;
        if (titleView) {
            if (majorVersion > 10) {
                // On iOS 11 titleView is wrapped in another view that is centered with constraints.
                View.layoutChild(this, titleView, 0, 0, titleView.getMeasuredWidth(), titleView.getMeasuredHeight());
            }
            else {
                // On iOS <11 titleView is direct child of UINavigationBar so we give it full width and leave
                // the layout to center it.
                View.layoutChild(this, titleView, 0, 0, right - left, bottom - top);
            }
        }
        (_a = this.actionItems) === null || _a === void 0 ? void 0 : _a.getItems().forEach(function (actionItem) {
            var actionView = actionItem.actionView;
            if (actionView && actionView.ios) {
                var measuredWidth = actionView.getMeasuredWidth();
                var measuredHeight = actionView.getMeasuredHeight();
                View.layoutChild(_this, actionView, 0, 0, measuredWidth, measuredHeight);
            }
        });
        _super.prototype.onLayout.call(this, left, top, right, bottom);
    };
    ActionBar.prototype.layoutNativeView = function (left, top, right, bottom) {
        return;
    };
    Object.defineProperty(ActionBar.prototype, "navBar", {
        get: function () {
            var _a, _b, _c;
            // Page should be attached to frame to update the action bar.
            if ((_c = (_b = (_a = this.page) === null || _a === void 0 ? void 0 : _a.frame) === null || _b === void 0 ? void 0 : _b.ios) === null || _c === void 0 ? void 0 : _c.controller) {
                return this.page.frame.ios.controller.navigationBar;
            }
            else {
                return undefined;
            }
        },
        enumerable: false,
        configurable: true
    });
    ActionBar.prototype[colorProperty.getDefault] = function () {
        return null;
    };
    ActionBar.prototype[colorProperty.setNative] = function (color) {
        var navBar = this.navBar;
        this.setColor(navBar, color);
    };
    ActionBar.prototype[backgroundColorProperty.getDefault] = function () {
        return this._getBackgroundColor(this.navBar);
    };
    ActionBar.prototype[backgroundColorProperty.setNative] = function (color) {
        this._setBackgroundColor(this.navBar, color);
    };
    ActionBar.prototype[backgroundImageProperty.getDefault] = function () {
        return this._getBackgroundImage(this.navBar);
    };
    ActionBar.prototype[backgroundImageProperty.setNative] = function (value) {
        var _this = this;
        var navBar = this.navBar;
        this._createBackgroundUIImage(navBar, value, function (image) {
            _this._setBackgroundImage(navBar, image);
        });
    };
    ActionBar.prototype[backgroundInternalProperty.getDefault] = function () {
        return null;
    };
    ActionBar.prototype[backgroundInternalProperty.setNative] = function (value) {
        // tslint:disable-line
    };
    ActionBar.prototype[flatProperty.setNative] = function (value) {
        var navBar = this.navBar;
        if (navBar) {
            this.updateFlatness(navBar);
        }
    };
    ActionBar.prototype[iosIconRenderingModeProperty.getDefault] = function () {
        return 'alwaysOriginal';
    };
    ActionBar.prototype[iosIconRenderingModeProperty.setNative] = function (value) {
        this.update();
    };
    return ActionBar;
}(ActionBarBase));
export { ActionBar };
//# sourceMappingURL=index.ios.js.map