import { ActivityIndicatorBase, busyProperty } from './activity-indicator-common';
import { CoreTypes } from '../../core-types';
import { Color } from '../../color';
import { colorProperty, visibilityProperty } from '../styling/style-properties';
import { AndroidHelper } from '../core/view';
export * from './activity-indicator-common';
var ActivityIndicator = /** @class */ (function (_super) {
    __extends(ActivityIndicator, _super);
    function ActivityIndicator() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    ActivityIndicator.prototype.createNativeView = function () {
        var progressBar = new android.widget.ProgressBar(this._context);
        progressBar.setVisibility(android.view.View.INVISIBLE);
        progressBar.setIndeterminate(true);
        return progressBar;
    };
    ActivityIndicator.prototype[busyProperty.getDefault] = function () {
        return false;
    };
    ActivityIndicator.prototype[busyProperty.setNative] = function (value) {
        if (this.visibility === CoreTypes.Visibility.visible) {
            this.nativeViewProtected.setVisibility(value ? android.view.View.VISIBLE : android.view.View.INVISIBLE);
        }
    };
    ActivityIndicator.prototype[visibilityProperty.getDefault] = function () {
        return CoreTypes.Visibility.hidden;
    };
    ActivityIndicator.prototype[visibilityProperty.setNative] = function (value) {
        switch (value) {
            case CoreTypes.Visibility.visible:
                this.nativeViewProtected.setVisibility(this.busy ? android.view.View.VISIBLE : android.view.View.INVISIBLE);
                break;
            case CoreTypes.Visibility.hidden:
                this.nativeViewProtected.setVisibility(android.view.View.INVISIBLE);
                break;
            case CoreTypes.Visibility.collapse:
                this.nativeViewProtected.setVisibility(android.view.View.GONE);
                break;
            default:
                throw new Error("Invalid visibility value: ".concat(value, ". Valid values are: \"").concat(CoreTypes.Visibility.visible, "\", \"").concat(CoreTypes.Visibility.hidden, "\", \"").concat(CoreTypes.Visibility.collapse, "\"."));
        }
    };
    ActivityIndicator.prototype[colorProperty.getDefault] = function () {
        return -1;
    };
    ActivityIndicator.prototype[colorProperty.setNative] = function (value) {
        var color = value instanceof Color ? value.android : value;
        var drawable = this.nativeViewProtected.getIndeterminateDrawable().mutate();
        if (color) {
            AndroidHelper.setDrawableColor(color, drawable);
        }
        else {
            AndroidHelper.clearDrawableColor(drawable);
        }
    };
    return ActivityIndicator;
}(ActivityIndicatorBase));
export { ActivityIndicator };
//# sourceMappingURL=index.android.js.map