import { CoreTypes } from '../../core-types';
import { Trace } from '../../trace';
// Types.
import { unsetValue } from '../core/properties';
import { Animation } from '.';
import { backgroundColorProperty, scaleXProperty, scaleYProperty, translateXProperty, translateYProperty, rotateProperty, opacityProperty, rotateXProperty, rotateYProperty, widthProperty, heightProperty } from '../styling/style-properties';
var UnparsedKeyframe = /** @class */ (function () {
    function UnparsedKeyframe() {
    }
    return UnparsedKeyframe;
}());
export { UnparsedKeyframe };
var KeyframeDeclaration = /** @class */ (function () {
    function KeyframeDeclaration() {
    }
    return KeyframeDeclaration;
}());
export { KeyframeDeclaration };
var KeyframeInfo = /** @class */ (function () {
    function KeyframeInfo() {
        this.curve = CoreTypes.AnimationCurve.ease;
    }
    return KeyframeInfo;
}());
export { KeyframeInfo };
var KeyframeAnimationInfo = /** @class */ (function () {
    function KeyframeAnimationInfo() {
        this.name = '';
        this.duration = 0.3;
        this.delay = 0;
        this.iterations = 1;
        this.curve = 'ease';
        this.isForwards = false;
        this.isReverse = false;
    }
    return KeyframeAnimationInfo;
}());
export { KeyframeAnimationInfo };
var KeyframeAnimation = /** @class */ (function () {
    function KeyframeAnimation() {
        this.delay = 0;
        this.iterations = 1;
    }
    KeyframeAnimation.keyframeAnimationFromInfo = function (info) {
        var _a;
        if (!((_a = info === null || info === void 0 ? void 0 : info.keyframes) === null || _a === void 0 ? void 0 : _a.length)) {
            Trace.write("No keyframes found for animation '".concat(info.name, "'."), Trace.categories.Animation, Trace.messageType.warn);
            return null;
        }
        var length = info.keyframes.length;
        var animations = new Array();
        var startDuration = 0;
        if (info.isReverse) {
            for (var index = length - 1; index >= 0; index--) {
                var keyframe = info.keyframes[index];
                startDuration = KeyframeAnimation.parseKeyframe(info, keyframe, animations, startDuration);
            }
        }
        else {
            for (var index = 0; index < length; index++) {
                var keyframe = info.keyframes[index];
                startDuration = KeyframeAnimation.parseKeyframe(info, keyframe, animations, startDuration);
            }
            for (var index = length - 1; index > 0; index--) {
                var a1 = animations[index];
                var a2 = animations[index - 1];
                if (a2['curve'] !== undefined) {
                    a1['curve'] = a2['curve'];
                    a2['curve'] = undefined;
                }
            }
        }
        animations.map(function (a) { return (a['curve'] ? a : Object.assign(a, { curve: info.curve })); });
        var animation = new KeyframeAnimation();
        animation.delay = info.delay;
        animation.iterations = info.iterations;
        animation.animations = animations;
        animation._isForwards = info.isForwards;
        return animation;
    };
    KeyframeAnimation.parseKeyframe = function (info, keyframe, animations, startDuration) {
        var animation = {};
        for (var _i = 0, _a = keyframe.declarations; _i < _a.length; _i++) {
            var declaration = _a[_i];
            animation[declaration.property] = declaration.value;
        }
        var duration = keyframe.duration;
        if (duration === 0) {
            duration = 0.01;
        }
        else {
            duration = info.duration * duration - startDuration;
            startDuration += duration;
        }
        animation.duration = info.isReverse ? info.duration - duration : duration;
        animation.curve = keyframe.curve;
        animation.forceLayer = true;
        animation.valueSource = 'keyframe';
        animations.push(animation);
        return startDuration;
    };
    Object.defineProperty(KeyframeAnimation.prototype, "isPlaying", {
        get: function () {
            return this._isPlaying;
        },
        enumerable: false,
        configurable: true
    });
    KeyframeAnimation.prototype.cancel = function () {
        if (!this.isPlaying) {
            Trace.write('Keyframe animation is already playing.', Trace.categories.Animation, Trace.messageType.warn);
            return;
        }
        this._isPlaying = false;
        for (var i = this._nativeAnimations.length - 1; i >= 0; i--) {
            var animation = this._nativeAnimations[i];
            if (animation.isPlaying) {
                animation.cancel();
            }
        }
        if (this._nativeAnimations.length > 0) {
            var animation = this._nativeAnimations[0];
            this._resetAnimationValues(this._target, animation);
        }
        this._resetAnimations();
    };
    KeyframeAnimation.prototype.play = function (view) {
        var _this = this;
        if (this._isPlaying) {
            Trace.write('Keyframe animation is already playing.', Trace.categories.Animation, Trace.messageType.warn);
            return new Promise(function (resolve) {
                resolve();
            });
        }
        var animationFinishedPromise = new Promise(function (resolve) {
            _this._resolve = resolve;
        });
        this._isPlaying = true;
        this._nativeAnimations = new Array();
        this._target = view;
        if (this.delay !== 0) {
            setTimeout(function () { return _this.animate(view, 0, _this.iterations); }, this.delay);
        }
        else {
            this.animate(view, 0, this.iterations);
        }
        return animationFinishedPromise;
    };
    KeyframeAnimation.prototype.animate = function (view, index, iterations) {
        var _this = this;
        if (!this._isPlaying) {
            return;
        }
        if (index === 0) {
            var animation = this.animations[0];
            if ('backgroundColor' in animation) {
                view.style[backgroundColorProperty.keyframe] = animation.backgroundColor;
            }
            if ('scale' in animation) {
                view.style[scaleXProperty.keyframe] = animation.scale.x;
                view.style[scaleYProperty.keyframe] = animation.scale.y;
            }
            if ('translate' in animation) {
                view.style[translateXProperty.keyframe] = animation.translate.x;
                view.style[translateYProperty.keyframe] = animation.translate.y;
            }
            if ('rotate' in animation) {
                view.style[rotateXProperty.keyframe] = animation.rotate.x;
                view.style[rotateYProperty.keyframe] = animation.rotate.y;
                view.style[rotateProperty.keyframe] = animation.rotate.z;
            }
            if ('opacity' in animation) {
                view.style[opacityProperty.keyframe] = animation.opacity;
            }
            if ('height' in animation) {
                view.style[heightProperty.keyframe] = animation.height;
            }
            if ('width' in animation) {
                view.style[widthProperty.keyframe] = animation.width;
            }
            setTimeout(function () { return _this.animate(view, 1, iterations); }, 1);
        }
        else if (index < 0 || index >= this.animations.length) {
            iterations -= 1;
            if (iterations > 0) {
                this.animate(view, 0, iterations);
            }
            else {
                if (this._isForwards === false) {
                    var animation = this.animations[this.animations.length - 1];
                    this._resetAnimationValues(view, animation);
                }
                this._resolveAnimationFinishedPromise();
            }
        }
        else {
            var animation = void 0;
            var cachedAnimation = this._nativeAnimations[index - 1];
            if (cachedAnimation) {
                animation = cachedAnimation;
            }
            else {
                var animationDef = __assign(__assign({}, this.animations[index]), { target: view });
                animation = new Animation([animationDef]);
                this._nativeAnimations.push(animation);
            }
            var isLastIteration = iterations - 1 <= 0;
            // Catch the animation cancel to prevent unhandled promise rejection warnings
            animation
                .play(isLastIteration)
                .then(function () {
                _this.animate(view, index + 1, iterations);
            }, function (error) {
                Trace.write(typeof error === 'string' ? error : error.message, Trace.categories.Animation, Trace.messageType.warn);
            })
                .catch(function (error) {
                Trace.write(typeof error === 'string' ? error : error.message, Trace.categories.Animation, Trace.messageType.warn);
            }); // tslint:disable-line
        }
    };
    KeyframeAnimation.prototype._resolveAnimationFinishedPromise = function () {
        this._nativeAnimations = new Array();
        this._isPlaying = false;
        this._target = null;
        this._resolve();
    };
    KeyframeAnimation.prototype._resetAnimations = function () {
        this._nativeAnimations = new Array();
        this._isPlaying = false;
        this._target = null;
    };
    KeyframeAnimation.prototype._resetAnimationValues = function (view, animation) {
        if ('backgroundColor' in animation) {
            view.style[backgroundColorProperty.keyframe] = unsetValue;
        }
        if ('scale' in animation) {
            view.style[scaleXProperty.keyframe] = unsetValue;
            view.style[scaleYProperty.keyframe] = unsetValue;
        }
        if ('translate' in animation) {
            view.style[translateXProperty.keyframe] = unsetValue;
            view.style[translateYProperty.keyframe] = unsetValue;
        }
        if ('rotate' in animation) {
            view.style[rotateProperty.keyframe] = unsetValue;
        }
        if ('opacity' in animation) {
            view.style[opacityProperty.keyframe] = unsetValue;
        }
        if ('height' in animation) {
            view.style[heightProperty.keyframe] = unsetValue;
        }
        if ('width' in animation) {
            view.style[widthProperty.keyframe] = unsetValue;
        }
    };
    return KeyframeAnimation;
}());
export { KeyframeAnimation };
//# sourceMappingURL=keyframe-animation.js.map