import { ButtonBase } from './button-common';
import { PseudoClassHandler } from '../core/view';
import { paddingLeftProperty, paddingTopProperty, paddingRightProperty, paddingBottomProperty, Length, zIndexProperty, minWidthProperty, minHeightProperty } from '../styling/style-properties';
import { textAlignmentProperty } from '../text-base';
import { profile } from '../../profiling';
import { TouchAction, GestureTypes } from '../gestures';
import { SDK_VERSION } from '../../utils/constants';
export * from './button-common';
var ClickListener;
var AndroidButton;
function initializeClickListener() {
    if (ClickListener) {
        return;
    }
    var ClickListenerImpl = /** @class */ (function (_super) {
    __extends(ClickListenerImpl, _super);
    function ClickListenerImpl(owner) {
        var _this = _super.call(this) || this;
        _this.owner = owner;
        return global.__native(_this);
    }
    ClickListenerImpl.prototype.onClick = function (v) {
        var owner = this.owner;
        if (owner) {
            owner._emit(ButtonBase.tapEvent);
        }
    };
    var _a;
    ClickListenerImpl = __decorate([
        Interfaces([android.view.View.OnClickListener]),
        __metadata("design:paramtypes", [typeof (_a = typeof Button !== "undefined" && Button) === "function" ? _a : Object])
    ], ClickListenerImpl);
    return ClickListenerImpl;
}(java.lang.Object));
    ClickListener = ClickListenerImpl;
}
function onButtonStateChange(args) {
    var button = args.object;
    switch (args.action) {
        case TouchAction.up:
        case TouchAction.cancel:
            button._removeVisualState('highlighted');
            break;
        case TouchAction.down:
            button._addVisualState('highlighted');
            break;
    }
}
var Button = /** @class */ (function (_super) {
    __extends(Button, _super);
    function Button() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    Button.prototype.createNativeView = function () {
        if (!AndroidButton) {
            AndroidButton = android.widget.Button;
        }
        return new AndroidButton(this._context);
    };
    Button.prototype.initNativeView = function () {
        _super.prototype.initNativeView.call(this);
        var nativeView = this.nativeViewProtected;
        initializeClickListener();
        var clickListener = new ClickListener(this);
        nativeView.setOnClickListener(clickListener);
        nativeView.clickListener = clickListener;
    };
    Button.prototype.disposeNativeView = function () {
        var _a;
        if ((_a = this.nativeViewProtected) === null || _a === void 0 ? void 0 : _a.clickListener) {
            this.nativeViewProtected.clickListener.owner = null;
        }
        _super.prototype.disposeNativeView.call(this);
    };
    Button.prototype.resetNativeView = function () {
        _super.prototype.resetNativeView.call(this);
        if (this._stateListAnimator && SDK_VERSION >= 21) {
            this.nativeViewProtected.setStateListAnimator(this._stateListAnimator);
            this._stateListAnimator = undefined;
        }
    };
    Button.prototype._updateButtonStateChangeHandler = function (subscribe) {
        if (subscribe) {
            this.on(GestureTypes[GestureTypes.touch], onButtonStateChange);
        }
        else {
            this.off(GestureTypes[GestureTypes.touch], onButtonStateChange);
        }
    };
    Object.defineProperty(Button.prototype, "needsNativeDrawableFill", {
        get: function () {
            return true;
        },
        enumerable: false,
        configurable: true
    });
    Button.prototype[minWidthProperty.getDefault] = function () {
        var dips = org.nativescript.widgets.ViewHelper.getMinWidth(this.nativeViewProtected);
        return { value: dips, unit: 'px' };
    };
    Button.prototype[minHeightProperty.getDefault] = function () {
        var dips = org.nativescript.widgets.ViewHelper.getMinHeight(this.nativeViewProtected);
        return { value: dips, unit: 'px' };
    };
    Button.prototype[paddingTopProperty.getDefault] = function () {
        return { value: this._defaultPaddingTop, unit: 'px' };
    };
    Button.prototype[paddingTopProperty.setNative] = function (value) {
        org.nativescript.widgets.ViewHelper.setPaddingTop(this.nativeViewProtected, Length.toDevicePixels(value, 0) + Length.toDevicePixels(this.style.borderTopWidth, 0));
    };
    Button.prototype[paddingRightProperty.getDefault] = function () {
        return { value: this._defaultPaddingRight, unit: 'px' };
    };
    Button.prototype[paddingRightProperty.setNative] = function (value) {
        org.nativescript.widgets.ViewHelper.setPaddingRight(this.nativeViewProtected, Length.toDevicePixels(value, 0) + Length.toDevicePixels(this.style.borderRightWidth, 0));
    };
    Button.prototype[paddingBottomProperty.getDefault] = function () {
        return { value: this._defaultPaddingBottom, unit: 'px' };
    };
    Button.prototype[paddingBottomProperty.setNative] = function (value) {
        org.nativescript.widgets.ViewHelper.setPaddingBottom(this.nativeViewProtected, Length.toDevicePixels(value, 0) + Length.toDevicePixels(this.style.borderBottomWidth, 0));
    };
    Button.prototype[paddingLeftProperty.getDefault] = function () {
        return { value: this._defaultPaddingLeft, unit: 'px' };
    };
    Button.prototype[paddingLeftProperty.setNative] = function (value) {
        org.nativescript.widgets.ViewHelper.setPaddingLeft(this.nativeViewProtected, Length.toDevicePixels(value, 0) + Length.toDevicePixels(this.style.borderLeftWidth, 0));
    };
    Button.prototype[zIndexProperty.setNative] = function (value) {
        if (SDK_VERSION >= 21) {
            var nativeView = this.nativeViewProtected;
            if (!this._stateListAnimator) {
                this._stateListAnimator = nativeView.getStateListAnimator();
            }
            nativeView.setStateListAnimator(null);
        }
        org.nativescript.widgets.ViewHelper.setZIndex(this.nativeViewProtected, value);
    };
    Button.prototype[textAlignmentProperty.setNative] = function (value) {
        // Button initial value is center.
        var newValue = value === 'initial' ? 'center' : value;
        _super.prototype[textAlignmentProperty.setNative].call(this, newValue);
    };
    Button.prototype.getDefaultElevation = function () {
        if (SDK_VERSION < 21) {
            return 0;
        }
        // NOTE: Button widget has StateListAnimator that defines the elevation value and
        // at the time of the getDefault() query the animator is not applied yet so we
        // return the hardcoded @dimen/button_elevation_material value 2dp here instead
        return 2;
    };
    Button.prototype.getDefaultDynamicElevationOffset = function () {
        if (SDK_VERSION < 21) {
            return 0;
        }
        return 4; // 4dp @dimen/button_pressed_z_material
    };
    __decorate([
        profile,
        __metadata("design:type", Function),
        __metadata("design:paramtypes", []),
        __metadata("design:returntype", void 0)
    ], Button.prototype, "createNativeView", null);
    __decorate([
        PseudoClassHandler('normal', 'highlighted', 'pressed', 'active'),
        __metadata("design:type", Function),
        __metadata("design:paramtypes", [Boolean]),
        __metadata("design:returntype", void 0)
    ], Button.prototype, "_updateButtonStateChangeHandler", null);
    return Button;
}(ButtonBase));
export { Button };
Button.prototype._ignoreFlexMinWidthHeightReset = true;
//# sourceMappingURL=index.android.js.map