import { ControlStateChangeListener } from '../core/control-state-change';
import { ButtonBase } from './button-common';
import { View, PseudoClassHandler } from '../core/view';
import { borderTopWidthProperty, borderRightWidthProperty, borderBottomWidthProperty, borderLeftWidthProperty, paddingLeftProperty, paddingTopProperty, paddingRightProperty, paddingBottomProperty } from '../styling/style-properties';
import { textAlignmentProperty, whiteSpaceProperty, textOverflowProperty } from '../text-base';
import { layout } from '../../utils';
export * from './button-common';
var observableVisualStates = ['highlighted']; // States like :disabled are handled elsewhere
var Button = /** @class */ (function (_super) {
    __extends(Button, _super);
    function Button() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    Button.prototype.createNativeView = function () {
        return UIButton.buttonWithType(1 /* UIButtonType.System */);
    };
    Button.prototype.initNativeView = function () {
        _super.prototype.initNativeView.call(this);
        this._tapHandler = TapHandlerImpl.initWithOwner(new WeakRef(this));
        this.nativeViewProtected.addTargetActionForControlEvents(this._tapHandler, 'tap', 64 /* UIControlEvents.TouchUpInside */);
    };
    Button.prototype.disposeNativeView = function () {
        this._tapHandler = null;
        _super.prototype.disposeNativeView.call(this);
    };
    Object.defineProperty(Button.prototype, "ios", {
        // @ts-ignore
        get: function () {
            return this.nativeViewProtected;
        },
        enumerable: false,
        configurable: true
    });
    Button.prototype.onUnloaded = function () {
        _super.prototype.onUnloaded.call(this);
        if (this._stateChangedHandler) {
            this._stateChangedHandler.stop();
        }
    };
    Button.prototype._updateButtonStateChangeHandler = function (subscribe) {
        var _this = this;
        if (subscribe) {
            if (!this._stateChangedHandler) {
                this._stateChangedHandler = new ControlStateChangeListener(this.nativeViewProtected, observableVisualStates, function (state, add) {
                    if (add) {
                        _this._addVisualState(state);
                    }
                    else {
                        _this._removeVisualState(state);
                    }
                });
            }
            this._stateChangedHandler.start();
        }
        else {
            this._stateChangedHandler.stop();
        }
    };
    Button.prototype[borderTopWidthProperty.getDefault] = function () {
        return {
            value: this.nativeViewProtected.contentEdgeInsets.top,
            unit: 'px',
        };
    };
    Button.prototype[borderTopWidthProperty.setNative] = function (value) {
        var inset = this.nativeViewProtected.contentEdgeInsets;
        var top = layout.toDeviceIndependentPixels(this.effectivePaddingTop + this.effectiveBorderTopWidth);
        this.nativeViewProtected.contentEdgeInsets = {
            top: top,
            left: inset.left,
            bottom: inset.bottom,
            right: inset.right,
        };
    };
    Button.prototype[borderRightWidthProperty.getDefault] = function () {
        return {
            value: this.nativeViewProtected.contentEdgeInsets.right,
            unit: 'px',
        };
    };
    Button.prototype[borderRightWidthProperty.setNative] = function (value) {
        var inset = this.nativeViewProtected.contentEdgeInsets;
        var right = layout.toDeviceIndependentPixels(this.effectivePaddingRight + this.effectiveBorderRightWidth);
        this.nativeViewProtected.contentEdgeInsets = {
            top: inset.top,
            left: inset.left,
            bottom: inset.bottom,
            right: right,
        };
    };
    Button.prototype[borderBottomWidthProperty.getDefault] = function () {
        return {
            value: this.nativeViewProtected.contentEdgeInsets.bottom,
            unit: 'px',
        };
    };
    Button.prototype[borderBottomWidthProperty.setNative] = function (value) {
        var inset = this.nativeViewProtected.contentEdgeInsets;
        var bottom = layout.toDeviceIndependentPixels(this.effectivePaddingBottom + this.effectiveBorderBottomWidth);
        this.nativeViewProtected.contentEdgeInsets = {
            top: inset.top,
            left: inset.left,
            bottom: bottom,
            right: inset.right,
        };
    };
    Button.prototype[borderLeftWidthProperty.getDefault] = function () {
        return {
            value: this.nativeViewProtected.contentEdgeInsets.left,
            unit: 'px',
        };
    };
    Button.prototype[borderLeftWidthProperty.setNative] = function (value) {
        var inset = this.nativeViewProtected.contentEdgeInsets;
        var left = layout.toDeviceIndependentPixels(this.effectivePaddingLeft + this.effectiveBorderLeftWidth);
        this.nativeViewProtected.contentEdgeInsets = {
            top: inset.top,
            left: left,
            bottom: inset.bottom,
            right: inset.right,
        };
    };
    Button.prototype[paddingTopProperty.getDefault] = function () {
        return {
            value: this.nativeViewProtected.contentEdgeInsets.top,
            unit: 'px',
        };
    };
    Button.prototype[paddingTopProperty.setNative] = function (value) {
        var inset = this.nativeViewProtected.contentEdgeInsets;
        var top = layout.toDeviceIndependentPixels(this.effectivePaddingTop + this.effectiveBorderTopWidth);
        this.nativeViewProtected.contentEdgeInsets = {
            top: top,
            left: inset.left,
            bottom: inset.bottom,
            right: inset.right,
        };
    };
    Button.prototype[paddingRightProperty.getDefault] = function () {
        return {
            value: this.nativeViewProtected.contentEdgeInsets.right,
            unit: 'px',
        };
    };
    Button.prototype[paddingRightProperty.setNative] = function (value) {
        var inset = this.nativeViewProtected.contentEdgeInsets;
        var right = layout.toDeviceIndependentPixels(this.effectivePaddingRight + this.effectiveBorderRightWidth);
        this.nativeViewProtected.contentEdgeInsets = {
            top: inset.top,
            left: inset.left,
            bottom: inset.bottom,
            right: right,
        };
    };
    Button.prototype[paddingBottomProperty.getDefault] = function () {
        return {
            value: this.nativeViewProtected.contentEdgeInsets.bottom,
            unit: 'px',
        };
    };
    Button.prototype[paddingBottomProperty.setNative] = function (value) {
        var inset = this.nativeViewProtected.contentEdgeInsets;
        var bottom = layout.toDeviceIndependentPixels(this.effectivePaddingBottom + this.effectiveBorderBottomWidth);
        this.nativeViewProtected.contentEdgeInsets = {
            top: inset.top,
            left: inset.left,
            bottom: bottom,
            right: inset.right,
        };
    };
    Button.prototype[paddingLeftProperty.getDefault] = function () {
        return {
            value: this.nativeViewProtected.contentEdgeInsets.left,
            unit: 'px',
        };
    };
    Button.prototype[paddingLeftProperty.setNative] = function (value) {
        var inset = this.nativeViewProtected.contentEdgeInsets;
        var left = layout.toDeviceIndependentPixels(this.effectivePaddingLeft + this.effectiveBorderLeftWidth);
        this.nativeViewProtected.contentEdgeInsets = {
            top: inset.top,
            left: left,
            bottom: inset.bottom,
            right: inset.right,
        };
    };
    Button.prototype[textAlignmentProperty.setNative] = function (value) {
        switch (value) {
            case 'left':
                this.nativeViewProtected.titleLabel.textAlignment = 0 /* NSTextAlignment.Left */;
                this.nativeViewProtected.contentHorizontalAlignment = 1 /* UIControlContentHorizontalAlignment.Left */;
                break;
            case 'initial':
            case 'center':
                this.nativeViewProtected.titleLabel.textAlignment = 1 /* NSTextAlignment.Center */;
                this.nativeViewProtected.contentHorizontalAlignment = 0 /* UIControlContentHorizontalAlignment.Center */;
                break;
            case 'right':
                this.nativeViewProtected.titleLabel.textAlignment = 2 /* NSTextAlignment.Right */;
                this.nativeViewProtected.contentHorizontalAlignment = 2 /* UIControlContentHorizontalAlignment.Right */;
                break;
        }
    };
    Button.prototype[whiteSpaceProperty.setNative] = function (value) {
        this.adjustLineBreak();
    };
    Button.prototype[textOverflowProperty.setNative] = function (value) {
        this.adjustLineBreak();
    };
    Button.prototype.adjustLineBreak = function () {
        var whiteSpace = this.whiteSpace;
        var textOverflow = this.textOverflow;
        var nativeView = this.nativeViewProtected.titleLabel;
        switch (whiteSpace) {
            case 'normal':
                nativeView.lineBreakMode = 0 /* NSLineBreakMode.ByWordWrapping */;
                nativeView.numberOfLines = this.maxLines;
                break;
            case 'initial':
                nativeView.lineBreakMode = 5 /* NSLineBreakMode.ByTruncatingMiddle */;
                nativeView.numberOfLines = 1;
                break;
            case 'nowrap':
                switch (textOverflow) {
                    case 'clip':
                        nativeView.lineBreakMode = 2 /* NSLineBreakMode.ByClipping */;
                        nativeView.numberOfLines = this.maxLines;
                        break;
                    case 'ellipsis':
                        nativeView.lineBreakMode = 4 /* NSLineBreakMode.ByTruncatingTail */;
                        nativeView.numberOfLines = 1;
                        break;
                    default:
                        nativeView.lineBreakMode = 5 /* NSLineBreakMode.ByTruncatingMiddle */;
                        nativeView.numberOfLines = 1;
                        break;
                }
                break;
        }
    };
    Button.prototype.onMeasure = function (widthMeasureSpec, heightMeasureSpec) {
        // If there is text-wrap UIButton.sizeThatFits will return wrong result (not respecting the text wrap).
        // So fallback to original onMeasure if there is no text-wrap and use custom measure otherwise.
        if (!this.textWrap) {
            return _super.prototype.onMeasure.call(this, widthMeasureSpec, heightMeasureSpec);
        }
        var nativeView = this.nativeViewProtected;
        if (nativeView) {
            var width = layout.getMeasureSpecSize(widthMeasureSpec);
            var widthMode = layout.getMeasureSpecMode(widthMeasureSpec);
            var height = layout.getMeasureSpecSize(heightMeasureSpec);
            var heightMode = layout.getMeasureSpecMode(heightMeasureSpec);
            var horizontalPadding = this.effectivePaddingLeft + this.effectiveBorderLeftWidth + this.effectivePaddingRight + this.effectiveBorderRightWidth;
            var verticalPadding = this.effectivePaddingTop + this.effectiveBorderTopWidth + this.effectivePaddingBottom + this.effectiveBorderBottomWidth;
            // The default button padding for UIButton - 6dip top and bottom.
            if (verticalPadding === 0) {
                verticalPadding = layout.toDevicePixels(12);
            }
            var desiredSize = layout.measureNativeView(nativeView.titleLabel, width - horizontalPadding, widthMode, height - verticalPadding, heightMode);
            desiredSize.width = desiredSize.width + horizontalPadding;
            desiredSize.height = desiredSize.height + verticalPadding;
            var measureWidth = Math.max(desiredSize.width, this.effectiveMinWidth);
            var measureHeight = Math.max(desiredSize.height, this.effectiveMinHeight);
            var widthAndState = View.resolveSizeAndState(measureWidth, width, widthMode, 0);
            var heightAndState = View.resolveSizeAndState(measureHeight, height, heightMode, 0);
            this.setMeasuredDimension(widthAndState, heightAndState);
        }
    };
    __decorate([
        PseudoClassHandler('normal', 'highlighted', 'pressed', 'active'),
        __metadata("design:type", Function),
        __metadata("design:paramtypes", [Boolean]),
        __metadata("design:returntype", void 0)
    ], Button.prototype, "_updateButtonStateChangeHandler", null);
    return Button;
}(ButtonBase));
export { Button };
var TapHandlerImpl = /** @class */ (function (_super) {
    __extends(TapHandlerImpl, _super);
    function TapHandlerImpl() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    TapHandlerImpl.initWithOwner = function (owner) {
        var handler = TapHandlerImpl.new();
        handler._owner = owner;
        return handler;
    };
    TapHandlerImpl.prototype.tap = function (args) {
        var _a;
        // _owner is a {N} view which could get destroyed when a tap initiates (protect!)
        if (this._owner) {
            var owner = (_a = this._owner) === null || _a === void 0 ? void 0 : _a.deref();
            if (owner) {
                owner._emit(ButtonBase.tapEvent);
            }
        }
    };
    TapHandlerImpl.ObjCExposedMethods = {
        tap: { returns: interop.types.void, params: [interop.types.id] },
    };
    return TapHandlerImpl;
}(NSObject));
//# sourceMappingURL=index.ios.js.map