import { View, CustomLayoutView } from '../core/view';
import { layout } from '../../utils';
/**
 * Represents a View that has a single child - content.
 * The View itself does not have visual representation and serves as a placeholder for its content in the logical tree.
 */
var ContentView = /** @class */ (function (_super) {
    __extends(ContentView, _super);
    function ContentView() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    Object.defineProperty(ContentView.prototype, "content", {
        /**
         * Gets or sets the single child of the view.
         */
        get: function () {
            return this._content;
        },
        set: function (value) {
            var oldView = this._content;
            if (this._content) {
                this._removeView(this._content);
            }
            this._content = value;
            if (this._content) {
                this._addView(this._content);
            }
            this._onContentChanged(oldView, value);
            if (__APPLE__ && oldView !== value) {
                this.requestLayout();
            }
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(ContentView.prototype, "layoutView", {
        get: function () {
            var result;
            if (this._content) {
                var first_1 = true;
                this._content._eachLayoutView(function (child) {
                    if (first_1) {
                        first_1 = false;
                        result = child;
                    }
                    else {
                        throw new Error('More than one layout child inside a ContentView');
                    }
                });
            }
            return result;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(ContentView.prototype, "_childrenCount", {
        get: function () {
            return this._content ? 1 : 0;
        },
        enumerable: false,
        configurable: true
    });
    //@private
    /**
     * Called when the content property has changed.
     * @private
     * @param oldView The previous content.
     * @param newView The new content.
     */
    ContentView.prototype._onContentChanged = function (oldView, newView) {
        //
    };
    ContentView.prototype._addChildFromBuilder = function (name, value) {
        if (value instanceof View) {
            this.content = value;
        }
    };
    ContentView.prototype.eachChildView = function (callback) {
        var content = this._content;
        if (content) {
            callback(content);
        }
    };
    // This method won't be called in Android because we use the native android layout.
    ContentView.prototype.onMeasure = function (widthMeasureSpec, heightMeasureSpec) {
        var result = View.measureChild(this, this.layoutView, widthMeasureSpec, heightMeasureSpec);
        var width = layout.getMeasureSpecSize(widthMeasureSpec);
        var widthMode = layout.getMeasureSpecMode(widthMeasureSpec);
        var height = layout.getMeasureSpecSize(heightMeasureSpec);
        var heightMode = layout.getMeasureSpecMode(heightMeasureSpec);
        var measureWidth = Math.max(result.measuredWidth, this.effectiveMinWidth);
        var measureHeight = Math.max(result.measuredHeight, this.effectiveMinHeight);
        var widthAndState = View.resolveSizeAndState(measureWidth, width, widthMode, 0);
        var heightAndState = View.resolveSizeAndState(measureHeight, height, heightMode, 0);
        this.setMeasuredDimension(widthAndState, heightAndState);
    };
    // This method won't be called in Android because we use the native android layout.
    ContentView.prototype.onLayout = function (left, top, right, bottom) {
        View.layoutChild(this, this.layoutView, 0, 0, right - left, bottom - top);
    };
    return ContentView;
}(CustomLayoutView));
export { ContentView };
ContentView.prototype.recycleNativeView = 'auto';
//# sourceMappingURL=index.js.map