import { DatePickerBase, yearProperty, monthProperty, dayProperty, dateProperty, maxDateProperty, minDateProperty } from './date-picker-common';
import { TimePicker } from '../time-picker';
export * from './date-picker-common';
var DateChangedListener;
function initializeDateChangedListener() {
    if (DateChangedListener) {
        return;
    }
    var DateChangedListenerImpl = /** @class */ (function (_super) {
    __extends(DateChangedListenerImpl, _super);
    function DateChangedListenerImpl(owner) {
        var _this = _super.call(this) || this;
        _this.owner = owner;
        return global.__native(_this);
    }
    DateChangedListenerImpl.prototype.onDateChanged = function (picker, year, month, day) {
        var owner = this.owner;
        var dateChanged = false;
        if (year !== owner.year) {
            yearProperty.nativeValueChange(owner, year);
            dateChanged = true;
        }
        if (month !== owner.month - 1) {
            monthProperty.nativeValueChange(owner, month + 1);
            dateChanged = true;
        }
        if (day !== owner.day) {
            dayProperty.nativeValueChange(owner, day);
            dateChanged = true;
        }
        if (dateChanged || (owner.showTime && owner.timePicker)) {
            var newDate = void 0;
            if (owner.showTime && owner.timePicker) {
                var dateTime = owner.timePicker.time;
                newDate = new Date(year, month, day, dateTime.getHours(), dateTime.getMinutes(), dateTime.getSeconds(), dateTime.getMilliseconds());
            }
            else {
                newDate = new Date(year, month, day);
            }
            dateProperty.nativeValueChange(owner, newDate);
        }
    };
    var _a;
    DateChangedListenerImpl = __decorate([
        Interfaces([android.widget.DatePicker.OnDateChangedListener]),
        __metadata("design:paramtypes", [typeof (_a = typeof DatePicker !== "undefined" && DatePicker) === "function" ? _a : Object])
    ], DateChangedListenerImpl);
    return DateChangedListenerImpl;
}(java.lang.Object));
    DateChangedListener = DateChangedListenerImpl;
}
var DatePicker = /** @class */ (function (_super) {
    __extends(DatePicker, _super);
    function DatePicker() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    DatePicker.prototype.createNativeView = function () {
        var picker = new android.widget.DatePicker(this._context);
        picker.setCalendarViewShown(false);
        return picker;
    };
    DatePicker.prototype.initNativeView = function () {
        var _this = this;
        _super.prototype.initNativeView.call(this);
        initializeDateChangedListener();
        var nativeView = this.nativeViewProtected;
        var listener = new DateChangedListener(this);
        nativeView.init(this.year, this.month - 1, this.day, listener);
        nativeView.listener = listener;
        if (this.showTime) {
            this.timePicker = new TimePicker();
            this.timePicker.width = this.width;
            this.timePicker.height = this.height;
            this.timePicker.on('timeChange', function (args) {
                _this.updateNativeDate();
            });
            this.parent.addChild(this.timePicker);
        }
    };
    DatePicker.prototype.disposeNativeView = function () {
        var _a;
        if (this.timePicker) {
            this.timePicker.disposeNativeView();
        }
        if ((_a = this.nativeViewProtected) === null || _a === void 0 ? void 0 : _a.listener) {
            this.nativeViewProtected.listener.owner = null;
        }
        _super.prototype.disposeNativeView.call(this);
    };
    DatePicker.prototype.updateNativeDate = function () {
        var nativeView = this.nativeViewProtected;
        var year = typeof this.year === 'number' ? this.year : nativeView.getYear();
        var month = typeof this.month === 'number' ? this.month - 1 : nativeView.getMonth();
        var day = typeof this.day === 'number' ? this.day : nativeView.getDayOfMonth();
        if (this.showTime && this.timePicker) {
            var time = this.timePicker.time || new Date();
            this.date = new Date(year, month, day, time.getHours(), time.getMinutes(), time.getSeconds(), time.getMilliseconds());
        }
        else {
            this.date = new Date(year, month, day);
        }
    };
    DatePicker.prototype[yearProperty.setNative] = function (value) {
        if (this.nativeViewProtected.getYear() !== value) {
            this.updateNativeDate();
        }
    };
    DatePicker.prototype[monthProperty.setNative] = function (value) {
        if (this.nativeViewProtected.getMonth() !== value - 1) {
            this.updateNativeDate();
        }
    };
    DatePicker.prototype[dayProperty.setNative] = function (value) {
        if (this.nativeViewProtected.getDayOfMonth() !== value) {
            this.updateNativeDate();
        }
    };
    DatePicker.prototype[dateProperty.setNative] = function (value) {
        var nativeView = this.nativeViewProtected;
        if (value && (nativeView.getDayOfMonth() !== value.getDate() || nativeView.getMonth() !== value.getMonth() || nativeView.getYear() !== value.getFullYear())) {
            nativeView.updateDate(value.getFullYear(), value.getMonth(), value.getDate());
        }
    };
    DatePicker.prototype[maxDateProperty.getDefault] = function () {
        return this.nativeViewProtected.getMaxDate();
    };
    DatePicker.prototype[maxDateProperty.setNative] = function (value) {
        var newValue = value instanceof Date ? value.getTime() : value;
        this.nativeViewProtected.setMaxDate(newValue);
    };
    DatePicker.prototype[minDateProperty.getDefault] = function () {
        return this.nativeViewProtected.getMinDate();
    };
    DatePicker.prototype[minDateProperty.setNative] = function (value) {
        var newValue = value instanceof Date ? value.getTime() : value;
        this.nativeViewProtected.setMinDate(newValue);
    };
    return DatePicker;
}(DatePickerBase));
export { DatePicker };
//# sourceMappingURL=index.android.js.map