import { DatePickerBase, yearProperty, monthProperty, dayProperty, dateProperty, maxDateProperty, minDateProperty, hourProperty, minuteProperty, secondProperty, showTimeProperty, iosPreferredDatePickerStyleProperty } from './date-picker-common';
import { colorProperty } from '../styling/style-properties';
import { Color } from '../../color';
import { Device } from '../../platform';
export * from './date-picker-common';
var SUPPORT_DATE_PICKER_STYLE = __VISIONOS__ || parseFloat(Device.osVersion) >= 13.4;
var SUPPORT_TEXT_COLOR = !__VISIONOS__ && parseFloat(Device.osVersion) < 14.0;
var DatePicker = /** @class */ (function (_super) {
    __extends(DatePicker, _super);
    function DatePicker() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    DatePicker.prototype.createNativeView = function () {
        var picker = UIDatePicker.new();
        picker.datePickerMode = this.showTime ? 2 /* UIDatePickerMode.DateAndTime */ : 1 /* UIDatePickerMode.Date */;
        if (SUPPORT_DATE_PICKER_STYLE) {
            picker.preferredDatePickerStyle = this.iosPreferredDatePickerStyle;
        }
        return picker;
    };
    DatePicker.prototype.initNativeView = function () {
        _super.prototype.initNativeView.call(this);
        var nativeView = this.nativeViewProtected;
        this._changeHandler = UIDatePickerChangeHandlerImpl.initWithOwner(new WeakRef(this));
        nativeView.addTargetActionForControlEvents(this._changeHandler, 'valueChanged', 4096 /* UIControlEvents.ValueChanged */);
    };
    DatePicker.prototype.disposeNativeView = function () {
        this._changeHandler = null;
        _super.prototype.disposeNativeView.call(this);
    };
    Object.defineProperty(DatePicker.prototype, "ios", {
        // @ts-ignore
        get: function () {
            return this.nativeViewProtected;
        },
        enumerable: false,
        configurable: true
    });
    DatePicker.prototype[showTimeProperty.setNative] = function (value) {
        this.showTime = value;
        if (this.nativeViewProtected) {
            this.nativeViewProtected.datePickerMode = this.showTime ? 2 /* UIDatePickerMode.DateAndTime */ : 1 /* UIDatePickerMode.Date */;
        }
    };
    DatePicker.prototype[iosPreferredDatePickerStyleProperty.setNative] = function (value) {
        this.iosPreferredDatePickerStyle = value;
        if (this.nativeViewProtected) {
            if (SUPPORT_DATE_PICKER_STYLE) {
                this.nativeViewProtected.preferredDatePickerStyle = this.iosPreferredDatePickerStyle;
            }
        }
    };
    DatePicker.prototype[yearProperty.setNative] = function (value) {
        this.date = new Date(value, this.month - 1, this.day, this.hour || 0, this.minute || 0, this.second || 0);
    };
    DatePicker.prototype[monthProperty.setNative] = function (value) {
        this.date = new Date(this.year, value - 1, this.day, this.hour || 0, this.minute || 0, this.second || 0);
    };
    DatePicker.prototype[dayProperty.setNative] = function (value) {
        this.date = new Date(this.year, this.month - 1, value, this.hour || 0, this.minute || 0, this.second || 0);
    };
    DatePicker.prototype[hourProperty.setNative] = function (value) {
        this.date = new Date(this.year, this.month - 1, this.day, value, this.minute || 0, this.second || 0);
    };
    DatePicker.prototype[minuteProperty.setNative] = function (value) {
        this.date = new Date(this.year, this.month - 1, this.day, this.hour || 0, value, this.second || 0);
    };
    DatePicker.prototype[secondProperty.setNative] = function (value) {
        this.date = new Date(this.year, this.month - 1, this.day, this.hour || 0, this.minute || 0, value);
    };
    DatePicker.prototype[dateProperty.setNative] = function (value) {
        if (value) {
            var comps = NSCalendar.currentCalendar.componentsFromDate(4 /* NSCalendarUnit.CalendarUnitYear */ | 8 /* NSCalendarUnit.CalendarUnitMonth */ | 16 /* NSCalendarUnit.CalendarUnitDay */ | 32 /* NSCalendarUnit.HourCalendarUnit */ | 64 /* NSCalendarUnit.MinuteCalendarUnit */ | 128 /* NSCalendarUnit.SecondCalendarUnit */, this.nativeViewProtected.date);
            comps.year = value.getFullYear();
            comps.month = value.getMonth() + 1;
            comps.day = value.getDate();
            comps.hour = value.getHours();
            comps.minute = value.getMinutes();
            comps.second = value.getSeconds();
            this.year = comps.year;
            this.month = comps.month;
            this.day = comps.day;
            this.hour = comps.hour;
            this.minute = comps.minute;
            this.second = comps.second;
            this.nativeViewProtected.setDateAnimated(NSCalendar.currentCalendar.dateFromComponents(comps), false);
        }
    };
    DatePicker.prototype[maxDateProperty.getDefault] = function () {
        return this.nativeViewProtected.maximumDate;
    };
    DatePicker.prototype[maxDateProperty.setNative] = function (value) {
        if (value) {
            var nsDate = NSDate.dateWithTimeIntervalSince1970(value.getTime() / 1000);
            this.nativeViewProtected.maximumDate = nsDate;
        }
        else {
            this.nativeViewProtected.maximumDate = null;
        }
    };
    DatePicker.prototype[minDateProperty.getDefault] = function () {
        return this.nativeViewProtected.minimumDate;
    };
    DatePicker.prototype[minDateProperty.setNative] = function (value) {
        if (value) {
            this.nativeViewProtected.minimumDate = NSDate.dateWithTimeIntervalSince1970(value.getTime() / 1000);
        }
        else {
            this.nativeViewProtected.minimumDate = null;
        }
    };
    DatePicker.prototype[colorProperty.getDefault] = function () {
        return SUPPORT_TEXT_COLOR ? this.nativeViewProtected.valueForKey('textColor') : UIColor.new();
    };
    DatePicker.prototype[colorProperty.setNative] = function (value) {
        if (SUPPORT_TEXT_COLOR) {
            var picker = this.nativeViewProtected;
            picker.setValueForKey(value instanceof Color ? value.ios : value, 'textColor');
        }
    };
    return DatePicker;
}(DatePickerBase));
export { DatePicker };
var UIDatePickerChangeHandlerImpl = /** @class */ (function (_super) {
    __extends(UIDatePickerChangeHandlerImpl, _super);
    function UIDatePickerChangeHandlerImpl() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    UIDatePickerChangeHandlerImpl.initWithOwner = function (owner) {
        var impl = UIDatePickerChangeHandlerImpl.new();
        impl._owner = owner;
        return impl;
    };
    UIDatePickerChangeHandlerImpl.prototype.valueChanged = function (sender) {
        var _a;
        var owner = (_a = this._owner) === null || _a === void 0 ? void 0 : _a.deref();
        if (!owner) {
            return;
        }
        var comps = NSCalendar.currentCalendar.componentsFromDate(NSCalendarUnit.CalendarUnitYear | NSCalendarUnit.CalendarUnitMonth | NSCalendarUnit.CalendarUnitDay | NSCalendarUnit.HourCalendarUnit | NSCalendarUnit.MinuteCalendarUnit | NSCalendarUnit.SecondCalendarUnit, sender.date);
        var dateChanged = false;
        if (comps.year !== owner.year) {
            yearProperty.nativeValueChange(owner, comps.year);
            dateChanged = true;
        }
        if (comps.month !== owner.month) {
            monthProperty.nativeValueChange(owner, comps.month);
            dateChanged = true;
        }
        if (comps.day !== owner.day) {
            dayProperty.nativeValueChange(owner, comps.day);
            dateChanged = true;
        }
        if (comps.hour !== owner.hour) {
            hourProperty.nativeValueChange(owner, comps.hour);
            dateChanged = true;
        }
        if (comps.minute !== owner.minute) {
            minuteProperty.nativeValueChange(owner, comps.minute);
            dateChanged = true;
        }
        if (comps.second !== owner.second) {
            secondProperty.nativeValueChange(owner, comps.second);
            dateChanged = true;
        }
        if (dateChanged) {
            dateProperty.nativeValueChange(owner, new Date(comps.year, comps.month - 1, comps.day, comps.hour, comps.minute, comps.second));
        }
    };
    UIDatePickerChangeHandlerImpl.ObjCExposedMethods = {
        valueChanged: { returns: interop.types.void, params: [UIDatePicker] },
    };
    return UIDatePickerChangeHandlerImpl;
}(NSObject));
//# sourceMappingURL=index.ios.js.map