import { TextBase } from '../text-base';
import { Property, CssProperty, makeValidator, makeParser } from '../core/properties';
import { PseudoClassHandler } from '../core/view';
import { booleanConverter } from '../core/view-base';
import { Style } from '../styling/style';
import { Color } from '../../color';
import { CoreTypes } from '../../core-types';
function focusChangeHandler(args) {
    var view = args.object;
    if (args.eventName === 'focus') {
        view._addVisualState('focus');
        view._removeVisualState('blur');
    }
    else {
        view._addVisualState('blur');
        view._removeVisualState('focus');
    }
}
var EditableTextBase = /** @class */ (function (_super) {
    __extends(EditableTextBase, _super);
    function EditableTextBase() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    EditableTextBase.prototype._updateTextBaseFocusStateHandler = function (subscribe) {
        var method = subscribe ? 'on' : 'off';
        this[method]('focus', focusChangeHandler);
        this[method]('blur', focusChangeHandler);
    };
    EditableTextBase.blurEvent = 'blur';
    EditableTextBase.focusEvent = 'focus';
    EditableTextBase.textChangeEvent = 'textChange';
    __decorate([
        PseudoClassHandler('focus', 'blur'),
        __metadata("design:type", Function),
        __metadata("design:paramtypes", [Object]),
        __metadata("design:returntype", void 0)
    ], EditableTextBase.prototype, "_updateTextBaseFocusStateHandler", null);
    return EditableTextBase;
}(TextBase));
export { EditableTextBase };
// TODO: Why not name it - hintColor property??
// TODO: Or rename hintProperty to 'placeholder' and make it CSSProperty??
// https://developer.mozilla.org/en-US/docs/Web/CSS/:-moz-placeholder
export var placeholderColorProperty = new CssProperty({
    name: 'placeholderColor',
    cssName: 'placeholder-color',
    equalityComparer: Color.equals,
    valueConverter: function (v) { return new Color(v); },
});
placeholderColorProperty.register(Style);
var keyboardTypeConverter = makeParser(makeValidator(CoreTypes.KeyboardType.datetime, CoreTypes.KeyboardType.phone, CoreTypes.KeyboardType.number, CoreTypes.KeyboardType.url, CoreTypes.KeyboardType.email, CoreTypes.KeyboardType.integer), true);
export var autofillTypeProperty = new Property({ name: 'autofillType' });
autofillTypeProperty.register(EditableTextBase);
export var keyboardTypeProperty = new Property({ name: 'keyboardType', valueConverter: keyboardTypeConverter });
keyboardTypeProperty.register(EditableTextBase);
var returnKeyTypeConverter = makeParser(makeValidator(CoreTypes.ReturnKeyType.done, CoreTypes.ReturnKeyType.next, CoreTypes.ReturnKeyType.go, CoreTypes.ReturnKeyType.search, CoreTypes.ReturnKeyType.send), true);
export var returnKeyTypeProperty = new Property({ name: 'returnKeyType', valueConverter: returnKeyTypeConverter });
returnKeyTypeProperty.register(EditableTextBase);
export var editableProperty = new Property({
    name: 'editable',
    defaultValue: true,
    valueConverter: booleanConverter,
});
editableProperty.register(EditableTextBase);
export var updateTextTriggerProperty = new Property({ name: 'updateTextTrigger', defaultValue: CoreTypes.UpdateTextTrigger.textChanged });
updateTextTriggerProperty.register(EditableTextBase);
var autocapitalizationTypeConverter = makeParser(makeValidator(CoreTypes.AutocapitalizationType.none, CoreTypes.AutocapitalizationType.words, CoreTypes.AutocapitalizationType.sentences, CoreTypes.AutocapitalizationType.allCharacters), true);
export var autocapitalizationTypeProperty = new Property({
    name: 'autocapitalizationType',
    defaultValue: CoreTypes.AutocapitalizationType.sentences,
    valueConverter: autocapitalizationTypeConverter,
});
autocapitalizationTypeProperty.register(EditableTextBase);
export var autocorrectProperty = new Property({
    name: 'autocorrect',
    valueConverter: booleanConverter,
});
autocorrectProperty.register(EditableTextBase);
export var hintProperty = new Property({
    name: 'hint',
    defaultValue: '',
});
hintProperty.register(EditableTextBase);
export var maxLengthProperty = new Property({
    name: 'maxLength',
    defaultValue: Number.POSITIVE_INFINITY,
    valueConverter: parseInt,
});
maxLengthProperty.register(EditableTextBase);
//# sourceMappingURL=editable-text-base-common.js.map