import '../../globals';
import { setActivityCallbacks } from '.';
import { Application } from '../../application';
import { isEmbedded } from '../embedding';
var EMPTY_FN = function () { };
if (!isEmbedded()) {
    /**
     * NOTE: We cannot use NativeClass here because this is used in appComponents in webpack.config
     * Whereby it bypasses the decorator transformation, hence pure es5 style written here
     */
    var superProto_1 = androidx.appcompat.app.AppCompatActivity.prototype;
    androidx.appcompat.app.AppCompatActivity.extend('com.tns.NativeScriptActivity', {
        init: function () {
            // init must at least be defined
        },
        onCreate: function (savedInstanceState) {
            Application.android.init(this.getApplication());
            // Set isNativeScriptActivity in onCreate.
            // The JS constructor might not be called because the activity is created from Android.
            this.isNativeScriptActivity = true;
            if (!this._callbacks) {
                setActivityCallbacks(this);
            }
            this._callbacks.onCreate(this, savedInstanceState, this.getIntent(), superProto_1.onCreate);
        },
        onNewIntent: function (intent) {
            this._callbacks.onNewIntent(this, intent, superProto_1.setIntent, superProto_1.onNewIntent);
        },
        onSaveInstanceState: function (outState) {
            this._callbacks.onSaveInstanceState(this, outState, superProto_1.onSaveInstanceState);
        },
        onStart: function () {
            this._callbacks.onStart(this, superProto_1.onStart);
        },
        onStop: function () {
            this._callbacks.onStop(this, superProto_1.onStop);
        },
        onDestroy: function () {
            this._callbacks.onDestroy(this, superProto_1.onDestroy);
        },
        onPostResume: function () {
            this._callbacks.onPostResume(this, superProto_1.onPostResume);
        },
        onBackPressed: function () {
            this._callbacks.onBackPressed(this, superProto_1.onBackPressed);
        },
        onRequestPermissionsResult: function (requestCode, permissions, grantResults) {
            this._callbacks.onRequestPermissionsResult(this, requestCode, permissions, grantResults, undefined /*TODO: Enable if needed*/);
        },
        onActivityResult: function (requestCode, resultCode, data) {
            this._callbacks.onActivityResult(this, requestCode, resultCode, data, superProto_1.onActivityResult);
        },
    });
}
else {
    var Callbacks = com.tns.embedding.EmbeddableActivityCallbacks.extend({
        init: function () {
            // init must at least be defined
        },
        onCreate: function (savedInstanceState) {
            var activity = this.getActivity();
            Application.android.init(activity.getApplication());
            // Set isNativeScriptActivity in onCreate.
            // The JS constructor might not be called because the activity is created from Android.
            activity.isNativeScriptActivity = true;
            if (!activity._callbacks) {
                setActivityCallbacks(activity);
            }
            activity._callbacks.onCreate(activity, savedInstanceState, activity.getIntent(), EMPTY_FN);
        },
        onNewIntent: function (intent) {
            var activity = this.getActivity();
            activity._callbacks.onNewIntent(activity, intent, EMPTY_FN, EMPTY_FN);
        },
        onSaveInstanceState: function (outState) {
            var activity = this.getActivity();
            activity._callbacks.onSaveInstanceState(activity, outState, EMPTY_FN);
        },
        onStart: function () {
            var activity = this.getActivity();
            activity._callbacks.onStart(activity, EMPTY_FN);
        },
        onStop: function () {
            var activity = this.getActivity();
            activity._callbacks.onStop(activity, EMPTY_FN);
        },
        onDestroy: function () {
            var activity = this.getActivity();
            activity._callbacks.onDestroy(activity, EMPTY_FN);
        },
        onPostResume: function () {
            var activity = this.getActivity();
            activity._callbacks.onPostResume(activity, EMPTY_FN);
        },
        onBackPressed: function () {
            var activity = this.getActivity();
            activity._callbacks.onBackPressed(activity, EMPTY_FN);
        },
        onRequestPermissionsResult: function (requestCode, permissions, grantResults) {
            var activity = this.getActivity();
            activity._callbacks.onRequestPermissionsResult(activity, requestCode, permissions, grantResults, undefined /*TODO: Enable if needed*/);
        },
        onActivityResult: function (requestCode, resultCode, data) {
            var activity = this.getActivity();
            activity._callbacks.onActivityResult(activity, requestCode, resultCode, data, EMPTY_FN);
        },
    });
    com.tns.embedding.CallbacksStore.setActivityCallbacks(new Callbacks());
}
//# sourceMappingURL=activity.android.js.map