import { isEmbedded, getEmbeddedView } from '../embedding';
import { setFragmentCallbacks } from '.';
var superProto = org.nativescript.widgets.FragmentBase.prototype;
var FragmentClass = org.nativescript.widgets.FragmentBase.extend('com.tns.FragmentClass', {
    init: function () { },
    onHiddenChanged: function (hidden) {
        this._callbacks.onHiddenChanged(this, hidden, superProto.onHiddenChanged);
    },
    onCreateAnimator: function (transit, enter, nextAnim) {
        return this._callbacks.onCreateAnimator(this, transit, enter, nextAnim, superProto.onCreateAnimator);
    },
    onStop: function () {
        this._callbacks.onStop(this, superProto.onStop);
    },
    onPause: function () {
        this._callbacks.onPause(this, superProto.onPause);
    },
    onResume: function () {
        this._callbacks.onResume(this, superProto.onResume);
    },
    onCreate: function (savedInstanceState) {
        if (!this._callbacks) {
            setFragmentCallbacks(this);
        }
        this.setHasOptionsMenu(true);
        this._callbacks.onCreate(this, savedInstanceState, superProto.onCreate);
    },
    onCreateView: function (inflater, container, savedInstanceState) {
        return this._callbacks.onCreateView(this, inflater, container, savedInstanceState, superProto.onCreateView);
    },
    onSaveInstanceState: function (outState) {
        this._callbacks.onSaveInstanceState(this, outState, superProto.onSaveInstanceState);
    },
    onDestroyView: function () {
        this._callbacks.onDestroyView(this, superProto.onDestroyView);
    },
    onDestroy: function () {
        this._callbacks.onDestroy(this, superProto.onDestroy);
        this._callbacks = null;
    },
    toString: function () {
        var callbacks = this._callbacks;
        if (callbacks) {
            return callbacks.toStringOverride(this, superProto.toString);
        }
        else {
            superProto.toString();
        }
    },
});
export var fragmentClass;
export function ensureFragmentClass() {
    if (fragmentClass) {
        return;
    }
    // this require will apply the FragmentClass implementation
    require('./fragment');
    if (!fragmentClass) {
        throw new Error('Failed to initialize the extended androidx.fragment.app.Fragment class');
    }
}
export function setFragmentClass(clazz) {
    if (fragmentClass) {
        throw new Error('Fragment class already initialized');
    }
    if (isEmbedded()) {
        attachEmbeddableFragmentCallbacks();
    }
    fragmentClass = clazz;
}
function attachEmbeddableFragmentCallbacks() {
    var Callbacks = com.tns.embedding.EmbeddableFragmentCallbacks.extend({
        init: function () {
            // init must at least be defined
        },
        onCreateView: function () {
            return getEmbeddedView().nativeViewProtected;
        },
        onResume: function () {
            getEmbeddedView().callLoaded();
        },
        onPause: function () {
            getEmbeddedView().callUnloaded();
        },
    });
    com.tns.embedding.CallbacksStore.setFragmentCallbacks(new Callbacks());
}
setFragmentClass(FragmentClass);
//# sourceMappingURL=fragment.android.js.map