// Definitions.
// Types.
import { GesturesObserverBase, toString, TouchAction, GestureStateTypes, GestureTypes, SwipeDirection, GestureEvents } from './gestures-common';
export * from './gestures-common';
export function observe(target, type, callback, context) {
    var observer = new GesturesObserver(target, callback, context);
    observer.observe(type);
    return observer;
}
var UIGestureRecognizerDelegateImpl = /** @class */ (function (_super) {
    __extends(UIGestureRecognizerDelegateImpl, _super);
    function UIGestureRecognizerDelegateImpl() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    UIGestureRecognizerDelegateImpl.prototype.gestureRecognizerShouldRecognizeSimultaneouslyWithGestureRecognizer = function (gestureRecognizer, otherGestureRecognizer) {
        return true;
    };
    UIGestureRecognizerDelegateImpl.prototype.gestureRecognizerShouldRequireFailureOfGestureRecognizer = function (gestureRecognizer, otherGestureRecognizer) {
        // If both gesture recognizers are of type UITapGestureRecognizer & one of them is a doubleTap,
        // we must require a failure.
        if (gestureRecognizer instanceof UITapGestureRecognizer && otherGestureRecognizer instanceof UITapGestureRecognizer && otherGestureRecognizer.numberOfTapsRequired === 2) {
            return true;
        }
        return false;
    };
    UIGestureRecognizerDelegateImpl.ObjCProtocols = [UIGestureRecognizerDelegate];
    return UIGestureRecognizerDelegateImpl;
}(NSObject));
var recognizerDelegateInstance = UIGestureRecognizerDelegateImpl.new();
var UIGestureRecognizerImpl = /** @class */ (function (_super) {
    __extends(UIGestureRecognizerImpl, _super);
    function UIGestureRecognizerImpl() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    UIGestureRecognizerImpl.initWithOwnerTypeCallback = function (owner, type, callback, thisArg) {
        var handler = UIGestureRecognizerImpl.new();
        handler._owner = owner;
        handler._type = type;
        if (callback) {
            handler._callback = callback;
        }
        if (thisArg) {
            handler._context = thisArg;
        }
        return handler;
    };
    UIGestureRecognizerImpl.prototype.recognize = function (recognizer) {
        var _a;
        var owner = (_a = this._owner) === null || _a === void 0 ? void 0 : _a.deref();
        var callback = this._callback ? this._callback : owner ? owner.callback : null;
        var typeParam = this._type;
        var target = owner ? owner.target : undefined;
        var args = {
            type: typeParam,
            view: target,
            ios: recognizer,
            android: undefined,
            object: target,
            eventName: toString(typeParam),
        };
        if (callback) {
            callback.call(this._context, args);
        }
    };
    UIGestureRecognizerImpl.ObjCExposedMethods = {
        recognize: {
            returns: interop.types.void,
            params: [UIGestureRecognizer],
        },
    };
    return UIGestureRecognizerImpl;
}(NSObject));
var GesturesObserver = /** @class */ (function (_super) {
    __extends(GesturesObserver, _super);
    function GesturesObserver() {
        var _this = _super !== null && _super.apply(this, arguments) || this;
        _this._recognizers = {};
        return _this;
    }
    GesturesObserver.prototype.androidOnTouchEvent = function (motionEvent) {
        //
    };
    /**
     * Observes a singular GestureTypes value (e.g. GestureTypes.tap).
     *
     * Does not support observing plural GestureTypes values, e.g.
     * GestureTypes.tap & GestureTypes.doubleTap.
     */
    GesturesObserver.prototype.observe = function (type) {
        var _this = this;
        this.type = type;
        if (!this.target) {
            return;
        }
        this._onTargetLoaded = function () {
            _this._attach(_this.target, type);
        };
        this._onTargetUnloaded = function () {
            _this._detach();
        };
        this.target.on('loaded', this._onTargetLoaded);
        this.target.on('unloaded', this._onTargetUnloaded);
        if (this.target.isLoaded) {
            this._attach(this.target, type);
        }
    };
    /**
     * Given a singular GestureTypes value (e.g. GestureTypes.tap), adds a
     * UIGestureRecognizer for it and populates a RecognizerCache entry in
     * this._recognizers.
     *
     * Does not support attaching plural GestureTypes values, e.g.
     * GestureTypes.tap & GestureTypes.doubleTap.
     */
    GesturesObserver.prototype._attach = function (target, type) {
        var _this = this;
        this._detach();
        var nativeView = target === null || target === void 0 ? void 0 : target.nativeViewProtected;
        if (!(nativeView === null || nativeView === void 0 ? void 0 : nativeView.addGestureRecognizer)) {
            return;
        }
        switch (type) {
            case GestureTypes.tap: {
                nativeView.addGestureRecognizer(this._createRecognizer(GestureTypes.tap, function (args) {
                    if (args.view) {
                        _this._executeCallback(_getTapData(args));
                    }
                }));
                break;
            }
            case GestureTypes.doubleTap: {
                nativeView.addGestureRecognizer(this._createRecognizer(GestureTypes.doubleTap, function (args) {
                    if (args.view) {
                        _this._executeCallback(_getTapData(args));
                    }
                }));
                break;
            }
            case GestureTypes.pinch: {
                nativeView.addGestureRecognizer(this._createRecognizer(GestureTypes.pinch, function (args) {
                    if (args.view) {
                        _this._executeCallback(_getPinchData(args));
                    }
                }));
                break;
            }
            case GestureTypes.pan: {
                nativeView.addGestureRecognizer(this._createRecognizer(GestureTypes.pan, function (args) {
                    if (args.view) {
                        _this._executeCallback(_getPanData(args, target.nativeViewProtected));
                    }
                }));
                break;
            }
            case GestureTypes.swipe: {
                nativeView.addGestureRecognizer(this._createRecognizer(GestureTypes.swipe, function (args) {
                    if (args.view) {
                        _this._executeCallback(_getSwipeData(args));
                    }
                }, 8 /* UISwipeGestureRecognizerDirection.Down */));
                nativeView.addGestureRecognizer(this._createRecognizer(GestureTypes.swipe, function (args) {
                    if (args.view) {
                        _this._executeCallback(_getSwipeData(args));
                    }
                }, 2 /* UISwipeGestureRecognizerDirection.Left */));
                nativeView.addGestureRecognizer(this._createRecognizer(GestureTypes.swipe, function (args) {
                    if (args.view) {
                        _this._executeCallback(_getSwipeData(args));
                    }
                }, 1 /* UISwipeGestureRecognizerDirection.Right */));
                nativeView.addGestureRecognizer(this._createRecognizer(GestureTypes.swipe, function (args) {
                    if (args.view) {
                        _this._executeCallback(_getSwipeData(args));
                    }
                }, 4 /* UISwipeGestureRecognizerDirection.Up */));
                break;
            }
            case GestureTypes.rotation: {
                nativeView.addGestureRecognizer(this._createRecognizer(GestureTypes.rotation, function (args) {
                    if (args.view) {
                        _this._executeCallback(_getRotationData(args));
                    }
                }));
                break;
            }
            case GestureTypes.longPress: {
                nativeView.addGestureRecognizer(this._createRecognizer(GestureTypes.longPress, function (args) {
                    if (args.view) {
                        _this._executeCallback(_getLongPressData(args));
                    }
                }));
                break;
            }
            case GestureTypes.touch: {
                nativeView.addGestureRecognizer(this._createRecognizer(GestureTypes.touch));
                break;
            }
        }
    };
    GesturesObserver.prototype._detach = function () {
        var _a, _b;
        for (var type in this._recognizers) {
            var item = this._recognizers[type];
            (_b = (_a = this.target) === null || _a === void 0 ? void 0 : _a.nativeViewProtected) === null || _b === void 0 ? void 0 : _b.removeGestureRecognizer(item.recognizer);
            item.recognizer = null;
            item.target = null;
            delete this._recognizers[type];
        }
    };
    GesturesObserver.prototype.disconnect = function () {
        this._detach();
        if (this.target) {
            this.target.off('loaded', this._onTargetLoaded);
            this.target.off('unloaded', this._onTargetUnloaded);
            this._onTargetLoaded = null;
            this._onTargetUnloaded = null;
        }
        // clears target, context and callback references
        _super.prototype.disconnect.call(this);
    };
    GesturesObserver.prototype._executeCallback = function (args) {
        if (this.callback) {
            this.callback.call(this.context, args);
        }
    };
    /**
     * Creates a UIGestureRecognizer (and populates a RecognizerCache entry in
     * this._recognizers) corresponding to the singular GestureTypes value passed
     * in.
     */
    GesturesObserver.prototype._createRecognizer = function (type, callback, swipeDirection) {
        var recognizer;
        var typeString = toString(type);
        var target = _createUIGestureRecognizerTarget(this, type, callback, this.context);
        var recognizerType = _getUIGestureRecognizerType(type);
        if (recognizerType) {
            recognizer = recognizerType.alloc().initWithTargetAction(target, 'recognize');
            if (type === GestureTypes.swipe && swipeDirection) {
                // e.g. "swipe1"
                typeString += swipeDirection.toString();
                recognizer.direction = swipeDirection;
            }
            else if (type === GestureTypes.touch) {
                recognizer.observer = this;
            }
            else if (type === GestureTypes.doubleTap) {
                recognizer.numberOfTapsRequired = 2;
            }
            if (recognizer) {
                recognizer.delegate = recognizerDelegateInstance;
                this._recognizers[typeString] = {
                    recognizer: recognizer,
                    target: target,
                };
            }
            this.target.notify({
                eventName: GestureEvents.gestureAttached,
                object: this.target,
                type: type,
                view: this.target,
                ios: recognizer,
            });
        }
        return recognizer;
    };
    return GesturesObserver;
}(GesturesObserverBase));
export { GesturesObserver };
function _createUIGestureRecognizerTarget(owner, type, callback, context) {
    return UIGestureRecognizerImpl.initWithOwnerTypeCallback(new WeakRef(owner), type, callback, context);
}
function _getUIGestureRecognizerType(type) {
    switch (type) {
        case GestureTypes.tap:
            return UITapGestureRecognizer;
        case GestureTypes.doubleTap:
            return UITapGestureRecognizer;
        case GestureTypes.pinch:
            return UIPinchGestureRecognizer;
        case GestureTypes.pan:
            return UIPanGestureRecognizer;
        case GestureTypes.swipe:
            return UISwipeGestureRecognizer;
        case GestureTypes.rotation:
            return UIRotationGestureRecognizer;
        case GestureTypes.longPress:
            return UILongPressGestureRecognizer;
        case GestureTypes.touch:
            return TouchGestureRecognizer;
        default:
            return null;
    }
}
function getState(recognizer) {
    if (recognizer.state === 1 /* UIGestureRecognizerState.Began */) {
        return GestureStateTypes.began;
    }
    else if (recognizer.state === 4 /* UIGestureRecognizerState.Cancelled */ || recognizer.state === 5 /* UIGestureRecognizerState.Failed */) {
        return GestureStateTypes.cancelled;
    }
    else if (recognizer.state === 2 /* UIGestureRecognizerState.Changed */) {
        return GestureStateTypes.changed;
    }
    else if (recognizer.state === 3 /* UIGestureRecognizerState.Ended */) {
        return GestureStateTypes.ended;
    }
}
function _getSwipeDirection(direction) {
    if (direction === 8 /* UISwipeGestureRecognizerDirection.Down */) {
        return SwipeDirection.down;
    }
    else if (direction === 2 /* UISwipeGestureRecognizerDirection.Left */) {
        return SwipeDirection.left;
    }
    else if (direction === 1 /* UISwipeGestureRecognizerDirection.Right */) {
        return SwipeDirection.right;
    }
    else if (direction === 4 /* UISwipeGestureRecognizerDirection.Up */) {
        return SwipeDirection.up;
    }
}
function _getTapData(args) {
    var recognizer = args.ios;
    var center = recognizer.locationInView(args.view.nativeViewProtected);
    return {
        type: args.type,
        view: args.view,
        ios: args.ios,
        android: undefined,
        eventName: args.eventName,
        object: args.object,
        getPointerCount: function () { return recognizer.numberOfTouches; },
        getX: function () { return center.x; },
        getY: function () { return center.y; },
    };
}
function _getPinchData(args) {
    var recognizer = args.ios;
    var center = recognizer.locationInView(args.view.nativeViewProtected);
    return {
        type: args.type,
        view: args.view,
        ios: args.ios,
        android: undefined,
        scale: recognizer.scale,
        getFocusX: function () { return center.x; },
        getFocusY: function () { return center.y; },
        object: args.view,
        eventName: toString(args.type),
        state: getState(recognizer),
    };
}
function _getSwipeData(args) {
    var recognizer = args.ios;
    return {
        type: args.type,
        view: args.view,
        ios: args.ios,
        android: undefined,
        direction: _getSwipeDirection(recognizer.direction),
        object: args.view,
        eventName: toString(args.type),
    };
}
function _getPanData(args, view) {
    var recognizer = args.ios;
    return {
        type: args.type,
        view: args.view,
        ios: args.ios,
        android: undefined,
        deltaX: recognizer.translationInView(view).x,
        deltaY: recognizer.translationInView(view).y,
        object: args.view,
        eventName: toString(args.type),
        state: getState(recognizer),
    };
}
function _getRotationData(args) {
    var recognizer = args.ios;
    return {
        type: args.type,
        view: args.view,
        ios: args.ios,
        android: undefined,
        rotation: recognizer.rotation * (180.0 / Math.PI),
        object: args.view,
        eventName: toString(args.type),
        state: getState(recognizer),
    };
}
function _getLongPressData(args) {
    var recognizer = args.ios;
    return {
        type: args.type,
        view: args.view,
        ios: args.ios,
        android: undefined,
        object: args.view,
        eventName: toString(args.type),
        state: getState(recognizer),
    };
}
var TouchGestureRecognizer = /** @class */ (function (_super) {
    __extends(TouchGestureRecognizer, _super);
    function TouchGestureRecognizer() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    TouchGestureRecognizer.prototype.touchesBeganWithEvent = function (touches, event) {
        this.executeCallback(TouchAction.down, touches, event);
        if (this.view) {
            this.view.touchesBeganWithEvent(touches, event);
        }
    };
    TouchGestureRecognizer.prototype.touchesMovedWithEvent = function (touches, event) {
        this.executeCallback(TouchAction.move, touches, event);
        if (this.view) {
            this.view.touchesMovedWithEvent(touches, event);
        }
    };
    TouchGestureRecognizer.prototype.touchesEndedWithEvent = function (touches, event) {
        this.executeCallback(TouchAction.up, touches, event);
        if (this.view) {
            this.view.touchesEndedWithEvent(touches, event);
        }
    };
    TouchGestureRecognizer.prototype.touchesCancelledWithEvent = function (touches, event) {
        this.executeCallback(TouchAction.cancel, touches, event);
        if (this.view) {
            this.view.touchesCancelledWithEvent(touches, event);
        }
    };
    TouchGestureRecognizer.prototype.executeCallback = function (action, touches, event) {
        if (!this._eventData) {
            this._eventData = new TouchGestureEventData();
        }
        this._eventData.prepare(this.observer.target, action, touches, event);
        this.observer._executeCallback(this._eventData);
    };
    return TouchGestureRecognizer;
}(UIGestureRecognizer));
var Pointer = /** @class */ (function () {
    function Pointer(touch, targetView) {
        this.android = undefined;
        this.ios = undefined;
        this.ios = touch;
        this._view = targetView;
    }
    Object.defineProperty(Pointer.prototype, "location", {
        get: function () {
            if (!this._location) {
                this._location = this.ios.locationInView(this._view.nativeViewProtected);
            }
            return this._location;
        },
        enumerable: false,
        configurable: true
    });
    Pointer.prototype.getX = function () {
        return this.location.x;
    };
    Pointer.prototype.getY = function () {
        return this.location.y;
    };
    return Pointer;
}());
var TouchGestureEventData = /** @class */ (function () {
    function TouchGestureEventData() {
        this.eventName = toString(GestureTypes.touch);
        this.type = GestureTypes.touch;
        this.android = undefined;
    }
    TouchGestureEventData.prototype.prepare = function (view, action, touches, event) {
        this.action = action;
        this.view = view;
        this.object = view;
        this.ios = {
            touches: touches,
            event: event,
        };
        this._mainPointer = undefined;
        this._activePointers = undefined;
        this._allPointers = undefined;
    };
    TouchGestureEventData.prototype.getPointerCount = function () {
        return this.ios.event.allTouches.count;
    };
    TouchGestureEventData.prototype.getMainPointer = function () {
        if (this._mainPointer === undefined) {
            this._mainPointer = this.ios.touches.anyObject();
        }
        return this._mainPointer;
    };
    TouchGestureEventData.prototype.getActivePointers = function () {
        if (!this._activePointers) {
            this._activePointers = [];
            for (var i = 0, nsArr = this.ios.touches.allObjects; i < nsArr.count; i++) {
                this._activePointers.push(new Pointer(nsArr.objectAtIndex(i), this.view));
            }
        }
        return this._activePointers;
    };
    TouchGestureEventData.prototype.getAllPointers = function () {
        if (!this._allPointers) {
            this._allPointers = [];
            var nsArr = this.ios.event.allTouches.allObjects;
            for (var i = 0; i < nsArr.count; i++) {
                this._allPointers.push(new Pointer(nsArr.objectAtIndex(i), this.view));
            }
        }
        return this._allPointers;
    };
    TouchGestureEventData.prototype.getX = function () {
        var offset = this.view.nativeViewProtected.contentOffset;
        var offsetX = offset ? offset.x : 0;
        return this.getMainPointer().locationInView(this.view.nativeViewProtected).x - offsetX;
    };
    TouchGestureEventData.prototype.getY = function () {
        var offset = this.view.nativeViewProtected.contentOffset;
        var offsetY = offset ? offset.y : 0;
        return this.getMainPointer().locationInView(this.view.nativeViewProtected).y - offsetY;
    };
    return TouchGestureEventData;
}());
//# sourceMappingURL=index.ios.js.map