import { Color } from '../../color';
import { SDK_VERSION } from '../../utils/constants';
import { Font } from '../styling/font';
import { colorProperty, fontSizeProperty, fontInternalProperty } from '../styling/style-properties';
import { HtmlViewBase, htmlProperty, selectableProperty, linkColorProperty } from './html-view-common';
export * from './html-view-common';
var HtmlView = /** @class */ (function (_super) {
    __extends(HtmlView, _super);
    function HtmlView() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    HtmlView.prototype.createNativeView = function () {
        return new android.widget.TextView(this._context);
    };
    HtmlView.prototype.initNativeView = function () {
        _super.prototype.initNativeView.call(this);
        var nativeView = this.nativeViewProtected;
        // Allow text selection
        nativeView.setTextIsSelectable(true);
        // This makes the html <a href...> work
        nativeView.setLinksClickable(true);
        nativeView.setMovementMethod(android.text.method.LinkMovementMethod.getInstance());
    };
    HtmlView.prototype.resetNativeView = function () {
        _super.prototype.resetNativeView.call(this);
        this.nativeViewProtected.setAutoLinkMask(0);
    };
    HtmlView.prototype[htmlProperty.getDefault] = function () {
        return '';
    };
    HtmlView.prototype[htmlProperty.setNative] = function (value) {
        // If the data.newValue actually has a <a...> in it; we need to disable autolink mask
        // it internally disables the coloring, but then the <a> links won't work..  So to support both
        // styles of links (html and just text based) we have to manually enable/disable the autolink mask
        var mask = 15;
        if (value.search(/<a\s/i) >= 0) {
            mask = 0;
        }
        this.nativeViewProtected.setAutoLinkMask(mask);
        if (SDK_VERSION >= 24) {
            this.nativeViewProtected.setText(android.text.Html.fromHtml(value, android.text.Html.FROM_HTML_MODE_LEGACY));
        }
        else {
            this.nativeViewProtected.setText(android.text.Html.fromHtml(value));
        }
    };
    HtmlView.prototype[selectableProperty.getDefault] = function () {
        return true;
    };
    HtmlView.prototype[selectableProperty.setNative] = function (value) {
        this.nativeViewProtected.setTextIsSelectable(value);
    };
    HtmlView.prototype[colorProperty.getDefault] = function () {
        return this.nativeViewProtected.getTextColors();
    };
    HtmlView.prototype[colorProperty.setNative] = function (value) {
        if (value instanceof Color) {
            this.nativeViewProtected.setTextColor(value.android);
        }
        else {
            this.nativeViewProtected.setTextColor(value);
        }
    };
    HtmlView.prototype[linkColorProperty.getDefault] = function () {
        return this.nativeViewProtected.getLinkTextColors();
    };
    HtmlView.prototype[linkColorProperty.setNative] = function (value) {
        if (value instanceof Color) {
            this.nativeViewProtected.setLinkTextColor(value.android);
        }
        else {
            this.nativeViewProtected.setLinkTextColor(value);
        }
    };
    HtmlView.prototype[fontInternalProperty.getDefault] = function () {
        return this.nativeViewProtected.getTypeface();
    };
    HtmlView.prototype[fontInternalProperty.setNative] = function (value) {
        var font = value instanceof Font ? value.getAndroidTypeface() : value;
        this.nativeViewProtected.setTypeface(font);
    };
    HtmlView.prototype[fontSizeProperty.getDefault] = function () {
        return { nativeSize: this.nativeViewProtected.getTextSize() };
    };
    HtmlView.prototype[fontSizeProperty.setNative] = function (value) {
        if (typeof value === 'number') {
            this.nativeViewProtected.setTextSize(value);
        }
        else {
            this.nativeViewProtected.setTextSize(android.util.TypedValue.COMPLEX_UNIT_PX, value.nativeSize);
        }
    };
    return HtmlView;
}(HtmlViewBase));
export { HtmlView };
//# sourceMappingURL=index.android.js.map