import { Color } from '../../color';
import { Font } from '../styling/font';
import { colorProperty, fontInternalProperty } from '../styling/style-properties';
import { HtmlViewBase, htmlProperty, selectableProperty, linkColorProperty } from './html-view-common';
import { View } from '../core/view';
import { iOSNativeHelper, layout } from '../../utils';
export * from './html-view-common';
var majorVersion = iOSNativeHelper.MajorVersion;
var HtmlView = /** @class */ (function (_super) {
    __extends(HtmlView, _super);
    function HtmlView() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    HtmlView.prototype.createNativeView = function () {
        var view = UITextView.new();
        view.scrollEnabled = false;
        view.editable = false;
        view.selectable = true;
        view.userInteractionEnabled = true;
        view.dataDetectorTypes = -1 /* UIDataDetectorTypes.All */;
        return view;
    };
    HtmlView.prototype.initNativeView = function () {
        _super.prototype.initNativeView.call(this);
        // Remove extra padding
        this.nativeViewProtected.textContainer.lineFragmentPadding = 0;
        this.nativeViewProtected.textContainerInset = UIEdgeInsets.zero;
    };
    Object.defineProperty(HtmlView.prototype, "ios", {
        // @ts-ignore
        get: function () {
            return this.nativeViewProtected;
        },
        enumerable: false,
        configurable: true
    });
    HtmlView.prototype.onMeasure = function (widthMeasureSpec, heightMeasureSpec) {
        var nativeView = this.nativeViewProtected;
        if (nativeView) {
            var width = layout.getMeasureSpecSize(widthMeasureSpec);
            var widthMode = layout.getMeasureSpecMode(widthMeasureSpec);
            var height = layout.getMeasureSpecSize(heightMeasureSpec);
            var heightMode = layout.getMeasureSpecMode(heightMeasureSpec);
            var desiredSize = layout.measureNativeView(nativeView, width, widthMode, height, heightMode);
            var labelWidth = widthMode === layout.AT_MOST ? Math.min(desiredSize.width, width) : desiredSize.width;
            var measureWidth = Math.max(labelWidth, this.effectiveMinWidth);
            var measureHeight = Math.max(desiredSize.height, this.effectiveMinHeight);
            var widthAndState = View.resolveSizeAndState(measureWidth, width, widthMode, 0);
            var heightAndState = View.resolveSizeAndState(measureHeight, height, heightMode, 0);
            this.setMeasuredDimension(widthAndState, heightAndState);
        }
    };
    HtmlView.prototype.renderWithStyles = function () {
        var _a;
        var html = this.currentHtml;
        var styles = [];
        if (this.nativeViewProtected.font) {
            styles.push("font-family: '".concat(this.nativeViewProtected.font.fontName, "';"));
            styles.push("font-size: ".concat(this.nativeViewProtected.font.pointSize, "px;"));
        }
        if (this.nativeViewProtected.textColor) {
            var textColor = Color.fromIosColor(this.nativeViewProtected.textColor);
            styles.push("color: ".concat(textColor.hex, ";"));
        }
        if (styles.length > 0) {
            html += "<style>body {".concat(styles.join(''), "}</style>");
        }
        var htmlString = NSString.stringWithString(html + '');
        var nsData = htmlString.dataUsingEncoding(NSUnicodeStringEncoding);
        this.nativeViewProtected.attributedText = NSAttributedString.alloc().initWithDataOptionsDocumentAttributesError(nsData, (_a = {}, _a[NSDocumentTypeDocumentAttribute] = NSHTMLTextDocumentType, _a), null);
        if (majorVersion >= 13 && UIColor.labelColor) {
            this.nativeViewProtected.textColor = UIColor.labelColor;
        }
    };
    HtmlView.prototype[htmlProperty.getDefault] = function () {
        return '';
    };
    HtmlView.prototype[htmlProperty.setNative] = function (value) {
        this.currentHtml = value;
        this.renderWithStyles();
    };
    HtmlView.prototype[selectableProperty.getDefault] = function () {
        return true;
    };
    HtmlView.prototype[selectableProperty.setNative] = function (value) {
        this.nativeViewProtected.selectable = value;
    };
    HtmlView.prototype[colorProperty.getDefault] = function () {
        return this.nativeViewProtected.textColor;
    };
    HtmlView.prototype[colorProperty.setNative] = function (value) {
        var color = value instanceof Color ? value.ios : value;
        this.nativeViewProtected.textColor = color;
        this.renderWithStyles();
    };
    HtmlView.prototype[linkColorProperty.getDefault] = function () {
        return this.nativeViewProtected.linkTextAttributes[NSForegroundColorAttributeName];
    };
    HtmlView.prototype[linkColorProperty.setNative] = function (value) {
        var color = value instanceof Color ? value.ios : value;
        var linkTextAttributes = NSDictionary.dictionaryWithObjectForKey(color, NSForegroundColorAttributeName);
        this.nativeViewProtected.linkTextAttributes = linkTextAttributes;
    };
    HtmlView.prototype[fontInternalProperty.getDefault] = function () {
        return this.nativeViewProtected.font;
    };
    HtmlView.prototype[fontInternalProperty.setNative] = function (value) {
        var font = value instanceof Font ? value.getUIFont(this.nativeViewProtected.font) : value;
        this.nativeViewProtected.font = font;
        this.renderWithStyles();
    };
    return HtmlView;
}(HtmlViewBase));
export { HtmlView };
//# sourceMappingURL=index.ios.js.map