import * as common from './image-cache-common';
import { Trace } from '../../trace';
var LruBitmapCacheClass;
function ensureLruBitmapCacheClass() {
    if (LruBitmapCacheClass) {
        return;
    }
    var LruBitmapCache = /** @class */ (function (_super) {
    __extends(LruBitmapCache, _super);
    function LruBitmapCache(cacheSize) {
        var _this = _super.call(this, cacheSize) || this;
        return global.__native(_this);
    }
    LruBitmapCache.prototype.sizeOf = function (key, bitmap) {
        // The cache size will be measured in kilobytes rather than
        // number of items.
        var result = Math.round(bitmap.getByteCount() / 1024);
        return result;
    };
    return LruBitmapCache;
}(android.util.LruCache));
    LruBitmapCacheClass = LruBitmapCache;
}
var Cache = /** @class */ (function (_super) {
    __extends(Cache, _super);
    function Cache() {
        var _this = _super.call(this) || this;
        ensureLruBitmapCacheClass();
        var maxMemory = java.lang.Runtime.getRuntime().maxMemory() / 1024;
        var cacheSize = maxMemory / 8;
        _this._cache = new LruBitmapCacheClass(cacheSize);
        var that = new WeakRef(_this);
        _this._callback = new org.nativescript.widgets.Async.CompleteCallback({
            onComplete: function (result, context) {
                var instance = that === null || that === void 0 ? void 0 : that.get();
                if (instance) {
                    if (result) {
                        instance._onDownloadCompleted(context, result);
                    }
                    else {
                        instance._onDownloadError(context, new Error('No result in CompletionCallback'));
                    }
                }
            },
            onError: function (err, context) {
                var instance = that === null || that === void 0 ? void 0 : that.get();
                if (instance) {
                    instance._onDownloadError(context, new Error(err));
                }
            },
        });
        return _this;
    }
    Cache.prototype._downloadCore = function (request) {
        org.nativescript.widgets.Async.Image.download(request.url, this._callback, request.key);
    };
    Cache.prototype.get = function (key) {
        var result = this._cache.get(key);
        return result;
    };
    Cache.prototype.set = function (key, image) {
        try {
            if (key && image) {
                this._cache.put(key, image);
            }
        }
        catch (err) {
            Trace.write('Cache set error: ' + err, Trace.categories.Error, Trace.messageType.error);
        }
    };
    Cache.prototype.remove = function (key) {
        this._cache.remove(key);
    };
    Cache.prototype.clear = function () {
        this._cache.evictAll();
    };
    return Cache;
}(common.Cache));
export { Cache };
//# sourceMappingURL=index.android.js.map