import * as common from './image-cache-common';
import { Trace } from '../../trace';
import * as utils from '../../utils';
var httpRequest;
function ensureHttpRequest() {
    if (!httpRequest) {
        httpRequest = require('../../http/http-request');
    }
}
var MemoryWarningHandler = /** @class */ (function (_super) {
    __extends(MemoryWarningHandler, _super);
    function MemoryWarningHandler() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    MemoryWarningHandler.new = function () {
        return _super.new.call(this);
    };
    MemoryWarningHandler.prototype.initWithCache = function (cache) {
        this._cache = cache;
        NSNotificationCenter.defaultCenter.addObserverSelectorNameObject(this, 'clearCache', 'UIApplicationDidReceiveMemoryWarningNotification', null);
        if (Trace.isEnabled()) {
            Trace.write('[MemoryWarningHandler] Added low memory observer.', Trace.categories.Debug);
        }
        return this;
    };
    MemoryWarningHandler.prototype.dealloc = function () {
        NSNotificationCenter.defaultCenter.removeObserverNameObject(this, 'UIApplicationDidReceiveMemoryWarningNotification', null);
        if (Trace.isEnabled()) {
            Trace.write('[MemoryWarningHandler] Removed low memory observer.', Trace.categories.Debug);
        }
        _super.prototype.dealloc.call(this);
    };
    MemoryWarningHandler.prototype.clearCache = function () {
        if (Trace.isEnabled()) {
            Trace.write('[MemoryWarningHandler] Clearing Image Cache.', Trace.categories.Debug);
        }
        this._cache.removeAllObjects();
        utils.GC();
    };
    MemoryWarningHandler.ObjCExposedMethods = {
        clearCache: { returns: interop.types.void, params: [] },
    };
    return MemoryWarningHandler;
}(NSObject));
var Cache = /** @class */ (function (_super) {
    __extends(Cache, _super);
    function Cache() {
        var _this = _super.call(this) || this;
        _this._cache = new NSCache();
        _this._memoryWarningHandler = MemoryWarningHandler.new().initWithCache(_this._cache);
        return _this;
    }
    Cache.prototype._downloadCore = function (request) {
        var _this = this;
        ensureHttpRequest();
        httpRequest.request({ url: request.url, method: 'GET' }).then(function (response) {
            try {
                var image = UIImage.alloc().initWithData(response.content.raw);
                if (image) {
                    _this._onDownloadCompleted(request.key, image);
                }
                else {
                    _this._onDownloadError(request.key, new Error('No result for provided url'));
                }
            }
            catch (err) {
                _this._onDownloadError(request.key, err);
            }
        }, function (err) {
            _this._onDownloadError(request.key, err);
        });
    };
    Cache.prototype.get = function (key) {
        return this._cache.objectForKey(key);
    };
    Cache.prototype.set = function (key, image) {
        this._cache.setObjectForKey(image, key);
    };
    Cache.prototype.remove = function (key) {
        this._cache.removeObjectForKey(key);
    };
    Cache.prototype.clear = function () {
        this._cache.removeAllObjects();
        utils.GC();
    };
    return Cache;
}(common.Cache));
export { Cache };
//# sourceMappingURL=index.ios.js.map