import { ImageBase, stretchProperty, imageSourceProperty, srcProperty, tintColorProperty } from './image-common';
import { isDataURI, isFontIconURI, isFileOrResourcePath, RESOURCE_PREFIX } from '../../utils';
import { ImageAsset } from '../../image-asset';
import { Length } from '../styling/style-properties';
import { knownFolders } from '../../file-system';
import { Screen } from '../../platform';
export * from './image-common';
var FILE_PREFIX = 'file:///';
var ASYNC = 'async';
var AndroidImageView;
var ImageLoadedListener;
function initializeImageLoadedListener() {
    if (ImageLoadedListener) {
        return;
    }
    var ImageLoadedListenerImpl = /** @class */ (function (_super) {
    __extends(ImageLoadedListenerImpl, _super);
    function ImageLoadedListenerImpl(owner) {
        var _this = _super.call(this) || this;
        _this.owner = owner;
        return global.__native(_this);
    }
    ImageLoadedListenerImpl.prototype.onImageLoaded = function (success) {
        var owner = this.owner;
        if (owner) {
            owner.isLoading = false;
        }
    };
    var _a;
    ImageLoadedListenerImpl = __decorate([
        Interfaces([org.nativescript.widgets.image.Worker.OnImageLoadedListener]),
        __metadata("design:paramtypes", [typeof (_a = typeof Image !== "undefined" && Image) === "function" ? _a : Object])
    ], ImageLoadedListenerImpl);
    return ImageLoadedListenerImpl;
}(java.lang.Object));
    ImageLoadedListener = ImageLoadedListenerImpl;
}
var Image = /** @class */ (function (_super) {
    __extends(Image, _super);
    function Image() {
        var _this = _super !== null && _super.apply(this, arguments) || this;
        _this.useCache = true;
        return _this;
    }
    Image.prototype.createNativeView = function () {
        if (!AndroidImageView) {
            AndroidImageView = org.nativescript.widgets.ImageView;
        }
        return new AndroidImageView(this._context);
    };
    Image.prototype.initNativeView = function () {
        _super.prototype.initNativeView.call(this);
        initializeImageLoadedListener();
        var nativeView = this.nativeViewProtected;
        var listener = new ImageLoadedListener(this);
        nativeView.setImageLoadedListener(listener);
        nativeView.listener = listener;
    };
    Image.prototype.disposeNativeView = function () {
        var _a;
        if ((_a = this.nativeViewProtected) === null || _a === void 0 ? void 0 : _a.listener) {
            this.nativeViewProtected.listener.owner = null;
        }
        _super.prototype.disposeNativeView.call(this);
    };
    Image.prototype.resetNativeView = function () {
        _super.prototype.resetNativeView.call(this);
        this.nativeViewProtected.setImageMatrix(new android.graphics.Matrix());
    };
    Image.prototype._createImageSourceFromSrc = function (value) {
        var imageView = this.nativeViewProtected;
        if (!imageView) {
            return;
        }
        if (!value) {
            imageView.setUri(null, 0, 0, false, false, true);
            return;
        }
        var decodeWidth = Math.min(Length.toDevicePixels(this.decodeWidth, 0), Screen.mainScreen.widthPixels);
        var decodeHeight = Math.min(Length.toDevicePixels(this.decodeHeight, 0), Screen.mainScreen.heightPixels);
        var keepAspectRatio = this._calculateKeepAspectRatio();
        if (value instanceof ImageAsset) {
            if (value.options) {
                decodeWidth = value.options.width || decodeWidth;
                decodeHeight = value.options.height || decodeHeight;
                keepAspectRatio = !!value.options.keepAspectRatio;
            }
            // handle assets as file paths natively
            value = value.android;
        }
        var async = this.loadMode === ASYNC;
        if (typeof value === 'string' || value instanceof String) {
            value = value.trim();
            this.isLoading = true;
            if (isFontIconURI(value) || isDataURI(value)) {
                // TODO: Check with runtime what should we do in case of base64 string.
                _super.prototype._createImageSourceFromSrc.call(this, value);
            }
            else if (isFileOrResourcePath(value)) {
                if (value.indexOf(RESOURCE_PREFIX) === 0) {
                    imageView.setUri(value, decodeWidth, decodeHeight, keepAspectRatio, this.useCache, async);
                }
                else {
                    var fileName = value;
                    if (fileName.indexOf('~/') === 0) {
                        fileName = knownFolders.currentApp().path + '/' + fileName.replace('~/', '');
                    }
                    imageView.setUri(FILE_PREFIX + fileName, decodeWidth, decodeHeight, keepAspectRatio, this.useCache, async);
                }
            }
            else {
                // For backwards compatibility http always use async loading.
                imageView.setUri(value, decodeWidth, decodeHeight, keepAspectRatio, this.useCache, true);
            }
        }
        else {
            _super.prototype._createImageSourceFromSrc.call(this, value);
        }
    };
    Image.prototype._calculateKeepAspectRatio = function () {
        return this.stretch === 'fill' ? false : true;
    };
    Image.prototype[stretchProperty.getDefault] = function () {
        return 'aspectFit';
    };
    Image.prototype[stretchProperty.setNative] = function (value) {
        switch (value) {
            case 'aspectFit':
                this.nativeViewProtected.setScaleType(android.widget.ImageView.ScaleType.FIT_CENTER);
                break;
            case 'aspectFill':
                this.nativeViewProtected.setScaleType(android.widget.ImageView.ScaleType.CENTER_CROP);
                break;
            case 'fill':
                this.nativeViewProtected.setScaleType(android.widget.ImageView.ScaleType.FIT_XY);
                break;
            case 'none':
            default:
                this.nativeViewProtected.setScaleType(android.widget.ImageView.ScaleType.MATRIX);
                break;
        }
    };
    Image.prototype[tintColorProperty.getDefault] = function () {
        return undefined;
    };
    Image.prototype[tintColorProperty.setNative] = function (value) {
        if (value) {
            this.nativeViewProtected.setColorFilter(value.android);
        }
        else {
            this.nativeViewProtected.clearColorFilter();
        }
    };
    Image.prototype[imageSourceProperty.getDefault] = function () {
        return undefined;
    };
    Image.prototype[imageSourceProperty.setNative] = function (value) {
        var nativeView = this.nativeViewProtected;
        if (value && value.android) {
            var rotation = value.rotationAngle ? value.rotationAngle : 0;
            nativeView.setRotationAngle(rotation);
            nativeView.setImageBitmap(value.android);
        }
        else {
            nativeView.setRotationAngle(0);
            nativeView.setImageBitmap(null);
        }
    };
    Image.prototype[srcProperty.getDefault] = function () {
        return undefined;
    };
    Image.prototype[srcProperty.setNative] = function (value) {
        this._createImageSourceFromSrc(value);
    };
    return Image;
}(ImageBase));
export { Image };
//# sourceMappingURL=index.android.js.map