import { ImageBase, stretchProperty, imageSourceProperty, tintColorProperty, srcProperty, iosSymbolEffectProperty, ImageSymbolEffect, iosSymbolScaleProperty } from './image-common';
import { Trace } from '../../trace';
import { layout, queueGC } from '../../utils';
import { SDK_VERSION } from '../../utils/constants';
export * from './image-common';
var Image = /** @class */ (function (_super) {
    __extends(Image, _super);
    function Image() {
        var _this = _super !== null && _super.apply(this, arguments) || this;
        _this._imageSourceAffectsLayout = true;
        return _this;
    }
    Image.prototype.createNativeView = function () {
        var imageView = UIImageView.new();
        imageView.contentMode = 1 /* UIViewContentMode.ScaleAspectFit */;
        imageView.userInteractionEnabled = true;
        return imageView;
    };
    Image.prototype.initNativeView = function () {
        _super.prototype.initNativeView.call(this);
        this._setNativeClipToBounds();
    };
    Image.prototype.disposeImageSource = function () {
        var _a, _b, _c;
        if (((_a = this.nativeViewProtected) === null || _a === void 0 ? void 0 : _a.image) === ((_b = this.imageSource) === null || _b === void 0 ? void 0 : _b.ios)) {
            this.nativeViewProtected.image = null;
        }
        if ((_c = this.imageSource) === null || _c === void 0 ? void 0 : _c.ios) {
            this.imageSource.ios = null;
        }
        this.imageSource = null;
        queueGC();
    };
    Image.prototype.disposeNativeView = function () {
        var _a;
        if ((_a = this.nativeViewProtected) === null || _a === void 0 ? void 0 : _a.image) {
            this.nativeViewProtected.image = null;
        }
        this.disposeImageSource();
        _super.prototype.disposeNativeView.call(this);
    };
    Image.prototype.setTintColor = function (value) {
        if (this.nativeViewProtected) {
            if (value && this.nativeViewProtected.image && !this._templateImageWasCreated) {
                this.nativeViewProtected.image = this.nativeViewProtected.image.imageWithRenderingMode(2 /* UIImageRenderingMode.AlwaysTemplate */);
                this._templateImageWasCreated = true;
                queueGC();
            }
            else if (!value && this.nativeViewProtected.image && this._templateImageWasCreated) {
                this._templateImageWasCreated = false;
                this.nativeViewProtected.image = this.nativeViewProtected.image.imageWithRenderingMode(0 /* UIImageRenderingMode.Automatic */);
                queueGC();
            }
            this.nativeViewProtected.tintColor = value ? value.ios : null;
        }
    };
    Image.prototype._setNativeImage = function (nativeImage) {
        var _a, _b;
        if ((_a = this.nativeViewProtected) === null || _a === void 0 ? void 0 : _a.image) {
            this.nativeViewProtected.image = null;
            queueGC();
        }
        if (this.nativeViewProtected) {
            this.nativeViewProtected.image = nativeImage;
        }
        this._templateImageWasCreated = false;
        this.setTintColor((_b = this.style) === null || _b === void 0 ? void 0 : _b.tintColor);
        if (this._imageSourceAffectsLayout) {
            this.requestLayout();
        }
    };
    Image.prototype._setNativeClipToBounds = function () {
        if (this.nativeViewProtected) {
            // Always set clipsToBounds for images
            this.nativeViewProtected.clipsToBounds = true;
        }
    };
    Image.prototype.onMeasure = function (widthMeasureSpec, heightMeasureSpec) {
        // We don't call super because we measure native view with specific size.
        var width = layout.getMeasureSpecSize(widthMeasureSpec);
        var widthMode = layout.getMeasureSpecMode(widthMeasureSpec);
        var height = layout.getMeasureSpecSize(heightMeasureSpec);
        var heightMode = layout.getMeasureSpecMode(heightMeasureSpec);
        var nativeWidth = this.imageSource ? layout.toDevicePixels(this.imageSource.width) : 0;
        var nativeHeight = this.imageSource ? layout.toDevicePixels(this.imageSource.height) : 0;
        var measureWidth = Math.max(nativeWidth, this.effectiveMinWidth);
        var measureHeight = Math.max(nativeHeight, this.effectiveMinHeight);
        var finiteWidth = widthMode !== layout.UNSPECIFIED;
        var finiteHeight = heightMode !== layout.UNSPECIFIED;
        this._imageSourceAffectsLayout = widthMode !== layout.EXACTLY || heightMode !== layout.EXACTLY;
        if (nativeWidth !== 0 && nativeHeight !== 0 && (finiteWidth || finiteHeight)) {
            var scale = Image.computeScaleFactor(width, height, finiteWidth, finiteHeight, nativeWidth, nativeHeight, this.stretch);
            var resultW = Math.round(nativeWidth * scale.width);
            var resultH = Math.round(nativeHeight * scale.height);
            measureWidth = finiteWidth ? Math.min(resultW, width) : resultW;
            measureHeight = finiteHeight ? Math.min(resultH, height) : resultH;
            if (Trace.isEnabled()) {
                Trace.write('Image stretch: ' + this.stretch + ', nativeWidth: ' + nativeWidth + ', nativeHeight: ' + nativeHeight, Trace.categories.Layout);
            }
        }
        var widthAndState = Image.resolveSizeAndState(measureWidth, width, widthMode, 0);
        var heightAndState = Image.resolveSizeAndState(measureHeight, height, heightMode, 0);
        this.setMeasuredDimension(widthAndState, heightAndState);
    };
    Image.computeScaleFactor = function (measureWidth, measureHeight, widthIsFinite, heightIsFinite, nativeWidth, nativeHeight, imageStretch) {
        var scaleW = 1;
        var scaleH = 1;
        if ((imageStretch === 'aspectFill' || imageStretch === 'aspectFit' || imageStretch === 'fill') && (widthIsFinite || heightIsFinite)) {
            scaleW = nativeWidth > 0 ? measureWidth / nativeWidth : 0;
            scaleH = nativeHeight > 0 ? measureHeight / nativeHeight : 0;
            if (!widthIsFinite) {
                scaleW = scaleH;
            }
            else if (!heightIsFinite) {
                scaleH = scaleW;
            }
            else {
                // No infinite dimensions.
                switch (imageStretch) {
                    case 'aspectFit':
                        scaleH = scaleW < scaleH ? scaleW : scaleH;
                        scaleW = scaleH;
                        break;
                    case 'aspectFill':
                        scaleH = scaleW > scaleH ? scaleW : scaleH;
                        scaleW = scaleH;
                        break;
                }
            }
        }
        return { width: scaleW, height: scaleH };
    };
    Image.prototype[stretchProperty.setNative] = function (value) {
        if (this.nativeViewProtected) {
            switch (value) {
                case 'aspectFit':
                    this.nativeViewProtected.contentMode = 1 /* UIViewContentMode.ScaleAspectFit */;
                    break;
                case 'aspectFill':
                    this.nativeViewProtected.contentMode = 2 /* UIViewContentMode.ScaleAspectFill */;
                    break;
                case 'fill':
                    this.nativeViewProtected.contentMode = 0 /* UIViewContentMode.ScaleToFill */;
                    break;
                case 'none':
                default:
                    this.nativeViewProtected.contentMode = 9 /* UIViewContentMode.TopLeft */;
                    break;
            }
        }
    };
    Image.prototype[tintColorProperty.setNative] = function (value) {
        this.setTintColor(value);
    };
    Image.prototype[imageSourceProperty.setNative] = function (value) {
        if (value !== this.imageSource) {
            this.disposeImageSource();
        }
        this._setNativeImage(value ? value.ios : null);
    };
    Image.prototype._setSrc = function (value) {
        this._createImageSourceFromSrc(value);
        if (this.iosSymbolScale) {
            // when applying symbol scale, contentMode must be center
            // https://stackoverflow.com/a/65787627
            this.nativeViewProtected.contentMode = 4 /* UIViewContentMode.Center */;
        }
    };
    Image.prototype[srcProperty.setNative] = function (value) {
        this._setSrc(value);
    };
    Image.prototype[iosSymbolEffectProperty.setNative] = function (value) {
        if (SDK_VERSION < 17) {
            return;
        }
        var symbol = typeof value === 'string' ? ImageSymbolEffect.fromSymbol(value) : value;
        if (symbol === null || symbol === void 0 ? void 0 : symbol.effect) {
            // Note: https://developer.apple.com/documentation/symbols/symboleffectoptions/4197883-repeating
            // Will want to move to https://developer.apple.com/documentation/symbols/nssymboleffectoptionsrepeatbehavior?language=objc as fallback once optionsWithRepeating is removed
            this.nativeViewProtected.addSymbolEffectOptionsAnimatedCompletion(symbol.effect, symbol.options || NSSymbolEffectOptions.optionsWithRepeating(), true, symbol.completion || null);
        }
        else {
            this.nativeViewProtected.removeAllSymbolEffects();
        }
    };
    Image.prototype[iosSymbolScaleProperty.setNative] = function (value) {
        // reset src to configure scale
        this._setSrc(this.src);
    };
    return Image;
}(ImageBase));
export { Image };
//# sourceMappingURL=index.ios.js.map