var _a, _b, _c, _d, _e, _f;
import { FlexDirection, FlexWrap, JustifyContent, AlignItems, AlignContent, FlexboxLayoutBase, orderProperty, flexGrowProperty, flexShrinkProperty, flexWrapBeforeProperty, alignSelfProperty, AlignSelf, flexDirectionProperty, flexWrapProperty, justifyContentProperty, alignItemsProperty, alignContentProperty } from './flexbox-layout-common';
import { View } from '../../core/view';
import { Length } from '../../styling/style-properties';
export * from './flexbox-layout-common';
var widgetFlexboxLayout;
var widgetLayoutParams;
function ensureNativeTypes() {
    if (!widgetFlexboxLayout) {
        widgetFlexboxLayout = org.nativescript.widgets.FlexboxLayout;
        widgetLayoutParams = widgetFlexboxLayout.LayoutParams;
    }
}
function makeNativeSetter(setter) {
    return function (value) {
        ensureNativeTypes();
        var nativeView = this.nativeViewProtected;
        var lp = nativeView.getLayoutParams() || new widgetLayoutParams();
        if (lp instanceof widgetLayoutParams) {
            setter(lp, value);
            nativeView.setLayoutParams(lp);
        }
    };
}
View.prototype[orderProperty.setNative] = makeNativeSetter(function (lp, value) { return (lp.order = value); });
View.prototype[flexGrowProperty.setNative] = makeNativeSetter(function (lp, value) { return (lp.flexGrow = value); });
View.prototype[flexShrinkProperty.setNative] = makeNativeSetter(function (lp, value) { return (lp.flexShrink = value); });
View.prototype[flexWrapBeforeProperty.setNative] = makeNativeSetter(function (lp, value) { return (lp.wrapBefore = value); });
View.prototype[alignSelfProperty.setNative] = makeNativeSetter(function (lp, value) { return (lp.alignSelf = alignSelfMap[value]); });
var flexDirectionMap = (_a = {},
    _a[FlexDirection.ROW] = 0,
    _a[FlexDirection.ROW_REVERSE] = 1,
    _a[FlexDirection.COLUMN] = 2,
    _a[FlexDirection.COLUMN_REVERSE] = 3,
    _a);
var flexWrapMap = (_b = {},
    _b[FlexWrap.NOWRAP] = 0,
    _b[FlexWrap.WRAP] = 1,
    _b[FlexWrap.WRAP_REVERSE] = 2,
    _b);
var justifyContentMap = (_c = {},
    _c[JustifyContent.FLEX_START] = 0,
    _c[JustifyContent.FLEX_END] = 1,
    _c[JustifyContent.CENTER] = 2,
    _c[JustifyContent.SPACE_BETWEEN] = 3,
    _c[JustifyContent.SPACE_AROUND] = 4,
    _c);
var alignItemsMap = (_d = {},
    _d[AlignItems.FLEX_START] = 0,
    _d[AlignItems.FLEX_END] = 1,
    _d[AlignItems.CENTER] = 2,
    _d[AlignItems.BASELINE] = 3,
    _d[AlignItems.STRETCH] = 4,
    _d);
var alignContentMap = (_e = {},
    _e[AlignContent.FLEX_START] = 0,
    _e[AlignContent.FLEX_END] = 1,
    _e[AlignContent.CENTER] = 2,
    _e[AlignContent.SPACE_BETWEEN] = 3,
    _e[AlignContent.SPACE_AROUND] = 4,
    _e[AlignContent.STRETCH] = 5,
    _e);
var alignSelfMap = (_f = {},
    _f[AlignSelf.AUTO] = -1,
    _f[AlignSelf.FLEX_START] = 0,
    _f[AlignSelf.FLEX_END] = 1,
    _f[AlignSelf.CENTER] = 2,
    _f[AlignSelf.BASELINE] = 3,
    _f[AlignSelf.STRETCH] = 4,
    _f);
var FlexboxLayout = /** @class */ (function (_super) {
    __extends(FlexboxLayout, _super);
    function FlexboxLayout() {
        var _this = _super.call(this) || this;
        ensureNativeTypes();
        return _this;
    }
    FlexboxLayout.prototype.createNativeView = function () {
        return new widgetFlexboxLayout(this._context);
    };
    FlexboxLayout.prototype.resetNativeView = function () {
        _super.prototype.resetNativeView.call(this);
        this.nativeViewProtected.invalidateOrdersCache();
    };
    FlexboxLayout.prototype[flexDirectionProperty.getDefault] = function () {
        return flexDirectionProperty.defaultValue;
    };
    FlexboxLayout.prototype[flexDirectionProperty.setNative] = function (flexDirection) {
        this.nativeViewProtected.setFlexDirection(flexDirectionMap[flexDirection]);
    };
    FlexboxLayout.prototype[flexWrapProperty.getDefault] = function () {
        return flexWrapProperty.defaultValue;
    };
    FlexboxLayout.prototype[flexWrapProperty.setNative] = function (flexWrap) {
        this.nativeViewProtected.setFlexWrap(flexWrapMap[flexWrap]);
    };
    FlexboxLayout.prototype[justifyContentProperty.getDefault] = function () {
        return justifyContentProperty.defaultValue;
    };
    FlexboxLayout.prototype[justifyContentProperty.setNative] = function (justifyContent) {
        this.nativeViewProtected.setJustifyContent(justifyContentMap[justifyContent]);
    };
    FlexboxLayout.prototype[alignItemsProperty.getDefault] = function () {
        return alignItemsProperty.defaultValue;
    };
    FlexboxLayout.prototype[alignItemsProperty.setNative] = function (alignItems) {
        this.nativeViewProtected.setAlignItems(alignItemsMap[alignItems]);
    };
    FlexboxLayout.prototype[alignContentProperty.getDefault] = function () {
        return alignContentProperty.defaultValue;
    };
    FlexboxLayout.prototype[alignContentProperty.setNative] = function (alignContent) {
        this.nativeViewProtected.setAlignContent(alignContentMap[alignContent]);
    };
    FlexboxLayout.prototype._updateNativeLayoutParams = function (child) {
        _super.prototype._updateNativeLayoutParams.call(this, child);
        // NOTE: If minWidth/Height is not set, the next code will clear the default native values for minWidth/Height.
        // Flex box will not respect the button default min width. Keeping this behavior for back-compatibility.
        this._setChildMinWidthNative(child, child.minWidth);
        this._setChildMinHeightNative(child, child.minHeight);
        var lp = child.nativeViewProtected.getLayoutParams();
        var style = child.style;
        lp.order = style.order;
        lp.flexGrow = style.flexGrow;
        lp.flexShrink = style.flexShrink;
        lp.wrapBefore = style.flexWrapBefore;
        lp.alignSelf = alignSelfMap[style.alignSelf];
        child.nativeViewProtected.setLayoutParams(lp);
    };
    FlexboxLayout.prototype._setChildMinWidthNative = function (child, value) {
        // Check needed to maintain back-compat after https://github.com/NativeScript/NativeScript/pull/7804
        if (!child._ignoreFlexMinWidthHeightReset) {
            child._setMinWidthNative(0);
        }
        var nativeView = child.nativeViewProtected;
        var lp = nativeView.getLayoutParams();
        if (lp instanceof widgetLayoutParams) {
            lp.minWidth = Length.toDevicePixels(value, 0);
            nativeView.setLayoutParams(lp);
        }
    };
    FlexboxLayout.prototype._setChildMinHeightNative = function (child, value) {
        // Check needed to maintain back-compat after https://github.com/NativeScript/NativeScript/pull/7804
        if (!child._ignoreFlexMinWidthHeightReset) {
            child._setMinHeightNative(0);
        }
        var nativeView = child.nativeViewProtected;
        var lp = nativeView.getLayoutParams();
        if (lp instanceof widgetLayoutParams) {
            lp.minHeight = Length.toDevicePixels(value, 0);
            nativeView.setLayoutParams(lp);
        }
    };
    return FlexboxLayout;
}(FlexboxLayoutBase));
export { FlexboxLayout };
//# sourceMappingURL=index.android.js.map