import { GridLayoutBase, ItemSpec as ItemSpecBase, rowProperty, columnProperty, rowSpanProperty, columnSpanProperty, GridUnitType } from './grid-layout-common';
import { View } from '../../core/view';
import { layout } from '../../../utils';
export * from './grid-layout-common';
function makeNativeSetter(setter) {
    return function (value) {
        var nativeView = this.nativeViewProtected;
        var lp = nativeView.getLayoutParams() || new org.nativescript.widgets.CommonLayoutParams();
        if (lp instanceof org.nativescript.widgets.CommonLayoutParams) {
            setter(lp, value);
            nativeView.setLayoutParams(lp);
        }
    };
}
View.prototype[rowProperty.setNative] = makeNativeSetter(function (lp, value) { return (lp.row = value); });
View.prototype[columnProperty.setNative] = makeNativeSetter(function (lp, value) { return (lp.column = value); });
View.prototype[rowSpanProperty.setNative] = makeNativeSetter(function (lp, value) { return (lp.rowSpan = value); });
View.prototype[columnSpanProperty.setNative] = makeNativeSetter(function (lp, value) { return (lp.columnSpan = value); });
ItemSpecBase.prototype.toJSON = function () {
    var result;
    switch (this.gridUnitType) {
        case GridUnitType.AUTO:
            result = { type: 0 /* org.nativescript.widgets.GridUnitType.auto */, value: this.value };
            break;
        case GridUnitType.PIXEL:
            result = { type: 1 /* org.nativescript.widgets.GridUnitType.pixel */, value: this.value * layout.getDisplayDensity() };
            break;
        case GridUnitType.STAR:
            result = { type: 2 /* org.nativescript.widgets.GridUnitType.star */, value: this.value };
            break;
        default:
            return null;
    }
    return result;
};
var GridLayout = /** @class */ (function (_super) {
    __extends(GridLayout, _super);
    function GridLayout() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    GridLayout.prototype.createNativeView = function () {
        return new org.nativescript.widgets.GridLayout(this._context);
    };
    GridLayout.prototype.initNativeView = function () {
        _super.prototype.initNativeView.call(this);
        // Update native GridLayout
        var jsonRows = JSON.stringify(this.rowsInternal.map(function (itemSpec) { return itemSpec.toJSON(); }).filter(function (j) { return !!j; }));
        var jsonColumns = JSON.stringify(this.columnsInternal.map(function (itemSpec) { return itemSpec.toJSON(); }).filter(function (j) { return !!j; }));
        this.nativeViewProtected.addRowsAndColumnsFromJSON(jsonRows, jsonColumns);
    };
    GridLayout.prototype.resetNativeView = function () {
        // Update native GridLayout
        this.nativeViewProtected.reset();
        _super.prototype.resetNativeView.call(this);
    };
    GridLayout.prototype._onRowAdded = function (itemSpec) {
        if (this.nativeViewProtected) {
            this.nativeViewProtected.addRowsFromJSON(JSON.stringify([itemSpec.toJSON()]));
        }
    };
    GridLayout.prototype.addRows = function (itemSpecs) {
        var jsonArray = [];
        var nativeView = this.nativeViewProtected;
        var initialized = !!nativeView;
        for (var index = 0; index < itemSpecs.length; index++) {
            var itemSpec = itemSpecs[index];
            this._addRow(itemSpec);
            if (initialized) {
                jsonArray.push(itemSpec.toJSON());
            }
        }
        if (initialized) {
            nativeView.addRowsFromJSON(JSON.stringify(jsonArray.filter(function (s) { return !!s; })));
        }
    };
    GridLayout.prototype.addColumns = function (itemSpecs) {
        var jsonArray = [];
        var nativeView = this.nativeViewProtected;
        var initialized = !!nativeView;
        for (var index = 0; index < itemSpecs.length; index++) {
            var itemSpec = itemSpecs[index];
            this._addColumn(itemSpec);
            if (initialized) {
                jsonArray.push(itemSpec.toJSON());
            }
        }
        if (initialized) {
            nativeView.addColumnsFromJSON(JSON.stringify(jsonArray.filter(function (s) { return !!s; })));
        }
    };
    GridLayout.prototype._onColumnAdded = function (itemSpec) {
        if (this.nativeViewProtected) {
            this.nativeViewProtected.addColumnsFromJSON(JSON.stringify([itemSpec.toJSON()]));
        }
    };
    GridLayout.prototype.removeColumns = function () {
        if (this._cols.length) {
            if (this.nativeViewProtected) {
                this.nativeViewProtected.clearColumns();
            }
            this._cols.length = 0;
        }
    };
    GridLayout.prototype.removeRows = function () {
        if (this._rows.length) {
            if (this.nativeViewProtected) {
                this.nativeViewProtected.clearRows();
            }
            this._rows.length = 0;
        }
    };
    GridLayout.prototype._onRowRemoved = function (itemSpec, index) {
        if (this.nativeViewProtected) {
            this.nativeViewProtected.removeRowAt(index);
        }
    };
    GridLayout.prototype._onColumnRemoved = function (itemSpec, index) {
        if (this.nativeViewProtected) {
            this.nativeViewProtected.removeColumnAt(index);
        }
    };
    GridLayout.prototype.invalidate = function () {
        // No need to request layout for android because it will be done in the native call.
    };
    return GridLayout;
}(GridLayoutBase));
export { GridLayout };
//# sourceMappingURL=index.android.js.map