import { LayoutBaseCommon, clipToBoundsProperty, isPassThroughParentEnabledProperty } from './layout-base-common';
import { Length, paddingLeftProperty, paddingTopProperty, paddingRightProperty, paddingBottomProperty } from '../styling/style-properties';
export * from './layout-base-common';
var LayoutBase = /** @class */ (function (_super) {
    __extends(LayoutBase, _super);
    function LayoutBase() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    LayoutBase.prototype[clipToBoundsProperty.getDefault] = function () {
        return true;
    };
    LayoutBase.prototype[clipToBoundsProperty.setNative] = function (value) {
        // TODO: Use ClipRectangle if API > 16!
        // We can't implement this without calling setClipChildren(false) on every ancestor up in the visual tree,
        // which will kill performance. It will also lead to unwanted side effects such as other totally unrelated
        // views being affected by setting the parents' setClipChildren to false.
        // The problem in Android is that a ViewGroup either clips ALL of its children or it does not. Unlike iOS, the clipping
        // cannot be controlled on a per view basis. So clipToBounds=false will have to be somehow achieved with stacking different
        // views on top of one another in an AbsoluteLayout or GridLayout. There is always a workaround when playing with layouts.
        //
        // The following article explains this in detail:
        // http://stackoverflow.com/questions/25044085/when-drawing-outside-the-view-clip-bounds-with-android-how-do-i-prevent-underli
        console.warn("clipToBounds with value false is not supported on Android. You can use this.android.getParent().setClipChildren(false) as an alternative");
    };
    LayoutBase.prototype[isPassThroughParentEnabledProperty.setNative] = function (value) {
        this.nativeViewProtected.setPassThroughParent(value);
    };
    LayoutBase.prototype[paddingTopProperty.getDefault] = function () {
        return { value: this._defaultPaddingTop, unit: 'px' };
    };
    LayoutBase.prototype[paddingTopProperty.setNative] = function (value) {
        org.nativescript.widgets.ViewHelper.setPaddingTop(this.nativeViewProtected, Length.toDevicePixels(value, 0) + Length.toDevicePixels(this.style.borderTopWidth, 0));
    };
    LayoutBase.prototype[paddingRightProperty.getDefault] = function () {
        return { value: this._defaultPaddingRight, unit: 'px' };
    };
    LayoutBase.prototype[paddingRightProperty.setNative] = function (value) {
        org.nativescript.widgets.ViewHelper.setPaddingRight(this.nativeViewProtected, Length.toDevicePixels(value, 0) + Length.toDevicePixels(this.style.borderRightWidth, 0));
    };
    LayoutBase.prototype[paddingBottomProperty.getDefault] = function () {
        return { value: this._defaultPaddingBottom, unit: 'px' };
    };
    LayoutBase.prototype[paddingBottomProperty.setNative] = function (value) {
        org.nativescript.widgets.ViewHelper.setPaddingBottom(this.nativeViewProtected, Length.toDevicePixels(value, 0) + Length.toDevicePixels(this.style.borderBottomWidth, 0));
    };
    LayoutBase.prototype[paddingLeftProperty.getDefault] = function () {
        return { value: this._defaultPaddingLeft, unit: 'px' };
    };
    LayoutBase.prototype[paddingLeftProperty.setNative] = function (value) {
        org.nativescript.widgets.ViewHelper.setPaddingLeft(this.nativeViewProtected, Length.toDevicePixels(value, 0) + Length.toDevicePixels(this.style.borderLeftWidth, 0));
    };
    return LayoutBase;
}(LayoutBaseCommon));
export { LayoutBase };
//# sourceMappingURL=layout-base.android.js.map