import { Color } from '../../../color';
import { RootLayoutBase, defaultShadeCoverOptions } from './root-layout-common';
import { parseLinearGradient } from '../../../css/parser';
import { LinearGradient } from '../../styling/linear-gradient';
export * from './root-layout-common';
var RootLayout = /** @class */ (function (_super) {
    __extends(RootLayout, _super);
    function RootLayout() {
        return _super.call(this) || this;
    }
    RootLayout.prototype.insertChild = function (view, atIndex) {
        _super.prototype.insertChild.call(this, view, atIndex);
        if (!view.hasGestureObservers()) {
            // block tap events from going through to layers behind the view
            if (view.nativeViewProtected) {
                view.nativeViewProtected.setOnTouchListener(new android.view.View.OnTouchListener({
                    onTouch: function (view, event) {
                        return true;
                    },
                }));
            }
        }
    };
    RootLayout.prototype.removeChild = function (view) {
        if (view.hasGestureObservers() && view.nativeViewProtected) {
            view.nativeViewProtected.setOnTouchListener(null);
        }
        _super.prototype.removeChild.call(this, view);
    };
    RootLayout.prototype._bringToFront = function (view) {
        view.nativeViewProtected.bringToFront();
    };
    RootLayout.prototype._initShadeCover = function (view, shadeOptions) {
        var _a;
        var initialState = __assign(__assign({}, defaultShadeCoverOptions.animation.enterFrom), (_a = shadeOptions === null || shadeOptions === void 0 ? void 0 : shadeOptions.animation) === null || _a === void 0 ? void 0 : _a.enterFrom);
        this._playAnimation(this._getAnimationSet(view, initialState));
    };
    RootLayout.prototype._updateShadeCover = function (view, shadeOptions) {
        var _a, _b;
        var options = __assign(__assign({}, defaultShadeCoverOptions), shadeOptions);
        var duration = ((_b = (_a = options.animation) === null || _a === void 0 ? void 0 : _a.enterFrom) === null || _b === void 0 ? void 0 : _b.duration) || defaultShadeCoverOptions.animation.enterFrom.duration;
        return this._playAnimation(this._getAnimationSet(view, {
            translateX: 0,
            translateY: 0,
            scaleX: 1,
            scaleY: 1,
            rotate: 0,
            opacity: options.opacity,
        }, options.color), duration);
    };
    RootLayout.prototype._closeShadeCover = function (view, shadeOptions) {
        var _a;
        var exitState = __assign(__assign({}, defaultShadeCoverOptions.animation.exitTo), (_a = shadeOptions === null || shadeOptions === void 0 ? void 0 : shadeOptions.animation) === null || _a === void 0 ? void 0 : _a.exitTo);
        return this._playAnimation(this._getAnimationSet(view, exitState), exitState === null || exitState === void 0 ? void 0 : exitState.duration);
    };
    RootLayout.prototype._getAnimationSet = function (view, shadeCoverAnimation, backgroundColor) {
        if (backgroundColor === void 0) { backgroundColor = defaultShadeCoverOptions.color; }
        var isBackgroundGradient = backgroundColor && backgroundColor.startsWith('linear-gradient');
        var animationSet = Array.create(android.animation.Animator, isBackgroundGradient ? 6 : 7);
        animationSet[0] = android.animation.ObjectAnimator.ofFloat(view.nativeViewProtected, 'translationX', [shadeCoverAnimation.translateX]);
        animationSet[1] = android.animation.ObjectAnimator.ofFloat(view.nativeViewProtected, 'translationY', [shadeCoverAnimation.translateY]);
        animationSet[2] = android.animation.ObjectAnimator.ofFloat(view.nativeViewProtected, 'scaleX', [shadeCoverAnimation.scaleX]);
        animationSet[3] = android.animation.ObjectAnimator.ofFloat(view.nativeViewProtected, 'scaleY', [shadeCoverAnimation.scaleY]);
        animationSet[4] = android.animation.ObjectAnimator.ofFloat(view.nativeViewProtected, 'rotation', [shadeCoverAnimation.rotate]);
        animationSet[5] = android.animation.ObjectAnimator.ofFloat(view.nativeViewProtected, 'alpha', [shadeCoverAnimation.opacity]);
        if (isBackgroundGradient) {
            if (view.backgroundColor) {
                view.backgroundColor = undefined;
            }
            var parsedGradient = parseLinearGradient(backgroundColor);
            view.backgroundImage = LinearGradient.parse(parsedGradient.value);
        }
        else {
            if (view.backgroundImage) {
                view.backgroundImage = undefined;
            }
            animationSet[6] = this._getBackgroundColorAnimator(view, backgroundColor);
        }
        return animationSet;
    };
    RootLayout.prototype._getBackgroundColorAnimator = function (view, backgroundColor) {
        var nativeArray = Array.create(java.lang.Object, 2);
        nativeArray[0] = java.lang.Integer.valueOf(view.backgroundColor ? view.backgroundColor.argb : -1);
        nativeArray[1] = java.lang.Integer.valueOf(backgroundColor ? new Color(backgroundColor).argb : -1);
        var backgroundColorAnimator = android.animation.ValueAnimator.ofObject(new android.animation.ArgbEvaluator(), nativeArray);
        backgroundColorAnimator.addUpdateListener(new android.animation.ValueAnimator.AnimatorUpdateListener({
            onAnimationUpdate: function (animator) {
                var argb = animator.getAnimatedValue().intValue();
                view.backgroundColor = new Color(argb);
            },
        }));
        return backgroundColorAnimator;
    };
    RootLayout.prototype._playAnimation = function (animationSet, duration) {
        if (duration === void 0) { duration = 0; }
        return new Promise(function (resolve) {
            var animatorSet = new android.animation.AnimatorSet();
            animatorSet.playTogether(animationSet);
            animatorSet.setDuration(duration);
            animatorSet.addListener(new android.animation.Animator.AnimatorListener({
                onAnimationStart: function (animator) { },
                onAnimationEnd: function (animator) {
                    resolve();
                },
                onAnimationRepeat: function (animator) { },
                onAnimationCancel: function (animator) { },
            }));
            animatorSet.start();
        });
    };
    return RootLayout;
}(RootLayoutBase));
export { RootLayout };
//# sourceMappingURL=index.android.js.map