import { Color } from '../../../color';
import { RootLayoutBase, defaultShadeCoverOptions } from './root-layout-common';
import { LinearGradient } from '../../styling/linear-gradient';
import { ios as iosViewUtils } from '../../utils';
import { parseLinearGradient } from '../../../css/parser';
export * from './root-layout-common';
var RootLayout = /** @class */ (function (_super) {
    __extends(RootLayout, _super);
    function RootLayout() {
        return _super.call(this) || this;
    }
    RootLayout.prototype._bringToFront = function (view) {
        this.nativeViewProtected.bringSubviewToFront(view.nativeViewProtected);
    };
    RootLayout.prototype._initShadeCover = function (view, shadeOptions) {
        var _a;
        var initialState = __assign(__assign({}, defaultShadeCoverOptions.animation.enterFrom), (_a = shadeOptions === null || shadeOptions === void 0 ? void 0 : shadeOptions.animation) === null || _a === void 0 ? void 0 : _a.enterFrom);
        this._applyAnimationProperties(view, initialState);
    };
    RootLayout.prototype._updateShadeCover = function (view, shadeOptions) {
        var _this = this;
        return new Promise(function (resolve) {
            var _a, _b;
            var options = __assign(__assign({}, defaultShadeCoverOptions), shadeOptions);
            if (view === null || view === void 0 ? void 0 : view.nativeViewProtected) {
                var duration = _this._convertDurationToSeconds(((_b = (_a = options.animation) === null || _a === void 0 ? void 0 : _a.enterFrom) === null || _b === void 0 ? void 0 : _b.duration) || defaultShadeCoverOptions.animation.enterFrom.duration);
                if (options.color && options.color.startsWith('linear-gradient')) {
                    if (options.color !== _this._currentGradient) {
                        _this._currentGradient = options.color;
                        var parsedGradient = parseLinearGradient(options.color);
                        _this._gradientLayer = CAGradientLayer.new();
                        iosViewUtils.drawGradient(view.nativeViewProtected, _this._gradientLayer, LinearGradient.parse(parsedGradient.value));
                        view.nativeViewProtected.layer.insertSublayerAtIndex(_this._gradientLayer, 0);
                    }
                }
                UIView.animateWithDurationAnimationsCompletion(duration, function () {
                    if (_this._gradientLayer) {
                        _this._gradientLayer.opacity = 1;
                    }
                    else if (options.color && (view === null || view === void 0 ? void 0 : view.nativeViewProtected)) {
                        view.nativeViewProtected.backgroundColor = new Color(options.color).ios;
                    }
                    _this._applyAnimationProperties(view, {
                        translateX: 0,
                        translateY: 0,
                        scaleX: 1,
                        scaleY: 1,
                        rotate: 0,
                        opacity: shadeOptions.opacity,
                    });
                }, function (completed) {
                    resolve();
                });
            }
        });
    };
    RootLayout.prototype._closeShadeCover = function (view, shadeOptions) {
        var _this = this;
        return new Promise(function (resolve) {
            var _a;
            var exitState = __assign(__assign({}, defaultShadeCoverOptions.animation.exitTo), (_a = shadeOptions === null || shadeOptions === void 0 ? void 0 : shadeOptions.animation) === null || _a === void 0 ? void 0 : _a.exitTo);
            if (view && view.nativeViewProtected) {
                UIView.animateWithDurationAnimationsCompletion(_this._convertDurationToSeconds(exitState.duration), function () {
                    _this._applyAnimationProperties(view, exitState);
                }, function (completed) {
                    resolve();
                });
            }
        });
    };
    RootLayout.prototype._cleanupPlatformShadeCover = function () {
        this._currentGradient = null;
        if (this._gradientLayer != null) {
            this._gradientLayer.removeFromSuperlayer();
            this._gradientLayer = null;
        }
    };
    RootLayout.prototype._applyAnimationProperties = function (view, shadeCoverAnimation) {
        if (view === null || view === void 0 ? void 0 : view.nativeViewProtected) {
            var translate = CGAffineTransformMakeTranslation(shadeCoverAnimation.translateX, shadeCoverAnimation.translateY);
            // ios doesn't like scale being 0, default it to a small number greater than 0
            var scale = CGAffineTransformMakeScale(shadeCoverAnimation.scaleX || 0.1, shadeCoverAnimation.scaleY || 0.1);
            var rotate = CGAffineTransformMakeRotation((shadeCoverAnimation.rotate * Math.PI) / 180); // convert degress to radians
            var translateAndScale = CGAffineTransformConcat(translate, scale);
            view.nativeViewProtected.transform = CGAffineTransformConcat(rotate, translateAndScale);
            view.nativeViewProtected.alpha = shadeCoverAnimation.opacity;
        }
    };
    RootLayout.prototype._convertDurationToSeconds = function (duration) {
        return duration / 1000;
    };
    return RootLayout;
}(RootLayoutBase));
export { RootLayout };
//# sourceMappingURL=index.ios.js.map