import { View, CSSType } from '../core/view';
import { Property, CoercibleProperty } from '../core/properties';
var ListPickerBase = /** @class */ (function (_super) {
    __extends(ListPickerBase, _super);
    function ListPickerBase() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    ListPickerBase.prototype._getItemAsString = function (index) {
        var items = this.items;
        if (!items) {
            return ' ';
        }
        var item = this.isItemsSource ? this.items.getItem(index) : this.items[index];
        return item === undefined || item === null ? index + '' : this.parseItem(item);
    };
    ListPickerBase.prototype.parseItem = function (item) {
        return this.textField ? item[this.textField] + '' : item + '';
    };
    ListPickerBase.prototype.updateSelectedValue = function (index) {
        var newVal = null;
        if (index >= 0) {
            var item = this.items[index];
            newVal = this.valueField ? item[this.valueField] : item;
        }
        if (this.selectedValue !== newVal) {
            this.set('selectedValue', newVal);
        }
    };
    ListPickerBase.selectedIndexChangeEvent = 'selectedIndexChange';
    ListPickerBase = __decorate([
        CSSType('ListPicker')
    ], ListPickerBase);
    return ListPickerBase;
}(View));
export { ListPickerBase };
ListPickerBase.prototype.recycleNativeView = 'auto';
export var selectedIndexProperty = new CoercibleProperty({
    name: 'selectedIndex',
    defaultValue: -1,
    valueConverter: function (v) { return parseInt(v); },
    coerceValue: function (target, value) {
        var items = target.items;
        if (items) {
            var max = items.length - 1;
            if (value < 0) {
                value = 0;
            }
            if (value > max) {
                value = max;
            }
        }
        else {
            value = -1;
        }
        target.updateSelectedValue(value);
        return value;
    },
});
selectedIndexProperty.register(ListPickerBase);
export var itemsProperty = new Property({
    name: 'items',
    valueChanged: function (target, oldValue, newValue) {
        var getItem = newValue && newValue.getItem;
        target.isItemsSource = typeof getItem === 'function';
    },
});
itemsProperty.register(ListPickerBase);
export var textFieldProperty = new Property({
    name: 'textField',
    defaultValue: '',
});
textFieldProperty.register(ListPickerBase);
export var valueFieldProperty = new Property({
    name: 'valueField',
    defaultValue: '',
});
valueFieldProperty.register(ListPickerBase);
export var selectedValueProperty = new Property({
    name: 'selectedValue',
    defaultValue: null,
});
selectedValueProperty.register(ListPickerBase);
//# sourceMappingURL=list-picker-common.js.map