import { PageBase, actionBarHiddenProperty, statusBarStyleProperty, androidStatusBarBackgroundProperty } from './page-common';
import { Color } from '../../color';
import { ActionBar } from '../action-bar';
import { GridLayout } from '../layouts/grid-layout';
import { SDK_VERSION } from '../../utils/constants';
import { profile } from '../../profiling';
import { AndroidAccessibilityEvent, getLastFocusedViewOnPage, isAccessibilityServiceEnabled } from '../../accessibility';
export * from './page-common';
var SYSTEM_UI_FLAG_LIGHT_STATUS_BAR = 0x00002000;
var STATUS_BAR_LIGHT_BCKG = -657931;
var STATUS_BAR_DARK_BCKG = 1711276032;
var Page = /** @class */ (function (_super) {
    __extends(Page, _super);
    function Page() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    Page.prototype.createNativeView = function () {
        var layout = new org.nativescript.widgets.GridLayout(this._context);
        layout.addRowsFromJSON(JSON.stringify([
            { value: 1, type: 0 /* org.nativescript.widgets.GridUnitType.auto */ },
            { value: 1, type: 2 /* org.nativescript.widgets.GridUnitType.star */ },
        ]));
        return layout;
    };
    Page.prototype.initNativeView = function () {
        _super.prototype.initNativeView.call(this);
        this.nativeViewProtected.setBackgroundColor(-1); // White color.
    };
    Page.prototype._addViewToNativeVisualTree = function (child, atIndex) {
        // Set the row property for the child
        if (this.nativeViewProtected && child.nativeViewProtected) {
            if (child instanceof ActionBar) {
                GridLayout.setRow(child, 0);
                child.horizontalAlignment = 'stretch';
                child.verticalAlignment = 'top';
            }
            else {
                GridLayout.setRow(child, 1);
            }
        }
        return _super.prototype._addViewToNativeVisualTree.call(this, child, atIndex);
    };
    Page.prototype.onLoaded = function () {
        _super.prototype.onLoaded.call(this);
        if (!this.hasActionBar && this.actionBarHidden !== true) {
            // ensure actionBar is created
            // but we only need to do that if the actionBarHidden is not hidden
            this.actionBar = new ActionBar();
        }
        if (this.hasActionBar) {
            this.updateActionBar();
        }
    };
    Page.prototype.updateActionBar = function () {
        // the test is actually to ensure the actionBar is created
        // it will be created if not
        if (this.actionBar) {
            this.actionBar.update();
        }
    };
    Page.prototype.getClosestWindow = function () {
        var _a;
        // When it comes to modals, check if page has a parent as it may not be the modal root view itself
        var view = (_a = this.parent) !== null && _a !== void 0 ? _a : this;
        var dialogFragment = view._dialogFragment;
        if (dialogFragment) {
            var dialog = dialogFragment.getDialog();
            if (dialog) {
                return dialog.getWindow();
            }
        }
        return this._context.getWindow();
    };
    Page.prototype[actionBarHiddenProperty.setNative] = function (value) {
        // in case the actionBar is not created and actionBarHidden is changed to true
        // the actionBar will be created by updateActionBar
        if (!value || this.hasActionBar) {
            this.updateActionBar();
        }
    };
    Page.prototype[statusBarStyleProperty.getDefault] = function () {
        if (SDK_VERSION >= 21) {
            var window_1 = this.getClosestWindow();
            var decorView = window_1.getDecorView();
            return {
                color: window_1.getStatusBarColor(),
                systemUiVisibility: decorView.getSystemUiVisibility(),
            };
        }
        return null;
    };
    Page.prototype[statusBarStyleProperty.setNative] = function (value) {
        if (SDK_VERSION >= 21) {
            var window_2 = this.getClosestWindow();
            var decorView = window_2.getDecorView();
            if (value === 'light') {
                window_2.setStatusBarColor(STATUS_BAR_LIGHT_BCKG);
                decorView.setSystemUiVisibility(SYSTEM_UI_FLAG_LIGHT_STATUS_BAR);
            }
            else if (value === 'dark') {
                window_2.setStatusBarColor(STATUS_BAR_DARK_BCKG);
                decorView.setSystemUiVisibility(0);
            }
            else {
                window_2.setStatusBarColor(value.color);
                decorView.setSystemUiVisibility(value.systemUiVisibility);
            }
        }
    };
    Page.prototype[androidStatusBarBackgroundProperty.getDefault] = function () {
        if (SDK_VERSION >= 21) {
            var window_3 = this.getClosestWindow();
            return window_3.getStatusBarColor();
        }
        return null;
    };
    Page.prototype[androidStatusBarBackgroundProperty.setNative] = function (value) {
        if (SDK_VERSION >= 21) {
            var window_4 = this.getClosestWindow();
            var color = value instanceof Color ? value.android : value;
            window_4.setStatusBarColor(color);
        }
    };
    Page.prototype.accessibilityScreenChanged = function (refocus) {
        if (refocus === void 0) { refocus = false; }
        if (!isAccessibilityServiceEnabled()) {
            return;
        }
        if (refocus) {
            var lastFocusedView = getLastFocusedViewOnPage(this);
            if (lastFocusedView) {
                var announceView = lastFocusedView.nativeViewProtected;
                if (announceView) {
                    announceView.sendAccessibilityEvent(android.view.accessibility.AccessibilityEvent.TYPE_VIEW_FOCUSED);
                    announceView.sendAccessibilityEvent(android.view.accessibility.AccessibilityEvent.TYPE_VIEW_ACCESSIBILITY_FOCUSED);
                    return;
                }
            }
        }
        if (this.actionBarHidden || this.accessibilityLabel) {
            this.sendAccessibilityEvent({
                androidAccessibilityEvent: AndroidAccessibilityEvent.WINDOW_STATE_CHANGED,
            });
            return;
        }
        this.actionBar.accessibilityScreenChanged();
    };
    __decorate([
        profile,
        __metadata("design:type", Function),
        __metadata("design:paramtypes", []),
        __metadata("design:returntype", void 0)
    ], Page.prototype, "onLoaded", null);
    return Page;
}(PageBase));
export { Page };
//# sourceMappingURL=index.android.js.map