import { ContentView } from '../content-view';
import { CSSType } from '../core/view';
import { booleanConverter } from '../core/view-base';
import { Property, CssProperty } from '../core/properties';
import { Style } from '../styling/style';
import { Color } from '../../color';
import { isFrame } from '../frame/frame-helpers';
import { ActionBar } from '../action-bar';
import { profile } from '../../profiling';
var PageBase = /** @class */ (function (_super) {
    __extends(PageBase, _super);
    function PageBase() {
        var _this = _super !== null && _super.apply(this, arguments) || this;
        _this.accessibilityAnnouncePageEnabled = true;
        return _this;
    }
    PageBase_1 = PageBase;
    Object.defineProperty(PageBase.prototype, "navigationContext", {
        get: function () {
            return this._navigationContext;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(PageBase.prototype, "actionBar", {
        get: function () {
            if (!this._actionBar) {
                this.hasActionBar = true;
                this._actionBar = new ActionBar();
                this._addView(this._actionBar);
            }
            return this._actionBar;
        },
        set: function (value) {
            if (!value) {
                throw new Error('ActionBar cannot be null or undefined.');
            }
            if (this._actionBar !== value) {
                if (this._actionBar) {
                    this._removeView(this._actionBar);
                }
                this.hasActionBar = true;
                this._actionBar = value;
                this._addView(this._actionBar);
            }
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(PageBase.prototype, "statusBarStyle", {
        get: function () {
            return this.style.statusBarStyle;
        },
        set: function (value) {
            this.style.statusBarStyle = value;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(PageBase.prototype, "androidStatusBarBackground", {
        get: function () {
            return this.style.androidStatusBarBackground;
        },
        set: function (value) {
            this.style.androidStatusBarBackground = value;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(PageBase.prototype, "page", {
        get: function () {
            return this;
        },
        enumerable: false,
        configurable: true
    });
    PageBase.prototype._parentChanged = function (oldParent) {
        var newParent = this.parent;
        if (newParent && !isFrame(newParent)) {
            throw new Error("Page can only be nested inside Frame. New parent: ".concat(newParent));
        }
        _super.prototype._parentChanged.call(this, oldParent);
    };
    PageBase.prototype._addChildFromBuilder = function (name, value) {
        if (value instanceof ActionBar) {
            this.actionBar = value;
        }
        else {
            _super.prototype._addChildFromBuilder.call(this, name, value);
        }
    };
    PageBase.prototype.getKeyframeAnimationWithName = function (animationName) {
        return this._styleScope.getKeyframeAnimationWithName(animationName);
    };
    Object.defineProperty(PageBase.prototype, "frame", {
        get: function () {
            return this.parent;
        },
        enumerable: false,
        configurable: true
    });
    PageBase.prototype.createNavigatedData = function (eventName, isBackNavigation) {
        return {
            eventName: eventName,
            object: this,
            context: this.navigationContext,
            isBackNavigation: isBackNavigation,
        };
    };
    PageBase.prototype.onNavigatingTo = function (context, isBackNavigation, bindingContext) {
        this._navigationContext = context;
        if (isBackNavigation && this._styleScope) {
            this._styleScope.ensureSelectors();
            if (!this._cssState.isSelectorsLatestVersionApplied()) {
                this._onCssStateChange();
            }
        }
        //https://github.com/NativeScript/NativeScript/issues/731
        if (!isBackNavigation && bindingContext !== undefined && bindingContext !== null) {
            this.bindingContext = bindingContext;
        }
        this.notify(this.createNavigatedData(PageBase_1.navigatingToEvent, isBackNavigation));
    };
    PageBase.prototype.onNavigatedTo = function (isBackNavigation) {
        this.notify(this.createNavigatedData(PageBase_1.navigatedToEvent, isBackNavigation));
        if (this.accessibilityAnnouncePageEnabled) {
            this.accessibilityScreenChanged(!!isBackNavigation);
        }
    };
    PageBase.prototype.onNavigatingFrom = function (isBackNavigation) {
        this.notify(this.createNavigatedData(PageBase_1.navigatingFromEvent, isBackNavigation));
    };
    PageBase.prototype.onNavigatedFrom = function (isBackNavigation) {
        this.notify(this.createNavigatedData(PageBase_1.navigatedFromEvent, isBackNavigation));
        this._navigationContext = undefined;
    };
    PageBase.prototype.eachChildView = function (callback) {
        _super.prototype.eachChildView.call(this, callback);
        if (this.hasActionBar) {
            callback(this.actionBar);
        }
    };
    Object.defineProperty(PageBase.prototype, "_childrenCount", {
        get: function () {
            return (this.content ? 1 : 0) + (this._actionBar ? 1 : 0);
        },
        enumerable: false,
        configurable: true
    });
    PageBase.prototype.accessibilityScreenChanged = function (refocus) {
        return;
    };
    var PageBase_1;
    PageBase.navigatingToEvent = 'navigatingTo';
    PageBase.navigatedToEvent = 'navigatedTo';
    PageBase.navigatingFromEvent = 'navigatingFrom';
    PageBase.navigatedFromEvent = 'navigatedFrom';
    __decorate([
        profile,
        __metadata("design:type", Function),
        __metadata("design:paramtypes", [Object, Boolean, Object]),
        __metadata("design:returntype", void 0)
    ], PageBase.prototype, "onNavigatingTo", null);
    __decorate([
        profile,
        __metadata("design:type", Function),
        __metadata("design:paramtypes", [Boolean]),
        __metadata("design:returntype", void 0)
    ], PageBase.prototype, "onNavigatedTo", null);
    __decorate([
        profile,
        __metadata("design:type", Function),
        __metadata("design:paramtypes", [Boolean]),
        __metadata("design:returntype", void 0)
    ], PageBase.prototype, "onNavigatingFrom", null);
    __decorate([
        profile,
        __metadata("design:type", Function),
        __metadata("design:paramtypes", [Boolean]),
        __metadata("design:returntype", void 0)
    ], PageBase.prototype, "onNavigatedFrom", null);
    PageBase = PageBase_1 = __decorate([
        CSSType('Page')
    ], PageBase);
    return PageBase;
}(ContentView));
export { PageBase };
PageBase.prototype.recycleNativeView = 'never';
/**
 * Dependency property used to hide the Navigation Bar in iOS and the Action Bar in Android.
 */
export var actionBarHiddenProperty = new Property({
    name: 'actionBarHidden',
    affectsLayout: __APPLE__,
    valueConverter: booleanConverter,
});
actionBarHiddenProperty.register(PageBase);
/**
 * Dependency property that specify if page background should span under status bar.
 */
export var backgroundSpanUnderStatusBarProperty = new Property({
    name: 'backgroundSpanUnderStatusBar',
    defaultValue: false,
    affectsLayout: __APPLE__,
    valueConverter: booleanConverter,
});
backgroundSpanUnderStatusBarProperty.register(PageBase);
/**
 * Dependency property used to control if swipe back navigation in iOS is enabled.
 * This property is iOS specific. Default value: true
 */
export var enableSwipeBackNavigationProperty = new Property({
    name: 'enableSwipeBackNavigation',
    defaultValue: true,
    valueConverter: booleanConverter,
});
enableSwipeBackNavigationProperty.register(PageBase);
/**
 * Property backing statusBarStyle.
 */
export var statusBarStyleProperty = new CssProperty({
    name: 'statusBarStyle',
    cssName: 'status-bar-style',
});
statusBarStyleProperty.register(Style);
/**
 * Property backing androidStatusBarBackground.
 */
export var androidStatusBarBackgroundProperty = new CssProperty({
    name: 'androidStatusBarBackground',
    cssName: 'android-status-bar-background',
    equalityComparer: Color.equals,
    valueConverter: function (v) { return new Color(v); },
});
androidStatusBarBackgroundProperty.register(Style);
//# sourceMappingURL=page-common.js.map