import { ContentView } from '../content-view';
import { Property, makeParser, makeValidator } from '../core/properties';
import { CSSType } from '../core/view';
import { booleanConverter } from '../core/view-base';
import { CoreTypes } from '../../core-types';
var ScrollViewBase = /** @class */ (function (_super) {
    __extends(ScrollViewBase, _super);
    function ScrollViewBase() {
        var _this = _super !== null && _super.apply(this, arguments) || this;
        _this._addedScrollEvent = false;
        return _this;
    }
    ScrollViewBase_1 = ScrollViewBase;
    ScrollViewBase.prototype.addEventListener = function (arg, callback, thisArg) {
        var hasExistingScrollListeners = this.hasListeners(ScrollViewBase_1.scrollEvent);
        _super.prototype.addEventListener.call(this, arg, callback, thisArg);
        // This indicates that a scroll listener was added for first time
        if (!hasExistingScrollListeners && this.hasListeners(ScrollViewBase_1.scrollEvent)) {
            this._addedScrollEvent = true;
            if (this.nativeViewProtected) {
                this.attachNative();
            }
        }
    };
    ScrollViewBase.prototype.removeEventListener = function (arg, callback, thisArg) {
        var hasExistingScrollListeners = this.hasListeners(ScrollViewBase_1.scrollEvent);
        _super.prototype.removeEventListener.call(this, arg, callback, thisArg);
        // This indicates that the final scroll listener was removed
        if (hasExistingScrollListeners && !this.hasListeners(ScrollViewBase_1.scrollEvent)) {
            this._addedScrollEvent = false;
            if (this.nativeViewProtected) {
                this.detachNative();
            }
        }
    };
    ScrollViewBase.prototype.initNativeView = function () {
        _super.prototype.initNativeView.call(this);
        if (this._addedScrollEvent) {
            this.attachNative();
        }
    };
    ScrollViewBase.prototype.disposeNativeView = function () {
        if (this._addedScrollEvent) {
            this.detachNative();
        }
        _super.prototype.disposeNativeView.call(this);
    };
    ScrollViewBase.prototype.attachNative = function () {
        //
    };
    ScrollViewBase.prototype.detachNative = function () {
        //
    };
    Object.defineProperty(ScrollViewBase.prototype, "horizontalOffset", {
        get: function () {
            return 0;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(ScrollViewBase.prototype, "verticalOffset", {
        get: function () {
            return 0;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(ScrollViewBase.prototype, "scrollableWidth", {
        get: function () {
            return 0;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(ScrollViewBase.prototype, "scrollableHeight", {
        get: function () {
            return 0;
        },
        enumerable: false,
        configurable: true
    });
    var ScrollViewBase_1;
    ScrollViewBase.scrollEvent = 'scroll';
    ScrollViewBase = ScrollViewBase_1 = __decorate([
        CSSType('ScrollView')
    ], ScrollViewBase);
    return ScrollViewBase;
}(ContentView));
export { ScrollViewBase };
var converter = makeParser(makeValidator(CoreTypes.Orientation.horizontal, CoreTypes.Orientation.vertical));
export var orientationProperty = new Property({
    name: 'orientation',
    defaultValue: CoreTypes.Orientation.vertical,
    affectsLayout: true,
    valueChanged: function (target, oldValue, newValue) {
        target._onOrientationChanged();
    },
    valueConverter: converter,
});
orientationProperty.register(ScrollViewBase);
export var scrollBarIndicatorVisibleProperty = new Property({
    name: 'scrollBarIndicatorVisible',
    defaultValue: true,
    valueConverter: booleanConverter,
});
scrollBarIndicatorVisibleProperty.register(ScrollViewBase);
export var isScrollEnabledProperty = new Property({
    name: 'isScrollEnabled',
    defaultValue: true,
    valueConverter: booleanConverter,
});
isScrollEnabledProperty.register(ScrollViewBase);
//# sourceMappingURL=scroll-view-common.js.map