import { Font } from '../styling/font';
import { SearchBarBase, textProperty, hintProperty, textFieldHintColorProperty, textFieldBackgroundColorProperty } from './search-bar-common';
import { isUserInteractionEnabledProperty, isEnabledProperty } from '../core/view';
import { ad } from '../../utils';
import { Color } from '../../color';
import { colorProperty, backgroundColorProperty, backgroundInternalProperty, fontInternalProperty, fontSizeProperty } from '../styling/style-properties';
export * from './search-bar-common';
var SEARCHTEXT = Symbol('searchText');
var QUERY = Symbol('query');
var QueryTextListener;
var CloseListener;
function initializeNativeClasses() {
    if (QueryTextListener) {
        return;
    }
    var CompatQueryTextListenerImpl = /** @class */ (function (_super) {
    __extends(CompatQueryTextListenerImpl, _super);
    function CompatQueryTextListenerImpl(owner) {
        var _this = _super.call(this) || this;
        _this.owner = owner;
        return global.__native(_this);
    }
    CompatQueryTextListenerImpl.prototype.onQueryTextChange = function (newText) {
        var owner = this.owner;
        textProperty.nativeValueChange(owner, newText);
        // This code is needed since sometimes OnCloseListener is not called!
        if (newText === '' && this[SEARCHTEXT] !== newText) {
            owner._emit(SearchBarBase.clearEvent);
        }
        this[SEARCHTEXT] = newText;
        this[QUERY] = undefined;
        return true;
    };
    CompatQueryTextListenerImpl.prototype.onQueryTextSubmit = function (query) {
        var owner = this.owner;
        // This code is needed since onQueryTextSubmit is called twice with same query!
        if (query !== '' && this[QUERY] !== query) {
            owner._emit(SearchBarBase.submitEvent);
        }
        this[QUERY] = query;
        return true;
    };
    var _a;
    CompatQueryTextListenerImpl = __decorate([
        Interfaces([androidx.appcompat.widget.SearchView.OnQueryTextListener]),
        __metadata("design:paramtypes", [typeof (_a = typeof SearchBar !== "undefined" && SearchBar) === "function" ? _a : Object])
    ], CompatQueryTextListenerImpl);
    return CompatQueryTextListenerImpl;
}(java.lang.Object));
    var CompatCloseListenerImpl = /** @class */ (function (_super) {
    __extends(CompatCloseListenerImpl, _super);
    function CompatCloseListenerImpl(owner) {
        var _this = _super.call(this) || this;
        _this.owner = owner;
        return global.__native(_this);
    }
    CompatCloseListenerImpl.prototype.onClose = function () {
        this.owner._emit(SearchBarBase.clearEvent);
        return true;
    };
    var _a;
    CompatCloseListenerImpl = __decorate([
        Interfaces([androidx.appcompat.widget.SearchView.OnCloseListener]),
        __metadata("design:paramtypes", [typeof (_a = typeof SearchBar !== "undefined" && SearchBar) === "function" ? _a : Object])
    ], CompatCloseListenerImpl);
    return CompatCloseListenerImpl;
}(java.lang.Object));
    QueryTextListener = CompatQueryTextListenerImpl;
    CloseListener = CompatCloseListenerImpl;
}
function enableSearchView(nativeView, value) {
    nativeView.setEnabled(value);
    if (!(nativeView instanceof android.view.ViewGroup)) {
        return;
    }
    for (var i = 0; i < nativeView.getChildCount(); i++) {
        var child = nativeView.getChildAt(i);
        enableSearchView(child, value);
    }
}
function enableUserInteractionSearchView(nativeView, value) {
    nativeView.setClickable(value);
    nativeView.setFocusable(value);
    if (!(nativeView instanceof android.view.ViewGroup)) {
        return;
    }
    for (var i = 0; i < nativeView.getChildCount(); i++) {
        var child = nativeView.getChildAt(i);
        enableUserInteractionSearchView(child, value);
    }
}
var SearchBar = /** @class */ (function (_super) {
    __extends(SearchBar, _super);
    function SearchBar() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    SearchBar.prototype.dismissSoftInput = function () {
        ad.dismissSoftInput(this.nativeViewProtected);
    };
    SearchBar.prototype.focus = function () {
        var result = _super.prototype.focus.call(this);
        if (result) {
            ad.showSoftInput(this.nativeViewProtected);
        }
        return result;
    };
    SearchBar.prototype.createNativeView = function () {
        var nativeView = new androidx.appcompat.widget.SearchView(this._context);
        nativeView.setIconified(false);
        return nativeView;
    };
    SearchBar.prototype.initNativeView = function () {
        _super.prototype.initNativeView.call(this);
        var nativeView = this.nativeViewProtected;
        initializeNativeClasses();
        var queryTextListener = new QueryTextListener(this);
        nativeView.setOnQueryTextListener(queryTextListener);
        nativeView.queryTextListener = queryTextListener;
        var closeListener = new CloseListener(this);
        nativeView.setOnCloseListener(closeListener);
        nativeView.closeListener = closeListener;
    };
    SearchBar.prototype.disposeNativeView = function () {
        var nativeView = this.nativeViewProtected;
        if (nativeView.closeListener) {
            nativeView.closeListener.owner = null;
        }
        if (nativeView.queryTextListener) {
            nativeView.queryTextListener.owner = null;
        }
        this._searchPlate = null;
        this._searchTextView = null;
        _super.prototype.disposeNativeView.call(this);
    };
    SearchBar.prototype[isEnabledProperty.setNative] = function (value) {
        enableSearchView(this.nativeViewProtected, value);
    };
    SearchBar.prototype[isUserInteractionEnabledProperty.setNative] = function (value) {
        enableUserInteractionSearchView(this.nativeViewProtected, value);
    };
    SearchBar.prototype[backgroundColorProperty.getDefault] = function () {
        // TODO: Why do we get DrawingCacheBackgroundColor but set backgroundColor?????
        var result = this.nativeViewProtected.getDrawingCacheBackgroundColor();
        return result;
    };
    SearchBar.prototype[backgroundColorProperty.setNative] = function (value) {
        var color;
        if (typeof value === 'number') {
            color = value;
        }
        else {
            color = value.android;
        }
        this.nativeViewProtected.setBackgroundColor(color);
        var searchPlate = this._getSearchPlate();
        searchPlate.setBackgroundColor(color);
    };
    SearchBar.prototype[colorProperty.getDefault] = function () {
        var textView = this._getTextView();
        return textView.getCurrentTextColor();
    };
    SearchBar.prototype[colorProperty.setNative] = function (value) {
        var color = typeof value === 'number' ? value : value.android;
        var textView = this._getTextView();
        textView.setTextColor(color);
    };
    SearchBar.prototype[fontSizeProperty.getDefault] = function () {
        return { nativeSize: this._getTextView().getTextSize() };
    };
    SearchBar.prototype[fontSizeProperty.setNative] = function (value) {
        if (typeof value === 'number') {
            this._getTextView().setTextSize(value);
        }
        else {
            this._getTextView().setTextSize(android.util.TypedValue.COMPLEX_UNIT_PX, value.nativeSize);
        }
    };
    SearchBar.prototype[fontInternalProperty.getDefault] = function () {
        return this._getTextView().getTypeface();
    };
    SearchBar.prototype[fontInternalProperty.setNative] = function (value) {
        this._getTextView().setTypeface(value instanceof Font ? value.getAndroidTypeface() : value);
    };
    SearchBar.prototype[backgroundInternalProperty.getDefault] = function () {
        return null;
    };
    SearchBar.prototype[backgroundInternalProperty.setNative] = function (value) {
        //
    };
    SearchBar.prototype[textProperty.getDefault] = function () {
        return '';
    };
    SearchBar.prototype[textProperty.setNative] = function (value) {
        var text = value === null || value === undefined ? '' : value.toString();
        this.nativeViewProtected.setQuery(text, false);
    };
    SearchBar.prototype[hintProperty.getDefault] = function () {
        return null;
    };
    SearchBar.prototype[hintProperty.setNative] = function (value) {
        if (value === null || value === undefined) {
            this.nativeViewProtected.setQueryHint(null);
        }
        else {
            this.nativeViewProtected.setQueryHint(value.toString());
        }
    };
    SearchBar.prototype[textFieldBackgroundColorProperty.getDefault] = function () {
        var textView = this._getTextView();
        return textView.getBackground();
    };
    SearchBar.prototype[textFieldBackgroundColorProperty.setNative] = function (value) {
        var textView = this._getTextView();
        if (value instanceof Color) {
            textView.setBackgroundColor(value.android);
        }
        else {
            textView.setBackground(value);
        }
    };
    SearchBar.prototype[textFieldHintColorProperty.getDefault] = function () {
        var textView = this._getTextView();
        return textView.getCurrentTextColor();
    };
    SearchBar.prototype[textFieldHintColorProperty.setNative] = function (value) {
        var textView = this._getTextView();
        var color = value instanceof Color ? value.android : value;
        textView.setHintTextColor(color);
    };
    SearchBar.prototype._getTextView = function () {
        if (!this._searchTextView) {
            var pkgName = this.nativeViewProtected.getContext().getPackageName();
            var id = this.nativeViewProtected.getContext().getResources().getIdentifier('search_src_text', 'id', pkgName);
            this._searchTextView = this.nativeViewProtected.findViewById(id);
        }
        return this._searchTextView;
    };
    SearchBar.prototype._getSearchPlate = function () {
        if (!this._searchPlate) {
            var pkgName = this.nativeViewProtected.getContext().getPackageName();
            var id = this.nativeViewProtected.getContext().getResources().getIdentifier('search_plate', 'id', pkgName);
            this._searchPlate = this.nativeViewProtected.findViewById(id);
        }
        return this._searchPlate;
    };
    return SearchBar;
}(SearchBarBase));
export { SearchBar };
//# sourceMappingURL=index.android.js.map