import { Font } from '../styling/font';
import { SearchBarBase, textProperty, hintProperty, textFieldHintColorProperty, textFieldBackgroundColorProperty } from './search-bar-common';
import { isEnabledProperty } from '../core/view';
import { Color } from '../../color';
import { colorProperty, backgroundColorProperty, backgroundInternalProperty, fontInternalProperty } from '../styling/style-properties';
import { iOSNativeHelper } from '../../utils';
export * from './search-bar-common';
var majorVersion = iOSNativeHelper.MajorVersion;
var UISearchBarDelegateImpl = /** @class */ (function (_super) {
    __extends(UISearchBarDelegateImpl, _super);
    function UISearchBarDelegateImpl() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    UISearchBarDelegateImpl.initWithOwner = function (owner) {
        var delegate = UISearchBarDelegateImpl.new();
        delegate._owner = owner;
        return delegate;
    };
    UISearchBarDelegateImpl.prototype.searchBarTextDidChange = function (searchBar, searchText) {
        var _a;
        var owner = (_a = this._owner) === null || _a === void 0 ? void 0 : _a.deref();
        if (!owner) {
            return;
        }
        textProperty.nativeValueChange(owner, searchText);
        // This code is needed since sometimes searchBarCancelButtonClicked is not called!
        if (searchText === '') {
            owner._emit(SearchBarBase.clearEvent);
        }
    };
    UISearchBarDelegateImpl.prototype.searchBarCancelButtonClicked = function (searchBar) {
        var _a;
        searchBar.resignFirstResponder();
        var owner = (_a = this._owner) === null || _a === void 0 ? void 0 : _a.deref();
        if (!owner) {
            return;
        }
        owner._emit(SearchBarBase.clearEvent);
    };
    UISearchBarDelegateImpl.prototype.searchBarSearchButtonClicked = function (searchBar) {
        var _a;
        searchBar.resignFirstResponder();
        var owner = (_a = this._owner) === null || _a === void 0 ? void 0 : _a.deref();
        if (!owner) {
            return;
        }
        owner._emit(SearchBarBase.submitEvent);
    };
    UISearchBarDelegateImpl.ObjCProtocols = [UISearchBarDelegate];
    return UISearchBarDelegateImpl;
}(NSObject));
var UISearchBarImpl = /** @class */ (function (_super) {
    __extends(UISearchBarImpl, _super);
    function UISearchBarImpl() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    UISearchBarImpl.prototype.sizeThatFits = function (size) {
        // iOS11 SDK does not support passing sizeThatFits(...) non-finite width value;
        // iOS layout system will take care to size the element properly when passed 0
        if (majorVersion >= 11 && size.width === Number.POSITIVE_INFINITY) {
            size.width = 0;
        }
        return _super.prototype.sizeThatFits.call(this, size);
    };
    return UISearchBarImpl;
}(UISearchBar));
var SearchBar = /** @class */ (function (_super) {
    __extends(SearchBar, _super);
    function SearchBar() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    SearchBar.prototype.createNativeView = function () {
        return UISearchBarImpl.new();
    };
    SearchBar.prototype.initNativeView = function () {
        _super.prototype.initNativeView.call(this);
        this._delegate = UISearchBarDelegateImpl.initWithOwner(new WeakRef(this));
        this.nativeViewProtected.delegate = this._delegate;
    };
    SearchBar.prototype.disposeNativeView = function () {
        this._delegate = null;
        _super.prototype.disposeNativeView.call(this);
    };
    SearchBar.prototype.dismissSoftInput = function () {
        this.ios.resignFirstResponder();
    };
    Object.defineProperty(SearchBar.prototype, "ios", {
        // @ts-ignore
        get: function () {
            return this.nativeViewProtected;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(SearchBar.prototype, "_textField", {
        get: function () {
            if (!this.__textField) {
                this.__textField = this.ios.valueForKey('searchField');
            }
            return this.__textField;
        },
        enumerable: false,
        configurable: true
    });
    SearchBar.prototype[isEnabledProperty.setNative] = function (value) {
        var nativeView = this.nativeViewProtected;
        if (nativeView instanceof UIControl) {
            nativeView.enabled = value;
        }
        var textField = this._textField;
        if (textField) {
            textField.enabled = value;
        }
    };
    SearchBar.prototype[backgroundColorProperty.getDefault] = function () {
        return this.ios.barTintColor;
    };
    SearchBar.prototype[backgroundColorProperty.setNative] = function (value) {
        var color = value instanceof Color ? value.ios : value;
        this.ios.barTintColor = color;
    };
    SearchBar.prototype[colorProperty.getDefault] = function () {
        var sf = this._textField;
        if (sf) {
            return sf.textColor;
        }
        return null;
    };
    SearchBar.prototype[colorProperty.setNative] = function (value) {
        var sf = this._textField;
        var color = value instanceof Color ? value.ios : value;
        if (sf) {
            sf.textColor = color;
            sf.tintColor = color;
        }
    };
    SearchBar.prototype[fontInternalProperty.getDefault] = function () {
        var sf = this._textField;
        return sf ? sf.font : null;
    };
    SearchBar.prototype[fontInternalProperty.setNative] = function (value) {
        var sf = this._textField;
        if (sf) {
            sf.font = value instanceof Font ? value.getUIFont(sf.font) : value;
        }
    };
    SearchBar.prototype[backgroundInternalProperty.getDefault] = function () {
        return null;
    };
    SearchBar.prototype[backgroundInternalProperty.setNative] = function (value) {
        //
    };
    SearchBar.prototype[textProperty.getDefault] = function () {
        return '';
    };
    SearchBar.prototype[textProperty.setNative] = function (value) {
        var text = value === null || value === undefined ? '' : value.toString();
        this.ios.text = text;
    };
    SearchBar.prototype[hintProperty.getDefault] = function () {
        return '';
    };
    SearchBar.prototype[hintProperty.setNative] = function (value) {
        this._updateAttributedPlaceholder();
    };
    SearchBar.prototype[textFieldBackgroundColorProperty.getDefault] = function () {
        var textField = this._textField;
        if (textField) {
            return textField.backgroundColor;
        }
        return null;
    };
    SearchBar.prototype[textFieldBackgroundColorProperty.setNative] = function (value) {
        var color = value instanceof Color ? value.ios : value;
        var textField = this._textField;
        if (textField) {
            textField.backgroundColor = color;
        }
    };
    SearchBar.prototype[textFieldHintColorProperty.getDefault] = function () {
        return null;
    };
    SearchBar.prototype[textFieldHintColorProperty.setNative] = function (value) {
        this._updateAttributedPlaceholder();
    };
    // Very similar to text-field.ios.ts implementation. Maybe unify APIs and base classes?
    SearchBar.prototype._updateAttributedPlaceholder = function () {
        var stringValue = this.hint;
        if (stringValue === null || stringValue === void 0) {
            stringValue = '';
        }
        else {
            stringValue = stringValue + '';
        }
        if (stringValue === '') {
            // we do not use empty string since initWithStringAttributes does not return proper value and
            // nativeView.attributedPlaceholder will be null
            stringValue = ' ';
        }
        var attributes = {};
        if (this.textFieldHintColor) {
            attributes[NSForegroundColorAttributeName] = this.textFieldHintColor.ios;
        }
        var attributedPlaceholder = NSAttributedString.alloc().initWithStringAttributes(stringValue, attributes);
        this._textField.attributedPlaceholder = attributedPlaceholder;
    };
    return SearchBar;
}(SearchBarBase));
export { SearchBar };
//# sourceMappingURL=index.ios.js.map