import { Font } from '../styling/font';
import { SegmentedBarItemBase, SegmentedBarBase, selectedIndexProperty, itemsProperty, selectedBackgroundColorProperty } from './segmented-bar-common';
import { AndroidHelper, isEnabledProperty } from '../core/view';
import { colorProperty, fontInternalProperty, fontSizeProperty } from '../styling/style-properties';
import { Color } from '../../color';
import { layout } from '../../utils';
import { SDK_VERSION } from '../../utils/constants';
import { Trace } from '../../trace';
export * from './segmented-bar-common';
var R_ID_TABS = 0x01020013;
var R_ID_TABCONTENT = 0x01020011;
var R_ATTR_STATE_SELECTED = 0x010100a1;
var TITLE_TEXT_VIEW_ID = 16908310; // http://developer.android.com/reference/android/R.id.html#title
var selectedIndicatorThickness;
var TabHost;
var TabChangeListener;
var TabContentFactory;
// TODO: All TabHost public methods become deprecated in API 30.
function initializeNativeClasses() {
    if (TabChangeListener) {
        return;
    }
    // Indicator thickness for material - 2dip. For pre-material - 5dip.
    selectedIndicatorThickness = layout.toDevicePixels(SDK_VERSION >= 21 ? 2 : 5);
    var TabChangeListenerImpl = /** @class */ (function (_super) {
    __extends(TabChangeListenerImpl, _super);
    function TabChangeListenerImpl(owner) {
        var _this = _super.call(this) || this;
        _this.owner = owner;
        return global.__native(_this);
    }
    TabChangeListenerImpl.prototype.onTabChanged = function (id) {
        var owner = this.owner;
        if (owner) {
            setTimeout(function () {
                owner.setTabColor(id);
            });
            if (owner.shouldChangeSelectedIndex()) {
                owner.selectedIndex = parseInt(id);
            }
        }
    };
    var _a;
    TabChangeListenerImpl = __decorate([
        Interfaces([android.widget.TabHost.OnTabChangeListener]),
        __metadata("design:paramtypes", [typeof (_a = typeof SegmentedBar !== "undefined" && SegmentedBar) === "function" ? _a : Object])
    ], TabChangeListenerImpl);
    return TabChangeListenerImpl;
}(java.lang.Object));
    var TabContentFactoryImpl = /** @class */ (function (_super) {
    __extends(TabContentFactoryImpl, _super);
    function TabContentFactoryImpl(owner) {
        var _this = _super.call(this) || this;
        _this.owner = owner;
        return global.__native(_this);
    }
    TabContentFactoryImpl.prototype.createTabContent = function (tag) {
        var tv = new android.widget.TextView(this.owner._context);
        // This is collapsed by default and made visible
        // by android when TabItem becomes visible/selected.
        // TODO: Try commenting visibility change.
        tv.setVisibility(android.view.View.GONE);
        tv.setMaxLines(1);
        tv.setEllipsize(android.text.TextUtils.TruncateAt.END);
        return tv;
    };
    var _a;
    TabContentFactoryImpl = __decorate([
        Interfaces([android.widget.TabHost.TabContentFactory]),
        __metadata("design:paramtypes", [typeof (_a = typeof SegmentedBar !== "undefined" && SegmentedBar) === "function" ? _a : Object])
    ], TabContentFactoryImpl);
    return TabContentFactoryImpl;
}(java.lang.Object));
    var TabHostImpl = /** @class */ (function (_super) {
    __extends(TabHostImpl, _super);
    function TabHostImpl(context, attrs) {
        var _this = _super.call(this, context, attrs) || this;
        return global.__native(_this);
    }
    TabHostImpl.prototype.onAttachedToWindow = function () {
        // overriden to remove the code that will steal the focus from edit fields.
    };
    return TabHostImpl;
}(android.widget.TabHost));
    TabHost = TabHostImpl;
    TabChangeListener = TabChangeListenerImpl;
    TabContentFactory = TabContentFactoryImpl;
}
var SegmentedBarItem = /** @class */ (function (_super) {
    __extends(SegmentedBarItem, _super);
    function SegmentedBarItem() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    SegmentedBarItem.prototype.setupNativeView = function (tabIndex) {
        // TabHost.TabSpec.setIndicator DOES NOT WORK once the title has been set.
        // http://stackoverflow.com/questions/2935781/modify-tab-indicator-dynamically-in-android
        var titleTextView = this.parent.nativeViewProtected.getTabWidget().getChildAt(tabIndex).findViewById(TITLE_TEXT_VIEW_ID);
        this.setNativeView(titleTextView);
        if (titleTextView) {
            if (this.titleDirty) {
                this._update();
            }
        }
    };
    SegmentedBarItem.prototype._update = function () {
        var tv = this.nativeViewProtected;
        if (tv) {
            var title = this.title;
            title = title === null || title === undefined ? '' : title;
            tv.setText(title);
            this.titleDirty = false;
        }
        else {
            this.titleDirty = true;
        }
    };
    SegmentedBarItem.prototype[colorProperty.getDefault] = function () {
        return this.nativeViewProtected.getCurrentTextColor();
    };
    SegmentedBarItem.prototype[colorProperty.setNative] = function (value) {
        var color = value instanceof Color ? value.android : value;
        this.nativeViewProtected.setTextColor(color);
    };
    SegmentedBarItem.prototype[fontSizeProperty.getDefault] = function () {
        return { nativeSize: this.nativeViewProtected.getTextSize() };
    };
    SegmentedBarItem.prototype[fontSizeProperty.setNative] = function (value) {
        if (typeof value === 'number') {
            this.nativeViewProtected.setTextSize(value);
        }
        else {
            this.nativeViewProtected.setTextSize(android.util.TypedValue.COMPLEX_UNIT_PX, value.nativeSize);
        }
    };
    SegmentedBarItem.prototype[fontInternalProperty.getDefault] = function () {
        return this.nativeViewProtected.getTypeface();
    };
    SegmentedBarItem.prototype[fontInternalProperty.setNative] = function (value) {
        this.nativeViewProtected.setTypeface(value instanceof Font ? value.getAndroidTypeface() : value);
    };
    SegmentedBarItem.prototype[selectedBackgroundColorProperty.getDefault] = function () {
        var viewGroup = this.nativeViewProtected.getParent();
        return viewGroup.getBackground();
    };
    SegmentedBarItem.prototype[selectedBackgroundColorProperty.setNative] = function (value) {
        var nativeView = this.nativeViewProtected;
        var viewGroup = nativeView.getParent();
        if (value instanceof Color) {
            var color = value.android;
            var backgroundDrawable = viewGroup.getBackground();
            if (SDK_VERSION > 21 && backgroundDrawable) {
                var newDrawable = AndroidHelper.getCopyOrDrawable(backgroundDrawable, nativeView.getResources());
                AndroidHelper.setDrawableColor(color, newDrawable);
                viewGroup.setBackground(newDrawable);
            }
            else {
                var stateDrawable = new android.graphics.drawable.StateListDrawable();
                var colorDrawable = new org.nativescript.widgets.SegmentedBarColorDrawable(color, selectedIndicatorThickness);
                var arr = Array.create('int', 1);
                arr[0] = R_ATTR_STATE_SELECTED;
                stateDrawable.addState(arr, colorDrawable);
                stateDrawable.setBounds(0, 15, viewGroup.getRight(), viewGroup.getBottom());
                viewGroup.setBackground(stateDrawable);
            }
        }
        else {
            var backgroundDrawable = AndroidHelper.getCopyOrDrawable(value, nativeView.getResources());
            viewGroup.setBackground(backgroundDrawable);
        }
    };
    return SegmentedBarItem;
}(SegmentedBarItemBase));
export { SegmentedBarItem };
var SegmentedBar = /** @class */ (function (_super) {
    __extends(SegmentedBar, _super);
    function SegmentedBar() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    SegmentedBar.prototype.shouldChangeSelectedIndex = function () {
        return !this._addingTab;
    };
    SegmentedBar.prototype.createNativeView = function () {
        initializeNativeClasses();
        var context = this._context;
        var nativeView = new TabHost(context, null);
        var tabHostLayout = new android.widget.LinearLayout(context);
        tabHostLayout.setOrientation(android.widget.LinearLayout.VERTICAL);
        var tabWidget = new android.widget.TabWidget(context);
        tabWidget.setId(R_ID_TABS);
        tabHostLayout.addView(tabWidget);
        var frame = new android.widget.FrameLayout(context);
        frame.setId(R_ID_TABCONTENT);
        frame.setVisibility(android.view.View.GONE);
        tabHostLayout.addView(frame);
        nativeView.addView(tabHostLayout);
        return nativeView;
    };
    SegmentedBar.prototype.initNativeView = function () {
        _super.prototype.initNativeView.call(this);
        var nativeView = this.nativeViewProtected;
        var listener = new TabChangeListener(this);
        nativeView.setOnTabChangedListener(listener);
        nativeView.listener = listener;
        nativeView.setup();
        this._tabContentFactory = this._tabContentFactory || new TabContentFactory(this);
    };
    SegmentedBar.prototype.disposeNativeView = function () {
        var nativeView = this.nativeViewProtected;
        if (nativeView === null || nativeView === void 0 ? void 0 : nativeView.listener) {
            nativeView.listener.owner = null;
        }
        _super.prototype.disposeNativeView.call(this);
    };
    SegmentedBar.prototype.onLoaded = function () {
        _super.prototype.onLoaded.call(this);
        // Can only be applied after view is loaded
        var tabWidget = this.nativeViewProtected.getTabWidget();
        if (tabWidget) {
            tabWidget.setEnabled(tabWidget.isEnabled());
        }
    };
    SegmentedBar.prototype.insertTab = function (tabItem, index) {
        var tabHost = this.nativeViewProtected;
        var tab = tabHost.newTabSpec(index + '');
        tab.setIndicator(tabItem.title + '');
        tab.setContent(this._tabContentFactory);
        this._addingTab = true;
        tabHost.addTab(tab);
        tabItem.setupNativeView(index);
        this._addingTab = false;
    };
    SegmentedBar.prototype[selectedIndexProperty.getDefault] = function () {
        return -1;
    };
    SegmentedBar.prototype[selectedIndexProperty.setNative] = function (value) {
        this.nativeViewProtected.setCurrentTab(value);
    };
    SegmentedBar.prototype[itemsProperty.getDefault] = function () {
        return null;
    };
    SegmentedBar.prototype[itemsProperty.setNative] = function (value) {
        var _this = this;
        this.nativeViewProtected.clearAllTabs();
        var newItems = value;
        if (newItems) {
            newItems.forEach(function (item, i, arr) { return _this.insertTab(item, i); });
        }
        selectedIndexProperty.coerce(this);
    };
    SegmentedBar.prototype[isEnabledProperty.setNative] = function (value) {
        var tabWidget = this.nativeViewProtected.getTabWidget();
        if (tabWidget) {
            tabWidget.setEnabled(value);
        }
    };
    SegmentedBar.prototype.setTabColor = function (index) {
        var _a, _b, _c, _d, _e, _f;
        try {
            var tabWidget = (_a = this.nativeViewProtected) === null || _a === void 0 ? void 0 : _a.getTabWidget();
            if (tabWidget) {
                var unselectedTextColor = this.getColorForAndroid((_b = this.color) !== null && _b !== void 0 ? _b : '#6e6e6e');
                var selectedTextColor = this.getColorForAndroid((_c = this === null || this === void 0 ? void 0 : this.selectedTextColor) !== null && _c !== void 0 ? _c : '#000000');
                var unselectedBackgroundColor = this.getColorForAndroid((_d = this === null || this === void 0 ? void 0 : this.backgroundColor) !== null && _d !== void 0 ? _d : '#dbdbdb');
                var selectedBackgroundColor = this.getColorForAndroid((_f = (_e = this === null || this === void 0 ? void 0 : this.selectedBackgroundColor) !== null && _e !== void 0 ? _e : this === null || this === void 0 ? void 0 : this.backgroundColor) !== null && _f !== void 0 ? _f : 'blue');
                if (tabWidget) {
                    for (var i = 0; i < tabWidget.getTabCount(); i++) {
                        var view = tabWidget.getChildTabViewAt(i);
                        var item = this.items[i];
                        var textView = item === null || item === void 0 ? void 0 : item.nativeViewProtected;
                        view.setBackgroundColor(unselectedBackgroundColor);
                        if (textView) {
                            textView.setTextColor(unselectedTextColor);
                        }
                        if (index == i) {
                            view.setBackgroundColor(selectedBackgroundColor);
                            if (textView) {
                                textView.setTextColor(selectedTextColor);
                            }
                            continue;
                        }
                    }
                }
            }
        }
        catch (e) {
            Trace.error(e);
        }
    };
    SegmentedBar.prototype.getColorForAndroid = function (color) {
        if (typeof color === 'string') {
            return new Color(color).android;
        }
        else if (color instanceof Color) {
            return color.android;
        }
    };
    return SegmentedBar;
}(SegmentedBarBase));
export { SegmentedBar };
//# sourceMappingURL=index.android.js.map