import { SegmentedBarItemBase, SegmentedBarBase, selectedIndexProperty, itemsProperty, selectedBackgroundColorProperty } from './segmented-bar-common';
import { colorProperty, fontInternalProperty } from '../styling/style-properties';
import { Color } from '../../color';
import { Trace } from '../../trace';
import { SDK_VERSION } from '../../utils';
export * from './segmented-bar-common';
var SegmentedBarItem = /** @class */ (function (_super) {
    __extends(SegmentedBarItem, _super);
    function SegmentedBarItem() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    SegmentedBarItem.prototype._update = function () {
        var parent = this.parent;
        if (parent === null || parent === void 0 ? void 0 : parent.ios) {
            var tabIndex = parent.items.indexOf(this);
            var title = this.title;
            title = title === null || title === undefined ? '' : title;
            parent.ios.setTitleForSegmentAtIndex(title, tabIndex);
        }
    };
    return SegmentedBarItem;
}(SegmentedBarItemBase));
export { SegmentedBarItem };
var SegmentedBar = /** @class */ (function (_super) {
    __extends(SegmentedBar, _super);
    function SegmentedBar() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    SegmentedBar.prototype.createNativeView = function () {
        return UISegmentedControl.new();
    };
    SegmentedBar.prototype.initNativeView = function () {
        _super.prototype.initNativeView.call(this);
        this._selectionHandler = SelectionHandlerImpl.initWithOwner(new WeakRef(this));
        this.nativeViewProtected.addTargetActionForControlEvents(this._selectionHandler, 'selected', 4096 /* UIControlEvents.ValueChanged */);
    };
    SegmentedBar.prototype.disposeNativeView = function () {
        this._selectionHandler = null;
        _super.prototype.disposeNativeView.call(this);
    };
    Object.defineProperty(SegmentedBar.prototype, "ios", {
        // @ts-ignore
        get: function () {
            return this.nativeViewProtected;
        },
        enumerable: false,
        configurable: true
    });
    SegmentedBar.prototype[selectedIndexProperty.getDefault] = function () {
        return -1;
    };
    SegmentedBar.prototype[selectedIndexProperty.setNative] = function (value) {
        this.ios.selectedSegmentIndex = value;
    };
    SegmentedBar.prototype[itemsProperty.getDefault] = function () {
        return null;
    };
    SegmentedBar.prototype[itemsProperty.setNative] = function (value) {
        var segmentedControl = this.ios;
        segmentedControl.removeAllSegments();
        var newItems = value;
        if (newItems && newItems.length) {
            newItems.forEach(function (item, index, arr) {
                var title = item.title;
                title = title === null || title === undefined ? '' : title;
                segmentedControl.insertSegmentWithTitleAtIndexAnimated(title, index, false);
            });
        }
        selectedIndexProperty.coerce(this);
    };
    SegmentedBar.prototype[selectedBackgroundColorProperty.getDefault] = function () {
        return SDK_VERSION < 13 ? this.ios.tintColor : this.ios.selectedSegmentTintColor;
    };
    SegmentedBar.prototype[selectedBackgroundColorProperty.setNative] = function (value) {
        var color = value instanceof Color ? value.ios : value;
        if (SDK_VERSION < 13) {
            this.ios.tintColor = color;
        }
        else {
            this.ios.selectedSegmentTintColor = color;
        }
    };
    SegmentedBar.prototype[colorProperty.getDefault] = function () {
        return null;
    };
    SegmentedBar.prototype[colorProperty.setNative] = function (value) {
        var color = value instanceof Color ? value.ios : value;
        var bar = this.ios;
        var currentAttrs = bar.titleTextAttributesForState(0 /* UIControlState.Normal */);
        var attrs = currentAttrs ? currentAttrs.mutableCopy() : NSMutableDictionary.new();
        attrs.setValueForKey(color, NSForegroundColorAttributeName);
        bar.setTitleTextAttributesForState(attrs, 0 /* UIControlState.Normal */);
        // Set the selected text color
        this.setSelectedTextColor(bar);
    };
    SegmentedBar.prototype[fontInternalProperty.getDefault] = function () {
        return null;
    };
    SegmentedBar.prototype[fontInternalProperty.setNative] = function (value) {
        var font = value ? value.getUIFont(UIFont.systemFontOfSize(UIFont.labelFontSize)) : null;
        var bar = this.ios;
        var currentAttrs = bar.titleTextAttributesForState(0 /* UIControlState.Normal */);
        var attrs = currentAttrs ? currentAttrs.mutableCopy() : NSMutableDictionary.new();
        attrs.setValueForKey(font, NSFontAttributeName);
        bar.setTitleTextAttributesForState(attrs, 0 /* UIControlState.Normal */);
    };
    SegmentedBar.prototype.setSelectedTextColor = function (bar) {
        var _a, _b;
        try {
            var selectedTextColor = this.getColorForIOS((_b = (_a = this === null || this === void 0 ? void 0 : this.selectedTextColor) !== null && _a !== void 0 ? _a : this === null || this === void 0 ? void 0 : this.color) !== null && _b !== void 0 ? _b : '#000000');
            if (!selectedTextColor) {
                Trace.write("unable te set selectedTextColor", Trace.categories.Error);
            }
            var selectedText = bar.titleTextAttributesForState(4 /* UIControlState.Selected */);
            var attrsSelected = selectedText ? selectedText.mutableCopy() : NSMutableDictionary.new();
            attrsSelected.setValueForKey(selectedTextColor, NSForegroundColorAttributeName);
            bar.setTitleTextAttributesForState(attrsSelected, 4 /* UIControlState.Selected */);
        }
        catch (e) {
            console.error("SegmentedBar:", e);
        }
    };
    SegmentedBar.prototype.getColorForIOS = function (color) {
        if (typeof color === 'string') {
            return new Color(color).ios;
        }
        else if (color instanceof Color) {
            return color.ios;
        }
    };
    return SegmentedBar;
}(SegmentedBarBase));
export { SegmentedBar };
var SelectionHandlerImpl = /** @class */ (function (_super) {
    __extends(SelectionHandlerImpl, _super);
    function SelectionHandlerImpl() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    SelectionHandlerImpl.initWithOwner = function (owner) {
        var handler = SelectionHandlerImpl.new();
        handler._owner = owner;
        return handler;
    };
    SelectionHandlerImpl.prototype.selected = function (sender) {
        var _a;
        var owner = (_a = this._owner) === null || _a === void 0 ? void 0 : _a.deref();
        if (owner) {
            owner.selectedIndex = sender.selectedSegmentIndex;
            owner.setSelectedTextColor(sender);
        }
    };
    SelectionHandlerImpl.ObjCExposedMethods = {
        selected: { returns: interop.types.void, params: [UISegmentedControl] },
    };
    return SelectionHandlerImpl;
}(NSObject));
//# sourceMappingURL=index.ios.js.map