import { View, CSSType } from '../core/view';
import { ViewBase } from '../core/view-base';
import { Property, CoercibleProperty, InheritedCssProperty } from '../core/properties';
import { Color } from '../../color';
import { Style } from '../styling/style';
var SegmentedBarItemBase = /** @class */ (function (_super) {
    __extends(SegmentedBarItemBase, _super);
    function SegmentedBarItemBase() {
        var _this = _super !== null && _super.apply(this, arguments) || this;
        _this._title = '';
        return _this;
    }
    Object.defineProperty(SegmentedBarItemBase.prototype, "title", {
        get: function () {
            return this._title;
        },
        set: function (value) {
            var strValue = value !== null && value !== undefined ? value.toString() : '';
            if (this._title !== strValue) {
                this._title = strValue;
                this._update();
            }
        },
        enumerable: false,
        configurable: true
    });
    SegmentedBarItemBase = __decorate([
        CSSType('SegmentedBarItem')
    ], SegmentedBarItemBase);
    return SegmentedBarItemBase;
}(ViewBase));
export { SegmentedBarItemBase };
var SegmentedBarBase = /** @class */ (function (_super) {
    __extends(SegmentedBarBase, _super);
    function SegmentedBarBase() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    Object.defineProperty(SegmentedBarBase.prototype, "selectedBackgroundColor", {
        get: function () {
            return this.style.selectedBackgroundColor;
        },
        set: function (value) {
            this.style.selectedBackgroundColor = value;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(SegmentedBarBase.prototype, "selectedTextColor", {
        get: function () {
            return this.style.selectedTabTextColor;
        },
        set: function (value) {
            this.style.selectedTabTextColor = value;
        },
        enumerable: false,
        configurable: true
    });
    SegmentedBarBase.prototype._addArrayFromBuilder = function (name, value) {
        if (name === 'items') {
            this.items = value;
        }
    };
    SegmentedBarBase.prototype._addChildFromBuilder = function (name, value) {
        if (name === 'SegmentedBarItem') {
            var item = value;
            var items = this.items;
            if (!items) {
                items = new Array();
                items.push(item);
                this.items = items;
            }
            else {
                items.push(item);
                this._addView(item);
            }
            if (this.nativeViewProtected) {
                this[itemsProperty.setNative](items);
            }
        }
    };
    SegmentedBarBase.prototype.onItemsChanged = function (oldItems, newItems) {
        if (oldItems) {
            for (var i = 0, count = oldItems.length; i < count; i++) {
                this._removeView(oldItems[i]);
            }
        }
        if (newItems) {
            for (var i = 0, count = newItems.length; i < count; i++) {
                this._addView(newItems[i]);
            }
        }
    };
    // TODO: Make _addView to keep its children so this method is not needed!
    SegmentedBarBase.prototype.eachChild = function (callback) {
        var items = this.items;
        if (items) {
            items.forEach(function (item, i) {
                callback(item);
            });
        }
    };
    SegmentedBarBase.selectedIndexChangedEvent = 'selectedIndexChanged';
    SegmentedBarBase = __decorate([
        CSSType('SegmentedBar')
    ], SegmentedBarBase);
    return SegmentedBarBase;
}(View));
export { SegmentedBarBase };
SegmentedBarBase.prototype.recycleNativeView = 'auto';
/**
 * Gets or sets the selected index dependency property of the SegmentedBar.
 */
export var selectedIndexProperty = new CoercibleProperty({
    name: 'selectedIndex',
    defaultValue: -1,
    valueChanged: function (target, oldValue, newValue) {
        target.notify({
            eventName: SegmentedBarBase.selectedIndexChangedEvent,
            object: target,
            oldIndex: oldValue,
            newIndex: newValue,
        });
    },
    coerceValue: function (target, value) {
        var items = target.items;
        if (items) {
            var max = items.length - 1;
            if (value < 0) {
                value = 0;
            }
            if (value > max) {
                value = max;
            }
        }
        else {
            value = -1;
        }
        return value;
    },
    valueConverter: function (v) { return parseInt(v); },
});
selectedIndexProperty.register(SegmentedBarBase);
export var itemsProperty = new Property({
    name: 'items',
    valueChanged: function (target, oldValue, newValue) {
        target.onItemsChanged(oldValue, newValue);
    },
});
itemsProperty.register(SegmentedBarBase);
export var selectedBackgroundColorProperty = new InheritedCssProperty({
    name: 'selectedBackgroundColor',
    cssName: 'selected-background-color',
    equalityComparer: Color.equals,
    defaultValue: new Color('blue'),
    valueConverter: function (v) { return new Color(v); },
});
selectedBackgroundColorProperty.register(Style);
export var selectedTextColorProperty = new InheritedCssProperty({
    name: 'selectedTextColor',
    cssName: 'selected-text-color',
    equalityComparer: Color.equals,
    defaultValue: new Color('black'),
    valueConverter: function (v) { return new Color(v); },
});
selectedTextColorProperty.register(Style);
//# sourceMappingURL=segmented-bar-common.js.map