import { SliderBase, valueProperty, minValueProperty, maxValueProperty } from './slider-common';
import { colorProperty, backgroundColorProperty, backgroundInternalProperty } from '../styling/style-properties';
import { Color } from '../../color';
import { AndroidHelper } from '../core/view';
export * from './slider-common';
var SeekBar;
var SeekBarChangeListener;
function initializeListenerClass() {
    if (!SeekBarChangeListener) {
        var SeekBarChangeListenerImpl = /** @class */ (function (_super) {
    __extends(SeekBarChangeListenerImpl, _super);
    function SeekBarChangeListenerImpl() {
        var _this = _super.call(this) || this;
        return global.__native(_this);
    }
    SeekBarChangeListenerImpl.prototype.onProgressChanged = function (seekBar, progress, fromUser) {
        var owner = seekBar.owner;
        if (owner && !owner._supressNativeValue) {
            var newValue = progress + owner.minValue;
            valueProperty.nativeValueChange(owner, newValue);
        }
    };
    SeekBarChangeListenerImpl.prototype.onStartTrackingTouch = function (seekBar) {
        //
    };
    SeekBarChangeListenerImpl.prototype.onStopTrackingTouch = function (seekBar) {
        //
    };
    SeekBarChangeListenerImpl = __decorate([
        Interfaces([android.widget.SeekBar.OnSeekBarChangeListener]),
        __metadata("design:paramtypes", [])
    ], SeekBarChangeListenerImpl);
    return SeekBarChangeListenerImpl;
}(java.lang.Object));
        SeekBarChangeListener = new SeekBarChangeListenerImpl();
    }
}
function getListener() {
    return SeekBarChangeListener;
}
var Slider = /** @class */ (function (_super) {
    __extends(Slider, _super);
    function Slider() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    Slider.prototype.createNativeView = function () {
        if (!SeekBar) {
            SeekBar = android.widget.SeekBar;
        }
        return new SeekBar(this._context);
    };
    Slider.prototype.initNativeView = function () {
        _super.prototype.initNativeView.call(this);
        var nativeView = this.nativeViewProtected;
        nativeView.owner = this;
        initializeListenerClass();
        var listener = getListener();
        nativeView.setOnSeekBarChangeListener(listener);
    };
    Slider.prototype.disposeNativeView = function () {
        this.nativeViewProtected.owner = null;
        _super.prototype.disposeNativeView.call(this);
    };
    Slider.prototype.resetNativeView = function () {
        _super.prototype.resetNativeView.call(this);
        var nativeView = this.nativeViewProtected;
        nativeView.setMax(100);
        nativeView.setProgress(0);
        nativeView.setKeyProgressIncrement(1);
    };
    /**
     * There is no minValue in Android. We simulate this by subtracting the minValue from the native value and maxValue.
     * We need this method to call native setMax and setProgress methods when minValue property is changed,
     * without handling the native value changed callback.
     */
    Slider.prototype.setNativeValuesSilently = function () {
        this._supressNativeValue = true;
        var nativeView = this.nativeViewProtected;
        try {
            nativeView.setMax(this.maxValue - this.minValue);
            nativeView.setProgress(this.value - this.minValue);
        }
        finally {
            this._supressNativeValue = false;
        }
    };
    Slider.prototype[valueProperty.setNative] = function (value) {
        this.setNativeValuesSilently();
    };
    Slider.prototype[minValueProperty.setNative] = function (value) {
        this.setNativeValuesSilently();
    };
    Slider.prototype[maxValueProperty.getDefault] = function () {
        return 100;
    };
    Slider.prototype[maxValueProperty.setNative] = function (value) {
        this.setNativeValuesSilently();
    };
    Slider.prototype[colorProperty.getDefault] = function () {
        return -1;
    };
    Slider.prototype[colorProperty.setNative] = function (value) {
        var drawable = this.nativeViewProtected.getThumb();
        if (value instanceof Color) {
            AndroidHelper.setDrawableColor(value.android, drawable);
        }
        else {
            AndroidHelper.clearDrawableColor(drawable);
        }
    };
    Slider.prototype[backgroundColorProperty.getDefault] = function () {
        return -1;
    };
    Slider.prototype[backgroundColorProperty.setNative] = function (value) {
        var drawable = this.nativeViewProtected.getProgressDrawable();
        if (value instanceof Color) {
            AndroidHelper.setDrawableColor(value.android, drawable);
        }
        else {
            AndroidHelper.clearDrawableColor(drawable);
        }
    };
    Slider.prototype[backgroundInternalProperty.getDefault] = function () {
        return null;
    };
    Slider.prototype[backgroundInternalProperty.setNative] = function (value) {
        //
    };
    return Slider;
}(SliderBase));
export { Slider };
//# sourceMappingURL=index.android.js.map