import { SliderBase, valueProperty, minValueProperty, maxValueProperty } from './slider-common';
import { colorProperty, backgroundColorProperty, backgroundInternalProperty } from '../styling/style-properties';
import { Color } from '../../color';
export * from './slider-common';
var TNSSlider = /** @class */ (function (_super) {
    __extends(TNSSlider, _super);
    function TNSSlider() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    TNSSlider.initWithOwner = function (owner) {
        var slider = TNSSlider.new();
        slider.owner = owner;
        return slider;
    };
    TNSSlider.prototype.accessibilityIncrement = function () {
        var _a;
        var owner = (_a = this.owner) === null || _a === void 0 ? void 0 : _a.deref();
        if (!owner) {
            this.value += 10;
        }
        else {
            this.value = owner._handlerAccessibilityIncrementEvent();
        }
        this.sendActionsForControlEvents(UIControlEvents.ValueChanged);
    };
    TNSSlider.prototype.accessibilityDecrement = function () {
        var _a;
        var owner = (_a = this.owner) === null || _a === void 0 ? void 0 : _a.deref();
        if (!owner) {
            this.value += 10;
        }
        else {
            this.value = owner._handlerAccessibilityDecrementEvent();
        }
        this.sendActionsForControlEvents(UIControlEvents.ValueChanged);
    };
    return TNSSlider;
}(UISlider));
var SliderChangeHandlerImpl = /** @class */ (function (_super) {
    __extends(SliderChangeHandlerImpl, _super);
    function SliderChangeHandlerImpl() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    SliderChangeHandlerImpl.initWithOwner = function (owner) {
        var handler = SliderChangeHandlerImpl.new();
        handler._owner = owner;
        return handler;
    };
    SliderChangeHandlerImpl.prototype.sliderValueChanged = function (sender) {
        var _a;
        var owner = (_a = this._owner) === null || _a === void 0 ? void 0 : _a.deref();
        if (owner) {
            valueProperty.nativeValueChange(owner, sender.value);
        }
    };
    SliderChangeHandlerImpl.ObjCExposedMethods = {
        sliderValueChanged: { returns: interop.types.void, params: [UISlider] },
    };
    return SliderChangeHandlerImpl;
}(NSObject));
var Slider = /** @class */ (function (_super) {
    __extends(Slider, _super);
    function Slider() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    Slider.prototype.createNativeView = function () {
        return TNSSlider.initWithOwner(new WeakRef(this));
    };
    Slider.prototype.initNativeView = function () {
        _super.prototype.initNativeView.call(this);
        var nativeView = this.nativeViewProtected;
        // default values
        nativeView.minimumValue = 0;
        nativeView.maximumValue = this.maxValue;
        this._changeHandler = SliderChangeHandlerImpl.initWithOwner(new WeakRef(this));
        nativeView.addTargetActionForControlEvents(this._changeHandler, 'sliderValueChanged', 4096 /* UIControlEvents.ValueChanged */);
    };
    Slider.prototype.disposeNativeView = function () {
        this._changeHandler = null;
        _super.prototype.disposeNativeView.call(this);
    };
    Object.defineProperty(Slider.prototype, "ios", {
        // @ts-ignore
        get: function () {
            return this.nativeViewProtected;
        },
        enumerable: false,
        configurable: true
    });
    Slider.prototype[valueProperty.getDefault] = function () {
        return 0;
    };
    Slider.prototype[valueProperty.setNative] = function (value) {
        this.ios.value = value;
    };
    Slider.prototype[minValueProperty.getDefault] = function () {
        return 0;
    };
    Slider.prototype[minValueProperty.setNative] = function (value) {
        this.ios.minimumValue = value;
    };
    Slider.prototype[maxValueProperty.getDefault] = function () {
        return 100;
    };
    Slider.prototype[maxValueProperty.setNative] = function (value) {
        this.ios.maximumValue = value;
    };
    Slider.prototype[colorProperty.getDefault] = function () {
        return this.ios.thumbTintColor;
    };
    Slider.prototype[colorProperty.setNative] = function (value) {
        var color = value instanceof Color ? value.ios : value;
        this.ios.thumbTintColor = color;
    };
    Slider.prototype[backgroundColorProperty.getDefault] = function () {
        return this.ios.minimumTrackTintColor;
    };
    Slider.prototype[backgroundColorProperty.setNative] = function (value) {
        var color = value instanceof Color ? value.ios : value;
        this.ios.minimumTrackTintColor = color;
    };
    Slider.prototype[backgroundInternalProperty.getDefault] = function () {
        return null;
    };
    Slider.prototype[backgroundInternalProperty.setNative] = function (value) {
        //
    };
    Slider.prototype.getAccessibilityStep = function () {
        if (!this.accessibilityStep || this.accessibilityStep <= 0) {
            return 10;
        }
        return this.accessibilityStep;
    };
    Slider.prototype._handlerAccessibilityIncrementEvent = function () {
        var args = {
            object: this,
            eventName: SliderBase.accessibilityIncrementEvent,
            value: this.value + this.getAccessibilityStep(),
        };
        this.notify(args);
        return args.value;
    };
    Slider.prototype._handlerAccessibilityDecrementEvent = function () {
        var args = {
            object: this,
            eventName: SliderBase.accessibilityIncrementEvent,
            value: this.value - this.getAccessibilityStep(),
        };
        this.notify(args);
        return args.value;
    };
    return Slider;
}(SliderBase));
export { Slider };
//# sourceMappingURL=index.ios.js.map