import { AccessibilityRole } from '../../accessibility';
import { CoercibleProperty, Property } from '../core/properties';
import { CSSType, View } from '../core/view';
// TODO: Extract base Range class for slider and progress
var SliderBase = /** @class */ (function (_super) {
    __extends(SliderBase, _super);
    function SliderBase() {
        var _this = _super !== null && _super.apply(this, arguments) || this;
        _this.accessibilityRole = AccessibilityRole.Adjustable;
        return _this;
    }
    Object.defineProperty(SliderBase.prototype, "accessibilityStep", {
        get: function () {
            return this.style.accessibilityStep;
        },
        set: function (value) {
            this.style.accessibilityStep = value;
        },
        enumerable: false,
        configurable: true
    });
    SliderBase.valueChangeEvent = 'valueChange';
    SliderBase.accessibilityIncrementEvent = 'accessibilityIncrement';
    SliderBase.accessibilityDecrementEvent = 'accessibilityDecrement';
    SliderBase = __decorate([
        CSSType('Slider')
    ], SliderBase);
    return SliderBase;
}(View));
export { SliderBase };
SliderBase.prototype.recycleNativeView = 'auto';
/**
 * Represents the observable property backing the value property of each Slider instance.
 */
export var valueProperty = new CoercibleProperty({
    name: 'value',
    defaultValue: 0,
    coerceValue: function (target, value) {
        value = Math.max(value, target.minValue);
        value = Math.min(value, target.maxValue);
        return value;
    },
    valueConverter: function (v) { return (__APPLE__ ? parseFloat(v) : parseInt(v)); },
});
valueProperty.register(SliderBase);
/**
 * Represents the observable property backing the minValue property of each Slider instance.
 */
export var minValueProperty = new Property({
    name: 'minValue',
    defaultValue: 0,
    valueChanged: function (target, oldValue, newValue) {
        maxValueProperty.coerce(target);
        valueProperty.coerce(target);
    },
    valueConverter: function (v) { return (__APPLE__ ? parseFloat(v) : parseInt(v)); },
});
minValueProperty.register(SliderBase);
/**
 * Represents the observable property backing the maxValue property of each Slider instance.
 */
export var maxValueProperty = new CoercibleProperty({
    name: 'maxValue',
    defaultValue: 100,
    coerceValue: function (target, value) {
        var minValue = target.minValue;
        if (value < minValue) {
            value = minValue;
        }
        return value;
    },
    valueChanged: function (target, oldValue, newValue) { return valueProperty.coerce(target); },
    valueConverter: function (v) { return (__APPLE__ ? parseFloat(v) : parseInt(v)); },
});
maxValueProperty.register(SliderBase);
//# sourceMappingURL=slider-common.js.map