import { LinearGradient } from './linear-gradient';
import { Screen } from '../../platform';
import { isDataURI, isFileOrResourcePath, layout } from '../../utils';
import { ios as iosViewUtils } from '../utils';
import { ImageSource } from '../../image-source';
import { parse as cssParse } from '../../css-value/reworkcss-value.js';
export * from './background-common';
var clearCGColor = UIColor.clearColor.CGColor;
var uriPattern = /url\(('|")(.*?)\1\)/;
var symbolUrl = Symbol('backgroundImageUrl');
export var CacheMode;
(function (CacheMode) {
    CacheMode[CacheMode["none"] = 0] = "none";
})(CacheMode || (CacheMode = {}));
export var ios;
(function (ios) {
    function createBackgroundUIColor(view, callback, flip) {
        var _a;
        var background = view.style.backgroundInternal;
        var nativeView = view.nativeViewProtected;
        if (!nativeView) {
            return;
        }
        // Unset this in case another layer handles background color (e.g. gradient)
        nativeView.layer.backgroundColor = null;
        // Cleanup of previous values
        clearBackgroundVisualEffects(view);
        // Borders, shadows, etc
        drawBackgroundVisualEffects(view);
        if (!background.image) {
            callback((_a = background === null || background === void 0 ? void 0 : background.color) === null || _a === void 0 ? void 0 : _a.ios);
        }
        else {
            if (!(background.image instanceof LinearGradient)) {
                createUIImageFromURI(view, background.image, flip, function (image) {
                    var _a;
                    callback(image ? UIColor.alloc().initWithPatternImage(image) : (_a = background === null || background === void 0 ? void 0 : background.color) === null || _a === void 0 ? void 0 : _a.ios);
                });
            }
        }
    }
    ios.createBackgroundUIColor = createBackgroundUIColor;
    function drawBackgroundVisualEffects(view) {
        var _a;
        var background = view.style.backgroundInternal;
        var nativeView = view.nativeViewProtected;
        var layer = nativeView.layer;
        var needsLayerAdjustmentOnScroll = false;
        // Add new gradient layer or update existing one
        if (background.image instanceof LinearGradient) {
            if (!nativeView.gradientLayer) {
                nativeView.gradientLayer = CAGradientLayer.new();
                layer.insertSublayerAtIndex(nativeView.gradientLayer, 0);
            }
            iosViewUtils.drawGradient(nativeView, nativeView.gradientLayer, background.image);
            needsLayerAdjustmentOnScroll = true;
        }
        // Initialize clipping mask (usually for clip-path and non-uniform rounded borders)
        maskLayerIfNeeded(nativeView, background);
        if (background.hasUniformBorder()) {
            var borderColor = background.getUniformBorderColor();
            layer.borderColor = (_a = borderColor === null || borderColor === void 0 ? void 0 : borderColor.ios) === null || _a === void 0 ? void 0 : _a.CGColor;
            layer.borderWidth = layout.toDeviceIndependentPixels(background.getUniformBorderWidth());
            layer.cornerRadius = getUniformBorderRadius(view, layer.bounds);
        }
        else {
            drawNonUniformBorders(nativeView, background);
            needsLayerAdjustmentOnScroll = true;
        }
        // Clip-path should be called after borders are applied
        if (nativeView.maskType === iosViewUtils.LayerMask.CLIP_PATH && layer.mask instanceof CAShapeLayer) {
            layer.mask.path = generateClipPath(view, layer.bounds);
        }
        if (background.hasBoxShadow()) {
            drawBoxShadow(view);
            needsLayerAdjustmentOnScroll = true;
        }
        if (needsLayerAdjustmentOnScroll) {
            registerAdjustLayersOnScrollListener(view);
        }
    }
    ios.drawBackgroundVisualEffects = drawBackgroundVisualEffects;
    function clearBackgroundVisualEffects(view) {
        var nativeView = view.nativeViewProtected;
        if (!nativeView) {
            return;
        }
        var background = view.style.backgroundInternal;
        var hasGradientBackground = background.image && background.image instanceof LinearGradient;
        // Remove mask if there is no clip path or non-uniform border with radius
        var needsMask;
        switch (nativeView.maskType) {
            case iosViewUtils.LayerMask.BORDER:
                needsMask = !background.hasUniformBorder() && background.hasBorderRadius();
                break;
            case iosViewUtils.LayerMask.CLIP_PATH:
                needsMask = !!background.clipPath;
                break;
            default:
                needsMask = false;
                break;
        }
        if (!needsMask) {
            clearLayerMask(nativeView);
        }
        // Clear box shadow if it's no longer needed
        if (background.clearFlags & 2 /* BackgroundClearFlags.CLEAR_BOX_SHADOW */) {
            clearBoxShadow(nativeView);
        }
        // Non-uniform borders cleanup
        if (nativeView.hasNonUniformBorder) {
            if (nativeView.hasNonUniformBorderColor && background.hasUniformBorderColor()) {
                clearNonUniformColorBorders(nativeView);
            }
            if (background.hasUniformBorder()) {
                clearNonUniformBorders(nativeView);
            }
        }
        if (nativeView.gradientLayer && !hasGradientBackground) {
            nativeView.gradientLayer.removeFromSuperlayer();
            nativeView.gradientLayer = null;
        }
        // Force unset scroll listener
        unregisterAdjustLayersOnScrollListener(view);
        // Reset clear flags
        background.clearFlags = 0 /* BackgroundClearFlags.NONE */;
    }
    ios.clearBackgroundVisualEffects = clearBackgroundVisualEffects;
    function createUIImageFromURI(view, imageURI, flip, callback) {
        var nativeView = view.nativeViewProtected;
        if (!nativeView) {
            return;
        }
        var frame = nativeView.frame;
        var boundsWidth = view.scaleX ? frame.size.width / view.scaleX : frame.size.width;
        var boundsHeight = view.scaleY ? frame.size.height / view.scaleY : frame.size.height;
        if (!boundsWidth || !boundsHeight) {
            return undefined;
        }
        var style = view.style;
        if (imageURI) {
            var match = imageURI.match(uriPattern);
            if (match && match[2]) {
                imageURI = match[2];
            }
        }
        var bitmap;
        if (isDataURI(imageURI)) {
            var base64Data = imageURI.split(',')[1];
            if (base64Data !== undefined) {
                var imageSource = ImageSource.fromBase64Sync(base64Data);
                bitmap = imageSource && imageSource.ios;
            }
        }
        else if (isFileOrResourcePath(imageURI)) {
            var imageSource = ImageSource.fromFileOrResourceSync(imageURI);
            bitmap = imageSource && imageSource.ios;
        }
        else if (imageURI.indexOf('http') !== -1) {
            style[symbolUrl] = imageURI;
            ImageSource.fromUrl(imageURI)
                .then(function (r) {
                if (style && style[symbolUrl] === imageURI) {
                    callback(generatePatternImage(r.ios, view, flip));
                }
            })
                .catch(function () { });
        }
        callback(generatePatternImage(bitmap, view, flip));
    }
    ios.createUIImageFromURI = createUIImageFromURI;
    function generateShadowLayerPaths(view, bounds) {
        var background = view.style.backgroundInternal;
        var nativeView = view.nativeViewProtected;
        var layer = nativeView.layer;
        var boxShadow = background.getBoxShadow();
        var spreadRadius = layout.toDeviceIndependentPixels(boxShadow.spreadRadius);
        var _a = bounds.size, width = _a.width, height = _a.height;
        var innerPath, shadowPath;
        // Generate more detailed paths if view has border radius
        if (background.hasBorderRadius()) {
            if (background.hasUniformBorder()) {
                var cornerRadius = layer.cornerRadius;
                var cappedRadius = getBorderCapRadius(cornerRadius, width / 2, height / 2);
                var cappedOuterRadii = {
                    topLeft: cappedRadius,
                    topRight: cappedRadius,
                    bottomLeft: cappedRadius,
                    bottomRight: cappedRadius,
                };
                var cappedOuterRadiiWithSpread = {
                    topLeft: cappedRadius + spreadRadius,
                    topRight: cappedRadius + spreadRadius,
                    bottomLeft: cappedRadius + spreadRadius,
                    bottomRight: cappedRadius + spreadRadius,
                };
                innerPath = generateNonUniformBorderOuterClipPath(bounds, cappedOuterRadii);
                shadowPath = generateNonUniformBorderOuterClipPath(bounds, cappedOuterRadiiWithSpread, spreadRadius);
            }
            else {
                var outerTopLeftRadius = layout.toDeviceIndependentPixels(background.borderTopLeftRadius);
                var outerTopRightRadius = layout.toDeviceIndependentPixels(background.borderTopRightRadius);
                var outerBottomRightRadius = layout.toDeviceIndependentPixels(background.borderBottomRightRadius);
                var outerBottomLeftRadius = layout.toDeviceIndependentPixels(background.borderBottomLeftRadius);
                var topRadii = outerTopLeftRadius + outerTopRightRadius;
                var rightRadii = outerTopRightRadius + outerBottomRightRadius;
                var bottomRadii = outerBottomRightRadius + outerBottomLeftRadius;
                var leftRadii = outerBottomLeftRadius + outerTopLeftRadius;
                var cappedOuterRadii = {
                    topLeft: getBorderCapRadius(outerTopLeftRadius, (outerTopLeftRadius / topRadii) * width, (outerTopLeftRadius / leftRadii) * height),
                    topRight: getBorderCapRadius(outerTopRightRadius, (outerTopRightRadius / topRadii) * width, (outerTopRightRadius / rightRadii) * height),
                    bottomLeft: getBorderCapRadius(outerBottomLeftRadius, (outerBottomLeftRadius / bottomRadii) * width, (outerBottomLeftRadius / leftRadii) * height),
                    bottomRight: getBorderCapRadius(outerBottomRightRadius, (outerBottomRightRadius / bottomRadii) * width, (outerBottomRightRadius / rightRadii) * height),
                };
                // Add spread radius to corners that actually have radius as shadow has grown larger
                // than view itself and needs to be rounded accordingly
                var cappedOuterRadiiWithSpread = {
                    topLeft: cappedOuterRadii.topLeft > 0 ? cappedOuterRadii.topLeft + spreadRadius : cappedOuterRadii.topLeft,
                    topRight: cappedOuterRadii.topRight > 0 ? cappedOuterRadii.topRight + spreadRadius : cappedOuterRadii.topRight,
                    bottomLeft: cappedOuterRadii.bottomLeft > 0 ? cappedOuterRadii.bottomLeft + spreadRadius : cappedOuterRadii.bottomLeft,
                    bottomRight: cappedOuterRadii.bottomRight > 0 ? cappedOuterRadii.bottomRight + spreadRadius : cappedOuterRadii.bottomRight,
                };
                innerPath = generateNonUniformBorderOuterClipPath(bounds, cappedOuterRadii);
                shadowPath = generateNonUniformBorderOuterClipPath(bounds, cappedOuterRadiiWithSpread, spreadRadius);
            }
        }
        else {
            innerPath = CGPathCreateWithRect(bounds, null);
            shadowPath = CGPathCreateWithRect(CGRectInset(bounds, -spreadRadius, -spreadRadius), null);
        }
        return {
            maskPath: generateShadowMaskPath(bounds, boxShadow, innerPath),
            shadowPath: shadowPath,
        };
    }
    ios.generateShadowLayerPaths = generateShadowLayerPaths;
    function generateClipPath(view, bounds) {
        var background = view.style.backgroundInternal;
        var origin = bounds.origin, size = bounds.size;
        var position = {
            left: origin.x,
            top: origin.y,
            bottom: size.height,
            right: size.width,
        };
        if (position.right === 0 || position.bottom === 0) {
            return;
        }
        var path;
        var clipPath = background.clipPath;
        var functionName = clipPath.substring(0, clipPath.indexOf('('));
        var value = clipPath.replace("".concat(functionName, "("), '').replace(')', '');
        switch (functionName) {
            case 'rect':
                path = rectPath(value, position);
                break;
            case 'inset':
                path = insetPath(value, position);
                break;
            case 'circle':
                path = circlePath(value, position);
                break;
            case 'ellipse':
                path = ellipsePath(value, position);
                break;
            case 'polygon':
                path = polygonPath(value, position);
                break;
        }
        return path;
    }
    ios.generateClipPath = generateClipPath;
    function getUniformBorderRadius(view, bounds) {
        var background = view.style.backgroundInternal;
        var _a = bounds.size, width = _a.width, height = _a.height;
        var cornerRadius = layout.toDeviceIndependentPixels(background.getUniformBorderRadius());
        return Math.min(Math.min(width / 2, height / 2), cornerRadius);
    }
    ios.getUniformBorderRadius = getUniformBorderRadius;
    function generateNonUniformBorderInnerClipRoundedPath(view, bounds) {
        var background = view.style.backgroundInternal;
        var cappedOuterRadii = calculateNonUniformBorderCappedRadii(bounds, background);
        return generateNonUniformBorderInnerClipPath(bounds, background, cappedOuterRadii);
    }
    ios.generateNonUniformBorderInnerClipRoundedPath = generateNonUniformBorderInnerClipRoundedPath;
    function generateNonUniformBorderOuterClipRoundedPath(view, bounds) {
        var background = view.style.backgroundInternal;
        var cappedOuterRadii = calculateNonUniformBorderCappedRadii(bounds, background);
        return generateNonUniformBorderOuterClipPath(bounds, cappedOuterRadii);
    }
    ios.generateNonUniformBorderOuterClipRoundedPath = generateNonUniformBorderOuterClipRoundedPath;
    function generateNonUniformMultiColorBorderRoundedPaths(view, bounds) {
        var background = view.style.backgroundInternal;
        return generateNonUniformMultiColorBorderPaths(bounds, background);
    }
    ios.generateNonUniformMultiColorBorderRoundedPaths = generateNonUniformMultiColorBorderRoundedPaths;
})(ios || (ios = {}));
function maskLayerIfNeeded(nativeView, background) {
    var layer = nativeView.layer;
    // Check if layer should be masked
    if (!(layer.mask instanceof CAShapeLayer)) {
        // Since layers can only accept up to a single mask at a time, clip path is given more priority
        if (background.clipPath) {
            nativeView.maskType = iosViewUtils.LayerMask.CLIP_PATH;
        }
        else if (!background.hasUniformBorder() && background.hasBorderRadius()) {
            nativeView.maskType = iosViewUtils.LayerMask.BORDER;
        }
        else {
            nativeView.maskType = null;
        }
        if (nativeView.maskType != null) {
            nativeView.originalMask = layer.mask;
            layer.mask = CAShapeLayer.new();
        }
    }
}
function clearLayerMask(nativeView) {
    if (nativeView.outerShadowContainerLayer) {
        nativeView.outerShadowContainerLayer.mask = null;
    }
    nativeView.layer.mask = nativeView.originalMask;
    nativeView.originalMask = null;
    nativeView.maskType = null;
}
function onBackgroundViewScroll(args) {
    var view = args.object;
    var nativeView = view.nativeViewProtected;
    if (nativeView instanceof UIScrollView) {
        adjustLayersForScrollView(nativeView);
    }
}
function adjustLayersForScrollView(nativeView) {
    // Compensates with transition for the background layers for scrolling in ScrollView based controls.
    CATransaction.begin();
    CATransaction.setDisableActions(true);
    var offset = nativeView.contentOffset;
    var transform = {
        a: 1,
        b: 0,
        c: 0,
        d: 1,
        tx: offset.x,
        ty: offset.y,
    };
    if (nativeView.layer.mask) {
        nativeView.layer.mask.setAffineTransform(transform);
    }
    // Nested layers
    if (nativeView.gradientLayer) {
        nativeView.gradientLayer.setAffineTransform(transform);
    }
    if (nativeView.borderLayer) {
        nativeView.borderLayer.setAffineTransform(transform);
    }
    if (nativeView.outerShadowContainerLayer) {
        // Update bounds of shadow layer as it belongs to parent view
        nativeView.outerShadowContainerLayer.bounds = nativeView.bounds;
        nativeView.outerShadowContainerLayer.setAffineTransform(transform);
    }
    CATransaction.setDisableActions(false);
    CATransaction.commit();
}
function unregisterAdjustLayersOnScrollListener(view) {
    if (view.nativeViewProtected instanceof UIScrollView) {
        view.off('scroll', onBackgroundViewScroll);
    }
}
function registerAdjustLayersOnScrollListener(view) {
    if (view.nativeViewProtected instanceof UIScrollView) {
        view.off('scroll', onBackgroundViewScroll);
        view.on('scroll', onBackgroundViewScroll);
        adjustLayersForScrollView(view.nativeViewProtected);
    }
}
function clearNonUniformColorBorders(nativeView) {
    if (nativeView.borderLayer) {
        nativeView.borderLayer.mask = null;
        nativeView.borderLayer.sublayers = null;
    }
    nativeView.hasNonUniformBorderColor = false;
}
function clearNonUniformBorders(nativeView) {
    if (nativeView.borderLayer) {
        nativeView.borderLayer.removeFromSuperlayer();
        nativeView.borderLayer = null;
    }
    nativeView.hasNonUniformBorder = false;
}
function parsePosition(pos) {
    var values = cssParse(pos);
    if (values.length === 2) {
        return { x: values[0], y: values[1] };
    }
    if (values.length === 1) {
        var center = { type: 'ident', string: 'center' };
        if (values[0].type === 'ident') {
            var val = values[0].string.toLocaleLowerCase();
            // If you only one keyword is specified, the other value is "center"
            if (val === 'left' || val === 'right') {
                return { x: values[0], y: center };
            }
            else if (val === 'top' || val === 'bottom') {
                return { x: center, y: values[0] };
            }
            else if (val === 'center') {
                return { x: center, y: center };
            }
        }
        else if (values[0].type === 'number') {
            return { x: values[0], y: center };
        }
    }
    return null;
}
function getDrawParams(image, background, width, height) {
    if (!image) {
        return null;
    }
    var res = {
        repeatX: true,
        repeatY: true,
        posX: 0,
        posY: 0,
    };
    // repeat
    if (background.repeat) {
        switch (background.repeat.toLowerCase()) {
            case 'no-repeat':
                res.repeatX = false;
                res.repeatY = false;
                break;
            case 'repeat-x':
                res.repeatY = false;
                break;
            case 'repeat-y':
                res.repeatX = false;
                break;
        }
    }
    var imageSize = image.size;
    var imageWidth = imageSize.width;
    var imageHeight = imageSize.height;
    // size
    var size = background.size;
    if (size) {
        var values = cssParse(size);
        if (values.length === 2) {
            var vx = values[0];
            var vy = values[1];
            if (vx.unit === '%' && vy.unit === '%') {
                imageWidth = (width * vx.value) / 100;
                imageHeight = (height * vy.value) / 100;
                res.sizeX = imageWidth;
                res.sizeY = imageHeight;
            }
            else if (vx.type === 'number' && vy.type === 'number' && ((vx.unit === 'px' && vy.unit === 'px') || (vx.unit === '' && vy.unit === ''))) {
                imageWidth = vx.value;
                imageHeight = vy.value;
                res.sizeX = imageWidth;
                res.sizeY = imageHeight;
            }
        }
        else if (values.length === 1 && values[0].type === 'ident') {
            var scale = 0;
            if (values[0].string === 'cover') {
                scale = Math.max(width / imageWidth, height / imageHeight);
            }
            else if (values[0].string === 'contain') {
                scale = Math.min(width / imageWidth, height / imageHeight);
            }
            if (scale > 0) {
                imageWidth *= scale;
                imageHeight *= scale;
                res.sizeX = imageWidth;
                res.sizeY = imageHeight;
            }
        }
    }
    // position
    var position = background.position;
    if (position) {
        var v = parsePosition(position);
        if (v) {
            var spaceX = width - imageWidth;
            var spaceY = height - imageHeight;
            if (v.x.unit === '%' && v.y.unit === '%') {
                res.posX = (spaceX * v.x.value) / 100;
                res.posY = (spaceY * v.y.value) / 100;
            }
            else if (v.x.type === 'number' && v.y.type === 'number' && ((v.x.unit === 'px' && v.y.unit === 'px') || (v.x.unit === '' && v.y.unit === ''))) {
                res.posX = v.x.value;
                res.posY = v.y.value;
            }
            else if (v.x.type === 'ident' && v.y.type === 'ident') {
                if (v.x.string.toLowerCase() === 'center') {
                    res.posX = spaceX / 2;
                }
                else if (v.x.string.toLowerCase() === 'right') {
                    res.posX = spaceX;
                }
                if (v.y.string.toLowerCase() === 'center') {
                    res.posY = spaceY / 2;
                }
                else if (v.y.string.toLowerCase() === 'bottom') {
                    res.posY = spaceY;
                }
            }
            else if (v.x.type === 'number' && v.y.type === 'ident') {
                if (v.x.unit === '%') {
                    res.posX = (spaceX * v.x.value) / 100;
                }
                else if (v.x.unit === 'px' || v.x.unit === '') {
                    res.posX = v.x.value;
                }
                if (v.y.string.toLowerCase() === 'center') {
                    res.posY = spaceY / 2;
                }
                else if (v.y.string.toLowerCase() === 'bottom') {
                    res.posY = spaceY;
                }
            }
        }
    }
    return res;
}
function generatePatternImage(img, view, flip) {
    var background = view.style.backgroundInternal;
    var nativeView = view.nativeViewProtected;
    if (!img || !nativeView) {
        return null;
    }
    var frame = nativeView.frame;
    var boundsWidth = view.scaleX ? frame.size.width / view.scaleX : frame.size.width;
    var boundsHeight = view.scaleY ? frame.size.height / view.scaleY : frame.size.height;
    var params = getDrawParams(img, background, boundsWidth, boundsHeight);
    if (params.sizeX > 0 && params.sizeY > 0) {
        var resizeRect = CGRectMake(0, 0, params.sizeX, params.sizeY);
        UIGraphicsBeginImageContextWithOptions(resizeRect.size, false, 0.0);
        img.drawInRect(resizeRect);
        img = UIGraphicsGetImageFromCurrentImageContext();
        UIGraphicsEndImageContext();
    }
    UIGraphicsBeginImageContextWithOptions(CGSizeFromString("{".concat(boundsWidth, ",").concat(boundsHeight, "}")), false, 0.0);
    var context = UIGraphicsGetCurrentContext();
    if (background.color && background.color.ios) {
        CGContextSetFillColorWithColor(context, background.color.ios.CGColor);
        CGContextFillRect(context, CGRectMake(0, 0, boundsWidth, boundsHeight));
    }
    if (!params.repeatX && !params.repeatY) {
        img.drawAtPoint(CGPointMake(params.posX, params.posY));
    }
    else {
        var w = params.repeatX ? boundsWidth : img.size.width;
        var h = params.repeatY ? boundsHeight : img.size.height;
        CGContextSetPatternPhase(context, CGSizeMake(params.posX, params.posY));
        params.posX = params.repeatX ? 0 : params.posX;
        params.posY = params.repeatY ? 0 : params.posY;
        var patternRect = CGRectMake(params.posX, params.posY, w, h);
        img.drawAsPatternInRect(patternRect);
    }
    var bgImage = UIGraphicsGetImageFromCurrentImageContext();
    UIGraphicsEndImageContext();
    return flip ? _flipImage(bgImage) : bgImage;
}
// Flipping the default coordinate system
// https://developer.apple.com/library/ios/documentation/2DDrawing/Conceptual/DrawingPrintingiOS/GraphicsDrawingOverview/GraphicsDrawingOverview.html
function _flipImage(originalImage) {
    UIGraphicsBeginImageContextWithOptions(originalImage.size, false, 0.0);
    var context = UIGraphicsGetCurrentContext();
    CGContextSaveGState(context);
    CGContextTranslateCTM(context, 0.0, originalImage.size.height);
    CGContextScaleCTM(context, 1.0, -1.0);
    originalImage.drawInRect(CGRectMake(0, 0, originalImage.size.width, originalImage.size.height));
    CGContextRestoreGState(context);
    var flippedImage = UIGraphicsGetImageFromCurrentImageContext();
    UIGraphicsEndImageContext();
    return flippedImage;
}
function cssValueToDeviceIndependentPixels(source, total) {
    source = source.trim();
    if (source.indexOf('px') !== -1) {
        return layout.toDeviceIndependentPixels(parseFloat(source.replace('px', '')));
    }
    else if (source.indexOf('%') !== -1 && total > 0) {
        return (parseFloat(source.replace('%', '')) / 100) * total;
    }
    else {
        return parseFloat(source);
    }
}
function getBorderCapRadius(a, b, c) {
    return a && Math.min(a, Math.min(b, c));
}
function calculateNonUniformBorderCappedRadii(bounds, background) {
    var _a = bounds.size, width = _a.width, height = _a.height;
    var _b = bounds.origin, x = _b.x, y = _b.y;
    var outerTopLeftRadius = layout.toDeviceIndependentPixels(background.borderTopLeftRadius);
    var outerTopRightRadius = layout.toDeviceIndependentPixels(background.borderTopRightRadius);
    var outerBottomRightRadius = layout.toDeviceIndependentPixels(background.borderBottomRightRadius);
    var outerBottomLeftRadius = layout.toDeviceIndependentPixels(background.borderBottomLeftRadius);
    var topRadii = outerTopLeftRadius + outerTopRightRadius;
    var rightRadii = outerTopRightRadius + outerBottomRightRadius;
    var bottomRadii = outerBottomRightRadius + outerBottomLeftRadius;
    var leftRadii = outerBottomLeftRadius + outerTopLeftRadius;
    var cappedOuterRadii = {
        topLeft: getBorderCapRadius(outerTopLeftRadius, (outerTopLeftRadius / topRadii) * width, (outerTopLeftRadius / leftRadii) * height),
        topRight: getBorderCapRadius(outerTopRightRadius, (outerTopRightRadius / topRadii) * width, (outerTopRightRadius / rightRadii) * height),
        bottomLeft: getBorderCapRadius(outerBottomLeftRadius, (outerBottomLeftRadius / bottomRadii) * width, (outerBottomLeftRadius / leftRadii) * height),
        bottomRight: getBorderCapRadius(outerBottomRightRadius, (outerBottomRightRadius / bottomRadii) * width, (outerBottomRightRadius / rightRadii) * height),
    };
    return cappedOuterRadii;
}
function drawNonUniformBorders(nativeView, background) {
    var _a, _b, _c, _d, _e, _f, _g, _h, _j, _k;
    var layer = nativeView.layer;
    var layerBounds = layer.bounds;
    layer.borderColor = null;
    layer.borderWidth = 0;
    layer.cornerRadius = 0;
    var cappedOuterRadii = calculateNonUniformBorderCappedRadii(layerBounds, background);
    if (nativeView.maskType === iosViewUtils.LayerMask.BORDER && layer.mask instanceof CAShapeLayer) {
        layer.mask.path = generateNonUniformBorderOuterClipPath(layerBounds, cappedOuterRadii);
    }
    if (background.hasBorderWidth()) {
        if (!nativeView.hasNonUniformBorder) {
            nativeView.borderLayer = CAShapeLayer.new();
            nativeView.borderLayer.fillRule = kCAFillRuleEvenOdd;
            layer.addSublayer(nativeView.borderLayer);
            nativeView.hasNonUniformBorder = true;
        }
        if (background.hasUniformBorderColor()) {
            // Use anti-aliasing or borders will draw incorrectly at times
            nativeView.borderLayer.shouldRasterize = true;
            nativeView.borderLayer.rasterizationScale = Screen.mainScreen.scale;
            nativeView.borderLayer.fillColor = ((_b = (_a = background.borderTopColor) === null || _a === void 0 ? void 0 : _a.ios) === null || _b === void 0 ? void 0 : _b.CGColor) || UIColor.blackColor.CGColor;
            nativeView.borderLayer.path = generateNonUniformBorderInnerClipPath(layerBounds, background, cappedOuterRadii);
        }
        else {
            // Non-uniform borders need more layers in order to display multiple colors at the same time
            var borderTopLayer = void 0, borderRightLayer = void 0, borderBottomLayer = void 0, borderLeftLayer = void 0;
            if (!nativeView.hasNonUniformBorderColor) {
                var maskLayer = CAShapeLayer.new();
                maskLayer.fillRule = kCAFillRuleEvenOdd;
                // Use anti-aliasing or borders will draw incorrectly at times
                maskLayer.shouldRasterize = true;
                maskLayer.rasterizationScale = Screen.mainScreen.scale;
                nativeView.borderLayer.mask = maskLayer;
                borderTopLayer = CAShapeLayer.new();
                borderRightLayer = CAShapeLayer.new();
                borderBottomLayer = CAShapeLayer.new();
                borderLeftLayer = CAShapeLayer.new();
                nativeView.borderLayer.addSublayer(borderTopLayer);
                nativeView.borderLayer.addSublayer(borderRightLayer);
                nativeView.borderLayer.addSublayer(borderBottomLayer);
                nativeView.borderLayer.addSublayer(borderLeftLayer);
                nativeView.hasNonUniformBorderColor = true;
            }
            else {
                borderTopLayer = nativeView.borderLayer.sublayers[0];
                borderRightLayer = nativeView.borderLayer.sublayers[1];
                borderBottomLayer = nativeView.borderLayer.sublayers[2];
                borderLeftLayer = nativeView.borderLayer.sublayers[3];
            }
            var paths = generateNonUniformMultiColorBorderPaths(layerBounds, background);
            borderTopLayer.fillColor = ((_d = (_c = background.borderTopColor) === null || _c === void 0 ? void 0 : _c.ios) === null || _d === void 0 ? void 0 : _d.CGColor) || UIColor.blackColor.CGColor;
            borderTopLayer.path = paths[0];
            borderRightLayer.fillColor = ((_f = (_e = background.borderRightColor) === null || _e === void 0 ? void 0 : _e.ios) === null || _f === void 0 ? void 0 : _f.CGColor) || UIColor.blackColor.CGColor;
            borderRightLayer.path = paths[1];
            borderBottomLayer.fillColor = ((_h = (_g = background.borderBottomColor) === null || _g === void 0 ? void 0 : _g.ios) === null || _h === void 0 ? void 0 : _h.CGColor) || UIColor.blackColor.CGColor;
            borderBottomLayer.path = paths[2];
            borderLeftLayer.fillColor = ((_k = (_j = background.borderLeftColor) === null || _j === void 0 ? void 0 : _j.ios) === null || _k === void 0 ? void 0 : _k.CGColor) || UIColor.blackColor.CGColor;
            borderLeftLayer.path = paths[3];
            // Clip inner area to create borders
            if (nativeView.borderLayer.mask instanceof CAShapeLayer) {
                nativeView.borderLayer.mask.path = generateNonUniformBorderInnerClipPath(layerBounds, background, cappedOuterRadii);
            }
        }
    }
}
function calculateInnerBorderClipRadius(radius, insetX, insetY) {
    var innerXRadius = Math.max(0, radius - insetX);
    var innerYRadius = Math.max(0, radius - insetY);
    var innerMaxRadius = Math.max(innerXRadius, innerYRadius);
    return {
        xRadius: innerXRadius,
        yRadius: innerYRadius,
        maxRadius: innerMaxRadius,
    };
}
/**
 * Generates a path that represents the rounded view area.
 *
 * @param bounds
 * @param cappedRadii
 * @param offset
 * @returns
 */
function generateNonUniformBorderOuterClipPath(bounds, cappedRadii, offset) {
    if (offset === void 0) { offset = 0; }
    var _a = bounds.size, width = _a.width, height = _a.height;
    var _b = bounds.origin, x = _b.x, y = _b.y;
    var left = x - offset;
    var top = y - offset;
    var right = x + width + offset;
    var bottom = y + height + offset;
    var clipPath = CGPathCreateMutable();
    CGPathMoveToPoint(clipPath, null, left + cappedRadii.topLeft, top);
    CGPathAddArcToPoint(clipPath, null, right, top, right, top + cappedRadii.topRight, cappedRadii.topRight);
    CGPathAddArcToPoint(clipPath, null, right, bottom, right - cappedRadii.bottomRight, bottom, cappedRadii.bottomRight);
    CGPathAddArcToPoint(clipPath, null, left, bottom, left, bottom - cappedRadii.bottomLeft, cappedRadii.bottomLeft);
    CGPathAddArcToPoint(clipPath, null, left, top, left + cappedRadii.topLeft, top, cappedRadii.topLeft);
    CGPathCloseSubpath(clipPath);
    return clipPath;
}
/**
 * Generates a path that represents the area inside borders.
 *
 * @param bounds
 * @param background
 * @param cappedOuterRadii
 * @returns
 */
function generateNonUniformBorderInnerClipPath(bounds, background, cappedOuterRadii) {
    var _a = bounds.size, width = _a.width, height = _a.height;
    var _b = bounds.origin, x = _b.x, y = _b.y;
    var position = {
        left: x,
        top: y,
        bottom: y + height,
        right: x + width,
    };
    var borderTopWidth = Math.max(0, layout.toDeviceIndependentPixels(background.borderTopWidth));
    var borderRightWidth = Math.max(0, layout.toDeviceIndependentPixels(background.borderRightWidth));
    var borderBottomWidth = Math.max(0, layout.toDeviceIndependentPixels(background.borderBottomWidth));
    var borderLeftWidth = Math.max(0, layout.toDeviceIndependentPixels(background.borderLeftWidth));
    var borderVWidth = borderTopWidth + borderBottomWidth;
    var borderHWidth = borderLeftWidth + borderRightWidth;
    var cappedBorderTopWidth = borderTopWidth && borderTopWidth * Math.min(1, height / borderVWidth);
    var cappedBorderRightWidth = borderRightWidth && borderRightWidth * Math.min(1, width / borderHWidth);
    var cappedBorderBottomWidth = borderBottomWidth && borderBottomWidth * Math.min(1, height / borderVWidth);
    var cappedBorderLeftWidth = borderLeftWidth && borderLeftWidth * Math.min(1, width / borderHWidth);
    var clipPath = CGPathCreateMutable();
    CGPathAddRect(clipPath, null, CGRectMake(x, y, width, height));
    // Inner clip paths
    if (cappedBorderTopWidth > 0 || cappedBorderLeftWidth > 0) {
        CGPathMoveToPoint(clipPath, null, position.left + cappedOuterRadii.topLeft, position.top + cappedBorderTopWidth);
    }
    else {
        CGPathMoveToPoint(clipPath, null, position.left, position.top);
    }
    if (cappedBorderTopWidth > 0 || cappedBorderRightWidth > 0) {
        var _c = calculateInnerBorderClipRadius(cappedOuterRadii.topRight, cappedBorderRightWidth, cappedBorderTopWidth), xRadius = _c.xRadius, yRadius = _c.yRadius, maxRadius = _c.maxRadius;
        var innerTopRightTransform = CGAffineTransformMake(maxRadius && xRadius / maxRadius, 0, 0, maxRadius && yRadius / maxRadius, position.right - cappedBorderRightWidth - xRadius, position.top + cappedBorderTopWidth + yRadius);
        CGPathAddArc(clipPath, innerTopRightTransform, 0, 0, maxRadius, (Math.PI * 3) / 2, 0, false);
    }
    else {
        CGPathAddLineToPoint(clipPath, null, position.right, position.top);
    }
    if (cappedBorderBottomWidth > 0 || cappedBorderRightWidth > 0) {
        var _d = calculateInnerBorderClipRadius(cappedOuterRadii.bottomRight, cappedBorderRightWidth, cappedBorderBottomWidth), xRadius = _d.xRadius, yRadius = _d.yRadius, maxRadius = _d.maxRadius;
        var innerBottomRightTransform = CGAffineTransformMake(maxRadius && xRadius / maxRadius, 0, 0, maxRadius && yRadius / maxRadius, position.right - cappedBorderRightWidth - xRadius, position.bottom - cappedBorderBottomWidth - yRadius);
        CGPathAddArc(clipPath, innerBottomRightTransform, 0, 0, maxRadius, 0, Math.PI / 2, false);
    }
    else {
        CGPathAddLineToPoint(clipPath, null, position.right, position.bottom);
    }
    if (cappedBorderBottomWidth > 0 || cappedBorderLeftWidth > 0) {
        var _e = calculateInnerBorderClipRadius(cappedOuterRadii.bottomLeft, cappedBorderLeftWidth, cappedBorderBottomWidth), xRadius = _e.xRadius, yRadius = _e.yRadius, maxRadius = _e.maxRadius;
        var innerBottomLeftTransform = CGAffineTransformMake(maxRadius && xRadius / maxRadius, 0, 0, maxRadius && yRadius / maxRadius, position.left + cappedBorderLeftWidth + xRadius, position.bottom - cappedBorderBottomWidth - yRadius);
        CGPathAddArc(clipPath, innerBottomLeftTransform, 0, 0, maxRadius, Math.PI / 2, Math.PI, false);
    }
    else {
        CGPathAddLineToPoint(clipPath, null, position.left, position.bottom);
    }
    if (cappedBorderTopWidth > 0 || cappedBorderLeftWidth > 0) {
        var _f = calculateInnerBorderClipRadius(cappedOuterRadii.topLeft, cappedBorderLeftWidth, cappedBorderTopWidth), xRadius = _f.xRadius, yRadius = _f.yRadius, maxRadius = _f.maxRadius;
        var innerTopLeftTransform = CGAffineTransformMake(maxRadius && xRadius / maxRadius, 0, 0, maxRadius && yRadius / maxRadius, position.left + cappedBorderLeftWidth + xRadius, position.top + cappedBorderTopWidth + yRadius);
        CGPathAddArc(clipPath, innerTopLeftTransform, 0, 0, maxRadius, Math.PI, (Math.PI * 3) / 2, false);
    }
    else {
        CGPathAddLineToPoint(clipPath, null, position.left, position.top);
    }
    CGPathCloseSubpath(clipPath);
    return clipPath;
}
/**
 * Calculates the needed widths for creating triangular shapes for each border.
 * To achieve this, all border widths are scaled according to view bounds.
 *
 * @param bounds
 * @param background
 * @returns
 */
function getBorderTriangleWidths(bounds, background) {
    var width = bounds.origin.x + bounds.size.width;
    var height = bounds.origin.y + bounds.size.height;
    var borderTopWidth = Math.max(0, layout.toDeviceIndependentPixels(background.borderTopWidth));
    var borderRightWidth = Math.max(0, layout.toDeviceIndependentPixels(background.borderRightWidth));
    var borderBottomWidth = Math.max(0, layout.toDeviceIndependentPixels(background.borderBottomWidth));
    var borderLeftWidth = Math.max(0, layout.toDeviceIndependentPixels(background.borderLeftWidth));
    var verticalBorderWidth = borderTopWidth + borderBottomWidth;
    var horizontalBorderWidth = borderLeftWidth + borderRightWidth;
    var verticalBorderMultiplier = verticalBorderWidth > 0 ? height / verticalBorderWidth : 0;
    var horizontalBorderMultiplier = horizontalBorderWidth > 0 ? width / horizontalBorderWidth : 0;
    // Both directions should consider each other in order to scale widths properly, as a view might have different width and height
    if (verticalBorderMultiplier > 0 && verticalBorderMultiplier < horizontalBorderMultiplier) {
        horizontalBorderMultiplier -= horizontalBorderMultiplier - verticalBorderMultiplier;
    }
    if (horizontalBorderMultiplier > 0 && horizontalBorderMultiplier < verticalBorderMultiplier) {
        verticalBorderMultiplier -= verticalBorderMultiplier - horizontalBorderMultiplier;
    }
    return {
        top: borderTopWidth * verticalBorderMultiplier,
        right: borderRightWidth * horizontalBorderMultiplier,
        bottom: borderBottomWidth * verticalBorderMultiplier,
        left: borderLeftWidth * horizontalBorderMultiplier,
    };
}
/**
 * Generates paths for visualizing borders with different colors per side.
 * This is achieved by extending all borders enough to consume entire view size,
 * then using an even-odd inner mask to clip and eventually render borders according to their corresponding width.
 *
 * @param bounds
 * @param background
 * @returns
 */
function generateNonUniformMultiColorBorderPaths(bounds, background) {
    var _a = bounds.size, width = _a.width, height = _a.height;
    var _b = bounds.origin, x = _b.x, y = _b.y;
    var position = {
        left: x,
        top: y,
        bottom: y + height,
        right: x + width,
    };
    var borderWidths = getBorderTriangleWidths(bounds, background);
    var paths = new Array(4);
    var lto = {
        x: position.left,
        y: position.top,
    }; // left-top-outside
    var lti = {
        x: position.left + borderWidths.left,
        y: position.top + borderWidths.top,
    }; // left-top-inside
    var rto = {
        x: position.right,
        y: position.top,
    }; // right-top-outside
    var rti = {
        x: position.right - borderWidths.right,
        y: position.top + borderWidths.top,
    }; // right-top-inside
    var rbo = {
        x: position.right,
        y: position.bottom,
    }; // right-bottom-outside
    var rbi = {
        x: position.right - borderWidths.right,
        y: position.bottom - borderWidths.bottom,
    }; // right-bottom-inside
    var lbo = {
        x: position.left,
        y: position.bottom,
    }; // left-bottom-outside
    var lbi = {
        x: position.left + borderWidths.left,
        y: position.bottom - borderWidths.bottom,
    }; // left-bottom-inside
    var borderTopColor = background.borderTopColor;
    var borderRightColor = background.borderRightColor;
    var borderBottomColor = background.borderBottomColor;
    var borderLeftColor = background.borderLeftColor;
    if (borderWidths.top > 0 && (borderTopColor === null || borderTopColor === void 0 ? void 0 : borderTopColor.ios)) {
        var topBorderPath = CGPathCreateMutable();
        CGPathMoveToPoint(topBorderPath, null, lto.x, lto.y);
        CGPathAddLineToPoint(topBorderPath, null, rto.x, rto.y);
        CGPathAddLineToPoint(topBorderPath, null, rti.x, rti.y);
        if (rti.x !== lti.x) {
            CGPathAddLineToPoint(topBorderPath, null, lti.x, lti.y);
        }
        CGPathAddLineToPoint(topBorderPath, null, lto.x, lto.y);
        paths[0] = topBorderPath;
    }
    if (borderWidths.right > 0 && (borderRightColor === null || borderRightColor === void 0 ? void 0 : borderRightColor.ios)) {
        var rightBorderPath = CGPathCreateMutable();
        CGPathMoveToPoint(rightBorderPath, null, rto.x, rto.y);
        CGPathAddLineToPoint(rightBorderPath, null, rbo.x, rbo.y);
        CGPathAddLineToPoint(rightBorderPath, null, rbi.x, rbi.y);
        if (rbi.y !== rti.y) {
            CGPathAddLineToPoint(rightBorderPath, null, rti.x, rti.y);
        }
        CGPathAddLineToPoint(rightBorderPath, null, rto.x, rto.y);
        paths[1] = rightBorderPath;
    }
    if (borderWidths.bottom > 0 && (borderBottomColor === null || borderBottomColor === void 0 ? void 0 : borderBottomColor.ios)) {
        var bottomBorderPath = CGPathCreateMutable();
        CGPathMoveToPoint(bottomBorderPath, null, rbo.x, rbo.y);
        CGPathAddLineToPoint(bottomBorderPath, null, lbo.x, lbo.y);
        CGPathAddLineToPoint(bottomBorderPath, null, lbi.x, lbi.y);
        if (lbi.x !== rbi.x) {
            CGPathAddLineToPoint(bottomBorderPath, null, rbi.x, rbi.y);
        }
        CGPathAddLineToPoint(bottomBorderPath, null, rbo.x, rbo.y);
        paths[2] = bottomBorderPath;
    }
    if (borderWidths.left > 0 && (borderLeftColor === null || borderLeftColor === void 0 ? void 0 : borderLeftColor.ios)) {
        var leftBorderPath = CGPathCreateMutable();
        CGPathMoveToPoint(leftBorderPath, null, lbo.x, lbo.y);
        CGPathAddLineToPoint(leftBorderPath, null, lto.x, lto.y);
        CGPathAddLineToPoint(leftBorderPath, null, lti.x, lti.y);
        if (lti.y !== lbi.y) {
            CGPathAddLineToPoint(leftBorderPath, null, lbi.x, lbi.y);
        }
        CGPathAddLineToPoint(leftBorderPath, null, lbo.x, lbo.y);
        paths[3] = leftBorderPath;
    }
    return paths;
}
function drawBoxShadow(view) {
    var _a, _b, _c;
    var background = view.style.backgroundInternal;
    var nativeView = view.nativeViewProtected;
    var layer = nativeView.layer;
    // There is no parent to add shadow to
    if (!layer.superlayer) {
        return;
    }
    var bounds = nativeView.bounds;
    var boxShadow = background.getBoxShadow();
    // Initialize outer shadows
    var outerShadowContainerLayer;
    if (nativeView.outerShadowContainerLayer) {
        outerShadowContainerLayer = nativeView.outerShadowContainerLayer;
    }
    else {
        outerShadowContainerLayer = CALayer.new();
        // TODO: Make this dynamic when views get support for multiple shadows
        var shadowLayer = CALayer.new();
        // This mask is necessary to maintain transparent background
        var maskLayer = CAShapeLayer.new();
        maskLayer.fillRule = kCAFillRuleEvenOdd;
        shadowLayer.mask = maskLayer;
        outerShadowContainerLayer.addSublayer(shadowLayer);
        // Instead of nesting it, add shadow container layer underneath view so that it's not affected by border masking
        layer.superlayer.insertSublayerBelow(outerShadowContainerLayer, layer);
        nativeView.outerShadowContainerLayer = outerShadowContainerLayer;
    }
    // Apply clip path to shadow
    if (nativeView.maskType === iosViewUtils.LayerMask.CLIP_PATH && layer.mask instanceof CAShapeLayer) {
        if (!outerShadowContainerLayer.mask) {
            outerShadowContainerLayer.mask = CAShapeLayer.new();
        }
        if (outerShadowContainerLayer.mask instanceof CAShapeLayer) {
            outerShadowContainerLayer.mask.path = layer.mask.path;
        }
    }
    outerShadowContainerLayer.bounds = bounds;
    outerShadowContainerLayer.anchorPoint = layer.anchorPoint;
    outerShadowContainerLayer.position = nativeView.center;
    outerShadowContainerLayer.zPosition = layer.zPosition;
    // Inherit view visibility values
    outerShadowContainerLayer.opacity = layer.opacity;
    outerShadowContainerLayer.hidden = layer.hidden;
    var outerShadowLayers = outerShadowContainerLayer.sublayers;
    if (outerShadowLayers === null || outerShadowLayers === void 0 ? void 0 : outerShadowLayers.count) {
        for (var i = 0, count = outerShadowLayers.count; i < count; i++) {
            var shadowLayer = outerShadowLayers[i];
            var shadowRadius = layout.toDeviceIndependentPixels(boxShadow.blurRadius);
            var spreadRadius = layout.toDeviceIndependentPixels(boxShadow.spreadRadius);
            var offsetX = layout.toDeviceIndependentPixels(boxShadow.offsetX);
            var offsetY = layout.toDeviceIndependentPixels(boxShadow.offsetY);
            var _d = ios.generateShadowLayerPaths(view, bounds), maskPath = _d.maskPath, shadowPath = _d.shadowPath;
            shadowLayer.allowsEdgeAntialiasing = true;
            shadowLayer.contentsScale = Screen.mainScreen.scale;
            // Shadow opacity is handled on the shadow's color instance
            shadowLayer.shadowOpacity = ((_a = boxShadow.color) === null || _a === void 0 ? void 0 : _a.a) ? boxShadow.color.a / 255 : 1;
            shadowLayer.shadowRadius = shadowRadius;
            shadowLayer.shadowColor = (_c = (_b = boxShadow.color) === null || _b === void 0 ? void 0 : _b.ios) === null || _c === void 0 ? void 0 : _c.CGColor;
            shadowLayer.shadowOffset = CGSizeMake(offsetX, offsetY);
            // Apply spread radius by expanding shadow layer bounds (this has a nice glow with radii set to 0)
            shadowLayer.shadowPath = shadowPath;
            // A mask that ensures that view maintains transparent background
            if (shadowLayer.mask instanceof CAShapeLayer) {
                shadowLayer.mask.path = maskPath;
            }
        }
    }
}
function clearBoxShadow(nativeView) {
    if (nativeView.outerShadowContainerLayer) {
        nativeView.outerShadowContainerLayer.removeFromSuperlayer();
        nativeView.outerShadowContainerLayer = null;
    }
}
/**
 * Creates a mask that ensures no shadow will be displayed underneath transparent backgrounds.
 *
 * @param bounds
 * @param boxShadow
 * @param bordersClipPath
 * @returns
 */
function generateShadowMaskPath(bounds, boxShadow, innerClipPath) {
    var shadowRadius = layout.toDeviceIndependentPixels(boxShadow.blurRadius);
    var spreadRadius = layout.toDeviceIndependentPixels(boxShadow.spreadRadius);
    var offsetX = layout.toDeviceIndependentPixels(boxShadow.offsetX);
    var offsetY = layout.toDeviceIndependentPixels(boxShadow.offsetY);
    // This value has to be large enough to avoid clipping shadow halo effect
    var outerRectRadius = shadowRadius * 3 + spreadRadius;
    var maskPath = CGPathCreateMutable();
    // Proper clip position and size
    var outerRect = CGRectOffset(CGRectInset(bounds, -outerRectRadius, -outerRectRadius), offsetX, offsetY);
    CGPathAddPath(maskPath, null, innerClipPath);
    CGPathAddRect(maskPath, null, outerRect);
    return maskPath;
}
function rectPath(value, position) {
    var arr = value.split(/[\s]+/);
    var top = cssValueToDeviceIndependentPixels(arr[0], position.top);
    var right = cssValueToDeviceIndependentPixels(arr[1], position.right);
    var bottom = cssValueToDeviceIndependentPixels(arr[2], position.bottom);
    var left = cssValueToDeviceIndependentPixels(arr[3], position.left);
    return UIBezierPath.bezierPathWithRect(CGRectMake(left, top, right - left, bottom - top)).CGPath;
}
function insetPath(value, position) {
    var arr = value.split(/[\s]+/);
    var topString;
    var rightString;
    var bottomString;
    var leftString;
    if (arr.length === 1) {
        topString = rightString = bottomString = leftString = arr[0];
    }
    else if (arr.length === 2) {
        topString = bottomString = arr[0];
        rightString = leftString = arr[1];
    }
    else if (arr.length === 3) {
        topString = arr[0];
        rightString = leftString = arr[1];
        bottomString = arr[2];
    }
    else if (arr.length === 4) {
        topString = arr[0];
        rightString = arr[1];
        bottomString = arr[2];
        leftString = arr[3];
    }
    var top = cssValueToDeviceIndependentPixels(topString, position.bottom);
    var right = cssValueToDeviceIndependentPixels('100%', position.right) - cssValueToDeviceIndependentPixels(rightString, position.right);
    var bottom = cssValueToDeviceIndependentPixels('100%', position.bottom) - cssValueToDeviceIndependentPixels(bottomString, position.bottom);
    var left = cssValueToDeviceIndependentPixels(leftString, position.right);
    return UIBezierPath.bezierPathWithRect(CGRectMake(left, top, right - left, bottom - top)).CGPath;
}
function circlePath(value, position) {
    var arr = value.split(/[\s]+/);
    var radius = cssValueToDeviceIndependentPixels(arr[0], (position.right > position.bottom ? position.bottom : position.right) / 2);
    var y = cssValueToDeviceIndependentPixels(arr[2], position.bottom);
    var x = cssValueToDeviceIndependentPixels(arr[3], position.right);
    return UIBezierPath.bezierPathWithArcCenterRadiusStartAngleEndAngleClockwise(CGPointMake(x, y), radius, 0, 360, true).CGPath;
}
function ellipsePath(value, position) {
    var arr = value.split(/[\s]+/);
    var rX = cssValueToDeviceIndependentPixels(arr[0], position.right);
    var rY = cssValueToDeviceIndependentPixels(arr[1], position.bottom);
    var cX = cssValueToDeviceIndependentPixels(arr[3], position.right);
    var cY = cssValueToDeviceIndependentPixels(arr[4], position.bottom);
    var left = cX - rX;
    var top = cY - rY;
    var width = rX * 2;
    var height = rY * 2;
    return UIBezierPath.bezierPathWithOvalInRect(CGRectMake(left, top, width, height)).CGPath;
}
function polygonPath(value, position) {
    var path = CGPathCreateMutable();
    var firstPoint;
    var arr = value.split(/[,]+/);
    for (var i = 0; i < arr.length; i++) {
        var xy = arr[i].trim().split(/[\s]+/);
        var point = {
            x: cssValueToDeviceIndependentPixels(xy[0], position.right),
            y: cssValueToDeviceIndependentPixels(xy[1], position.bottom),
        };
        if (!firstPoint) {
            firstPoint = point;
            CGPathMoveToPoint(path, null, point.x, point.y);
        }
        CGPathAddLineToPoint(path, null, point.x, point.y);
    }
    CGPathAddLineToPoint(path, null, firstPoint.x, firstPoint.y);
    return path;
}
//# sourceMappingURL=background.ios.js.map