import { parse as convertToCSSWhatSelector } from 'css-what';
import '../../globals';
import { isCssVariable } from '../core/properties';
import { isNullOrUndefined } from '../../utils/types';
import { checkIfMediaQueryMatches } from '../../media-query-list';
export var MEDIA_QUERY_SEPARATOR = '&&';
var Combinator;
(function (Combinator) {
    Combinator["descendant"] = " ";
    Combinator["child"] = ">";
    Combinator["adjacent"] = "+";
    Combinator["sibling"] = "~";
    // Not supported
    Combinator["parent"] = "<";
    Combinator["column-combinator"] = "||";
})(Combinator || (Combinator = {}));
var AttributeSelectorOperator;
(function (AttributeSelectorOperator) {
    AttributeSelectorOperator["exists"] = "";
    AttributeSelectorOperator["equals"] = "=";
    AttributeSelectorOperator["start"] = "^=";
    AttributeSelectorOperator["end"] = "$=";
    AttributeSelectorOperator["any"] = "*=";
    AttributeSelectorOperator["element"] = "~=";
    AttributeSelectorOperator["hyphen"] = "|=";
})(AttributeSelectorOperator || (AttributeSelectorOperator = {}));
var Match;
(function (Match) {
    /**
     * Depends on attributes or pseudoclasses state;
     */
    Match.Dynamic = true;
    /**
     * Depends only on the tree structure.
     */
    Match.Static = false;
})(Match || (Match = {}));
function eachNodePreviousGeneralSibling(node, callback) {
    if (!node.parent || !node.parent.getChildIndex || !node.parent.getChildAt || !node.parent.getChildrenCount) {
        return;
    }
    var nodeIndex = node.parent.getChildIndex(node);
    if (nodeIndex === 0) {
        return;
    }
    var count = node.parent.getChildrenCount();
    var retVal = true;
    for (var i = nodeIndex - 1; i >= 0 && retVal; i--) {
        var sibling = node.parent.getChildAt(i);
        retVal = callback(sibling);
    }
}
function getNodePreviousDirectSibling(node) {
    if (!node.parent || !node.parent.getChildIndex || !node.parent.getChildAt) {
        return null;
    }
    var nodeIndex = node.parent.getChildIndex(node);
    if (nodeIndex === 0) {
        return null;
    }
    return node.parent.getChildAt(nodeIndex - 1);
}
function SelectorProperties(specificity, rarity, dynamic) {
    if (dynamic === void 0) { dynamic = false; }
    return function (cls) {
        cls.prototype.specificity = specificity;
        cls.prototype.rarity = rarity;
        cls.prototype.combinator = undefined;
        cls.prototype.dynamic = dynamic;
        return cls;
    };
}
function FunctionalPseudoClassProperties(specificity, rarity, pseudoSelectorListType) {
    return function (cls) {
        cls.prototype.specificity = specificity;
        cls.prototype.rarity = rarity;
        cls.prototype.combinator = undefined;
        cls.prototype.dynamic = false;
        cls.prototype.pseudoSelectorListType = pseudoSelectorListType;
        return cls;
    };
}
var SelectorBase = /** @class */ (function () {
    function SelectorBase() {
    }
    return SelectorBase;
}());
export { SelectorBase };
var SelectorCore = /** @class */ (function (_super) {
    __extends(SelectorCore, _super);
    function SelectorCore() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    SelectorCore.prototype.lookupSort = function (sorter, base) {
        sorter.sortAsUniversal(base || this);
    };
    SelectorCore = __decorate([
        SelectorProperties(0 /* Specificity.Universal */, 0 /* Rarity.Universal */, Match.Static)
    ], SelectorCore);
    return SelectorCore;
}(SelectorBase));
export { SelectorCore };
var SimpleSelector = /** @class */ (function (_super) {
    __extends(SimpleSelector, _super);
    function SimpleSelector() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    SimpleSelector.prototype.accumulateChanges = function (node, map) {
        if (!this.dynamic) {
            return this.match(node);
        }
        else if (this.mayMatch(node)) {
            this.trackChanges(node, map);
            return true;
        }
        return false;
    };
    SimpleSelector.prototype.mayMatch = function (node) {
        return this.match(node);
    };
    SimpleSelector.prototype.trackChanges = function (node, map) {
        // No-op, silence the tslint 'block is empty'.
        // Some derived classes (dynamic) will actually fill the map with stuff here, some (static) won't do anything.
    };
    return SimpleSelector;
}(SelectorCore));
export { SimpleSelector };
function wrap(text) {
    return text ? " ".concat(text, " ") : '';
}
var InvalidSelector = /** @class */ (function (_super) {
    __extends(InvalidSelector, _super);
    function InvalidSelector(e) {
        var _this = _super.call(this) || this;
        _this.e = e;
        return _this;
    }
    InvalidSelector.prototype.toString = function () {
        return "<".concat(this.e, ">");
    };
    InvalidSelector.prototype.match = function (node) {
        return false;
    };
    InvalidSelector.prototype.lookupSort = function (sorter, base) {
        // No-op, silence the tslint 'block is empty'.
        // It feels like tslint has problems with simple polymorphism...
        // This selector is invalid and will never match so we won't bother sorting it to further appear in queries.
    };
    InvalidSelector = __decorate([
        SelectorProperties(0 /* Specificity.Invalid */, 4 /* Rarity.Invalid */, Match.Static),
        __metadata("design:paramtypes", [Error])
    ], InvalidSelector);
    return InvalidSelector;
}(SimpleSelector));
export { InvalidSelector };
var UniversalSelector = /** @class */ (function (_super) {
    __extends(UniversalSelector, _super);
    function UniversalSelector() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    UniversalSelector.prototype.toString = function () {
        return "*".concat(wrap(this.combinator));
    };
    UniversalSelector.prototype.match = function (node) {
        return true;
    };
    UniversalSelector = __decorate([
        SelectorProperties(0 /* Specificity.Universal */, 0 /* Rarity.Universal */, Match.Static)
    ], UniversalSelector);
    return UniversalSelector;
}(SimpleSelector));
export { UniversalSelector };
var IdSelector = /** @class */ (function (_super) {
    __extends(IdSelector, _super);
    function IdSelector(id) {
        var _this = _super.call(this) || this;
        _this.id = id;
        return _this;
    }
    IdSelector.prototype.toString = function () {
        return "#".concat(this.id).concat(wrap(this.combinator));
    };
    IdSelector.prototype.match = function (node) {
        return node.id === this.id;
    };
    IdSelector.prototype.lookupSort = function (sorter, base) {
        sorter.sortById(this.id, base || this);
    };
    IdSelector = __decorate([
        SelectorProperties(100 /* Specificity.Id */, 3 /* Rarity.Id */, Match.Static),
        __metadata("design:paramtypes", [String])
    ], IdSelector);
    return IdSelector;
}(SimpleSelector));
export { IdSelector };
var TypeSelector = /** @class */ (function (_super) {
    __extends(TypeSelector, _super);
    function TypeSelector(cssType) {
        var _this = _super.call(this) || this;
        _this.cssType = cssType;
        return _this;
    }
    TypeSelector.prototype.toString = function () {
        return "".concat(this.cssType).concat(wrap(this.combinator));
    };
    TypeSelector.prototype.match = function (node) {
        return node.cssType === this.cssType;
    };
    TypeSelector.prototype.lookupSort = function (sorter, base) {
        sorter.sortByType(this.cssType, base || this);
    };
    TypeSelector = __decorate([
        SelectorProperties(1 /* Specificity.Type */, 1 /* Rarity.Type */, Match.Static),
        __metadata("design:paramtypes", [String])
    ], TypeSelector);
    return TypeSelector;
}(SimpleSelector));
export { TypeSelector };
var ClassSelector = /** @class */ (function (_super) {
    __extends(ClassSelector, _super);
    function ClassSelector(cssClass) {
        var _this = _super.call(this) || this;
        _this.cssClass = cssClass;
        return _this;
    }
    ClassSelector.prototype.toString = function () {
        return ".".concat(this.cssClass).concat(wrap(this.combinator));
    };
    ClassSelector.prototype.match = function (node) {
        return node.cssClasses && node.cssClasses.has(this.cssClass);
    };
    ClassSelector.prototype.lookupSort = function (sorter, base) {
        sorter.sortByClass(this.cssClass, base || this);
    };
    ClassSelector = __decorate([
        SelectorProperties(10 /* Specificity.Class */, 2 /* Rarity.Class */, Match.Static),
        __metadata("design:paramtypes", [String])
    ], ClassSelector);
    return ClassSelector;
}(SimpleSelector));
export { ClassSelector };
var AttributeSelector = /** @class */ (function (_super) {
    __extends(AttributeSelector, _super);
    function AttributeSelector(attribute, test, value, ignoreCase) {
        var _this = _super.call(this) || this;
        _this.attribute = attribute;
        _this.test = test;
        _this.value = value;
        _this.ignoreCase = ignoreCase;
        return _this;
    }
    AttributeSelector.prototype.toString = function () {
        var _a;
        return "[".concat(this.attribute).concat(wrap((_a = AttributeSelectorOperator[this.test]) !== null && _a !== void 0 ? _a : this.test)).concat(this.value || '', "]").concat(wrap(this.combinator));
    };
    AttributeSelector.prototype.match = function (node) {
        var attr = node[this.attribute];
        if (this.test === 'exists') {
            return !isNullOrUndefined(attr);
        }
        if (!this.value) {
            return false;
        }
        // Now, convert value to string
        attr += '';
        if (this.ignoreCase) {
            attr = attr.toLowerCase();
            this.value = this.value.toLowerCase();
        }
        // =
        if (this.test === 'equals') {
            return attr === this.value;
        }
        // ^=
        if (this.test === 'start') {
            return attr.startsWith(this.value);
        }
        // $=
        if (this.test === 'end') {
            return attr.endsWith(this.value);
        }
        // *=
        if (this.test === 'any') {
            return attr.indexOf(this.value) !== -1;
        }
        // ~=
        if (this.test === 'element') {
            var words = attr.split(' ');
            return words && words.indexOf(this.value) !== -1;
        }
        // |=
        if (this.test === 'hyphen') {
            return attr === this.value || attr.startsWith(this.value + '-');
        }
        return false;
    };
    AttributeSelector.prototype.mayMatch = function (node) {
        return true;
    };
    AttributeSelector.prototype.trackChanges = function (node, map) {
        map.addAttribute(node, this.attribute);
    };
    AttributeSelector = __decorate([
        SelectorProperties(10 /* Specificity.Attribute */, 0 /* Rarity.Attribute */, Match.Dynamic),
        __metadata("design:paramtypes", [String, String, String, Boolean])
    ], AttributeSelector);
    return AttributeSelector;
}(SimpleSelector));
export { AttributeSelector };
var PseudoClassSelector = /** @class */ (function (_super) {
    __extends(PseudoClassSelector, _super);
    function PseudoClassSelector(cssPseudoClass) {
        var _this = _super.call(this) || this;
        _this.cssPseudoClass = cssPseudoClass;
        return _this;
    }
    PseudoClassSelector.prototype.toString = function () {
        return ":".concat(this.cssPseudoClass).concat(wrap(this.combinator));
    };
    PseudoClassSelector.prototype.match = function (node) {
        return node.cssPseudoClasses && node.cssPseudoClasses.has(this.cssPseudoClass);
    };
    PseudoClassSelector.prototype.mayMatch = function (node) {
        return true;
    };
    PseudoClassSelector.prototype.trackChanges = function (node, map) {
        map.addPseudoClass(node, this.cssPseudoClass);
    };
    PseudoClassSelector = __decorate([
        SelectorProperties(10 /* Specificity.PseudoClass */, 0 /* Rarity.PseudoClass */, Match.Dynamic),
        __metadata("design:paramtypes", [String])
    ], PseudoClassSelector);
    return PseudoClassSelector;
}(SimpleSelector));
export { PseudoClassSelector };
var FunctionalPseudoClassSelector = /** @class */ (function (_super) {
    __extends(FunctionalPseudoClassSelector, _super);
    function FunctionalPseudoClassSelector(cssPseudoClass, dataType) {
        var _this = _super.call(this, cssPseudoClass) || this;
        var selectors = [];
        var needsHighestSpecificity = _this.specificity === -1 /* Specificity.SelectorListHighest */;
        var specificity = 0;
        if (Array.isArray(dataType)) {
            for (var _i = 0, dataType_1 = dataType; _i < dataType_1.length; _i++) {
                var asts = dataType_1[_i];
                var selector = createSelectorFromAst(asts);
                if (selector instanceof InvalidSelector) {
                    // Only forgiving selector list can ignore invalid selectors
                    if (_this.selectorListType !== 1 /* PseudoClassSelectorList.Forgiving */) {
                        selectors.splice(0);
                        specificity = 0;
                        break;
                    }
                    continue;
                }
                // The specificity of some pseudo-classes is replaced by the specificity of the most specific selector in its comma-separated argument of selectors
                if (needsHighestSpecificity && selector.specificity > specificity) {
                    specificity = selector.specificity;
                }
                selectors.push(selector);
            }
        }
        _this.selectors = selectors;
        _this.specificity = specificity;
        // Functional pseudo-classes become dynamic based on selectors in selector list
        _this.dynamic = _this.selectors.some(function (sel) { return sel.dynamic; });
        return _this;
    }
    FunctionalPseudoClassSelector.prototype.toString = function () {
        return ":".concat(this.cssPseudoClass, "(").concat(this.selectors.join(', '), ")").concat(wrap(this.combinator));
    };
    FunctionalPseudoClassSelector.prototype.match = function (node) {
        return false;
    };
    FunctionalPseudoClassSelector.prototype.mayMatch = function (node) {
        return true;
    };
    FunctionalPseudoClassSelector.prototype.trackChanges = function (node, map) {
        this.selectors.forEach(function (sel) { return sel.trackChanges(node, map); });
    };
    return FunctionalPseudoClassSelector;
}(PseudoClassSelector));
export { FunctionalPseudoClassSelector };
var NotFunctionalPseudoClassSelector = /** @class */ (function (_super) {
    __extends(NotFunctionalPseudoClassSelector, _super);
    function NotFunctionalPseudoClassSelector() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    NotFunctionalPseudoClassSelector.prototype.match = function (node) {
        return !this.selectors.some(function (sel) { return sel.match(node); });
    };
    NotFunctionalPseudoClassSelector = __decorate([
        FunctionalPseudoClassProperties(-1 /* Specificity.SelectorListHighest */, 0 /* Rarity.PseudoClass */, 0 /* PseudoClassSelectorList.Regular */)
    ], NotFunctionalPseudoClassSelector);
    return NotFunctionalPseudoClassSelector;
}(FunctionalPseudoClassSelector));
export { NotFunctionalPseudoClassSelector };
var IsFunctionalPseudoClassSelector = /** @class */ (function (_super) {
    __extends(IsFunctionalPseudoClassSelector, _super);
    function IsFunctionalPseudoClassSelector() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    IsFunctionalPseudoClassSelector.prototype.match = function (node) {
        return this.selectors.some(function (sel) { return sel.match(node); });
    };
    IsFunctionalPseudoClassSelector.prototype.lookupSort = function (sorter, base) {
        // A faster lookup can be performed when selector list contains just a single selector
        if (this.selectors.length === 1) {
            this.selectors[0].lookupSort(sorter, base || this);
        }
        else {
            _super.prototype.lookupSort.call(this, sorter, base || this);
        }
    };
    IsFunctionalPseudoClassSelector = __decorate([
        FunctionalPseudoClassProperties(-1 /* Specificity.SelectorListHighest */, 0 /* Rarity.PseudoClass */, 1 /* PseudoClassSelectorList.Forgiving */)
    ], IsFunctionalPseudoClassSelector);
    return IsFunctionalPseudoClassSelector;
}(FunctionalPseudoClassSelector));
export { IsFunctionalPseudoClassSelector };
var WhereFunctionalPseudoClassSelector = /** @class */ (function (_super) {
    __extends(WhereFunctionalPseudoClassSelector, _super);
    function WhereFunctionalPseudoClassSelector() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    WhereFunctionalPseudoClassSelector.prototype.match = function (node) {
        return this.selectors.some(function (sel) { return sel.match(node); });
    };
    WhereFunctionalPseudoClassSelector.prototype.lookupSort = function (sorter, base) {
        // A faster lookup can be performed when selector list contains just a single selector
        if (this.selectors.length === 1) {
            this.selectors[0].lookupSort(sorter, base || this);
        }
        else {
            _super.prototype.lookupSort.call(this, sorter, base || this);
        }
    };
    WhereFunctionalPseudoClassSelector = __decorate([
        FunctionalPseudoClassProperties(0 /* Specificity.Zero */, 0 /* Rarity.PseudoClass */, 1 /* PseudoClassSelectorList.Forgiving */)
    ], WhereFunctionalPseudoClassSelector);
    return WhereFunctionalPseudoClassSelector;
}(FunctionalPseudoClassSelector));
export { WhereFunctionalPseudoClassSelector };
var SimpleSelectorSequence = /** @class */ (function (_super) {
    __extends(SimpleSelectorSequence, _super);
    function SimpleSelectorSequence(selectors) {
        var _this = _super.call(this) || this;
        _this.selectors = selectors;
        _this.specificity = selectors.reduce(function (sum, sel) { return sel.specificity + sum; }, 0);
        _this.head = selectors.reduce(function (prev, curr) { return (!prev || curr.rarity > prev.rarity ? curr : prev); }, null);
        _this.dynamic = selectors.some(function (sel) { return sel.dynamic; });
        return _this;
    }
    SimpleSelectorSequence.prototype.toString = function () {
        return "".concat(this.selectors.join('')).concat(wrap(this.combinator));
    };
    SimpleSelectorSequence.prototype.match = function (node) {
        return this.selectors.every(function (sel) { return sel.match(node); });
    };
    SimpleSelectorSequence.prototype.mayMatch = function (node) {
        return this.selectors.every(function (sel) { return sel.mayMatch(node); });
    };
    SimpleSelectorSequence.prototype.trackChanges = function (node, map) {
        this.selectors.forEach(function (sel) { return sel.trackChanges(node, map); });
    };
    SimpleSelectorSequence.prototype.lookupSort = function (sorter, base) {
        this.head.lookupSort(sorter, base || this);
    };
    return SimpleSelectorSequence;
}(SimpleSelector));
export { SimpleSelectorSequence };
var ComplexSelector = /** @class */ (function (_super) {
    __extends(ComplexSelector, _super);
    function ComplexSelector(selectors) {
        var _this = _super.call(this) || this;
        _this.selectors = selectors;
        var siblingsToGroup;
        var currentGroup;
        var groups = [];
        _this.specificity = 0;
        _this.dynamic = false;
        for (var i = selectors.length - 1; i >= 0; i--) {
            var sel = selectors[i];
            switch (sel.combinator) {
                case undefined:
                case Combinator.descendant:
                    siblingsToGroup = [];
                    currentGroup = [siblingsToGroup];
                    groups.push(currentGroup);
                    break;
                case Combinator.child:
                    siblingsToGroup = [];
                    currentGroup.push(siblingsToGroup);
                    break;
                case Combinator.adjacent:
                case Combinator.sibling:
                    break;
                default:
                    throw new Error("Unsupported combinator \"".concat(sel.combinator, "\" for selector ").concat(sel, "."));
            }
            _this.specificity += sel.specificity;
            if (sel.dynamic) {
                _this.dynamic = true;
            }
            siblingsToGroup.push(sel);
        }
        _this.groups = groups.map(function (g) { return new Selector.ChildGroup(g.map(function (selectors) { return (selectors.length > 1 ? new Selector.SiblingGroup(selectors) : selectors[0]); })); });
        _this.last = selectors[selectors.length - 1];
        return _this;
    }
    ComplexSelector.prototype.toString = function () {
        return this.selectors.join('');
    };
    ComplexSelector.prototype.match = function (node) {
        return this.groups.every(function (group, i) {
            var _a;
            if (i === 0) {
                node = group.getMatchingNode(node, true);
                return !!node;
            }
            else {
                var ancestor = node;
                while ((ancestor = (_a = ancestor.parent) !== null && _a !== void 0 ? _a : ancestor._modalParent)) {
                    if ((node = group.getMatchingNode(ancestor, true))) {
                        return true;
                    }
                }
                return false;
            }
        });
    };
    ComplexSelector.prototype.mayMatch = function (node) {
        return false;
    };
    ComplexSelector.prototype.trackChanges = function (node, map) {
        this.selectors.forEach(function (sel) { return sel.trackChanges(node, map); });
    };
    ComplexSelector.prototype.lookupSort = function (sorter, base) {
        this.last.lookupSort(sorter, base || this);
    };
    ComplexSelector.prototype.accumulateChanges = function (node, map) {
        if (!this.dynamic) {
            return this.match(node);
        }
        var bounds = [];
        var mayMatch = this.groups.every(function (group, i) {
            if (i === 0) {
                var nextNode = group.getMatchingNode(node, false);
                bounds.push({ left: node, right: node });
                node = nextNode;
                return !!node;
            }
            else {
                var ancestor = node;
                while ((ancestor = ancestor.parent)) {
                    var nextNode = group.getMatchingNode(ancestor, false);
                    if (nextNode) {
                        bounds.push({ left: ancestor, right: null });
                        node = nextNode;
                        return true;
                    }
                }
                return false;
            }
        });
        // Calculating the right bounds for each selectors won't save much
        if (!mayMatch) {
            return false;
        }
        if (!map) {
            return mayMatch;
        }
        for (var i = 0; i < this.groups.length; i++) {
            var group_1 = this.groups[i];
            if (!group_1.dynamic) {
                continue;
            }
            var bound = bounds[i];
            var node_1 = bound.left;
            do {
                if (group_1.mayMatch(node_1)) {
                    group_1.trackChanges(node_1, map);
                }
            } while (node_1 !== bound.right && (node_1 = node_1.parent));
        }
        return mayMatch;
    };
    return ComplexSelector;
}(SelectorCore));
export { ComplexSelector };
export var Selector;
(function (Selector) {
    // Non-spec. Selector sequences are grouped by ancestor then by child combinators for easier backtracking.
    var ChildGroup = /** @class */ (function (_super) {
        __extends(ChildGroup, _super);
        function ChildGroup(selectors) {
            var _this = _super.call(this) || this;
            _this.selectors = selectors;
            _this.dynamic = selectors.some(function (sel) { return sel.dynamic; });
            return _this;
        }
        ChildGroup.prototype.getMatchingNode = function (node, strict) {
            var funcName = strict ? 'match' : 'mayMatch';
            return this.selectors.every(function (sel, i) { return (node = i === 0 ? node : node.parent) && sel[funcName](node); }) ? node : null;
        };
        ChildGroup.prototype.match = function (node) {
            return this.getMatchingNode(node, true) != null;
        };
        ChildGroup.prototype.mayMatch = function (node) {
            return this.getMatchingNode(node, false) != null;
        };
        ChildGroup.prototype.trackChanges = function (node, map) {
            this.selectors.forEach(function (sel, i) {
                if (i === 0) {
                    node && sel.trackChanges(node, map);
                }
                else {
                    node = node.parent;
                    if (node && sel.mayMatch(node)) {
                        sel.trackChanges(node, map);
                    }
                }
            });
        };
        return ChildGroup;
    }(SelectorBase));
    Selector.ChildGroup = ChildGroup;
    var SiblingGroup = /** @class */ (function (_super) {
        __extends(SiblingGroup, _super);
        function SiblingGroup(selectors) {
            var _this = _super.call(this) || this;
            _this.selectors = selectors;
            _this.dynamic = selectors.some(function (sel) { return sel.dynamic; });
            return _this;
        }
        SiblingGroup.prototype.match = function (node) {
            return this.selectors.every(function (sel, i) {
                if (i === 0) {
                    return node && sel.match(node);
                }
                if (sel.combinator === Combinator.adjacent) {
                    node = getNodePreviousDirectSibling(node);
                    return node && sel.match(node);
                }
                // Sibling combinator
                var isMatching = false;
                eachNodePreviousGeneralSibling(node, function (sibling) {
                    isMatching = sel.match(sibling);
                    return !isMatching;
                });
                return isMatching;
            });
        };
        SiblingGroup.prototype.mayMatch = function (node) {
            return this.selectors.every(function (sel, i) {
                if (i === 0) {
                    return node && sel.mayMatch(node);
                }
                if (sel.combinator === Combinator.adjacent) {
                    node = getNodePreviousDirectSibling(node);
                    return node && sel.mayMatch(node);
                }
                // Sibling combinator
                var isMatching = false;
                eachNodePreviousGeneralSibling(node, function (sibling) {
                    isMatching = sel.mayMatch(sibling);
                    return !isMatching;
                });
                return isMatching;
            });
        };
        SiblingGroup.prototype.trackChanges = function (node, map) {
            this.selectors.forEach(function (sel, i) {
                if (i === 0) {
                    if (node) {
                        sel.trackChanges(node, map);
                    }
                }
                else {
                    if (sel.combinator === Combinator.adjacent) {
                        node = getNodePreviousDirectSibling(node);
                        if (node && sel.mayMatch(node)) {
                            sel.trackChanges(node, map);
                        }
                    }
                    else {
                        // Sibling combinator
                        var matchingSibling_1;
                        eachNodePreviousGeneralSibling(node, function (sibling) {
                            var isMatching = sel.mayMatch(sibling);
                            if (isMatching) {
                                matchingSibling_1 = sibling;
                            }
                            return !isMatching;
                        });
                        if (matchingSibling_1) {
                            sel.trackChanges(matchingSibling_1, map);
                        }
                    }
                }
            });
        };
        return SiblingGroup;
    }(SelectorBase));
    Selector.SiblingGroup = SiblingGroup;
})(Selector || (Selector = {}));
var RuleSet = /** @class */ (function () {
    function RuleSet(selectors, declarations) {
        var _this = this;
        this.selectors = selectors;
        this.declarations = declarations;
        this.selectors.forEach(function (sel) { return (sel.ruleset = _this); });
    }
    RuleSet.prototype.toString = function () {
        var desc = "".concat(this.selectors.join(', '), " {").concat(this.declarations.map(function (d, i) { return "".concat(i === 0 ? ' ' : '').concat(d.property, ": ").concat(d.value); }).join('; '), " }");
        if (this.mediaQueryString) {
            desc = "@media ".concat(this.mediaQueryString, " { ").concat(desc, " }");
        }
        return desc;
    };
    RuleSet.prototype.lookupSort = function (sorter) {
        this.selectors.forEach(function (sel) { return sel.lookupSort(sorter); });
    };
    return RuleSet;
}());
export { RuleSet };
export function fromAstNode(astRule) {
    var declarations = astRule.declarations.filter(isDeclaration).map(createDeclaration);
    var selectors = astRule.selectors.map(createSelector);
    return new RuleSet(selectors, declarations);
}
function createDeclaration(decl) {
    return { property: isCssVariable(decl.property) ? decl.property : decl.property.toLowerCase(), value: decl.value };
}
function createSimpleSelectorFromAst(ast) {
    if (ast.type === 'attribute') {
        if (ast.name === 'class') {
            return new ClassSelector(ast.value);
        }
        if (ast.name === 'id') {
            return new IdSelector(ast.value);
        }
        return new AttributeSelector(ast.name, ast.action, ast.value, !!ast.ignoreCase);
    }
    if (ast.type === 'tag') {
        return new TypeSelector(ast.name.replace('-', '').toLowerCase());
    }
    if (ast.type === 'pseudo') {
        if (ast.name === 'is') {
            return new IsFunctionalPseudoClassSelector(ast.name, ast.data);
        }
        if (ast.name === 'where') {
            return new WhereFunctionalPseudoClassSelector(ast.name, ast.data);
        }
        if (ast.name === 'not') {
            return new NotFunctionalPseudoClassSelector(ast.name, ast.data);
        }
        return new PseudoClassSelector(ast.name);
    }
    if (ast.type === 'universal') {
        return new UniversalSelector();
    }
    return new InvalidSelector(new Error(ast.type));
}
function createSimpleSelectorSequenceFromAst(asts) {
    if (asts.length === 0) {
        return new InvalidSelector(new Error('Empty simple selector sequence.'));
    }
    if (asts.length === 1) {
        return createSimpleSelectorFromAst(asts[0]);
    }
    var sequenceSelectors = [];
    for (var _i = 0, asts_1 = asts; _i < asts_1.length; _i++) {
        var ast = asts_1[_i];
        var selector = createSimpleSelectorFromAst(ast);
        if (selector instanceof InvalidSelector) {
            return selector;
        }
        sequenceSelectors.push(selector);
    }
    return new SimpleSelectorSequence(sequenceSelectors);
}
function createSelectorFromAst(asts) {
    var result;
    if (asts.length === 0) {
        return new InvalidSelector(new Error('Empty selector.'));
    }
    if (asts.length === 1) {
        return createSimpleSelectorFromAst(asts[0]);
    }
    var simpleSelectorSequences = [];
    var sequenceAsts = [];
    var combinatorCount = 0;
    for (var _i = 0, asts_2 = asts; _i < asts_2.length; _i++) {
        var ast = asts_2[_i];
        var combinator = Combinator[ast.type];
        // Combinator means the end of a sequence
        if (combinator != null) {
            var selector = createSimpleSelectorSequenceFromAst(sequenceAsts);
            if (selector instanceof InvalidSelector) {
                return selector;
            }
            selector.combinator = combinator;
            simpleSelectorSequences.push(selector);
            combinatorCount++;
            // Cleanup stored selectors for the new sequence to take place
            sequenceAsts = [];
        }
        else {
            sequenceAsts.push(ast);
        }
    }
    if (combinatorCount > 0) {
        // Create a sequence using the remaining selectors after the last combinator
        if (sequenceAsts.length) {
            var selector = createSimpleSelectorSequenceFromAst(sequenceAsts);
            if (selector instanceof InvalidSelector) {
                return selector;
            }
            simpleSelectorSequences.push(selector);
        }
        return new ComplexSelector(simpleSelectorSequences);
    }
    return createSimpleSelectorSequenceFromAst(sequenceAsts);
}
export function createSelector(sel) {
    try {
        var result = convertToCSSWhatSelector(sel);
        if (!(result === null || result === void 0 ? void 0 : result.length)) {
            return new InvalidSelector(new Error('Empty selector'));
        }
        return createSelectorFromAst(result[0]);
    }
    catch (e) {
        return new InvalidSelector(e);
    }
}
function isDeclaration(node) {
    return node.type === 'declaration';
}
export function matchMediaQueryString(mediaQueryString, cachedQueries) {
    // It can be a single or multiple queries in case of nested media queries
    var mediaQueryStrings = mediaQueryString.split(MEDIA_QUERY_SEPARATOR);
    return mediaQueryStrings.every(function (mq) {
        var isMatching;
        // Query has already been validated
        if (cachedQueries.includes(mq)) {
            isMatching = true;
        }
        else {
            isMatching = checkIfMediaQueryMatches(mq);
            if (isMatching) {
                cachedQueries.push(mq);
            }
        }
        return isMatching;
    });
}
var SelectorScope = /** @class */ (function () {
    function SelectorScope() {
        this.id = {};
        this.class = {};
        this.type = {};
        this.universal = [];
        this.position = 0;
    }
    SelectorScope.prototype.getSelectorCandidates = function (node) {
        var _this = this;
        var cssClasses = node.cssClasses, id = node.id, cssType = node.cssType;
        var selectorClasses = [this.universal, this.id[id], this.type[cssType]];
        if (cssClasses && cssClasses.size) {
            cssClasses.forEach(function (c) { return selectorClasses.push(_this.class[c]); });
        }
        return selectorClasses.reduce(function (cur, next) { return cur.concat(next || []); }, []);
    };
    SelectorScope.prototype.sortById = function (id, sel) {
        this.addToMap(this.id, id, sel);
    };
    SelectorScope.prototype.sortByClass = function (cssClass, sel) {
        this.addToMap(this.class, cssClass, sel);
    };
    SelectorScope.prototype.sortByType = function (cssType, sel) {
        this.addToMap(this.type, cssType, sel);
    };
    SelectorScope.prototype.sortAsUniversal = function (sel) {
        this.universal.push(this.makeDocSelector(sel));
    };
    SelectorScope.prototype.addToMap = function (map, head, sel) {
        if (!map[head]) {
            map[head] = [];
        }
        map[head].push(this.makeDocSelector(sel));
    };
    SelectorScope.prototype.makeDocSelector = function (sel) {
        sel.pos = this.position++;
        return sel;
    };
    return SelectorScope;
}());
export { SelectorScope };
var MediaQuerySelectorScope = /** @class */ (function (_super) {
    __extends(MediaQuerySelectorScope, _super);
    function MediaQuerySelectorScope(mediaQueryString) {
        var _this = _super.call(this) || this;
        _this._mediaQueryString = mediaQueryString;
        return _this;
    }
    Object.defineProperty(MediaQuerySelectorScope.prototype, "mediaQueryString", {
        get: function () {
            return this._mediaQueryString;
        },
        enumerable: false,
        configurable: true
    });
    return MediaQuerySelectorScope;
}(SelectorScope));
export { MediaQuerySelectorScope };
var StyleSheetSelectorScope = /** @class */ (function (_super) {
    __extends(StyleSheetSelectorScope, _super);
    function StyleSheetSelectorScope(rulesets) {
        var _this = _super.call(this) || this;
        _this.lookupRulesets(rulesets);
        return _this;
    }
    StyleSheetSelectorScope.prototype.createMediaQuerySelectorScope = function (mediaQueryString) {
        var selectorScope = new MediaQuerySelectorScope(mediaQueryString);
        selectorScope.position = this.position;
        if (this.mediaQuerySelectorScopes) {
            this.mediaQuerySelectorScopes.push(selectorScope);
        }
        else {
            this.mediaQuerySelectorScopes = [selectorScope];
        }
        return selectorScope;
    };
    StyleSheetSelectorScope.prototype.lookupRulesets = function (rulesets) {
        var lastMediaSelectorScope;
        for (var i = 0, length_1 = rulesets.length; i < length_1; i++) {
            var ruleset = rulesets[i];
            if (lastMediaSelectorScope && lastMediaSelectorScope.mediaQueryString !== ruleset.mediaQueryString) {
                // Once done with current media query scope, update stylesheet scope position
                this.position = lastMediaSelectorScope.position;
                lastMediaSelectorScope = null;
            }
            if (ruleset.mediaQueryString) {
                // Create media query selector scope and register selector lookups there
                if (!lastMediaSelectorScope) {
                    lastMediaSelectorScope = this.createMediaQuerySelectorScope(ruleset.mediaQueryString);
                }
                ruleset.lookupSort(lastMediaSelectorScope);
            }
            else {
                ruleset.lookupSort(this);
            }
        }
        // If reference of last media selector scope is still kept, update stylesheet scope position
        if (lastMediaSelectorScope) {
            this.position = lastMediaSelectorScope.position;
            lastMediaSelectorScope = null;
        }
    };
    StyleSheetSelectorScope.prototype.query = function (node) {
        var selectorsMatch = new SelectorsMatch();
        var selectors = this.getSelectorCandidates(node);
        // Validate media queries and include their selectors if needed
        if (this.mediaQuerySelectorScopes) {
            // Cache media query results to avoid validations of other identical queries
            var validatedMediaQueries = [];
            for (var i = 0, length_2 = this.mediaQuerySelectorScopes.length; i < length_2; i++) {
                var selectorScope = this.mediaQuerySelectorScopes[i];
                var isMatchingAllQueries = matchMediaQueryString(selectorScope.mediaQueryString, validatedMediaQueries);
                if (isMatchingAllQueries) {
                    var mediaQuerySelectors = selectorScope.getSelectorCandidates(node);
                    selectors.push.apply(selectors, mediaQuerySelectors);
                }
            }
        }
        selectorsMatch.selectors = selectors.filter(function (sel) { return sel.accumulateChanges(node, selectorsMatch); }).sort(function (a, b) { return a.specificity - b.specificity || a.pos - b.pos; });
        return selectorsMatch;
    };
    return StyleSheetSelectorScope;
}(SelectorScope));
export { StyleSheetSelectorScope };
var SelectorsMatch = /** @class */ (function () {
    function SelectorsMatch() {
        this.changeMap = new Map();
    }
    SelectorsMatch.prototype.addAttribute = function (node, attribute) {
        var deps = this.properties(node);
        if (!deps.attributes) {
            deps.attributes = new Set();
        }
        deps.attributes.add(attribute);
    };
    SelectorsMatch.prototype.addPseudoClass = function (node, pseudoClass) {
        var deps = this.properties(node);
        if (!deps.pseudoClasses) {
            deps.pseudoClasses = new Set();
        }
        deps.pseudoClasses.add(pseudoClass);
    };
    SelectorsMatch.prototype.properties = function (node) {
        var set = this.changeMap.get(node);
        if (!set) {
            this.changeMap.set(node, (set = {}));
        }
        return set;
    };
    return SelectorsMatch;
}());
export { SelectorsMatch };
export var CSSHelper = {
    createSelector: createSelector,
    SelectorCore: SelectorCore,
    SimpleSelector: SimpleSelector,
    InvalidSelector: InvalidSelector,
    UniversalSelector: UniversalSelector,
    TypeSelector: TypeSelector,
    ClassSelector: ClassSelector,
    AttributeSelector: AttributeSelector,
    PseudoClassSelector: PseudoClassSelector,
    SimpleSelectorSequence: SimpleSelectorSequence,
    Selector: Selector,
    RuleSet: RuleSet,
    SelectorScope: SelectorScope,
    MediaQuerySelectorScope: MediaQuerySelectorScope,
    StyleSheetSelectorScope: StyleSheetSelectorScope,
    fromAstNode: fromAstNode,
    SelectorsMatch: SelectorsMatch,
};
//# sourceMappingURL=css-selector.js.map