import { Trace } from '../../trace';
import { CoreTypes } from '../../core-types';
import { Length } from './style-properties';
export function cleanupImportantFlags(value, propertyName) {
    if (typeof value !== 'string') {
        return '' + value;
    }
    var index = value === null || value === void 0 ? void 0 : value.indexOf('!important');
    if (index >= 0) {
        if (Trace.isEnabled()) {
            Trace.write("The !important css rule is currently not supported. Property: ".concat(propertyName), Trace.categories.Style, Trace.messageType.warn);
        }
        return value.substring(0, index).trim();
    }
    return value;
}
/**
 * Matches whitespace except if the whitespace is contained in parenthesis - ex. rgb(a), hsl color.
 */
var PARTS_RE = /\s(?![^(]*\))/;
/**
 * Matches a Length value with or without a unit
 */
var LENGTH_RE = /^-?[0-9]+[a-zA-Z%]*?$/;
/**
 * Checks if the value is a Length or 0
 */
var isLength = function (v) { return v === '0' || LENGTH_RE.test(v); };
export function parseCSSShorthand(value) {
    var parts = value.trim().split(PARTS_RE);
    var first = parts[0];
    if (['', 'none', 'unset'].includes(first)) {
        return null;
    }
    else {
        var invalidColors_1 = ['inset', 'unset'];
        var inset = parts.includes('inset');
        var last = parts[parts.length - 1];
        var color_1 = 'black';
        if (first && !isLength(first) && !invalidColors_1.includes(first)) {
            color_1 = first;
        }
        else if (last && !isLength(last) && !invalidColors_1.includes(last)) {
            color_1 = last;
        }
        var values = parts
            .filter(function (n) { return !invalidColors_1.includes(n); })
            .filter(function (n) { return n !== color_1; })
            .map(function (val) {
            try {
                return Length.parse(val);
            }
            catch (err) {
                return CoreTypes.zeroLength;
            }
        });
        return {
            inset: inset,
            color: color_1,
            values: values,
        };
    }
}
//# sourceMappingURL=css-utils.js.map