import { Observable } from '../../../data/observable';
import { Trace } from '../../../trace';
var Style = /** @class */ (function (_super) {
    __extends(Style, _super);
    function Style(ownerView) {
        var _this = _super.call(this) || this;
        _this.unscopedCssVariables = new Map();
        _this.scopedCssVariables = new Map();
        // HACK: Could not find better way for cross platform WeakRef type checking.
        if (ownerView.constructor.toString().indexOf('[native code]') !== -1) {
            _this.viewRef = ownerView;
        }
        else {
            _this.viewRef = new WeakRef(ownerView);
        }
        return _this;
    }
    Style.prototype.setScopedCssVariable = function (varName, value) {
        this.scopedCssVariables.set(varName, value);
    };
    Style.prototype.setUnscopedCssVariable = function (varName, value) {
        this.unscopedCssVariables.set(varName, value);
    };
    Style.prototype.removeScopedCssVariable = function (varName) {
        this.scopedCssVariables.delete(varName);
    };
    Style.prototype.removeUnscopedCssVariable = function (varName) {
        this.unscopedCssVariables.delete(varName);
    };
    Style.prototype.getCssVariable = function (varName) {
        var view = this.view;
        if (!view) {
            return null;
        }
        if (this.unscopedCssVariables.has(varName)) {
            return this.unscopedCssVariables.get(varName);
        }
        if (this.scopedCssVariables.has(varName)) {
            return this.scopedCssVariables.get(varName);
        }
        if (!view.parent || !view.parent.style) {
            return null;
        }
        return view.parent.style.getCssVariable(varName);
    };
    Style.prototype.resetScopedCssVariables = function () {
        this.scopedCssVariables.clear();
    };
    Style.prototype.resetUnscopedCssVariables = function () {
        this.unscopedCssVariables.clear();
    };
    Style.prototype.toString = function () {
        var view = this.viewRef.get();
        if (!view) {
            Trace.write("toString() of Style cannot execute correctly because \".viewRef\" is cleared", Trace.categories.Animation, Trace.messageType.warn);
            return '';
        }
        return "".concat(view, ".style");
    };
    Object.defineProperty(Style.prototype, "view", {
        get: function () {
            if (this.viewRef) {
                return this.viewRef.get();
            }
            return undefined;
        },
        enumerable: false,
        configurable: true
    });
    return Style;
}(Observable));
export { Style };
Style.prototype.PropertyBag = /** @class */ (function () {
    function class_1() {
    }
    return class_1;
}());
//# sourceMappingURL=index.js.map