import { SwitchBase, checkedProperty, offBackgroundColorProperty } from './switch-common';
import { colorProperty, backgroundColorProperty, backgroundInternalProperty } from '../styling/style-properties';
import { Color } from '../../color';
import { AndroidHelper } from '../core/view';
export * from './switch-common';
var CheckedChangeListener;
function initializeCheckedChangeListener() {
    if (CheckedChangeListener) {
        return;
    }
    var CheckedChangeListenerImpl = /** @class */ (function (_super) {
    __extends(CheckedChangeListenerImpl, _super);
    function CheckedChangeListenerImpl(owner) {
        var _this = _super.call(this) || this;
        _this.owner = owner;
        return global.__native(_this);
    }
    CheckedChangeListenerImpl.prototype.onCheckedChanged = function (buttonView, isChecked) {
        var owner = this.owner;
        checkedProperty.nativeValueChange(owner, isChecked);
    };
    var _a;
    CheckedChangeListenerImpl = __decorate([
        Interfaces([android.widget.CompoundButton.OnCheckedChangeListener]),
        __metadata("design:paramtypes", [typeof (_a = typeof Switch !== "undefined" && Switch) === "function" ? _a : Object])
    ], CheckedChangeListenerImpl);
    return CheckedChangeListenerImpl;
}(java.lang.Object));
    CheckedChangeListener = CheckedChangeListenerImpl;
}
var Switch = /** @class */ (function (_super) {
    __extends(Switch, _super);
    function Switch() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    Switch.prototype.createNativeView = function () {
        return new android.widget.Switch(this._context);
    };
    Switch.prototype.initNativeView = function () {
        _super.prototype.initNativeView.call(this);
        var nativeView = this.nativeViewProtected;
        initializeCheckedChangeListener();
        var listener = new CheckedChangeListener(this);
        nativeView.setOnCheckedChangeListener(listener);
        nativeView.listener = listener;
    };
    Switch.prototype.disposeNativeView = function () {
        var nativeView = this.nativeViewProtected;
        if (nativeView.listener) {
            nativeView.listener.owner = null;
        }
        _super.prototype.disposeNativeView.call(this);
    };
    Switch.prototype.setNativeBackgroundColor = function (value) {
        if (this.nativeViewProtected) {
            var drawable = this.nativeViewProtected.getTrackDrawable();
            if (value instanceof Color) {
                AndroidHelper.setDrawableColor(value.android, drawable, androidx.core.graphics.BlendModeCompat.SRC_OVER);
            }
            else {
                AndroidHelper.clearDrawableColor(drawable);
            }
        }
    };
    Switch.prototype._onCheckedPropertyChanged = function (newValue) {
        _super.prototype._onCheckedPropertyChanged.call(this, newValue);
        if (this.offBackgroundColor) {
            if (!newValue) {
                this.setNativeBackgroundColor(this.offBackgroundColor);
            }
            else {
                this.setNativeBackgroundColor(this.backgroundColor);
            }
        }
    };
    Switch.prototype[checkedProperty.getDefault] = function () {
        return false;
    };
    Switch.prototype[checkedProperty.setNative] = function (value) {
        this.nativeViewProtected.setChecked(value);
    };
    Switch.prototype[colorProperty.getDefault] = function () {
        return -1;
    };
    Switch.prototype[colorProperty.setNative] = function (value) {
        var drawable = this.nativeViewProtected.getThumbDrawable();
        if (value instanceof Color) {
            AndroidHelper.setDrawableColor(value.android, drawable, androidx.core.graphics.BlendModeCompat.SRC_ATOP);
        }
        else {
            AndroidHelper.clearDrawableColor(drawable);
        }
    };
    Switch.prototype[backgroundColorProperty.getDefault] = function () {
        return -1;
    };
    Switch.prototype[backgroundColorProperty.setNative] = function (value) {
        if (!this.offBackgroundColor || this.checked) {
            this.setNativeBackgroundColor(value);
        }
    };
    Switch.prototype[backgroundInternalProperty.getDefault] = function () {
        return null;
    };
    Switch.prototype[backgroundInternalProperty.setNative] = function (value) {
        //
    };
    Switch.prototype[offBackgroundColorProperty.getDefault] = function () {
        return -1;
    };
    Switch.prototype[offBackgroundColorProperty.setNative] = function (value) {
        if (!this.checked) {
            this.setNativeBackgroundColor(value);
        }
    };
    return Switch;
}(SwitchBase));
export { Switch };
//# sourceMappingURL=index.android.js.map