import { SwitchBase, checkedProperty, offBackgroundColorProperty } from './switch-common';
import { colorProperty, backgroundColorProperty, backgroundInternalProperty } from '../styling/style-properties';
import { Color } from '../../color';
import { iOSNativeHelper, layout } from '../../utils';
export * from './switch-common';
var majorVersion = iOSNativeHelper.MajorVersion;
var SwitchChangeHandlerImpl = /** @class */ (function (_super) {
    __extends(SwitchChangeHandlerImpl, _super);
    function SwitchChangeHandlerImpl() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    SwitchChangeHandlerImpl.initWithOwner = function (owner) {
        var handler = SwitchChangeHandlerImpl.new();
        handler._owner = owner;
        return handler;
    };
    SwitchChangeHandlerImpl.prototype.valueChanged = function (sender) {
        var _a;
        var owner = (_a = this._owner) === null || _a === void 0 ? void 0 : _a.deref();
        if (owner) {
            checkedProperty.nativeValueChange(owner, sender.on);
        }
    };
    SwitchChangeHandlerImpl.ObjCExposedMethods = {
        valueChanged: { returns: interop.types.void, params: [UISwitch] },
    };
    return SwitchChangeHandlerImpl;
}(NSObject));
var zeroSize = { width: 0, height: 0 };
var Switch = /** @class */ (function (_super) {
    __extends(Switch, _super);
    function Switch() {
        var _this = _super.call(this) || this;
        _this.width = 51;
        _this.height = 31;
        return _this;
    }
    Switch.prototype.createNativeView = function () {
        return UISwitch.new();
    };
    Switch.prototype.initNativeView = function () {
        _super.prototype.initNativeView.call(this);
        var nativeView = this.nativeViewProtected;
        this._handler = SwitchChangeHandlerImpl.initWithOwner(new WeakRef(this));
        nativeView.addTargetActionForControlEvents(this._handler, 'valueChanged', 4096 /* UIControlEvents.ValueChanged */);
    };
    Switch.prototype.disposeNativeView = function () {
        this._handler = null;
        _super.prototype.disposeNativeView.call(this);
    };
    Switch.prototype.setNativeBackgroundColor = function (value) {
        if (value) {
            this.nativeViewProtected.onTintColor = value instanceof Color ? value.ios : value;
            this.nativeViewProtected.tintColor = value instanceof Color ? value.ios : value;
            this.nativeViewProtected.backgroundColor = value instanceof Color ? value.ios : value;
            this.nativeViewProtected.layer.cornerRadius = this.nativeViewProtected.frame.size.height / 2;
        }
        else {
            this.nativeViewProtected.onTintColor = null;
            this.nativeViewProtected.tintColor = null;
            this.nativeViewProtected.backgroundColor = null;
            this.nativeViewProtected.layer.cornerRadius = 0;
        }
    };
    Switch.prototype._onCheckedPropertyChanged = function (newValue) {
        // only add :checked pseudo handling on supported iOS versions
        // ios <13 works but causes glitchy animations when toggling
        // so we decided to keep the old behavior on older versions.
        if (majorVersion >= 13) {
            _super.prototype._onCheckedPropertyChanged.call(this, newValue);
            if (this.offBackgroundColor) {
                if (!newValue) {
                    this.setNativeBackgroundColor(this.offBackgroundColor);
                }
                else {
                    this.setNativeBackgroundColor(this.backgroundColor instanceof Color ? this.backgroundColor : new Color(this.backgroundColor));
                }
            }
        }
    };
    Object.defineProperty(Switch.prototype, "ios", {
        // @ts-ignore
        get: function () {
            return this.nativeViewProtected;
        },
        enumerable: false,
        configurable: true
    });
    Switch.prototype.onMeasure = function (widthMeasureSpec, heightMeasureSpec) {
        // It can't be anything different from 51x31
        var nativeSize = this.nativeViewProtected.sizeThatFits(zeroSize);
        this.width = nativeSize.width;
        this.height = nativeSize.height;
        var widthAndState = Switch.resolveSizeAndState(layout.toDevicePixels(nativeSize.width), layout.toDevicePixels(51), layout.EXACTLY, 0);
        var heightAndState = Switch.resolveSizeAndState(layout.toDevicePixels(nativeSize.height), layout.toDevicePixels(31), layout.EXACTLY, 0);
        this.setMeasuredDimension(widthAndState, heightAndState);
    };
    Switch.prototype[checkedProperty.getDefault] = function () {
        return false;
    };
    Switch.prototype[checkedProperty.setNative] = function (value) {
        this.nativeViewProtected.on = value;
    };
    Switch.prototype[colorProperty.getDefault] = function () {
        return this.nativeViewProtected.thumbTintColor;
    };
    Switch.prototype[colorProperty.setNative] = function (value) {
        var _a;
        var color = value instanceof Color ? value.ios : value;
        this.nativeViewProtected.thumbTintColor = color;
        if (color && this.nativeViewProtected.subviews.count > 0) {
            var alpha = new interop.Reference(1.0);
            var res = color.getRedGreenBlueAlpha(null, null, null, alpha);
            this.nativeViewProtected.subviews[0].alpha = (_a = (res && alpha.value)) !== null && _a !== void 0 ? _a : 1;
        }
    };
    Switch.prototype[backgroundColorProperty.getDefault] = function () {
        return this.nativeViewProtected.onTintColor;
    };
    Switch.prototype[backgroundColorProperty.setNative] = function (value) {
        if (majorVersion >= 13) {
            if (!this.offBackgroundColor || this.checked) {
                this.setNativeBackgroundColor(value);
            }
        }
        else {
            // old behavior on unsupported iOS versions
            this.nativeViewProtected.onTintColor = value instanceof Color ? value.ios : value;
        }
    };
    Switch.prototype[backgroundInternalProperty.getDefault] = function () {
        return null;
    };
    Switch.prototype[backgroundInternalProperty.setNative] = function (value) {
        //
    };
    Switch.prototype[offBackgroundColorProperty.getDefault] = function () {
        return this.nativeViewProtected.backgroundColor;
    };
    Switch.prototype[offBackgroundColorProperty.setNative] = function (value) {
        if (majorVersion >= 13) {
            if (!this.checked) {
                this.setNativeBackgroundColor(value);
            }
        }
        else {
            // old behavior on unsupported iOS versions...
            var nativeValue = value instanceof Color ? value.ios : value;
            this.nativeViewProtected.tintColor = nativeValue;
            this.nativeViewProtected.backgroundColor = nativeValue;
            this.nativeViewProtected.layer.cornerRadius = this.nativeViewProtected.frame.size.height / 2;
        }
    };
    return Switch;
}(SwitchBase));
export { Switch };
//# sourceMappingURL=index.ios.js.map