import { View, CSSType } from '../core/view';
import { ViewBase, booleanConverter } from '../core/view-base';
import { Style } from '../styling/style';
import { Color } from '../../color';
import { Property, CssProperty, CoercibleProperty } from '../core/properties';
import { Trace } from '../../trace';
export var traceCategory = 'TabView';
var TabViewItemBase = /** @class */ (function (_super) {
    __extends(TabViewItemBase, _super);
    function TabViewItemBase() {
        var _this = _super !== null && _super.apply(this, arguments) || this;
        _this._title = '';
        return _this;
    }
    Object.defineProperty(TabViewItemBase.prototype, "textTransform", {
        get: function () {
            return this.style.textTransform;
        },
        set: function (value) {
            this.style.textTransform = value;
        },
        enumerable: false,
        configurable: true
    });
    TabViewItemBase.prototype._addChildFromBuilder = function (name, value) {
        if (value instanceof View) {
            this.view = value;
        }
    };
    Object.defineProperty(TabViewItemBase.prototype, "title", {
        get: function () {
            return this._title;
        },
        set: function (value) {
            if (this._title !== value) {
                this._title = value;
                this._update();
            }
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(TabViewItemBase.prototype, "view", {
        get: function () {
            return this._view;
        },
        set: function (value) {
            if (this._view !== value) {
                if (this._view) {
                    throw new Error('Changing the view of an already loaded TabViewItem is not currently supported.');
                }
                this._view = value;
                this._addView(value);
            }
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(TabViewItemBase.prototype, "iconSource", {
        get: function () {
            return this._iconSource;
        },
        set: function (value) {
            if (this._iconSource !== value) {
                this._iconSource = value;
                this._update();
            }
        },
        enumerable: false,
        configurable: true
    });
    TabViewItemBase.prototype.eachChild = function (callback) {
        var view = this._view;
        if (view) {
            callback(view);
        }
    };
    TabViewItemBase.prototype.loadView = function (view) {
        var tabView = this.parent;
        if (tabView && tabView.items) {
            // Don't load items until their fragments are instantiated.
            if (this.canBeLoaded) {
                _super.prototype.loadView.call(this, view);
            }
        }
    };
    TabViewItemBase = __decorate([
        CSSType('TabViewItem')
    ], TabViewItemBase);
    return TabViewItemBase;
}(ViewBase));
export { TabViewItemBase };
var TabViewBase = /** @class */ (function (_super) {
    __extends(TabViewBase, _super);
    function TabViewBase() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    TabViewBase_1 = TabViewBase;
    Object.defineProperty(TabViewBase.prototype, "androidSelectedTabHighlightColor", {
        get: function () {
            return this.style.androidSelectedTabHighlightColor;
        },
        set: function (value) {
            this.style.androidSelectedTabHighlightColor = value;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(TabViewBase.prototype, "tabTextFontSize", {
        get: function () {
            return this.style.tabTextFontSize;
        },
        set: function (value) {
            this.style.tabTextFontSize = value;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(TabViewBase.prototype, "tabTextColor", {
        get: function () {
            return this.style.tabTextColor;
        },
        set: function (value) {
            this.style.tabTextColor = value;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(TabViewBase.prototype, "tabBackgroundColor", {
        get: function () {
            return this.style.tabBackgroundColor;
        },
        set: function (value) {
            this.style.tabBackgroundColor = value;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(TabViewBase.prototype, "selectedTabTextColor", {
        get: function () {
            return this.style.selectedTabTextColor;
        },
        set: function (value) {
            this.style.selectedTabTextColor = value;
        },
        enumerable: false,
        configurable: true
    });
    TabViewBase.prototype._addArrayFromBuilder = function (name, value) {
        if (name === 'items') {
            this.items = value;
        }
    };
    TabViewBase.prototype._addChildFromBuilder = function (name, value) {
        if (value instanceof TabViewItemBase) {
            if (!this.items) {
                this.items = new Array();
            }
            this.items.push(value);
            this._addView(value);
            selectedIndexProperty.coerce(this);
        }
    };
    Object.defineProperty(TabViewBase.prototype, "_selectedView", {
        get: function () {
            var selectedIndex = this.selectedIndex;
            return selectedIndex > -1 ? this.items[selectedIndex].view : null;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(TabViewBase.prototype, "_childrenCount", {
        get: function () {
            var items = this.items;
            return items ? items.length : 0;
        },
        enumerable: false,
        configurable: true
    });
    TabViewBase.prototype.eachChild = function (callback) {
        var items = this.items;
        if (items) {
            items.forEach(function (item, i) {
                callback(item);
            });
        }
    };
    TabViewBase.prototype.eachChildView = function (callback) {
        var items = this.items;
        if (items) {
            items.forEach(function (item, i) {
                callback(item.view);
            });
        }
    };
    TabViewBase.prototype.onItemsChanged = function (oldItems, newItems) {
        var _this = this;
        if (oldItems) {
            oldItems.forEach(function (item) { return _this._removeView(item); });
        }
        if (newItems) {
            newItems.forEach(function (item) {
                if (!item.view) {
                    throw new Error("TabViewItem must have a view.");
                }
                _this._addView(item);
            });
        }
    };
    TabViewBase.prototype.onSelectedIndexChanged = function (oldIndex, newIndex) {
        // to be overridden in platform specific files
        this.notify({
            eventName: TabViewBase_1.selectedIndexChangedEvent,
            object: this,
            oldIndex: oldIndex,
            newIndex: newIndex,
        });
    };
    var TabViewBase_1;
    TabViewBase.selectedIndexChangedEvent = 'selectedIndexChanged';
    TabViewBase = TabViewBase_1 = __decorate([
        CSSType('TabView')
    ], TabViewBase);
    return TabViewBase;
}(View));
export { TabViewBase };
export function traceMissingIcon(icon) {
    Trace.write('Could not load tab bar icon: ' + icon, Trace.categories.Error, Trace.messageType.error);
}
export var selectedIndexProperty = new CoercibleProperty({
    name: 'selectedIndex',
    defaultValue: -1,
    affectsLayout: __APPLE__,
    valueChanged: function (target, oldValue, newValue) {
        target.onSelectedIndexChanged(oldValue, newValue);
    },
    coerceValue: function (target, value) {
        var items = target.items;
        if (items) {
            var max = items.length - 1;
            if (value < 0) {
                value = 0;
            }
            if (value > max) {
                value = max;
            }
        }
        else {
            value = -1;
        }
        return value;
    },
    valueConverter: function (v) { return parseInt(v); },
});
selectedIndexProperty.register(TabViewBase);
export var itemsProperty = new Property({
    name: 'items',
    valueChanged: function (target, oldValue, newValue) {
        target.onItemsChanged(oldValue, newValue);
    },
});
itemsProperty.register(TabViewBase);
export var iosIconRenderingModeProperty = new Property({ name: 'iosIconRenderingMode', defaultValue: 'automatic' });
iosIconRenderingModeProperty.register(TabViewBase);
export var androidIconRenderingModeProperty = new Property({ name: 'androidIconRenderingMode', defaultValue: 'alwaysOriginal' });
androidIconRenderingModeProperty.register(TabViewBase);
export var androidOffscreenTabLimitProperty = new Property({
    name: 'androidOffscreenTabLimit',
    defaultValue: 1,
    affectsLayout: __APPLE__,
    valueConverter: function (v) { return parseInt(v); },
});
androidOffscreenTabLimitProperty.register(TabViewBase);
export var androidTabsPositionProperty = new Property({ name: 'androidTabsPosition', defaultValue: 'top' });
androidTabsPositionProperty.register(TabViewBase);
export var androidSwipeEnabledProperty = new Property({
    name: 'androidSwipeEnabled',
    defaultValue: true,
    valueConverter: booleanConverter,
});
androidSwipeEnabledProperty.register(TabViewBase);
export var tabTextFontSizeProperty = new CssProperty({
    name: 'tabTextFontSize',
    cssName: 'tab-text-font-size',
    valueConverter: function (v) { return parseFloat(v); },
});
tabTextFontSizeProperty.register(Style);
export var tabTextColorProperty = new CssProperty({
    name: 'tabTextColor',
    cssName: 'tab-text-color',
    equalityComparer: Color.equals,
    valueConverter: function (v) { return new Color(v); },
});
tabTextColorProperty.register(Style);
export var tabBackgroundColorProperty = new CssProperty({
    name: 'tabBackgroundColor',
    cssName: 'tab-background-color',
    equalityComparer: Color.equals,
    valueConverter: function (v) { return new Color(v); },
});
tabBackgroundColorProperty.register(Style);
export var selectedTabTextColorProperty = new CssProperty({
    name: 'selectedTabTextColor',
    cssName: 'selected-tab-text-color',
    equalityComparer: Color.equals,
    valueConverter: function (v) { return new Color(v); },
});
selectedTabTextColorProperty.register(Style);
export var androidSelectedTabHighlightColorProperty = new CssProperty({
    name: 'androidSelectedTabHighlightColor',
    cssName: 'android-selected-tab-highlight-color',
    equalityComparer: Color.equals,
    valueConverter: function (v) { return new Color(v); },
});
androidSelectedTabHighlightColorProperty.register(Style);
//# sourceMappingURL=tab-view-common.js.map