import { Span } from './span';
import { Observable } from '../../data/observable';
import { ObservableArray } from '../../data/observable-array';
import { ViewBase } from '../core/view-base';
var FormattedString = /** @class */ (function (_super) {
    __extends(FormattedString, _super);
    function FormattedString() {
        var _this = _super.call(this) || this;
        _this._spans = new ObservableArray();
        _this._spans.addEventListener(ObservableArray.changeEvent, _this.onSpansCollectionChanged, _this);
        return _this;
    }
    Object.defineProperty(FormattedString.prototype, "fontFamily", {
        get: function () {
            return this.style.fontFamily;
        },
        set: function (value) {
            this.style.fontFamily = value;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(FormattedString.prototype, "fontSize", {
        get: function () {
            return this.style.fontSize;
        },
        set: function (value) {
            this.style.fontSize = value;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(FormattedString.prototype, "fontStyle", {
        get: function () {
            return this.style.fontStyle;
        },
        set: function (value) {
            this.style.fontStyle = value;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(FormattedString.prototype, "fontWeight", {
        get: function () {
            return this.style.fontWeight;
        },
        set: function (value) {
            this.style.fontWeight = value;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(FormattedString.prototype, "fontVariationSettings", {
        get: function () {
            return this.style.fontVariationSettings;
        },
        set: function (value) {
            this.style.fontVariationSettings = value;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(FormattedString.prototype, "textDecoration", {
        get: function () {
            return this.style.textDecoration;
        },
        set: function (value) {
            this.style.textDecoration = value;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(FormattedString.prototype, "color", {
        get: function () {
            return this.style.color;
        },
        set: function (value) {
            this.style.color = value;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(FormattedString.prototype, "backgroundColor", {
        get: function () {
            return this.style.backgroundColor;
        },
        set: function (value) {
            this.style.backgroundColor = value;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(FormattedString.prototype, "iosAccessibilityAdjustsFontSize", {
        get: function () {
            return this.style.iosAccessibilityAdjustsFontSize;
        },
        set: function (value) {
            this.style.iosAccessibilityAdjustsFontSize = value;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(FormattedString.prototype, "iosAccessibilityMinFontScale", {
        get: function () {
            return this.style.iosAccessibilityMinFontScale;
        },
        set: function (value) {
            this.style.iosAccessibilityMinFontScale = value;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(FormattedString.prototype, "iosAccessibilityMaxFontScale", {
        get: function () {
            return this.style.iosAccessibilityMaxFontScale;
        },
        set: function (value) {
            this.style.iosAccessibilityMaxFontScale = value;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(FormattedString.prototype, "spans", {
        get: function () {
            if (!this._spans) {
                this._spans = new ObservableArray();
            }
            return this._spans;
        },
        enumerable: false,
        configurable: true
    });
    FormattedString.prototype.toString = function () {
        var result = '';
        for (var i = 0, length_1 = this.spans.length; i < length_1; i++) {
            result += this.spans.getItem(i).text;
        }
        return result;
    };
    FormattedString.prototype._addArrayFromBuilder = function (name, value) {
        var _a;
        if (name === 'spans') {
            (_a = this.spans).push.apply(_a, value);
        }
    };
    FormattedString.prototype._addChildFromBuilder = function (name, value) {
        if (value instanceof Span) {
            this.spans.push(value);
        }
    };
    FormattedString.prototype.onSpansCollectionChanged = function (eventData) {
        if (eventData.addedCount > 0) {
            for (var i = 0; i < eventData.addedCount; i++) {
                var span = eventData.object.getItem(eventData.index + i);
                // First add to logical tree so that inherited properties are set.
                this._addView(span);
                // Then attach handlers - we skip the first notification because
                // we raise change for the whole instance.
                this.addPropertyChangeHandler(span);
            }
        }
        if (eventData.removed && eventData.removed.length > 0) {
            for (var p = 0; p < eventData.removed.length; p++) {
                var span = eventData.removed[p];
                // First remove handlers so that we don't listen for changes
                // on inherited properties.
                this.removePropertyChangeHandler(span);
                // Then remove the element.
                this._removeView(span);
            }
        }
        this.notifyPropertyChange('.', this);
    };
    FormattedString.prototype.addPropertyChangeHandler = function (span) {
        var style = span.style;
        span.on(Observable.propertyChangeEvent, this.onPropertyChange, this);
        style.on('fontFamilyChange', this.onPropertyChange, this);
        style.on('fontSizeChange', this.onPropertyChange, this);
        style.on('fontStyleChange', this.onPropertyChange, this);
        style.on('fontWeightChange', this.onPropertyChange, this);
        style.on('fontVariationSettingsChange', this.onPropertyChange, this);
        style.on('textDecorationChange', this.onPropertyChange, this);
        style.on('colorChange', this.onPropertyChange, this);
        style.on('backgroundColorChange', this.onPropertyChange, this);
        // These handlers will trigger font scale update
        style.on('iosAccessibilityAdjustsFontSizeChange', this.onPropertyChange, this);
        style.on('iosAccessibilityMinFontScaleChange', this.onPropertyChange, this);
        style.on('iosAccessibilityMaxFontScaleChange', this.onPropertyChange, this);
        style.on('fontScaleInternalChange', this.onPropertyChange, this);
    };
    FormattedString.prototype.removePropertyChangeHandler = function (span) {
        var style = span.style;
        span.off(Observable.propertyChangeEvent, this.onPropertyChange, this);
        style.off('fontFamilyChange', this.onPropertyChange, this);
        style.off('fontSizeChange', this.onPropertyChange, this);
        style.off('fontStyleChange', this.onPropertyChange, this);
        style.off('fontWeightChange', this.onPropertyChange, this);
        style.off('fontVariationSettingsChange', this.onPropertyChange, this);
        style.off('textDecorationChange', this.onPropertyChange, this);
        style.off('colorChange', this.onPropertyChange, this);
        style.off('backgroundColorChange', this.onPropertyChange, this);
        style.off('iosAccessibilityAdjustsFontSizeChange', this.onPropertyChange, this);
        style.off('iosAccessibilityMinFontScaleChange', this.onPropertyChange, this);
        style.off('iosAccessibilityMaxFontScaleChange', this.onPropertyChange, this);
        style.off('fontScaleInternalChange', this.onPropertyChange, this);
    };
    FormattedString.prototype.onPropertyChange = function (data) {
        this.notifyPropertyChange(data.propertyName, this);
    };
    FormattedString.prototype.eachChild = function (callback) {
        this.spans.forEach(function (v, i, arr) { return callback(v); });
    };
    return FormattedString;
}(ViewBase));
export { FormattedString };
//# sourceMappingURL=formatted-string.js.map