import { booleanConverter } from '../core/view-base';
// Requires.
import { FormattedString } from './formatted-string';
import { View } from '../core/view';
import { Property, CssProperty, InheritedCssProperty, makeValidator, makeParser } from '../core/properties';
import { Style } from '../styling/style';
import { Observable } from '../../data/observable';
import { parseCSSShadow } from '../styling/css-shadow';
import { parseCSSStroke } from '../styling/css-stroke';
var CHILD_SPAN = 'Span';
var CHILD_FORMATTED_TEXT = 'formattedText';
var CHILD_FORMATTED_STRING = 'FormattedString';
var TextBaseCommon = /** @class */ (function (_super) {
    __extends(TextBaseCommon, _super);
    function TextBaseCommon() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    Object.defineProperty(TextBaseCommon.prototype, "nativeTextViewProtected", {
        /***
         * In the NativeScript Core; by default the nativeTextViewProtected points to the same value as nativeViewProtected.
         * At this point no internal NS components need this indirection functionality.
         * This indirection is used to allow support usage by third party components so they don't have to duplicate functionality.
         *
         * A third party component can just override the `nativeTextViewProtected` getter and return a different internal view and that view would be
         * what all TextView/TextInput class features would be applied to.
         *
         * A example is the Android MaterialDesign TextInput class, it has a wrapper view of a TextInputLayout
         *    https://developer.android.com/reference/com/google/android/material/textfield/TextInputLayout
         * which wraps the actual TextInput.  This wrapper layout (TextInputLayout) must be assigned to the nativeViewProtected as the entire
         * NS Core uses nativeViewProtected for everything related to layout, so that it can be measured, added to the parent view as a child, ect.
         *
         * However, its internal view would be the actual TextView/TextInput and to allow that sub-view to have the normal TextView/TextInput
         * class features, which we expose and to allow them to work on it, the internal TextView/TextInput is what the needs to have the class values applied to it.
         *
         * So all code that works on what is expected to be a TextView/TextInput should use `nativeTextViewProtected` so that any third party
         * components that need to have two separate components can work properly without them having to duplicate all the TextBase (and decendants) functionality
         * by just overriding the nativeTextViewProtected getter.
         **/
        get: function () {
            return this.nativeViewProtected;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(TextBaseCommon.prototype, "fontFamily", {
        get: function () {
            return this.style.fontFamily;
        },
        set: function (value) {
            this.style.fontFamily = value;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(TextBaseCommon.prototype, "fontSize", {
        get: function () {
            return this.style.fontSize;
        },
        set: function (value) {
            this.style.fontSize = value;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(TextBaseCommon.prototype, "fontStyle", {
        get: function () {
            return this.style.fontStyle;
        },
        set: function (value) {
            this.style.fontStyle = value;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(TextBaseCommon.prototype, "fontWeight", {
        get: function () {
            return this.style.fontWeight;
        },
        set: function (value) {
            this.style.fontWeight = value;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(TextBaseCommon.prototype, "letterSpacing", {
        get: function () {
            return this.style.letterSpacing;
        },
        set: function (value) {
            this.style.letterSpacing = value;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(TextBaseCommon.prototype, "lineHeight", {
        get: function () {
            return this.style.lineHeight;
        },
        set: function (value) {
            this.style.lineHeight = value;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(TextBaseCommon.prototype, "maxLines", {
        get: function () {
            return this.style.maxLines;
        },
        set: function (value) {
            this.style.maxLines = value;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(TextBaseCommon.prototype, "textAlignment", {
        get: function () {
            return this.style.textAlignment;
        },
        set: function (value) {
            this.style.textAlignment = value;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(TextBaseCommon.prototype, "textDecoration", {
        get: function () {
            return this.style.textDecoration;
        },
        set: function (value) {
            this.style.textDecoration = value;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(TextBaseCommon.prototype, "textTransform", {
        get: function () {
            return this.style.textTransform;
        },
        set: function (value) {
            this.style.textTransform = value;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(TextBaseCommon.prototype, "textShadow", {
        get: function () {
            return this.style.textShadow;
        },
        set: function (value) {
            this.style.textShadow = value;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(TextBaseCommon.prototype, "textStroke", {
        get: function () {
            return this.style.textStroke;
        },
        set: function (value) {
            this.style.textStroke = value;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(TextBaseCommon.prototype, "whiteSpace", {
        get: function () {
            return this.style.whiteSpace;
        },
        set: function (value) {
            this.style.whiteSpace = value;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(TextBaseCommon.prototype, "textOverflow", {
        get: function () {
            return this.style.textOverflow;
        },
        set: function (value) {
            this.style.textOverflow = value;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(TextBaseCommon.prototype, "padding", {
        get: function () {
            return this.style.padding;
        },
        set: function (value) {
            this.style.padding = value;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(TextBaseCommon.prototype, "paddingTop", {
        get: function () {
            return this.style.paddingTop;
        },
        set: function (value) {
            this.style.paddingTop = value;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(TextBaseCommon.prototype, "paddingRight", {
        get: function () {
            return this.style.paddingRight;
        },
        set: function (value) {
            this.style.paddingRight = value;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(TextBaseCommon.prototype, "paddingBottom", {
        get: function () {
            return this.style.paddingBottom;
        },
        set: function (value) {
            this.style.paddingBottom = value;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(TextBaseCommon.prototype, "paddingLeft", {
        get: function () {
            return this.style.paddingLeft;
        },
        set: function (value) {
            this.style.paddingLeft = value;
        },
        enumerable: false,
        configurable: true
    });
    TextBaseCommon.prototype._onFormattedTextContentsChanged = function (data) {
        if (this.nativeViewProtected) {
            // Notifications from the FormattedString start arriving before the Android view is even created.
            this[formattedTextProperty.setNative](data.value);
        }
    };
    TextBaseCommon.prototype._addChildFromBuilder = function (name, value) {
        if (name === CHILD_SPAN) {
            if (!this.formattedText) {
                var formattedText = new FormattedString();
                formattedText.spans.push(value);
                this.formattedText = formattedText;
            }
            else {
                this.formattedText.spans.push(value);
            }
        }
        else if (name === CHILD_FORMATTED_TEXT || name === CHILD_FORMATTED_STRING) {
            this.formattedText = value;
        }
    };
    TextBaseCommon.prototype._requestLayoutOnTextChanged = function () {
        this.requestLayout();
    };
    TextBaseCommon.prototype.eachChild = function (callback) {
        var text = this.formattedText;
        if (text) {
            callback(text);
        }
    };
    TextBaseCommon.prototype._setNativeText = function (reset) {
        if (reset === void 0) { reset = false; }
        //
    };
    TextBaseCommon.iosTextAnimationFallback = true;
    return TextBaseCommon;
}(View));
export { TextBaseCommon };
TextBaseCommon.prototype._isSingleLine = false;
export function isBold(fontWeight) {
    return fontWeight === 'bold' || fontWeight === '700' || fontWeight === '800' || fontWeight === '900';
}
export var textProperty = new Property({
    name: 'text',
    defaultValue: '',
    affectsLayout: __ANDROID__,
});
textProperty.register(TextBaseCommon);
export var formattedTextProperty = new Property({
    name: 'formattedText',
    affectsLayout: true,
    valueChanged: onFormattedTextPropertyChanged,
});
formattedTextProperty.register(TextBaseCommon);
export var iosTextAnimationProperty = new Property({
    name: 'iosTextAnimation',
    defaultValue: 'inherit',
    affectsLayout: false,
    valueConverter: function (value) {
        try {
            return booleanConverter(value);
        }
        catch (e) {
            return 'inherit';
        }
    },
});
iosTextAnimationProperty.register(TextBaseCommon);
function onFormattedTextPropertyChanged(textBase, oldValue, newValue) {
    if (oldValue) {
        oldValue.off(Observable.propertyChangeEvent, textBase._onFormattedTextContentsChanged, textBase);
        textBase._removeView(oldValue);
    }
    if (newValue) {
        var oldParent = newValue.parent;
        // In case formattedString is attached to new TextBase
        if (oldParent) {
            oldParent._removeView(newValue);
        }
        textBase._addView(newValue);
        newValue.on(Observable.propertyChangeEvent, textBase._onFormattedTextContentsChanged, textBase);
    }
}
export function getClosestPropertyValue(property, span) {
    if (property.isSet(span.style)) {
        return span.style[property.name];
    }
    else if (property.isSet(span.parent.style)) {
        // parent is FormattedString
        return span.parent.style[property.name];
    }
    else if (property.isSet(span.parent.parent.style)) {
        // parent.parent is TextBase
        return span.parent.parent.style[property.name];
    }
}
var textAlignmentConverter = makeParser(makeValidator('initial', 'left', 'center', 'right', 'justify'));
export var textAlignmentProperty = new InheritedCssProperty({
    name: 'textAlignment',
    cssName: 'text-align',
    defaultValue: 'initial',
    valueConverter: textAlignmentConverter,
});
textAlignmentProperty.register(Style);
var textTransformConverter = makeParser(makeValidator('initial', 'none', 'capitalize', 'uppercase', 'lowercase'));
export var textTransformProperty = new CssProperty({
    name: 'textTransform',
    cssName: 'text-transform',
    defaultValue: 'initial',
    valueConverter: textTransformConverter,
});
textTransformProperty.register(Style);
export var textShadowProperty = new CssProperty({
    name: 'textShadow',
    cssName: 'text-shadow',
    affectsLayout: __APPLE__,
    valueConverter: function (value) {
        return parseCSSShadow(value);
    },
});
textShadowProperty.register(Style);
export var textStrokeProperty = new CssProperty({
    name: 'textStroke',
    cssName: 'text-stroke',
    affectsLayout: __APPLE__,
    valueConverter: function (value) {
        return parseCSSStroke(value);
    },
});
textStrokeProperty.register(Style);
var whiteSpaceConverter = makeParser(makeValidator('initial', 'normal', 'nowrap'));
export var whiteSpaceProperty = new CssProperty({
    name: 'whiteSpace',
    cssName: 'white-space',
    defaultValue: 'initial',
    affectsLayout: __APPLE__,
    valueConverter: whiteSpaceConverter,
});
whiteSpaceProperty.register(Style);
var textOverflowConverter = makeParser(makeValidator('clip', 'ellipsis', 'initial', 'unset'));
export var textOverflowProperty = new CssProperty({
    name: 'textOverflow',
    cssName: 'text-overflow',
    defaultValue: 'initial',
    affectsLayout: __APPLE__,
    valueConverter: textOverflowConverter,
});
textOverflowProperty.register(Style);
var textDecorationConverter = makeParser(makeValidator('none', 'underline', 'line-through', 'underline line-through'));
export var textDecorationProperty = new CssProperty({
    name: 'textDecoration',
    cssName: 'text-decoration',
    defaultValue: 'none',
    valueConverter: textDecorationConverter,
});
textDecorationProperty.register(Style);
export var letterSpacingProperty = new InheritedCssProperty({
    name: 'letterSpacing',
    cssName: 'letter-spacing',
    defaultValue: 0,
    affectsLayout: __APPLE__,
    valueConverter: function (v) { return parseFloat(v); },
});
letterSpacingProperty.register(Style);
export var lineHeightProperty = new InheritedCssProperty({
    name: 'lineHeight',
    cssName: 'line-height',
    affectsLayout: __APPLE__,
    valueConverter: function (v) { return parseFloat(v); },
});
lineHeightProperty.register(Style);
export var maxLinesProperty = new CssProperty({
    name: 'maxLines',
    cssName: 'max-lines',
    valueConverter: function (v) { return (v === 'none' ? 0 : parseInt(v, 10)); },
});
maxLinesProperty.register(Style);
export var resetSymbol = Symbol('textPropertyDefault');
//# sourceMappingURL=text-base-common.js.map