import { TimePickerBase, getValidTime, timeProperty, hourProperty, minuteProperty } from './time-picker-common';
import { SDK_VERSION } from '../../utils/constants';
export * from './time-picker-common';
var TimeChangedListener;
function initializeTimeChangedListener() {
    if (TimeChangedListener) {
        return;
    }
    var TimeChangedListenerImpl = /** @class */ (function (_super) {
    __extends(TimeChangedListenerImpl, _super);
    function TimeChangedListenerImpl(owner) {
        var _this = _super.call(this) || this;
        _this.owner = owner;
        return global.__native(_this);
    }
    TimeChangedListenerImpl.prototype.onTimeChanged = function (picker, hour, minute) {
        var timePicker = this.owner;
        if (timePicker.updatingNativeValue) {
            return;
        }
        var validTime = getValidTime(timePicker, hour, minute);
        timeProperty.nativeValueChange(timePicker, new Date(0, 0, 0, validTime.hour, validTime.minute));
    };
    var _a;
    TimeChangedListenerImpl = __decorate([
        Interfaces([android.widget.TimePicker.OnTimeChangedListener]),
        __metadata("design:paramtypes", [typeof (_a = typeof TimePicker !== "undefined" && TimePicker) === "function" ? _a : Object])
    ], TimeChangedListenerImpl);
    return TimeChangedListenerImpl;
}(java.lang.Object));
    TimeChangedListener = TimeChangedListenerImpl;
}
var TimePicker = /** @class */ (function (_super) {
    __extends(TimePicker, _super);
    function TimePicker() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    TimePicker.prototype.createNativeView = function () {
        return new android.widget.TimePicker(this._context);
    };
    TimePicker.prototype.initNativeView = function () {
        _super.prototype.initNativeView.call(this);
        var nativeView = this.nativeViewProtected;
        initializeTimeChangedListener();
        var listener = new TimeChangedListener(this);
        nativeView.setOnTimeChangedListener(listener);
        nativeView.listener = listener;
        var calendar = (nativeView.calendar = java.util.Calendar.getInstance());
        var hour = hourProperty.isSet(this) ? this.hour : calendar.get(java.util.Calendar.HOUR_OF_DAY);
        var minute = minuteProperty.isSet(this) ? this.minute : calendar.get(java.util.Calendar.MINUTE);
        var validTime = getValidTime(this, hour, minute);
        if (!timeProperty.isSet(this)) {
            this.time = new Date(0, 0, 0, validTime.hour, validTime.minute);
        }
    };
    TimePicker.prototype[minuteProperty.setNative] = function (value) {
        this.updatingNativeValue = true;
        try {
            if (SDK_VERSION >= 23) {
                this.nativeViewProtected.setMinute(value);
            }
            else {
                this.nativeViewProtected.setCurrentMinute(new java.lang.Integer(value));
            }
        }
        finally {
            this.updatingNativeValue = false;
        }
    };
    TimePicker.prototype[hourProperty.setNative] = function (value) {
        this.updatingNativeValue = true;
        try {
            if (SDK_VERSION >= 23) {
                this.nativeViewProtected.setHour(value);
            }
            else {
                this.nativeViewProtected.setCurrentHour(new java.lang.Integer(value));
            }
        }
        finally {
            this.updatingNativeValue = false;
        }
    };
    return TimePicker;
}(TimePickerBase));
export { TimePicker };
//# sourceMappingURL=index.android.js.map