import { ContainerView, CSSType } from '../core/view';
import { Property } from '../core/properties';
import { knownFolders } from '../../file-system';
import { booleanConverter } from '../core/view-base';
export * from './web-view-interfaces';
export var srcProperty = new Property({ name: 'src' });
export var disableZoomProperty = new Property({ name: 'disableZoom', defaultValue: false, valueConverter: booleanConverter });
var WebViewBase = /** @class */ (function (_super) {
    __extends(WebViewBase, _super);
    function WebViewBase() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    WebViewBase_1 = WebViewBase;
    WebViewBase.prototype._onLoadFinished = function (url, error) {
        var args = {
            eventName: WebViewBase_1.loadFinishedEvent,
            object: this,
            url: url,
            navigationType: undefined,
            error: error,
        };
        this.notify(args);
    };
    WebViewBase.prototype._onLoadStarted = function (url, navigationType) {
        var args = {
            eventName: WebViewBase_1.loadStartedEvent,
            object: this,
            url: url,
            navigationType: navigationType,
            error: undefined,
        };
        this.notify(args);
    };
    Object.defineProperty(WebViewBase.prototype, "canGoBack", {
        get: function () {
            throw new Error('This member is abstract.');
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(WebViewBase.prototype, "canGoForward", {
        get: function () {
            throw new Error('This member is abstract.');
        },
        enumerable: false,
        configurable: true
    });
    WebViewBase.prototype[srcProperty.getDefault] = function () {
        return '';
    };
    WebViewBase.prototype[srcProperty.setNative] = function (src) {
        this.stopLoading();
        // Add file:// prefix for local files.
        // They should be loaded with _loadUrl() method as it handles query params.
        if (src.indexOf('~/') === 0) {
            var appPath = knownFolders.currentApp().path;
            if (appPath && appPath.indexOf('/') !== 0) {
                // ensure slash is correct
                appPath = "/".concat(appPath);
            }
            src = "file://".concat(appPath, "/") + src.substr(2);
        }
        else if (src.indexOf('/') === 0) {
            src = 'file://' + src;
        }
        // loading local files from paths with spaces may fail
        if (src.toLowerCase().indexOf('file:///') === 0) {
            src = encodeURI(src);
        }
        if (src.toLowerCase().indexOf('http://') === 0 || src.toLowerCase().indexOf('https://') === 0 || src.toLowerCase().indexOf('file:///') === 0) {
            this._loadUrl(src);
        }
        else {
            this._loadData(src);
        }
    };
    var WebViewBase_1;
    WebViewBase.loadStartedEvent = 'loadStarted';
    WebViewBase.loadFinishedEvent = 'loadFinished';
    WebViewBase = WebViewBase_1 = __decorate([
        CSSType('WebView')
    ], WebViewBase);
    return WebViewBase;
}(ContainerView));
export { WebViewBase };
srcProperty.register(WebViewBase);
disableZoomProperty.register(WebViewBase);
//# sourceMappingURL=web-view-common.js.map