import { Color } from '../../color';
import { Trace } from '../../trace';
import { CORE_ANIMATION_DEFAULTS, getDurationWithDampingFromSpring } from '../common';
import { SDK_VERSION } from '../constants';
var radToDeg = Math.PI / 180;
function isOrientationLandscape(orientation) {
    return orientation === 3 /* UIDeviceOrientation.LandscapeLeft */ /* 3 */ || orientation === 4 /* UIDeviceOrientation.LandscapeRight */ /* 4 */;
}
function openFileAtRootModule(filePath) {
    try {
        var appPath = getCurrentAppPath();
        var path = isRealDevice() ? filePath.replace('~', appPath) : filePath;
        var controller = UIDocumentInteractionController.interactionControllerWithURL(NSURL.fileURLWithPath(path));
        controller.delegate = createUIDocumentInteractionControllerDelegate();
        return controller.presentPreviewAnimated(true);
    }
    catch (e) {
        Trace.write('Error in openFile', Trace.categories.Error, Trace.messageType.error);
    }
    return false;
}
// TODO: remove for NativeScript 9.0
export function getter(_this, property) {
    console.log('utils.ios.getter() is deprecated; use the respective native property instead');
    if (typeof property === 'function') {
        return property.call(_this);
    }
    else {
        return property;
    }
}
export var collections;
(function (collections) {
    function jsArrayToNSArray(str) {
        return NSArray.arrayWithArray(str);
    }
    collections.jsArrayToNSArray = jsArrayToNSArray;
    function nsArrayToJSArray(a) {
        var arr = [];
        if (a !== undefined) {
            var count = a.count;
            for (var i = 0; i < count; i++) {
                arr.push(a.objectAtIndex(i));
            }
        }
        return arr;
    }
    collections.nsArrayToJSArray = nsArrayToJSArray;
})(collections || (collections = {}));
export function getRootViewController() {
    var win = getWindow();
    var vc = win && win.rootViewController;
    while (vc && vc.presentedViewController) {
        vc = vc.presentedViewController;
    }
    return vc;
}
export function getWindow() {
    var window;
    if (SDK_VERSION >= 15 && typeof NativeScriptViewFactory !== 'undefined') {
        // UIWindowScene.keyWindow is only available 15+
        window = NativeScriptViewFactory.getKeyWindow();
    }
    if (window) {
        return window;
    }
    var app = UIApplication.sharedApplication;
    if (!app) {
        return;
    }
    return app.keyWindow || (app.windows && app.windows.count > 0 && app.windows.objectAtIndex(0));
}
export function getMainScreen() {
    var window = getWindow();
    return window ? window.screen : UIScreen.mainScreen;
}
export function setWindowBackgroundColor(value) {
    var win = getWindow();
    if (win) {
        var bgColor = new Color(value);
        win.backgroundColor = bgColor.ios;
        var rootVc = getRootViewController();
        if (rootVc === null || rootVc === void 0 ? void 0 : rootVc.view) {
            rootVc.view.backgroundColor = bgColor.ios;
        }
    }
}
export function isLandscape() {
    console.log('utils.ios.isLandscape() is deprecated; use application.orientation instead');
    var deviceOrientation = UIDevice.currentDevice.orientation;
    var statusBarOrientation = UIApplication.sharedApplication.statusBarOrientation;
    var isDeviceOrientationLandscape = isOrientationLandscape(deviceOrientation);
    var isStatusBarOrientationLandscape = isOrientationLandscape(statusBarOrientation);
    return isDeviceOrientationLandscape || isStatusBarOrientationLandscape;
}
/**
 * @deprecated use Utils.SDK_VERSION instead which is a float of the {major}.{minor} verison
 */
export var MajorVersion = NSString.stringWithString(UIDevice.currentDevice.systemVersion).intValue;
export function openFile(filePath) {
    console.log('utils.ios.openFile() is deprecated; use utils.openFile() instead');
    return openFileAtRootModule(filePath);
}
export function getCurrentAppPath() {
    var currentDir = __dirname;
    var tnsModulesIndex = currentDir.indexOf('/tns_modules');
    // Module not hosted in ~/tns_modules when bundled. Use current dir.
    var appPath = currentDir;
    if (tnsModulesIndex !== -1) {
        // Strip part after tns_modules to obtain app root
        appPath = currentDir.substring(0, tnsModulesIndex);
    }
    return appPath;
}
export function joinPaths() {
    var paths = [];
    for (var _i = 0; _i < arguments.length; _i++) {
        paths[_i] = arguments[_i];
    }
    if (!paths || paths.length === 0) {
        return '';
    }
    return NSString.stringWithString(NSString.pathWithComponents(paths)).stringByStandardizingPath;
}
export function getVisibleViewController(rootViewController) {
    var viewController = rootViewController;
    while (viewController && viewController.presentedViewController) {
        viewController = viewController.presentedViewController;
    }
    return viewController;
}
export function applyRotateTransform(transform, x, y, z) {
    if (x) {
        transform = CATransform3DRotate(transform, x * radToDeg, 1, 0, 0);
    }
    if (y) {
        transform = CATransform3DRotate(transform, y * radToDeg, 0, 1, 0);
    }
    if (z) {
        transform = CATransform3DRotate(transform, z * radToDeg, 0, 0, 1);
    }
    return transform;
}
export function createUIDocumentInteractionControllerDelegate() {
    var UIDocumentInteractionControllerDelegateImpl = /** @class */ (function (_super) {
    __extends(UIDocumentInteractionControllerDelegateImpl, _super);
    function UIDocumentInteractionControllerDelegateImpl() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    UIDocumentInteractionControllerDelegateImpl.prototype.getViewController = function () {
        return getWindow().rootViewController;
    };
    UIDocumentInteractionControllerDelegateImpl.prototype.documentInteractionControllerViewControllerForPreview = function (controller) {
        return this.getViewController();
    };
    UIDocumentInteractionControllerDelegateImpl.prototype.documentInteractionControllerViewForPreview = function (controller) {
        return this.getViewController().view;
    };
    UIDocumentInteractionControllerDelegateImpl.prototype.documentInteractionControllerRectForPreview = function (controller) {
        return this.getViewController().view.frame;
    };
    UIDocumentInteractionControllerDelegateImpl.ObjCProtocols = [UIDocumentInteractionControllerDelegate];
    return UIDocumentInteractionControllerDelegateImpl;
}(NSObject));
    return new UIDocumentInteractionControllerDelegateImpl();
}
export function isRealDevice() {
    try {
        // https://stackoverflow.com/a/5093092/4936697
        var sourceType = UIImagePickerControllerSourceType.UIImagePickerControllerSourceTypeCamera;
        var mediaTypes = UIImagePickerController.availableMediaTypesForSourceType(sourceType);
        return !!mediaTypes;
    }
    catch (e) {
        return true;
    }
}
export function printCGRect(rect) {
    if (rect) {
        return "CGRect(".concat(rect.origin.x, " ").concat(rect.origin.y, " ").concat(rect.size.width, " ").concat(rect.size.height, ")");
    }
}
export function snapshotView(view, scale) {
    if (view instanceof UIImageView) {
        return view.image;
    }
    // console.log('snapshotView view.frame:', printRect(view.frame));
    var originalOpacity = view.layer.opacity;
    view.layer.opacity = originalOpacity > 0 ? originalOpacity : 1;
    UIGraphicsBeginImageContextWithOptions(CGSizeMake(view.frame.size.width, view.frame.size.height), false, scale);
    view.layer.renderInContext(UIGraphicsGetCurrentContext());
    var image = UIGraphicsGetImageFromCurrentImageContext();
    UIGraphicsEndImageContext();
    setTimeout(function () {
        // ensure set back properly on next tick
        view.layer.opacity = originalOpacity;
    });
    return image;
}
export function copyLayerProperties(view, toView, customProperties) {
    var viewPropertiesToMatch = (customProperties === null || customProperties === void 0 ? void 0 : customProperties.view) || ['backgroundColor'];
    var layerPropertiesToMatch = (customProperties === null || customProperties === void 0 ? void 0 : customProperties.layer) || ['cornerRadius', 'borderWidth', 'borderColor'];
    viewPropertiesToMatch.forEach(function (property) {
        if (view[property] !== toView[property]) {
            // console.log('|    -- matching view property:', property);
            view[property] = toView[property];
        }
    });
    layerPropertiesToMatch.forEach(function (property) {
        if (view.layer[property] !== toView.layer[property]) {
            // console.log('|    -- matching layer property:', property);
            view.layer[property] = toView.layer[property];
        }
    });
}
export function animateWithSpring(options) {
    // for convenience, default spring settings are provided
    var opt = __assign(__assign(__assign({}, CORE_ANIMATION_DEFAULTS.spring), { delay: 0, animateOptions: null, animations: null, completion: null }), (options || {}));
    var _a = getDurationWithDampingFromSpring(opt), duration = _a.duration, damping = _a.damping;
    if (duration === 0) {
        UIView.animateWithDurationAnimationsCompletion(0, opt.animations, opt.completion);
        return;
    }
    UIView.animateWithDurationDelayUsingSpringWithDampingInitialSpringVelocityOptionsAnimationsCompletion(duration, opt.delay, damping, opt.velocity, opt.animateOptions, opt.animations, opt.completion);
}
//# sourceMappingURL=index.js.map