import * as iOSUtils from './ios';
import { platformCheck } from './platform-check';
import { getClass, isNullOrUndefined, numberHasDecimals, numberIs64Bit } from './types';
export function dataDeserialize(nativeData) {
    if (isNullOrUndefined(nativeData)) {
        // some native values will already be js null values
        // calling types.getClass below on null/undefined will cause crash
        return null;
    }
    else {
        switch (getClass(nativeData)) {
            case 'NSNull':
                return null;
            case 'NSMutableDictionary':
            case 'NSDictionary': {
                var obj = {};
                var length_1 = nativeData.count;
                var keysArray = nativeData.allKeys;
                for (var i = 0; i < length_1; i++) {
                    var nativeKey = keysArray.objectAtIndex(i);
                    obj[nativeKey] = dataDeserialize(nativeData.objectForKey(nativeKey));
                }
                return obj;
            }
            case 'NSMutableArray':
            case 'NSArray': {
                var array = [];
                var len = nativeData.count;
                for (var i = 0; i < len; i++) {
                    array[i] = dataDeserialize(nativeData.objectAtIndex(i));
                }
                return array;
            }
            default:
                return nativeData;
        }
    }
}
export function dataSerialize(data, wrapPrimitives) {
    if (wrapPrimitives === void 0) { wrapPrimitives = false; }
    switch (typeof data) {
        case 'string':
        case 'boolean': {
            return data;
        }
        case 'number': {
            var hasDecimals = numberHasDecimals(data);
            if (numberIs64Bit(data)) {
                if (hasDecimals) {
                    return NSNumber.alloc().initWithDouble(data);
                }
                else {
                    return NSNumber.alloc().initWithLongLong(data);
                }
            }
            else {
                if (hasDecimals) {
                    return NSNumber.alloc().initWithFloat(data);
                }
                else {
                    return data;
                }
            }
        }
        case 'object': {
            if (data instanceof Date) {
                return NSDate.dateWithTimeIntervalSince1970(data.getTime() / 1000);
            }
            if (!data) {
                return null;
            }
            if (Array.isArray(data)) {
                return NSArray.arrayWithArray(data.map(function (el) { return dataSerialize(el, wrapPrimitives); }).filter(function (el) { return el !== null; }));
            }
            var node = Object.fromEntries(Object.entries(data)
                .map(function (_a) {
                var key = _a[0], value = _a[1];
                return [key, dataSerialize(value, wrapPrimitives)];
            })
                .filter(function (_a) {
                var value = _a[1];
                return value !== null;
            }));
            // cast to any avoids signature overload on tsc build
            return NSDictionary.dictionaryWithDictionary(node);
        }
        default:
            return null;
    }
}
// these don't exist on iOS. Stub them to empty functions.
export var ad = platformCheck('Utils.ad');
export var android = platformCheck('Utils.android');
export var ios = iOSUtils;
/**
 * @deprecated Use `Utils.ios` instead.
 */
export var iOSNativeHelper = iOSUtils;
//# sourceMappingURL=native-helper.ios.js.map