export function isString(value) {
    return typeof value === 'string' || value instanceof String;
}
export function isNumber(value) {
    return typeof value === 'number' || value instanceof Number;
}
export function isBoolean(value) {
    return typeof value === 'boolean' || value instanceof Boolean;
}
export function isFunction(value) {
    if (!value) {
        return false;
    }
    return typeof value === 'function';
}
export function isObject(value) {
    if (!value) {
        return false;
    }
    return typeof value === 'object';
}
export function isUndefined(value) {
    return typeof value === 'undefined';
}
export function isDefined(value) {
    return typeof value !== 'undefined';
}
export function isNullOrUndefined(value) {
    return typeof value === 'undefined' || value === null;
}
export function verifyCallback(value) {
    if (value && !isFunction(value)) {
        throw new TypeError('Callback must be a valid function.');
    }
}
export function numberHasDecimals(value) {
    return !(value % 1 === 0);
}
export function numberIs64Bit(value) {
    return value < -Math.pow(2, 31) + 1 || value > Math.pow(2, 31) - 1;
}
var classInfosMap = new Map();
// ES3-5 type classes are "function blah()", new ES6+ classes can be "class blah"
var funcNameRegex = /(?:function|class)\s+(\w+).*/;
export function getClass(object) {
    return getClassInfo(object).name;
}
export function getClassInfo(object) {
    var constructor = object.constructor;
    var result = classInfosMap.get(constructor);
    if (!result) {
        result = new ClassInfo(constructor);
        classInfosMap.set(constructor, result);
    }
    return result;
}
export function getBaseClasses(object) {
    var result = [];
    var info = getClassInfo(object);
    while (info) {
        result.push(info.name);
        info = info.baseClassInfo;
    }
    return result;
}
var ClassInfo = /** @class */ (function () {
    function ClassInfo(typeConstructor) {
        this._typeConstructor = typeConstructor;
    }
    Object.defineProperty(ClassInfo.prototype, "name", {
        get: function () {
            if (!this._name) {
                if (this._typeConstructor.name) {
                    this._name = this._typeConstructor.name;
                }
                else {
                    var results = funcNameRegex.exec(this._typeConstructor.toString());
                    this._name = results && results.length > 1 ? results[1] : '';
                }
            }
            return this._name;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(ClassInfo.prototype, "baseClassInfo", {
        get: function () {
            if (isUndefined(this._baseClassInfo)) {
                this._baseClassInfo = ClassInfo._getBase(this);
                // While extending some classes for platform specific versions results in duplicate class types in hierarchy.
                if (this._baseClassInfo && this._baseClassInfo.name === this.name) {
                    this._baseClassInfo = ClassInfo._getBase(this._baseClassInfo);
                }
            }
            return this._baseClassInfo;
        },
        enumerable: false,
        configurable: true
    });
    ClassInfo._getBase = function (info) {
        var result = null;
        var constructorProto = info._typeConstructor.prototype;
        if (constructorProto.__proto__) {
            result = getClassInfo(constructorProto.__proto__);
        }
        return result;
    };
    return ClassInfo;
}());
export { ClassInfo };
export function toUIString(obj) {
    return isNullOrUndefined(obj) ? '' : obj + '';
}
//# sourceMappingURL=types.js.map