var _a, _b;
var parent_ = Symbol('[[parent]]');
var native_ = Symbol('[[native]]');
var algorithm_ = Symbol('[[algorithm]]');
var usages_ = Symbol('[[usages]]');
var extractable_ = Symbol('[[extractable]]');
var type_ = Symbol('[[type]]');
var privateKey_ = Symbol('[[privateKey]]');
var publicKey_ = Symbol('[[publicKey]]');
function parseHash(hash) {
    switch (hash) {
        case 'SHA-1':
            return 0;
        case 'SHA-256':
            return 1;
        case 'SHA-384':
            return 2;
        case 'SHA-512':
            return 3;
        default:
            return null;
    }
}
function parseUsages(usages) {
    var ret = NSMutableArray.new();
    if (Array.isArray(usages)) {
        /*

    kNSCCryptoDecrypt,
    kNSCCryptoEncrypt,
    kNSCCryptoSign,
    kNSCCryptoVerify,
    kNSCCryptoDeriveKey,
    kNSCCryptoDeriveBits,
    kNSCCryptoWrapKey,
    kNSCCryptoUnwrapKey,
*/
        for (var _i = 0, usages_1 = usages; _i < usages_1.length; _i++) {
            var usage = usages_1[_i];
            switch (usage) {
                case 'encrypt':
                    ret.addObject(1);
                    break;
                case 'decrypt':
                    ret.addObject(0);
                    break;
                case 'sign':
                    ret.addObject(2);
                    break;
                case 'verify':
                    ret.addObject(3);
                    break;
                case 'deriveKey':
                    ret.addObject(4);
                    break;
                case 'deriveBits':
                    ret.addObject(5);
                    break;
                case 'wrapKey':
                    ret.addObject(6);
                    break;
                case 'unwrapKey':
                    ret.addObject(7);
                    break;
            }
        }
    }
    return ret;
}
var CryptoKey = /** @class */ (function () {
    function CryptoKey() {
    }
    Object.defineProperty(CryptoKey.prototype, "algorithm", {
        get: function () {
            return this[algorithm_];
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(CryptoKey.prototype, "usages", {
        get: function () {
            return this[usages_];
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(CryptoKey.prototype, "extractable", {
        get: function () {
            return this[extractable_];
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(CryptoKey.prototype, "type", {
        get: function () {
            return this[type_];
        },
        enumerable: false,
        configurable: true
    });
    CryptoKey.fromNative = function (key) {
        if (key) {
            var ret = new CryptoKey();
            ret[native_] = key;
            return ret;
        }
        return null;
    };
    return CryptoKey;
}());
export { CryptoKey };
var CryptoKeyPair = /** @class */ (function () {
    function CryptoKeyPair() {
        this[_a] = null;
        this[_b] = null;
    }
    Object.defineProperty(CryptoKeyPair.prototype, "privateKey", {
        get: function () {
            if (__IOS__) {
                var kp = this[native_];
                if (!this[privateKey_]) {
                    this[privateKey_] = CryptoKey.fromNative(kp.privateKey);
                }
            }
            if (__ANDROID__) {
                var kp = this[native_];
                if (!this[privateKey_]) {
                    this[privateKey_] = CryptoKey.fromNative(kp.getPrivate());
                }
            }
            return this[privateKey_];
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(CryptoKeyPair.prototype, "publicKey", {
        get: function () {
            if (__IOS__) {
                var kp = this[native_];
                if (!this[publicKey_]) {
                    this[publicKey_] = CryptoKey.fromNative(kp.publicKey);
                }
            }
            if (__ANDROID__) {
                var kp = this[native_];
                if (!this[publicKey_]) {
                    this[publicKey_] = CryptoKey.fromNative(kp.getPublic());
                }
            }
            return this[publicKey_];
        },
        enumerable: false,
        configurable: true
    });
    CryptoKeyPair.fromNative = function (keyPair) {
        if (keyPair) {
            var ret = new CryptoKeyPair();
            ret[native_] = keyPair;
            return ret;
        }
        return null;
    };
    return CryptoKeyPair;
}());
export { CryptoKeyPair };
_a = privateKey_, _b = publicKey_;
var SubtleCrypto = /** @class */ (function () {
    function SubtleCrypto() {
    }
    SubtleCrypto.prototype.digest = function (algorithm, data) {
        return new Promise(function (resolve, reject) {
            var error;
            var mode;
            switch (algorithm) {
                case 'SHA-1':
                    mode = 0;
                    break;
                case 'SHA-256':
                    mode = 1;
                    break;
                case 'SHA-384':
                    mode = 2;
                    break;
                case 'SHA-512':
                    mode = 3;
                    break;
                default:
                    error = new Error('Operation is not supported');
                    break;
            }
            if (error !== undefined) {
                reject(error);
                return;
            }
            if (data instanceof ArrayBuffer || data instanceof Uint8Array || data instanceof Uint8ClampedArray) {
                // noop
            }
            else if ((data === null || data === void 0 ? void 0 : data.BYTES_PER_ELEMENT) !== 1) {
                data = new Uint8Array(data.buffer, data.byteOffset);
            }
            else {
                reject(new TypeError('Argument 2 could not be converted to any of: ArrayBufferView, ArrayBuffer.'));
                return;
            }
            if (__ANDROID__) {
                //const instance = java.security.MessageDigest.getInstance(algorithm);
                var buffer = org.nativescript.winter_tc.Crypto.digest(mode, data);
                var ab = ArrayBuffer.from(buffer);
                if (!ab) {
                    // todo throw failure
                }
                resolve(ab);
                /* instance.update(data as any);
                const digest = instance.digest();
    
                const ab = new ArrayBuffer(digest.length);
                (<any>org).nativescript.winter_tc.Utils.copyToBuffer(ab, digest);
    
                resolve(ab);

                */
            }
            if (__IOS__) {
                var d = NSData.dataWithData(data);
                NSCCrypto.digestModeCompletion(d, mode, function (ab, error) {
                    if (!ab) {
                        // todo throw failure
                    }
                    resolve(interop.bufferFromData(ab));
                });
                // const ab = NSCCrypto.digestMode(d, mode);
                // if (!ab) {
                // 	// todo throw failure
                // }
                // resolve(interop.bufferFromData(ab));
            }
        });
    };
    SubtleCrypto.prototype.encrypt = function (algorithm, key, data) {
        return new Promise(function (resolve, reject) {
            var error;
            switch (algorithm === null || algorithm === void 0 ? void 0 : algorithm.name) {
                case 'RSA-OAEP':
                    {
                        try {
                            if (__IOS__) {
                                var hash = parseHash(key.algorithm.hash.name);
                                var d = NSData.dataWithData(data);
                                NSCCrypto.encryptRsaKeyHashDataCompletion(key.type === 'private', key[parent_], hash, d, function (ret, error) {
                                    if (ret) {
                                        resolve(interop.bufferFromData(ret));
                                    }
                                    else {
                                        reject(new Error('Failed to encrypt data'));
                                    }
                                });
                            }
                            if (__ANDROID__) {
                                var hash = parseHash(key.algorithm.hash.name);
                                var ret = org.nativescript.winter_tc.Crypto.encryptRsaOAEP(key[parent_], hash, data);
                                if (ret) {
                                    resolve(ArrayBuffer.from(ret));
                                }
                                else {
                                    reject(new Error('Failed to encrypt data'));
                                }
                            }
                        }
                        catch (error) {
                            reject(error);
                        }
                    }
                    break;
                default:
                    error = new Error('Operation is not supported');
                    break;
            }
            if (error) {
                reject(error);
                return;
            }
        });
    };
    SubtleCrypto.prototype.decrypt = function (algorithm, key, data) {
        return new Promise(function (resolve, reject) {
            var error;
            switch (algorithm === null || algorithm === void 0 ? void 0 : algorithm.name) {
                case 'RSA-OAEP':
                    {
                        try {
                            if (__IOS__) {
                                var hash = parseHash(key.algorithm.hash.name);
                                var d = NSData.dataWithData(data);
                                NSCCrypto.decryptRsaKeyHashDataCompletion(key.type === 'private', key[parent_], hash, d, function (ret, error) {
                                    if (ret) {
                                        resolve(interop.bufferFromData(ret));
                                    }
                                    else {
                                        reject(new Error('Failed to decrypt data'));
                                    }
                                });
                            }
                            if (__ANDROID__) {
                                var hash = parseHash(key.algorithm.hash.name);
                                var ret = org.nativescript.winter_tc.Crypto.decryptRsaOAEP(key[parent_], hash, data);
                                if (ret) {
                                    resolve(ArrayBuffer.from(ret));
                                }
                                else {
                                    reject(new Error('Failed to decrypt data'));
                                }
                            }
                        }
                        catch (error) {
                            reject(error);
                        }
                    }
                    break;
                default:
                    error = new Error('Operation is not supported');
                    break;
            }
            if (error) {
                reject(error);
                return;
            }
        });
    };
    SubtleCrypto.prototype.sign = function (algorithm, key, data) {
        return new Promise(function (resolve, reject) {
            var error;
            var algorithmName = typeof algorithm === 'object' ? algorithm === null || algorithm === void 0 ? void 0 : algorithm.name : algorithm;
            switch (algorithmName) {
                case 'HMAC':
                    {
                        try {
                            if (__IOS__) {
                                var hash = parseHash(key.algorithm.hash.name);
                                var d = NSData.dataWithData(data);
                                var ret = NSCCrypto.signHmacHashData(key[native_], hash, d);
                                if (ret) {
                                    resolve(interop.bufferFromData(ret));
                                }
                                else {
                                    reject(new Error('Failed to sign data'));
                                }
                            }
                            if (__ANDROID__) {
                                var algo = void 0;
                                switch (key.algorithm.hash.name) {
                                    case 'SHA-1':
                                        algo = 'HmacSHA1';
                                        break;
                                    case 'SHA-256':
                                        algo = 'HmacSHA256';
                                        break;
                                    case 'SHA-384':
                                        algo = 'HmacSHA384';
                                        break;
                                    case 'SHA-512':
                                        algo = 'HmacSHA512';
                                        break;
                                    default:
                                        error = new Error('Operation is not supported');
                                        break;
                                }
                                if (error) {
                                    reject(error);
                                    return;
                                }
                                var ab = org.nativescript.winter_tc.Crypto.signHMAC(algo, key[native_], data);
                                resolve(ArrayBuffer.from(ab));
                            }
                        }
                        catch (error) {
                            reject(error);
                        }
                    }
                    break;
                default:
                    error = new Error('Operation is not supported');
                    break;
            }
            if (error) {
                reject(error);
                return;
            }
        });
    };
    SubtleCrypto.prototype.verify = function (algorithm, key, signature, data) {
        return new Promise(function (resolve, reject) {
            var error;
            var algorithmName = typeof algorithm === 'object' ? algorithm === null || algorithm === void 0 ? void 0 : algorithm.name : algorithm;
            switch (algorithmName) {
                case 'HMAC':
                    {
                        try {
                            if (__IOS__) {
                                var hash = parseHash(key.algorithm.hash.name);
                                var s = NSData.dataWithData(signature);
                                var d = NSData.dataWithData(data);
                                var ret = NSCCrypto.verifyHmacHashSignatureData(key[native_], hash, s, d);
                                resolve(ret);
                            }
                            if (__ANDROID__) {
                                var algo = void 0;
                                switch (key.algorithm.hash.name) {
                                    case 'SHA-1':
                                        algo = 'HmacSHA1';
                                        break;
                                    case 'SHA-256':
                                        algo = 'HmacSHA256';
                                        break;
                                    case 'SHA-384':
                                        algo = 'HmacSHA384';
                                        break;
                                    case 'SHA-512':
                                        algo = 'HmacSHA512';
                                        break;
                                    default:
                                        error = new Error('Operation is not supported');
                                        break;
                                }
                                if (error) {
                                    reject(error);
                                    return;
                                }
                                var ret = org.nativescript.winter_tc.Crypto.verifyHMAC(algo, key[native_], signature, data);
                                resolve(ret);
                            }
                        }
                        catch (error) {
                            reject(error);
                        }
                    }
                    break;
                default:
                    error = new Error('Operation is not supported');
                    break;
            }
            if (error) {
                reject(error);
                return;
            }
        });
    };
    SubtleCrypto.prototype.generateKey = function (algorithm, extractable, keyUsages) {
        return new Promise(function (resolve, reject) {
            var _c, _d;
            var algorithmHash = typeof (algorithm === null || algorithm === void 0 ? void 0 : algorithm.hash) === 'object' ? (_c = algorithm === null || algorithm === void 0 ? void 0 : algorithm.hash) === null || _c === void 0 ? void 0 : _c.name : algorithm.hash;
            switch (algorithm === null || algorithm === void 0 ? void 0 : algorithm.name) {
                case 'HMAC':
                    {
                        var algo = void 0;
                        var error = void 0;
                        switch (algorithmHash) {
                            case 'SHA-1':
                                algo = 'HmacSHA1';
                                break;
                            case 'SHA-256':
                                algo = 'HmacSHA256';
                                break;
                            case 'SHA-384':
                                algo = 'HmacSHA384';
                                break;
                            case 'SHA-512':
                                algo = 'HmacSHA512';
                                break;
                            default:
                                error = new Error('Operation is not supported');
                                break;
                        }
                        if (error) {
                            reject(error);
                            return;
                        }
                        if (__ANDROID__) {
                            // const mac = javax.crypto.KeyGenerator.getInstance(algo);
                            // const key = mac.generateKey();
                            var key = org.nativescript.winter_tc.Crypto.generateKeyHMAC(algo);
                            var ret = new CryptoKey();
                            ret[algorithm_] = { name: algorithm.name, hash: { name: algorithmHash } };
                            ret[native_] = key;
                            ret[usages_] = keyUsages;
                            ret[extractable_] = extractable;
                            ret[type_] = 'secret';
                            resolve(ret);
                        }
                        if (__IOS__) {
                            var hash = parseHash(algorithmHash);
                            var key = NSCCrypto.generateKeyHmacLength(hash, (_d = algorithm === null || algorithm === void 0 ? void 0 : algorithm.length) !== null && _d !== void 0 ? _d : -1);
                            var ret = new CryptoKey();
                            ret[algorithm_] = { name: algorithm.name, hash: { name: algorithmHash } };
                            ret[native_] = key;
                            ret[usages_] = keyUsages;
                            ret[extractable_] = extractable;
                            ret[type_] = 'secret';
                            resolve(ret);
                        }
                    }
                    break;
                case 'RSA-OAEP':
                    {
                        if (__IOS__) {
                            var hash = parseHash(algorithm.hash);
                            if (hash === null) {
                                // todo throw invalid hash
                            }
                            var usages = parseUsages(keyUsages);
                            NSCCrypto.generateKeyRsaModulusLengthPublicExponentSizeHashExtractableKeyUsagesCompletion(2, algorithm.modulusLength, null, 0, hash, !!extractable, usages, function (kp, error) {
                                if (!kp) {
                                    reject(new Error('Failed to generateKey'));
                                }
                                else {
                                    var ret = CryptoKeyPair.fromNative(kp);
                                    ret.privateKey[parent_] = kp;
                                    ret.privateKey[algorithm_] = { name: algorithm.name, hash: { name: algorithm.hash }, modulusLength: algorithm.modulusLength, publicExponent: new Uint8Array([1, 0, 1]) };
                                    ret.privateKey[type_] = 'private';
                                    ret.privateKey[extractable_] = extractable;
                                    ret.publicKey[parent_] = kp;
                                    ret.publicKey[algorithm_] = { name: algorithm.name, hash: { name: algorithm.hash }, modulusLength: algorithm.modulusLength, publicExponent: new Uint8Array([1, 0, 1]) };
                                    ret.publicKey[type_] = 'public';
                                    ret.publicKey[extractable_] = extractable;
                                    resolve(ret);
                                }
                            });
                            /*
                            // ignore publicExponent for now
                            const kp = NSCCrypto.generateKeyRsaModulusLengthPublicExponentSizeHashExtractableKeyUsages(2, algorithm.modulusLength, null, 0, hash, !!extractable, usages);

                            if (!kp) {
                                reject(new Error('Failed to generateKey'));
                            } else {
                                const ret = CryptoKeyPair.fromNative(kp);
                                ret.privateKey[parent_] = kp;
                                ret.privateKey[algorithm_] = { name: algorithm.name, hash: { name: algorithm.hash }, modulusLength: algorithm.modulusLength, publicExponent: new Uint8Array([1, 0, 1]) };
                                ret.privateKey[type_] = 'private';
                                ret.privateKey[extractable_] = extractable;

                                ret.publicKey[parent_] = kp;
                                ret.publicKey[algorithm_] = { name: algorithm.name, hash: { name: algorithm.hash }, modulusLength: algorithm.modulusLength, publicExponent: new Uint8Array([1, 0, 1]) };
                                ret.publicKey[type_] = 'public';
                                ret.publicKey[extractable_] = extractable;

                                resolve(ret);
                            }
                            */
                        }
                        if (__ANDROID__) {
                            var hash = parseHash(algorithm.hash);
                            if (hash === null) {
                                // todo throw invalid hash
                            }
                            //	const usages = parseUsages(keyUsages);
                            // ignore publicExponent for now
                            var kp = org.nativescript.winter_tc.Crypto.generateKeyRsaOAEP(algorithm.modulusLength);
                            if (!kp) {
                                reject(new Error('Failed to generateKey'));
                            }
                            else {
                                var ret = CryptoKeyPair.fromNative(kp);
                                ret.privateKey[parent_] = kp;
                                ret.privateKey[algorithm_] = { name: algorithm.name, hash: { name: algorithm.hash }, modulusLength: algorithm.modulusLength, publicExponent: new Uint8Array([1, 0, 1]) };
                                ret.privateKey[type_] = 'private';
                                ret.privateKey[extractable_] = extractable;
                                ret.publicKey[parent_] = kp;
                                ret.publicKey[algorithm_] = { name: algorithm.name, hash: { name: algorithm.hash }, modulusLength: algorithm.modulusLength, publicExponent: new Uint8Array([1, 0, 1]) };
                                ret.publicKey[type_] = 'public';
                                ret.publicKey[extractable_] = extractable;
                                resolve(ret);
                            }
                        }
                    }
                    break;
                default:
                    reject(new Error(
                    // @ts-expect-error
                    "'subtle.generateKey()' is not implemented for ".concat(algorithm === null || algorithm === void 0 ? void 0 : algorithm.name, ".\n\t\t\t\t\t\t  Unrecognized algorithm name")));
                    break;
            }
        });
    };
    return SubtleCrypto;
}());
export { SubtleCrypto };
//# sourceMappingURL=SubtleCrypto.js.map